/* global top, WE */

/**
 * webEdition SDK
 *
 * webEdition CMS
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *
 * @category   we
 * @package    we_ui
 * @subpackage we_ui_layout
 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */
'use strict';

var hot = false;
WE().util.loadConsts(document, "g_l.banner");

function doUnload() {
	WE().util.jsWindow.prototype.closeAll(window);
}

function setHot() {
	hot = true;
}

function unsetHot() {
	hot = false;
}


function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window);
	var args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments));
	var url = WE().util.getWe_cmdArgsUrl(args);

	switch (args[0]) {
		case 'unsetHot':
			unsetHot();
			break;
		case "exit_banner":
			if (hot) {
				WE().util.showConfirm(window, '', WE().consts.g_l.banner.view.save_changed_banner, ["processConfirmHot", "save_banner"], ["processConfirmHot", "unsetHot"].concat(args), WE().consts.g_l.button.save, WE().consts.g_l.button.revert);
				break;
			}
			top.opener.top.we_cmd('exit_modules');
			break;
		case "new_banner":
			if (hot) {
				WE().util.showConfirm(window, '', WE().consts.g_l.banner.view.save_changed_banner, ["processConfirmHot", "save_banner"], ["processConfirmHot", "unsetHot"].concat(args), WE().consts.g_l.button.save, WE().consts.g_l.button.revert);
				break;
			}
			if (top.content.editor.edbody.loaded) {
				top.content.editor.edbody.document.we_form.ncmd.value = args[0];
				top.content.editor.edbody.submitForm();
			} else {
				window.setTimeout(we_cmd, 10, "new_banner");
			}
			break;
		case "new_bannergroup":
			if (hot) {
				WE().util.showConfirm(window, '', WE().consts.g_l.banner.view.save_changed_banner, ["processConfirmHot", "save_banner"], ["processConfirmHot", "unsetHot"].concat(args), WE().consts.g_l.button.save, WE().consts.g_l.button.revert);
				break;
			}
			if (top.content.editor.edbody.loaded) {
				top.content.editor.edbody.document.we_form.ncmd.value = args[0];
				top.content.editor.edbody.submitForm();
			} else {
				window.setTimeout(we_cmd, 10, "new_bannergroup");
			}
			break;
		case "delete_banner":
			if (!WE().util.hasPerm("DELETE_BANNER")) {
				WE().util.showMessage(WE().consts.g_l.main.no_perms, WE().consts.message.WE_MESSAGE_ERROR, window);
				return;
			}
			if (!top.content.editor.edbody.loaded && top.content.editor.edbody.we_is_home === undefined) {
				WE().util.showMessage(WE().consts.g_l.banner.view.nothing_to_delete, WE().consts.message.WE_MESSAGE_WARNING, window);
				return;
			}
			WE().util.showConfirm(window, "", WE().consts.g_l.banner.view.delete_question, ["delete_banner_do"]);
			break;
		case "delete_banner_do":
			top.content.editor.edbody.document.we_form.ncmd.value = "delete_banner";
			top.content.editor.edbody.submitForm();

			break;
		case "save_banner":
			if (WE().util.hasPerm("EDIT_BANNER") || WE().util.hasPerm("NEW_BANNER")) {
				if (top.content.editor.edbody.loaded && top.content.editor.edbody.we_is_home === undefined) {
					if (!top.content.editor.edbody.checkData()) {
						return;
					}
				} else {
					WE().util.showMessage(WE().consts.g_l.banner.view.nothing_to_save, WE().consts.message.WE_MESSAGE_WARNING, window);
					return;
				}

				top.content.editor.edbody.document.we_form.ncmd.value = args[0];
				top.content.editor.edbody.submitForm();
			} else {
				WE().util.showMessage(WE().consts.g_l.main.no_perms, WE().consts.message.WE_MESSAGE_ERROR, window);
			}
			top.content.unsetHot();
			break;
		case "banner_edit":
			if (hot) {
				WE().util.showConfirm(window, '', WE().consts.g_l.banner.view.save_changed_banner, ["processConfirmHot", "save_banner"], ["processConfirmHot", "unsetHot"].concat(args), WE().consts.g_l.button.save, WE().consts.g_l.button.revert);
				break;
			}
			top.content.editor.edbody.document.we_form.ncmd.value = args[0];
			top.content.editor.edbody.document.we_form.bid.value = args[1];
			top.content.editor.edbody.submitForm();
			break;
		case "banner_load":
			if (hot) {
				WE().util.showConfirm(window, '', WE().consts.g_l.banner.view.save_changed_banner, ["processConfirmHot", "save_banner"], ["processConfirmHot", "unsetHot"].concat(args), WE().consts.g_l.button.save, WE().consts.g_l.button.revert);
				break;
			}
			top.content.editor.edheader.location = WE().consts.dirs.WEBEDITION_DIR + 'we_showMod.php?mod=banner&pnt=edheader&page=' + args[1] + '&txt=' + args[2] + '&isFolder=' + args[3];
			top.content.editor.edfooter.location = WE().consts.dirs.WEBEDITION_DIR + 'we_showMod.php?mod=banner&pnt=edfooter';
			break;
		case "banner_code":
			WE().util.jsWindow.prototype.focus('edit_module');
			new (WE().util.jsWindow)(caller, url, "bannercode", WE().consts.size.dialog.small, WE().consts.size.dialog.smaller, true, true, true, false);
			break;
		default:
			top.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
	}
}

function setTab(tab) {
	top.content.editor.edbody.we_cmd("switchPage", tab);
}