/* global WE, weSelect, weInput, categories_edit */

/**
 * webEdition SDK
 *
 * webEdition CMS
 * $Rev: 15029 $
 * $Author: lukasimhof $
 * $Date: 2018-03-10 18:53:45 +0100 (Sat, 10 Mar 2018) $
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *
 * @category   we
 * @package    we_ui
 * @subpackage we_ui_controls
 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */
'use strict';
var ruleData = WE().util.getDynamicVar(document, 'loadVarNavigationRules', 'data-rules');

var resetFields = [
	"NavigationName", "NavigationID", "NavigationIDPath", "FolderID", "FolderIDPath", "DoctypeID", "ClassID", "ClassIDPath", "WorkspaceID"
];
var categories_edit;

function switchType(value) {
	// 1st hide all
	var allFields = document.getElementsByClassName("switch");
	var switchFields = document.getElementsByClassName("switch_" + value);
	var i;

	for (i = 0; i < allFields.length; i++) {
		allFields[i].style.display = "none";
	}

	// show needed

	for (i = 0; i < switchFields.length; i++) {
		switchFields[i].style.display = "";
	}
}

function clearNavigationForm() {
	for (var i = 0; i < resetFields.length; i++) {
		if (document.we_form[resetFields[i]]) {
			document.we_form[resetFields[i]].value = "";
		}
	}

	document.we_form.ID.value = "0";
	weSelect.removeOptions("WorkspaceID");
	removeAllCats();
}

function removeAllCats() {
	if (categories_edit.itemCount > 0) {
		while (categories_edit.itemCount > 0) {
			categories_edit.delItem(categories_edit.itemCount);
		}
	}
}

function addCat(paths) {
	for (var i = 0; i < paths.length; i++) {
		if (paths[i] !== "") {
			categories_edit.addItem();
			categories_edit.setItem(0, (categories_edit.itemCount - 1), paths[i]);
		}
	}
	categories_edit.showVariant(0);
}

function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window);
	var args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments));
	var url = WE().util.getWe_cmdArgsUrl(args);

	switch (args[0]) {
		case "addCat":
			addCat(args[1].allPaths);
			break;
		case "switchType":
			switchType(args[1]);
			break;
		case "new_navigation_rule":
			clearNavigationForm();
			break;
		case "save_navigation_rule":
			var isValid = true;
			if (document.we_form.SelectionType.options[0].selected === true) {
				isValid = WE().layout.weSuggest.checkRequired(window, "yuiAcInputFolderIDPath").valid;
			} else if (document.we_form.SelectionType.options[1] !== undefined && document.we_form.SelectionType.options[1].selected === true) {
				isValid = WE().layout.weSuggest.checkRequired(window, "yuiAcInputClassIDPath").valid;
			}
			if (isValid && WE().layout.weSuggest.checkRequired(window, "yuiAcInputNavigationIDPath").valid) {
				weInput.setValue("cmd", "save_navigation_rule");
				document.we_form.submit();
			} else {
				WE().util.showMessage(WE().consts.g_l.navigation.rule.save_error_fields_value_not_valid, WE().consts.message.WE_MESSAGE_ERROR, window);
				return false;
			}
			break;
		case "delete_navigation_rule":
			var navId;
			if ((navId = document.we_form.navigationRules.value)) {
				document.we_form.NavigationName.value = "";
				weInput.setValue("cmd", "delete_navigation_rule");
				weInput.setValue("ID", navId);
				document.we_form.submit();
			}
			break;
		case "navigation_edit_rule":
			weInput.setValue("cmd", "navigation_edit_rule");
			weInput.setValue("ID", args[1]);
			document.we_form.submit();
			break;
		case "get_workspaces":
			weInput.setValue("cmd", "get_workspaces");
			document.we_form.submit();
			break;
		case "we_selector_directory":
			new (WE().util.jsWindow)(caller, url, "we_selector_directory", WE().consts.size.dialog.big, WE().consts.size.dialog.small, true, true, true, true);
			break;
		case "we_selector_file":
		case "we_selector_image":
		case "we_selector_document":
			new (WE().util.jsWindow)(caller, url, args[0], WE().consts.size.dialog.big, WE().consts.size.dialog.medium, true, true, true, true);
			break;
		default:
			top.opener.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
	}
}

function initCat() {
	categories_edit = new (WE().util.multi_edit)("categories", window, 0, ruleData.trashButton, 400, false, '', 'we/category');
	categories_edit.addVariant();
	document.we_form.CategoriesControl.value = categories_edit.name;
	categories_edit.showVariant(0);
}

$(function () {
	initCat();
});