<?php
/**
 * webEdition CMS
 *
 * $Rev: 15003 $
 * $Author: mokraemer $
 * $Date: 2018-03-06 15:43:48 +0100 (Tue, 06 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * This class deals with the frames for the liveUpdate
 * Not much functionality here, just show the requested frame
 */
class liveUpdateFrames{
	var $Section;
	var $Data;

	function __construct(){

		// depending on section variables execute different stuff to gather
		// data for the frame


		$this->Section = we_base_request::_(we_base_request::STRING, 'section', 'frameset');

		switch($this->Section){
			case 'frameset':
				// open frameset
				$this->Data['activeTab'] = $this->getValidTab(we_base_request::_(we_base_request::STRING, 'active', ''));

				break;
			case 'tabs':
				// frame with tabs
				$this->Data['activeTab'] = we_base_request::_(we_base_request::STRING, 'active', '');
				break;

			case 'update':
				$this->processUpdateVariables();
				break;

			case 'beta':
				$this->processBeta();
				break;

			case 'updatelog':
				$this->processUpdateLogVariables();
				break;

			case 'languages':
				$this->processDeleteLanguages();
				break;
		}
	}

	function getFrame(){

		switch($this->Section){
			case 'tabs':
				return include(LIVEUPDATE_TEMPLATE_DIR . 'tabs.inc.php');
			case 'frameset':
				return $this->htmlFrameset();
			case 'beta':
				return include(LIVEUPDATE_TEMPLATE_DIR . 'beta.inc.php');
			case 'update':
				return include(LIVEUPDATE_TEMPLATE_DIR . 'update.inc.php');
			case 'languages':
				return include(LIVEUPDATE_TEMPLATE_DIR . 'languages.inc.php');
			case 'updatelog':
				return include(LIVEUPDATE_TEMPLATE_DIR . 'updatelog.inc.php');
			case 'connect':
				return include(LIVEUPDATE_TEMPLATE_DIR . 'connect.inc.php');
			default:
				return "Frame $this->Section is not known!";
		}
	}

	function getData($name){
		if(isset($this->Data[$name])){
			return $this->Data[$name];
		}
		return false;
	}

	function processBeta(){
		if(($setTestUpdate = we_base_request::_(we_base_request::BOOL, 'setTestUpdate', '-1')) !== '-1'){
			$_SESSION['weS']['testUpdate'] = $setTestUpdate;
		}
	}

	function processUpdateVariables(){
		$this->Data['lastUpdate'] = g_l('liveUpdate', '[update][neverUpdated]');

		if(($date = f('SELECT DATE_FORMAT(datum, "%d.%m.%y - %T ") FROM ' . UPDATE_LOG_TABLE . ' WHERE error=0 ORDER BY ID DESC LIMIT 1'))){
			$this->Data['lastUpdate'] = $date;
		}
	}

	function processDeleteLanguages(){
		$deletedLngs = [];
		$notDeletedLngs = [];

		if(($langs = we_base_request::_(we_base_request::STRING, 'deleteLanguages'))){
			// update prefs_table

			$GLOBALS['DB_WE']->query('UPDATE ' . PREFS_TABLE . ' SET value="' . WE_LANGUAGE . '" WHERE `key`="Language" AND value IN ("","' . implode('","', array_map('escape_sql_query', $langs)) . '")');

			// delete folders
			foreach($langs as $lng){
				if(strpos($lng, "..") === false && $lng != ''){
					if(we_base_file::deleteLocalFolder(LIVEUPDATE_SOFTWARE_DIR . '/webEdition/we/include/we_language/' . $lng, true, true)){
						$deletedLngs[] = $lng;
					} else {
						$notDeletedLngs[] = $lng;
					}
				}
			}
		}
		$this->Data['deletedLngs'] = $deletedLngs;
		$this->Data['notDeletedLngs'] = $notDeletedLngs;
	}

	function processUpdateLogVariables(){
		$db = $GLOBALS['DB_WE'];
		$this->Data['amountPerPage'] = 5;
		$show = [
			'msg' => we_base_request::_(we_base_request::BOOL, 'messages', false),
			'err' => we_base_request::_(we_base_request::BOOL, 'errors', false),
			'notice' => we_base_request::_(we_base_request::BOOL, 'notices', false),
		];
		$errors = [];
		$tmp = 0;
		foreach($show as $cur){
			if($cur){
				$errors[] = $tmp;
			}
			$tmp++;
		}
		$condition = ' WHERE ' . ($errors ? ' error IN (' . implode(',', $errors) . ')' : ' FALSE ');

		/*
		 * process update_cmd
		 */

		switch(we_base_request::_(we_base_request::STRING, 'log_cmd', '')){
			case 'deleteEntries':
				$db->query('DELETE FROM ' . UPDATE_LOG_TABLE . $condition);
				$start = 0;
				break;
			case 'nextEntries':
				$start = we_base_request::_(we_base_request::INT, 'start', 0) + $this->Data['amountPerPage'];
				break;
			case 'lastEntries':
				$start = we_base_request::_(we_base_request::INT, 'start', 0) - $this->Data['amountPerPage'];
				break;
			default:
				$start = 0;
				break;
		}

		/*
		 * Check if there are Log-Entries
		 */
		// complete amount

		$this->Data['amountMessages'] = $this->Data['amountNotices'] = $this->Data['amountErrors'] = $this->Data['allEntries'] = $this->Data['amountEntries'] = 0;

		$db->query('SELECT COUNT(ID) as amount, error FROM ' . UPDATE_LOG_TABLE . ' GROUP BY error');
		while($db->next_record()){
			$this->Data['allEntries'] += $db->f('amount');

			switch($db->f('error')){
				case 0:
					$this->Data['amountMessages'] = $db->f('amount');
					if($show['msg']){
						$this->Data['amountEntries'] += $db->f('amount');
					}
					break;
				case 1:
					$this->Data['amountErrors'] = $db->f('amount');
					if($show['err']){
						$this->Data['amountEntries'] += $db->f('amount');
					}
					break;
				case 2:
					$this->Data['amountNotices'] = $db->f('amount');
					if($show['notice']){
						$this->Data['amountEntries'] += $db->f('amount');
					}
			}
		}
		$this->Data['start'] = max(0, $start);

		if($this->Data['allEntries']){
			//There are entries available, get them
			$this->Data['logEntries'] = $db->getAllq('SELECT DATE_FORMAT(datum, "' . str_replace(' ', '&nbsp;/&nbsp;', g_l('date', '[format][mysql]')) . '") AS date, aktion AS action, versionsnummer AS version, error AS `state` FROM ' . UPDATE_LOG_TABLE . ' ' . $condition . ' ORDER BY ID DESC LIMIT ' . $this->Data['start'] . ',' . abs($this->Data['amountPerPage']));
		}
	}

	/**
	 * @return string
	 */
	private function htmlFrameset(){
		$activeTab = self::getValidTab($this->Data['activeTab']);

		we_html_tools::headerCtCharset('text/html', $GLOBALS['we']['Charset']['Backend']);
		return we_html_tools::getHtmlTop('webEdition Update', '', 'frameset', ' ') . '<body style="overflow:hidden">' .
			we_html_element::iframe('updatetabs', WEBEDITION_DIR . 'liveUpdate/liveUpdate.php?section=tabs&active=' . $activeTab, 'position: absolute;top:0;left:0;right:0;height:25px;', '', '', false) .
			we_html_element::iframe('updatecontent', WEBEDITION_DIR . 'liveUpdate/liveUpdate.php?section=' . $activeTab, 'position: absolute;top:25px;left:0;right:0;bottom:0;', '', '', false) .
			we_html_element::iframe('updateload', 'about:blank', 'display:none;', '', '', false) . '</body>
</html>';
	}

	static function htmlConnectionSuccess($errorMessage = ''){
		return liveUpdateTemplates::getHtml(g_l('liveUpdate', '[connect][headline]'), we_html_element::div([], ($errorMessage ? // servers response is error string
					g_l('liveUpdate', '[connect][connectionSuccessError]') . we_html_element::div(['class' => 'errorDiv'], '<code>' . $errorMessage . '</code>') :
					g_l('liveUpdate', '[connect][connectionSuccess]')
		)));
	}

	static function htmlConnectionError(){
		return include(LIVEUPDATE_TEMPLATE_DIR . 'connectError.inc.php');
	}

	static function htmlStateMessage(){
		$description = ($this->State === 'true' ?
			g_l('liveUpdate', '[state][descriptionTrue]') :
			g_l('liveUpdate', '[state][descriptionError]'));

		$content = we_html_element::div([], $description .
				we_html_element::div(['class' => 'errorDiv'], '<code>' . $this->Message . '</code>')
		);

		return liveUpdateTemplates::getHtml(g_l('liveUpdate', '[state][headline]'), $content);
	}

	static function getValidTab($showTab = ''){
		if(in_array($showTab, $GLOBALS['updatecmds'])){
			return $showTab;
		}
		return $GLOBALS['updatecmds'][0];
	}

	public static function getJSLangConsts(){
		return we_base_util::setLangString([
				'confirmDelete' => g_l('liveUpdate', '[updatelog][confirmDelete]'),
				'languagesDeleted' => g_l('liveUpdate', '[languages][languagesDeleted]'),
				'languagesNotDeleted' => g_l('liveUpdate', '[languages][languagesNotDeleted]'),
		]);
	}

}
