<?php
/**
 * webEdition CMS
 *
 * $Rev: 15035 $
 * $Author: mokraemer $
 * $Date: 2018-03-12 16:26:53 +0100 (Mon, 12 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */

/** the parent class of storagable webEdition classes */
abstract class we_contents_base{
	//constants for retrieving data from DB

	const LOAD_MAID_DB = 0; //default DB
	const LOAD_TEMP_DB = 1; //temporary saved files
	const LOAD_REVERT_DB = 2; //we_temporaryDocument::revert gibst nicht mehr siehe #5789
	const LOAD_SCHEDULE_DB = 3; //FIXME: remove this. We should load all data from temp table
	//constants define where to write document (guess)
	const SUB_DIR_NO = '-';
	const SUB_DIR_YEAR = 'y';
	const SUB_DIR_YEAR_MONTH = 'ym';
	const SUB_DIR_YEAR_MONTH_DAY = 'ymd';
	const ETYPE_UNKNOWN = 'ALLKEYS';
	const ETYPE_NUMBER = 'bdid';
	const ETYPE_TEXT = 'dat';
	const INTERNAL_VAR = 'vars';

	/* Name of the class => important for reconstructing the class from outside the class */
	var $ClassName = __CLASS__;
	/* In this array are all storagable class variables */
	var $persistent_slots = ['ClassName', 'Name', 'ID', 'Table', 'wasUpdate', 'InWebEdition'];
	/* Name of the Object that was createt from this class */
	var $Name = '';

	/* ID from the database record */
	var $ID = 0;

	/* database table in which the object is stored */
	var $Table = '';
	protected $LangLinks = [];

	/* Database Object */
	protected $DB_WE;

	/* optional ID for being able to manipulate the ID for DB inserts, requires a getter and setter */
	protected $insertID = 0;

	/* Flag which is set when the file is not new */
	var $wasUpdate = false;
	public $InWebEdition = false;
	var $PublWhenSave = 1;
	var $IsTextContentDoc = false;
	var $fileExists = 1;
	protected $errMsg = '';
	public $View = null;

	/* Constructor */

	function __construct(){
		$this->Name = md5(uniqid(__FILE__, true));
		$this->ClassName = get_class($this); //$this is different from self!
		$this->DB_WE = new DB_WE();
		if(isWE() || defined('WE_SIDEBAR')){
			$this->InWebEdition = true;
			//FIXME: add in 8.1
			//$this->View = we_view_base::getView($this);
		}
	}

	//Overwrite
	public function we_new(){

	}

	//Overwrite
	abstract public function we_initSessDat($sessDat);

	/* Intialize the class. If $sessDat (array) is set, the class will be initialized from this array */

	function init($sessDat = ''){
		$this->we_new();
	}

	/* set the protected variable insertID */

	function setInsertID($insertID){
		$this->insertID = $insertID;
	}

	/* get the protected variable insertID */

	function getInsertID(){
		return $this->insertID;
	}

	/* returns the url $in with $we_transaction appended */

	public static function url($in){
		return $in . (strpos($in, '?') !== FALSE ? '&' : '?') . 'we_transaction=' . $GLOBALS['we_transaction'];
	}

	/* returns the code for a hidden " we_transaction-field" */

	public static function hiddenTrans(){
		return we_html_element::hidden('we_transaction', $GLOBALS['we_transaction']);
	}

	/* must be overwritten by child */

	abstract function saveInSession(&$save, $toFile = false);

	/* creates a text-input field for entering Data that will be stored at the $elements Array */

	function formInput($name, $size = 25, $type = 'txt'){
		return $this->formTextInput($type, $name, (g_l('weClass', '[' . $name . ']') ?: $name), $size);
	}

	/* creates a color field. when user clicks, a colorchooser opens. Data that will be stored at the $elements Array */

	function formColor($width, $name, $type = 'txt', $height = 18, $isTag = false){
		$value = $this->getElement($name);
		if(!$isTag){
			$width -= 4;
		}
		$formname = 'we_' . $this->Name . '_' . $type . '[' . $name . ']';
		$out = we_html_element::hidden($formname, $this->getElement($name)) .
			we_html_element::span([
				'style' => 'display:block;width:' . $width . 'px;height:' . $height . 'px;background-color: ' . ($value ?: 'inherit') . ';border:1px solid black;',
				'onclick' => "setScrollTo();we_cmd('openColorChooser','" . $formname . "',document.we_form.elements['" . $formname . "'].value);"
				], '&nbsp;');
		return g_l('weClass', '[' . $name . ']', true) !== false ? we_html_tools::formElementTable($out, g_l('weClass', '[' . $name . ']')) : $out;
	}

	function formTextInput($elementtype, $name, $text, $size = 24, $maxlength = '', array $attribs = [], $textalign = 'left', $textclass = 'defaultfont'){
		if(!$elementtype){
			$ps = $this->$name;
		}
		return we_html_tools::formElementTable(we_html_tools::textInput(($elementtype ? ('we_' . $this->Name . '_' . $elementtype . '[' . $name . ']') : ('we_' . $this->Name . '_' . $name)), ($elementtype ? $this->getElement($name) : $ps), $maxlength, $attribs), $text, $textalign, $textclass);
	}

	function formInputField($elementtype, $name, $text, $width, $maxlength = '', array $attribs = []){
		list($realName) = explode('#', $name);
		if(!$elementtype){
			$ps = $this->$realName;
		}
		return we_html_element::div(['style' => "position:relative"], $text .
				we_html_element::br() .
				we_html_tools::textInput('we_' . $this->Name . '_' . ($elementtype ? $elementtype . '[' . $name . ']' : $name), ($elementtype && ($elVal = $this->getElement($realName)) ? $elVal : (isset($GLOBALS['meta'][$realName]) ? $GLOBALS['meta'][$realName]['default'] : (isset($ps) ? $ps : '') )), $maxlength, $attribs, isset($attribs['type']) ? $attribs['type'] : 'txt', $width));
	}

	protected function formTextArea($elementtype, $name, $text, $rows = 0, $cols = 0, array $attribs = [], $optional = ''){
		return we_html_element::div(['style' => "position:relative"], $text .
				we_html_element::br() .
				self::htmlTextArea(($elementtype ? ('we_' . $this->Name . '_' . $elementtype . '[' . $name . ']') : ('we_' . $this->Name . '_' . $name)), $rows, $cols, $this->getElement($name), $attribs) .
				$optional
		);
	}

	protected function formSelectFromArray($elementtype, $name, array $vals, $text, $size = 1, $multiple = false, array $attribs = []){
		$pop = we_html_tools::select('we_' . $this->Name . '_' . ($elementtype ? $elementtype . '[' . $name . ']' : $name), $vals, $size, ($elementtype ? [$this->getElement($name)] : $this->$name), $multiple, $attribs);
		return we_html_tools::formElementTable($pop, $text, 'left', 'defaultfont');
	}

	static function htmlTextArea($name, $rows = 10, $cols = 30, $value = '', array $attribs = []){
		return we_html_element::textArea(array_merge(['name' => trim($name),
				'class' => 'defaultfont wetextarea',
				($rows ? 'rows' : '') => abs($rows),
				($cols ? 'cols' : '') => abs($cols),
					], $attribs
				), ($value ? (oldHtmlspecialchars($value)) : ''));
	}

	/* creates a select field for entering Data that will be stored at the $elements Array */

	protected function formSelectElement($width, $name, $values, $type = 'txt', $size = 1, array $attribs = []){
		return we_html_tools::formElementTable(
				we_html_tools::select('we_' . $this->Name . '_' . $type . '[' . $name . ']', $values, $size, $this->getElement($name), false, array_merge(['class' => 'defaultfont',
					'width' => $width,
						], $attribs))
				, g_l('weClass', '[' . $name . ']'));
	}

	/* creates a text-input field for entering Data that will be stored at the $elements Array and shows information from another Element */

	protected function formInput2($width, $name, $size, $type = 'txt', array $attribs = [], $infoname = ''){
		$infotext = $infoname ? ' (' . (g_l('weClass', '[' . $infoname . ']', true) ?: $infoname) . ': ' . $this->getElement($infoname) . ')' : '';
		list($origname) = explode('#', $name);
		return $this->formInputField($type, $name, (g_l('weClass', '[' . $origname . ']', true) ?: $origname) . $infotext, $width, '', $attribs);
	}

	protected function formSelect2($width, $name, $table, $sqlFrom, $text, $sqlTail = '', $size = 1, $selectedIndex = '', $multiple = false, $onChange = '', array $attribs = [
	], $textalign = 'left', $textclass = 'defaultfont', $precode = '', $postcode = '', $firstEntry = '', $gap = 20){
		$vals = [];
		if($firstEntry){
			$vals[$firstEntry[0]] = $firstEntry[1];
		}
		//FIXME: remove $sqlFrom or val/txt
		$vals = $vals + $this->DB_WE->getAllFirstq('SELECT ' . $sqlFrom . ' FROM ' . $table . ' WHERE ' . $sqlTail, false);

		$vals = we_html_tools::groupArray($vals, false, 1);
		$myname = 'we_' . $this->Name . '_' . $name;

		$ps = $this->$name;

		$pop = we_html_tools::select($myname . ($multiple ? 'Tmp' : ''), $vals, $size, $ps, $multiple, array_merge(['onchange' => $onChange . ($multiple ? ";var we_sel='';for(i=0;i<this.options.length;i++){if(this.options[i].selected){we_sel += (this.options[i].value + ',');};};if(we_sel){we_sel=we_sel.substring(0,we_sel.length-1)};this.form.elements['" . $myname . "'].value=we_sel;" : '')
					], $attribs), 'value', $width);

		if($precode || $postcode){
			$pop = '<table class="default"><tr>' . ($precode ? ('<td style="padding-right:' . $gap . 'px;">' . $precode . '</td>') : '') . '<td>' . $pop . '</td>' . ($postcode ? ('<td>' . $postcode . '</td>') : '') . '</tr></table>';
		}
		return ($multiple ? we_html_element::hidden($myname, $selectedIndex) : '') . we_html_tools::formElementTable($pop, $text, $textalign, $textclass);
	}

	protected function formSelect4($width, $name, $table, $sqlFrom, $text, $sqlTail = '', $size = 1, $selectedIndex = '', $multiple = false, $onChange = '', array $attribs = [
	], $textalign = 'left', $textclass = 'defaultfont', $precode = '', $postcode = '', array $firstEntry = [], $gap = 20){
		$vals = [];
		if($firstEntry){
			$vals[$firstEntry[0]] = $firstEntry[1];
		}
		//array_merge would change the numeric order of keys!
		$vals = $vals + $this->DB_WE->getAllFirstq('SELECT ' . $sqlFrom . ' FROM ' . $this->DB_WE->escape($table) . ' WHERE ' . $sqlTail, false);

		$myname = 'we_' . $this->Name . '_' . $name;

		$pop = we_html_tools::select($myname, $vals, $size, $selectedIndex, $multiple, array_merge(['onchange' => $onChange], $attribs), 'value', $width);
		if($precode || $postcode){
			$pop = '<table class="default"><tr>' . ($precode ? ('<td style="padding-right:' . $gap . 'px;">' . $precode . '</td>') : '') . '<td>' . $pop . '</td>' . ($postcode ? ('<td>' . $postcode . '</td>') : '') . '</tr></table>';
		}
		return we_html_tools::formElementTable($pop, $text, $textalign, $textclass);
	}

	public function initByID($ID, $Table = FILE_TABLE, $from = self::LOAD_MAID_DB){
		$this->ID = intval($ID);
		$this->Table = ($Table ?: FILE_TABLE);
		$this->we_load($from);
		//$GLOBALS['we_ID'] = intval($ID); //FIXME: check if we need this !
		//$GLOBALS['we_Table'] = $this->Table;
		// init Customer Filter !
		$this->initWeDocumentCustomerFilterFromDB();
	}

	/**
	 * inits weDocumentCustomerFilter from db regarding the modelId
	 * is called from "we_textContentDocument::we_load"
	 * @see we_textContentDocument::we_load
	 */
	protected function initWeDocumentCustomerFilterFromDB(){
		if(isset($this->documentCustomerFilter) && we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$this->documentCustomerFilter = we_customer_documentFilter::getFilterOfDocument($this, $this->DB_WE);
		}
	}

	public function we_load($from = self::LOAD_MAID_DB){
		$this->getPersistentSlotsFromDB();
	}

	public function we_save($resave = 0){
		$this->wasUpdate = $this->ID > 0;
		return $this->savePersistentSlotsToDB();
	}

	public function we_delete(){
		we_base_delete::deleteEntry($this->ID, $this->Table, true, false, $this->DB_WE);
	}

	public function we_publish(/* $DoNotMark = false, $saveinMainDB = true */){
		return true; // overwrite
	}

	public function we_unpublish(/* $DoNotMark = false */){
		return true; // overwrite
	}

	public function we_republish(){
		return true;
	}

	protected function setElementsFromHTTP(){
		if($_REQUEST){
			// do not set REQUEST VARS into the document
			switch(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0)){
				case 'switch_edit_page':
					if(we_base_request::_(we_base_request::STRING, 'we_cmd', false, 3)){
						return true;
					}
					break;
				case 'save_document':
					if(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 7) === 'save_document'){
						return true;
					}
					break;
			}

			$regs = [];
			foreach($_REQUEST as $n => $v){
				if(preg_match('#^we_' . preg_quote($this->Name, '#') . '_([^\[]+)$#', $n, $regs)){
					$this->setPersistentSlot($regs[1], $v);
				}
			}
		}
	}

	protected function setPersistentSlot($name, $value){
		if(in_array($name, $this->persistent_slots)){
			$this->$name = $value;
		}
	}

	protected function getPersistentSlotsFromDB($felder = '*'){
		$fields = getHash('SELECT ' . $felder . ' FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->ID), $this->DB_WE);
		if($fields){
			foreach($fields as $k => $v){
				if($k && in_array($k, $this->persistent_slots)){
					$this->{$k} = $v;
				}
			}
		} else {
			$this->fileExists = 0;
		}
	}

	protected function savePersistentSlotsToDB(array $felder = []){
		$tableInfo = $this->DB_WE->metadata($this->Table, we_database_base::META_NAME);
		$feldArr = $felder ? $felder : $this->persistent_slots;
		$fields = [];
		if(!$this->wasUpdate && $this->insertID && f('SELECT 1 FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->insertID) . ' LIMIT 1', '', $this->DB_WE)){
			return false;
		}
		foreach($tableInfo as $fieldName){
			if(in_array($fieldName, $feldArr)){
				$val = isset($this->$fieldName) ? $this->$fieldName : '';

				switch($fieldName){
					case 'ID':
						if(!$this->wasUpdate && $this->insertID){//for Apps to be able to manipulate Insert-ID
							$fields['ID'] = $this->insertID;
							$this->insertID = 0;
						}
						break;
					case 'Filehash':
						$fields['Filehash'] = $val !== null ? sql_function('x\'' . $val . '\'') : sql_function('NULL');
						break;
					default:
						$fields[$fieldName] = $val;
				}
			}
		}
		if($fields){
			$where = ($this->wasUpdate || $this->ID ? ' WHERE ID=' . intval($this->ID) : '');
			$ret = (bool) ($this->DB_WE->query(($where ? 'UPDATE ' : 'INSERT INTO ') . $this->DB_WE->escape($this->Table) . ' SET ' . we_database_base::arraySetter($fields) . $where));
			$this->ID = ($this->ID ?: $this->DB_WE->getInsertId());
			return $ret;
		}
		return false;
	}

	function setDocumentControlElements(){
		//	function is overwritten in we_webEditionDocument
	}

	function executeDocumentControlElements(){
		//	function is overwritten in we_webEditionDocument
	}

	function isValidEditPage($editPageNr){
		return (is_array($this->EditPageNrs) ?
			in_array($editPageNr, $this->EditPageNrs) :
			false);
	}

	protected function updateRemoteLang($db, $id, $lang, $type){
		//overwrite if needed <= diese verwenden!
	}

	/**
	 * If documents, objects, folders and docTypes are saved and there is no LANGLINK_SUPPORT we must check, whether there is a change of language:
	 * if so, we must delete eventual entries in tblLangLink (entered before LANGLINK_SUPPORT wa stopped. <= TODO: Merge this with next method!
	 */
	protected function checkRemoteLanguage($table, $isfolder = false){
		if(($newLang = we_base_request::_(we_base_request::STRING, 'we_' . $this->Name . '_Language'))){
			$isobject = ($table === OBJECT_FILES_TABLE);
			$type = ($isfolder && !$isobject) ? stripTblPrefix(FILE_TABLE) : ($isobject ? 'tblObjectFiles' : stripTblPrefix($table));

			$delete = f('SELECT 1 FROM ' . LANGLINK_TABLE . ' WHERE DLocale!="' . $this->DB_WE->escape($newLang) . '" AND DocumentTable="' . $this->DB_WE->escape($type) . '" AND IsObject=' . intval($isobject) . ' AND IsFolder=' . intval($isfolder) . ' AND DID=' . intval($this->ID) . ' LIMIT 1', '', $this->DB_WE);

			if($delete){
				$this->DB_WE->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DID=' . intval($this->ID) . ' AND DocumentTable="' . $this->DB_WE->escape($type) . '" AND IsFolder=' . intval($isfolder) . ' AND IsObject=' . intval($isobject));
				if(!$isfolder){
					$this->DB_WE->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE LDID=' . intval($this->ID) . ' AND DocumentTable="' . $this->DB_WE->escape($type) . '" AND IsFolder=0 AND IsObject=' . intval($isobject));
				}
			}
		}
	}

	/**
	 * Before writing LangLinks to the db, we must check the Document-Locale: if it has changed, we must update or clear
	 * existing LangLinks from and to this document.
	 */
	protected function setLanguageLink(array $LinkArray, $documentTable, $isfolder = false, $isobject = false){
		if(!(LANGLINK_SUPPORT)){
			return true;
		}
		$newLang = $this->Language;
		if(!$newLang){
			return false;
		}
		$type = stripTblPrefix($documentTable);
		if($documentTable !== DOC_TYPES_TABLE){
			$LangLinkArray = [];
			foreach($LinkArray as $lang => $link){
				$LangLinkArray[$lang] = $link['id'];
			}
		} else {
			$LangLinkArray = $LinkArray;
		}

		$db = new DB_WE();
		$ownDocumentTable = ($isfolder && $isobject) ? FILE_TABLE : $documentTable;
		$origLinks = [];

		if(!$isfolder){
			$oldLang = f('SELECT Language FROM ' . $db->escape($ownDocumentTable) . ' WHERE ID=' . intval($this->ID), '', $db);
			if($newLang != $oldLang){// language changed
				// what langs where linked before document-language changed?
				$this->DB_WE->query('SELECT * FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $this->DB_WE->escape($type) . '" AND DID=' . intval($this->ID) . ' AND IsObject = ' . ($isobject ? 1 : 0) . ' AND IsFolder = ' . intval($isfolder));
				$origLinks = $this->DB_WE->getAllFirst(false);
				$origLangs = array_keys($origLinks);
				// because of UNIQUE-Indexes we do first delete obsolete entries in tblLangLink
				// => after optimizing executeSetLanguageLink() this will be obsolete!
				$this->DB_WE->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE (DID=' . intval($this->ID) . ' OR LDID=' . intval($this->ID) . ') AND IsFolder=0 AND IsObject=' . ($isobject ? 1 : 0) . ' AND DocumentTable="' . $this->DB_WE->escape($type) . '"');

				// links FROM folders to the actual we_document/object must be updated right here.
				// if updating leads to conflict, we must delete a link
				$DB_WE2 = new DB_WE();
				$this->DB_WE->query('SELECT DID,(DLocale="' . $this->DB_WE->escape($newLang) . '"||(SELECT 1 FROM ' . LANGLINK_TABLE . ' WHERE DID=l.DID AND IsFolder=1 AND Locale="' . $this->DB_WE->escape($newLang) . '" LIMIT 1)) as `del` FROM ' . LANGLINK_TABLE . ' l WHERE LDID=' . intval($this->ID) . ' AND IsFolder=1');
				while($this->DB_WE->next_record()){
					if($this->DB_WE->f('del')){
						$DB_WE2->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE LDID=' . intval($this->ID) . ' AND DID=' . $this->DB_WE->f('DID') . ' AND IsFolder=1');
					} else {
						$DB_WE2->query('UPDATE ' . LANGLINK_TABLE . ' SET Locale="' . $this->DB_WE->escape($newLang) . '" WHERE LDID=' . intval($this->ID) . ' AND DID=' . $this->DB_WE->f('DID') . ' AND IsFolder=1');
					}
				}
				// if there is no conflict we can set new links and call prepareSetLanguageLinks()
				if(!in_array($newLang, $origLangs)){
					return ($this->prepareSetLanguageLink($LangLinkArray, $origLinks, true, $newLang, $type, $isfolder, $isobject, $ownDocumentTable)) ? true : false;
				}
				/*
				  $jsCmd = new we_base_jsCmd();
				  $jsCmd->addMsg(g_l('weClass', '[languageLinksLocaleChanged]'), we_base_util::WE_MESSAGE_NOTICE);
				  echo we_html_tools::getHtmlTop('', '', '', $jsCmd->getCmds(), we_html_element::body()); */
				return true;
			}
			//default case: there was now change of page language. Loop method call to another method, preparing LangLinks
			return ($this->prepareSetLanguageLink($LangLinkArray, $origLinks, false, $oldLang, $type, $isfolder, $isobject, $ownDocumentTable)) ? true : false;
		}//isfolder
		if(f('SELECT DLocale FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $this->DB_WE->escape($type) . '" AND IsObject=' . ($isobject ? 1 : 0) . ' AND IsFolder=1 AND DID=' . intval($this->ID), '', $this->DB_WE) != $newLang){
			$this->DB_WE->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DID=' . intval($this->ID) . ' AND DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND IsFolder=1 AND IsObject=' . ($isobject ? 1 : 0));
		}
		return ($this->prepareSetLanguageLink($LangLinkArray, $origLinks, false, $newLang, $type, $isfolder, $isobject, $ownDocumentTable)) ? true : false;
	}

	/**
	 * In this method the links of $LangLinkArray are testet twice:
	 * 1) We only write new or changed LangLinks to db, if LangLink-Locale and Locale of the targe-document/object fit together.
	 * 2) In recursive-mode we only one document/object to another, if their respective link-chains are not in conflict.
	 */
	private function prepareSetLanguageLink(array $LangLinkArray, $origLinks, $langChange, $ownLocale, $type, $isfolder, $isobject, $documentTable){

		if(in_array(0, $LangLinkArray) || in_array('', $LangLinkArray)){
			if(!$langChange){
				$origLinks = $this->DB_WE->getAllFirstq('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $this->DB_WE->escape($type) . '" AND DID=' . intval($this->ID) . ' AND IsObject=' . ($isobject ? 1 : 0) . ' AND IsFolder=' . ($isfolder ? 1 : 0), false);
			}
			$tmpLangLinkArray = [];
			foreach($LangLinkArray as $locale => $LDID){

				if(!($LDID == '' || $LDID == 0 || $LDID == -1)){
					$tmpLangLinkArray[$locale] = $LDID;
				} elseif(array_key_exists($locale, $origLinks)){
					$tmpLangLinkArray[$locale] = -1;
				}
			}
			$LangLinkArray = $tmpLangLinkArray;
		}

		$k = 0;
		foreach($LangLinkArray as $locale => $LDID){
			$k = ($locale == $ownLocale) ? $k : $k + 1;
			$newOrChanged = (($actualLDID = f('SELECT LDID FROM ' . LANGLINK_TABLE . ' WHERE Locale="' . $this->DB_WE->escape($locale) . '" AND DID=' . intval($this->ID) . ' AND DocumentTable="' . $this->DB_WE->escape($type) . '" AND IsObject=' . ($isobject ? 1 : 0) . ' AND IsFolder=' . ($isfolder ? 1 : 0), '', $this->DB_WE)) ?
				($actualLDID != $LDID) : //changed
				true); //new

			if(($newOrChanged || $langChange) && !($LDID == -1)){

				// from Folders, links lead only to documents, never to objects
				//$fileTable = $isfolder ? FILE_TABLE : ($isobject ? OBJECT_FILES_TABLE : FILE_TABLE);

				if(($fileLang = f('SELECT Language FROM ' . $this->DB_WE->escape($documentTable) . ' WHERE ID=' . intval($LDID), '', $this->DB_WE))){
					if($fileLang != $locale){
						t_e(sprintf(g_l('weClass', '[languageLinksLangNotok]'), $locale, $fileLang, $locale));
						/* $jsCmd = new we_base_jsCmd();
						  $jsCmd->addMsg(sprintf(g_l('weClass', '[languageLinksLangNotok]'), $locale, $fileLang, $locale), we_base_util::WE_MESSAGE_NOTICE);
						  echo we_html_tools::getHtmlTop('', '', '', $jsCmd->getCmds(), we_html_element::body()); */
						return true;
					}
					if(!$isfolder){
						$setThisLink = true;
						$this->DB_WE->query('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $this->DB_WE->escape($type) . '" AND DID=' . intval($this->ID) . ' AND IsObject=' . ($isobject ? 1 : 0) . ' AND IsFolder=' . ($isfolder ? 1 : 0));
						$actualLinks = $this->DB_WE->getAllFirst(false);
						$actualLangs = array_keys($actualLinks);
						$actualLangs[] = $ownLocale;

						$this->DB_WE->query('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $this->DB_WE->escape($type) . '" AND DID=' . intval($LDID) . ' AND IsObject=' . ($isobject ? 1 : 0) . ' AND IsFolder=' . ($isfolder ? 1 : 0));
						$targetLinks = $this->DB_WE->getAllFirst(false);
						$targetLangs = array_keys($targetLinks);

						if(count($actualLangs) > 1 || $targetLangs){
							$intersect = array_intersect($actualLangs, $targetLangs);
							$setThisLink = $intersect ? false : true;
						}

						if(!$newOrChanged){
							$setThisLink = true;
						}

						if(!$setThisLink){
							/* $jsCmd = new we_base_jsCmd();
							  $jsCmd->addMsg(sprintf(g_l('weClass', '[languageLinksConflicts]'), $locale), we_base_util::WE_MESSAGE_NOTICE);
							  echo we_html_tools::getHtmlTop('', '', '', $jsCmd->getCmds(), we_html_element::body()); */
							t_e(sprintf(g_l('weClass', '[languageLinksConflicts]'), $locale));
							return true;
						}
						// instead of modifying db-Enries, we delete them and create new ones
						if(!empty($actualLinks[$locale])){
							$deleteObsoleteArray = $actualLinks;
							$deleteObsoleteArray[$locale] = -1;
							$this->executeSetLanguageLink($deleteObsoleteArray, $type, $isfolder, $isobject);
						}

						$preparedLinkArray = $actualLinks;
						$preparedLinkArray[$locale] = $LDID;
						foreach($targetLinks as $targetLocale => $targetLDID){
							$preparedLinkArray[$targetLocale] = $targetLDID;
						}
						$this->executeSetLanguageLink($preparedLinkArray, $type, $isfolder, $isobject);
					} else {//!isfolder
						if(f('SELECT 1 FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $this->DB_WE->escape($type) . '" AND DLocale="' . $this->DB_WE->escape($ownLocale) . '" AND Locale="' . $this->DB_WE->escape($locale) . '" AND LDID=' . intval($LDID) . ' AND IsObject=' . ($isobject ? 1 : 0) . ' AND IsFolder=1 LIMIT 1', '', $this->DB_WE)){//conflict
							/* $jsCmd = new we_base_jsCmd();
							  $jsCmd->addMsg(sprintf(g_l('weClass', '[languageLinksConflicts]'), $locale), we_base_util::WE_MESSAGE_NOTICE);
							  echo we_html_tools::getHtmlTop('', '', '', $jsCmd->getCmds(), we_html_element::body()); */
							t_e(sprintf(g_l('weClass', '[languageLinksConflicts]'), $locale));
							return true;
						}
						$actualLinks = [];
						$this->DB_WE->query('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $this->DB_WE->escape($type) . '" AND DID=' . intval($this->ID) . ' AND IsObject=' . ($isobject ? 1 : 0) . ' AND IsFolder=' . ($isfolder ? 1 : 0));
						while($this->DB_WE->next_record()){
							$actualLinks[$this->DB_WE->f('Locale')] = $this->DB_WE->f('LDID');
						}
						$actualLinks[$locale] = $LDID;
						$this->executeSetLanguageLink($actualLinks, $type, $isfolder, $isobject);
					}
				}
			}// end of new or changed link
			else {//delete links
				$actualLinks = [];
				$this->DB_WE->query('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $this->DB_WE->escape($type) . '" AND DID=' . intval($this->ID) . ' AND IsObject=' . intval($isobject) . ' AND IsFolder=' . intval($isfolder));
				while($this->DB_WE->next_record()){
					$actualLinks[$this->DB_WE->f('Locale')] = $this->DB_WE->f('LDID');
				}
				$preparedLinkArray = $actualLinks;
				$preparedLinkArray[$locale] = $LDID;
				$this->executeSetLanguageLink($preparedLinkArray, $type, $isfolder, $isobject);
			}
		}//foreach
		return true;
	}

	//FIXME: in this method tblLangLink.ID is used and a lot of things are obsolete => to be cleaned in 6.3.1
	private function executeSetLanguageLink($LangLinkArray, $type, $isfolder = false, $isobject = false){
		if(!is_array($LangLinkArray)){
			return;
		}
		$this->DB_WE->query('SELECT * FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $this->DB_WE->escape($type) . '" AND DID=' . intval($this->ID) . ' AND IsFolder=' . ($isfolder ? 1 : 0) . ' AND IsObject=' . ($isobject ? 1 : 0));
		$orig = $this->DB_WE->getAll();
		if(!$isfolder){//folders never have backlinks BUT the document linked to the folder CAN have them if linked to another document
			$lids = [];
			foreach($orig as $cur){
				$lids[] = $cur['LDID'];
			}
			if($lids){
				$this->DB_WE->query('SELECT * FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $this->DB_WE->escape($type) . '" AND DID IN (' . implode(',', $lids) . ')');
			}
			$orig = array_merge($orig, $this->DB_WE->getAll());
		}

		foreach($LangLinkArray as $locale => $LDID){ //obsolete if we call executeSetLanguageLink with only the link to bechanged (instead of whole $LangLinkArray)
			if(($ID = f('SELECT ID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $this->DB_WE->escape($type) . '" AND DID=' . intval($this->ID) . ' AND Locale="' . $this->DB_WE->escape($locale) . '" AND isFolder=' . intval($isfolder) . ' AND IsObject=' . intval($isobject), 'ID', $this->DB_WE))){
				$this->DB_WE->query('UPDATE ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter(['LDID' => $LDID,
						'DLocale' => $this->Language
					]) . ' WHERE ID=' . intval($ID) . ' AND DocumentTable="' . $this->DB_WE->escape($type) . '"');
			} elseif($locale != $this->Language && $LDID > 0){
				$this->DB_WE->query('INSERT INTO ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter(['DID' => $this->ID,
						'DLocale' => $this->Language,
						'IsFolder' => $isfolder,
						'IsObject' => $isobject,
						'LDID' => $LDID,
						'Locale' => $locale,
						'DocumentTable' => $type
				]));
			}

			if(!$isfolder && $LDID && $LDID != $this->ID){
				if(($ID = f('SELECT ID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $this->DB_WE->escape($type) . '" AND DID=' . intval($LDID) . ' AND Locale="' . $this->DB_WE->escape($this->Language) . '" AND IsObject=' . ($isobject ? 1 : 0), '', $this->DB_WE))){
					if($LDID > 0){
						$this->DB_WE->query('UPDATE ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter(['DID' => $LDID,
								'DLocale' => $locale,
								'LDID' => $this->ID,
								'Locale' => $this->Language
							]) . ' WHERE ID=' . intval($ID) . ' AND DocumentTable="' . $this->DB_WE->escape($type) . '"');
					} elseif($LDID < 0){// here we could delete istead of update (and then delete later...)
						$this->DB_WE->query('UPDATE ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter(['DID' => $LDID, //FIXME: DID is unsigned => result=0!
								'DLocale' => $locale,
								'LDID' => 0,
								'Locale' => $this->Language,
							]) . ' WHERE ID=' . intval($ID) . ' AND DocumentTable="' . $this->DB_WE->escape($type) . '"');
					}
				} elseif($LDID > 0){
					$this->DB_WE->query('INSERT INTO ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter(['DID' => $LDID,
							'DLocale' => $locale,
							'LDID' => $this->ID,
							'Locale' => $this->Language,
							'IsObject' => ($isobject ? 1 : 0),
							'DocumentTable' => $type
					]));
				}
			}

			if(!$isfolder && $LDID < 0 && $LDID != $this->ID){
				if($LDID > 0){// never happens!
					$this->DB_WE->query('REPLACE INTO ' . LANGLINK_TABLE . ' SET DID=' . intval($LDID) . ', DLocale="' . $this->DB_WE->escape($locale) . '", LDID=' . intval($this->ID) . ', Locale="' . $this->DB_WE->escape($this->Language) . '",IsObject=' . ($isobject ? 1 : 0) . ', DocumentTable="' . $this->DB_WE->escape($type) . '"');
				}
			}
		}//foreach

		if(!$isfolder){
			foreach($LangLinkArray as $locale => $LDID){
				if($LDID > 0){
					$rows = $this->DB_WE->getAllq('SELECT * FROM ' . LANGLINK_TABLE . ' WHERE DID=' . intval($this->ID) . ' AND DocumentTable="' . $this->DB_WE->escape($type) . '" AND IsObject=' . ($isobject ? 1 : 0));
					if(count($rows) > 1){
						foreach($rows as $i => $row){
							$j = ($i + 1) % count($rows);
							if($rows[$i]['LDID'] && $rows[$j]['LDID']){
								$this->DB_WE->query('REPLACE INTO ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter(['DID' => $rows[$i]['LDID'],
										'DLocale' => $rows[$i]['Locale'],
										'LDID' => $rows[$j]['LDID'],
										'Locale' => $rows[$j]['Locale'],
										'IsObject' => ($isobject ? 1 : 0),
										'DocumentTable' => $type
								]));
							}
						}
					}
				} elseif($LDID < 0){
					foreach($orig as $origrow){
						if($origrow['DLocale'] == $locale){
							$this->DB_WE->query('SELECT ID FROM ' . LANGLINK_TABLE . ' WHERE DID=' . intval($origrow['DID']) . ' AND DLocale="' . $this->DB_WE->escape($locale) . '" AND DocumentTable="' . $this->DB_WE->escape($type) . '" AND IsObject=' . ($isobject ? 1 : 0));
							if(($ids = $this->DB_WE->getAll(true))){
								$this->DB_WE->query('UPDATE ' . LANGLINK_TABLE . ' SET LDID=0 WHERE ID IN(' . implode(',', $ids) . ') AND DocumentTable="' . $this->DB_WE->escape($type) . '"');
							}
						}
						if($origrow['Locale'] == $locale){
							$this->DB_WE->query('SELECT ID FROM ' . LANGLINK_TABLE . ' WHERE LDID=' . intval($origrow['LDID']) . ' AND Locale="' . $this->DB_WE->escape($locale) . '" AND DocumentTable="' . $this->DB_WE->escape($type) . '" AND IsObject=' . ($isobject ? 1 : 0));
							if(($ids = $this->DB_WE->getAll(true))){
								$this->DB_WE->query('UPDATE ' . LANGLINK_TABLE . ' SET LDID=0 WHERE ID IN(' . implode(',', $ids) . ') AND DocumentTable="' . $this->DB_WE->escape($type) . '"');
							}
						}
					}
				}
			}
		}
		$this->DB_WE->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DID=0 OR LDID=0');
	}

	/*	 * returns error-messages recorded during an operation, currently only save is used */

	public function getErrMsg(){
		return ($this->errMsg ? '\n' . str_replace("\n", '\n', $this->errMsg) : '');
	}

}
