<?php

/**
 * webEdition CMS
 *
 * $Rev: 15016 $
 * $Author: mokraemer $
 * $Date: 2018-03-08 11:35:51 +0100 (Thu, 08 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* the parent class for tree-objects */
abstract class we_contents_root extends we_contents_base{
	const USER_HASACCESS = 1;
	const FILE_LOCKED = -3;
	const USER_NO_PERM = -2;
	const USER_NO_SAVE = -4;
	const FILE_NOT_IN_USER_WORKSPACE = -1;
	const EDITOR_HEADER = 1;
	const EDITOR_FOOTER = 2;

	var $IsClassFolder = 0;
	/* ParentID of the object (ID of the Parent-Folder of the Object) */
	var $ParentID = 0;

	/* Parent Path of the object (Path of the Parent-Folder of the Object) */
	var $ParentPath = '/';

	/* The Text that will be shown in the tree-menue */
	var $Text = '';

	/* Filename of the file */
	var $Filename = '';

	/* Path of the File  */
	var $Path = '';

	/* OldPath of the File => used internal  */
	var $OldPath = '';

	/* Creation Date as UnixTimestamp  */
	var $CreationDate = 0;

	/* Modification Date as UnixTimestamp  */
	var $ModDate = 0;

	/* Flag which is set, when the file is a folder  */
	var $IsFolder = 0;

	/* ContentType of the Object  */
	public $ContentType = '';

	/* array which holds the content of the Object */
	var $elements = [];
	private $wasMoved = false;

	/* Number of the EditPage when editor(we_base_jsCmd $jsCmd = null) is called */
	public $EditPageNr = we_base_constants::WE_EDITPAGE_CONTENT;
	var $CopyID;
	var $EditPageNrs = [];
	var $Owners = '';
	var $OwnersReadOnly = '';
	var $WebUserID = '';

	/* ID of the Autor who created the document */
	var $CreatorID = 0;

	/* ID of the user who last modify the document */
	var $ModifierID = 0;
	var $RestrictOwners = 0;
	protected $LockUser = 0;
	protected $MediaLinks = [];
	protected $LangLinks = [];

	/* Constructor */

	function __construct(){
		parent::__construct();
		$this->CreationDate = time();
		$this->ModDate = time();
		array_push($this->persistent_slots, 'OwnersReadOnly', 'ParentID', 'ParentPath', 'Text', 'Filename', 'Path', 'OldPath', 'CreationDate', 'ModDate', 'IsFolder', 'ContentType', 'elements', 'EditPageNr', 'CopyID', 'Owners', 'CreatorID', 'ModifierID', 'RestrictOwners', 'WebUserID', 'LockUser', 'LangLinks');
	}

	public function makeSameNew(array $keep = []){
		$keep = array_merge($keep, ['ParentID', 'ParentPath', 'EditPageNr',]);
		$tempDoc = new static();
		$tempDoc->we_new();
		foreach($tempDoc->persistent_slots as $name){
			if(!in_array($name, $keep)){
				$this->{$name} = isset($tempDoc->{$name}) ? $tempDoc->{$name} : '';
			}
		}
		//t_e($this->ClassName, get_class($this), $tempDoc->persistent_slots, );
		$this->InWebEdition = true;
	}

	function setParentID($newID){
		$this->ParentID = $newID;
		$this->ParentPath = $this->getParentPath();
	}

	function ModifyPathInformation($parentID){
		$this->setParentID($parentID);
		$this->Path = $this->getPath();
		$this->wasUpdate = true;
		$this->we_save(); //savePersistentSlotsToDB("Filename,Extension,Text,Path,ParentID");
		$this->modifyChildrenPath(); // only on folders, because on other classes this function is empty
	}

	function modifyChildrenPath(){
		// do nothing, only in Folder-Classes this Function schould have code!
	}

	function checkIfPathOk(){
		//check if Path has changed
		$Path = $this->getPath();
		if($Path != $this->Path){

			//check if Path exists in db
			if(f('SELECT Path FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE Path="' . $this->DB_WE->escape($Path) . '"', '', $this->DB_WE)){
				$GLOBALS['we_responseText'] = sprintf(g_l('weClass', '[response_path_exists]'), $Path);
				return false;
			}
			$this->Path = $Path;
		}
		return true;
	}

	/**
	 * @desc	the function modifies document EditPageNrs set
	 */
	function checkTabs(){
		//to be overriden
	}

	//FIXME: make this __sleep
	function saveInSession(&$save, $forTempDoc = false){
		//NOTE: this is used for temporary documents! so be carefull when changing
		$save = [[],
			$this->elements
		];
		foreach($this->persistent_slots as $slot){
			switch($slot){
				case 'elements'://elements are saved in slot 1
					break;
				//if saved to scheduler/temporary table we don't need these fields, they are only relevant in editor - or reread on load from current state table
				//FIXME: use primaryDBFiels or better
				case 'EditPageNr':
				case 'schedArr':
				case 'Path':
				case 'Text':
				case 'Filename':
				case 'Extension':
				case 'ParentID':
				case 'Published':
				case 'ModDate':
				case 'CreatorID':
				case 'ModifierID':
				case 'Owners':
				case 'RestrictOwners':
				case 'WebUserID':
				case 'Language':
				case 'temp_template_id':
				case 'DocType':
				case 'TemplateID':
				case 'OwnersReadOnly':
				case 'temp_category':
				case 'urlMap':
				case 'viewType':
				case 'IsProtected':
				case 'LockUser':
				case 'wasUpdate':
				case 'InWebEdition':
				case 'CreationDate':
				case 'CopyID':
				case 'hidePages':
				case 'controlElement':
				case 'usedElementNames':
				case 'editorSaves':
					if($forTempDoc){
						break;
					}


				default:
					$bb = isset($this->{$slot}) ? $this->{$slot} : '';
//note these are objects: for searchclass, searchclassFolder
					$save[0][$slot] = $bb;
			}
		}
		// save weDocumentCustomerFilter in Session
		//NOTE: this is an object!
		//if we save to file, we load the new content from db instead
		if(!$forTempDoc && !empty($this->documentCustomerFilter) && we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$save[3] = $this->documentCustomerFilter;
		}
		return true;
	}

	function applyWeDocumentCustomerFilterFromFolder(){
		if(isset($this->documentCustomerFilter) && we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$tmpFolder = new we_folder();
			$tmpFolder->initByID($this->ParentID, $this->Table);
			$this->documentCustomerFilter = $tmpFolder->documentCustomerFilter;

			if($this->IsFolder && $this->ID != 0){
				$this->ApplyWeDocumentCustomerFiltersToChilds = true;
			}
			unset($tmpFolder);
		}
	}

	/* init the object with data from the database */

	function copyDoc($id){
		return false;
		// overwrite
	}

//Form functions for generating the html of the input fields

	/* creates a text-input field for entering Data that will be stored at the $elements Array */

	/* creates the filename input-field */

	function formFilename($text = ''){
		return $this->formTextInput('', 'Filename', $text ?: g_l('weClass', '[filename]'), 24, 255);
	}

	/* creates the DirectoryChoooser field with the "browse"-Button. Clicking on the Button opens the fileselector */

	protected function formDirChooser($rootDirID = 0, $table = '', $Pathname = 'ParentPath', $IDName = 'ParentID', $cmd = '', $label = true, $disabled = false){
		$weSuggest = &we_gui_suggest::getInstance();
		$label = ($label === true ? g_l('weClass', '[dir]') : $label);

		if(!$table){
			$table = $this->Table;
		}
		$textname = 'we_' . $this->Name . '_' . $Pathname;
		$idname = 'we_' . $this->Name . '_' . $IDName;
		$path = $this->$Pathname;
		$myid = $this->$IDName;
		if($disabled){
			return we_html_tools::formElementTable(['text' => we_html_element::hidden($idname, $myid, $idname) .
					we_html_element::hidden($textname, $path, $textname) .
					we_html_element::input(['name' => 'disabled', 'value' => $path, 'type' => 'text', 'disabled' => 1]),
					'style' => 'vertical-align:top;height:10px;'], g_l('weClass', '[dir]')
			);
		}

		$weSuggest->setAcId('Path', id_to_path([$rootDirID], $table));
		$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::CLASS_FOLDER]);
		$weSuggest->setInput($textname, $path);
		if($Pathname === 'ParentPath'){
			$weSuggest->setJsCommandOnItemSelect('pathOfDocumentChanged');
		}
		//FIXME: onblur!
		$weSuggest->setLabel($label ?: '');
		$weSuggest->setMaxResults(10);
		//$weSuggest->setRequired(true);
		$weSuggest->setResult($idname, $myid);
		$weSuggest->setSelector(we_gui_suggest::DirSelector);
		$weSuggest->setTable($table);
		$weSuggest->setWidth(0);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',document.we_form.elements['" . $idname . "'].value,'" . $table . "','" . $idname . "','" . $textname . "','dirChooser_callback," . $Pathname . ($cmd ? ',' . $cmd : '') . "','" . $rootDirID . "')"));
		return $weSuggest->getHTML();
	}

	function htmlTextInput_formDirChooser($attribs = [], $addAttribs = []){
		$attribs = ['class' => 'wetextinput',
			'size' => 30,
			'value' => '',
		];

		foreach($addAttribs as $key => $value){
			if(isset($attribs[$key])){
				$attribs[$key] .= $value;
			} else {
				$attribs[$key] = $value;
			}
		}

		foreach($attribs as $key => $value){
			$attribs[$key] = $value;
		}

		$attribs['type'] = 'text';

		return getHtmlTag('input', $attribs);
	}

	function formCreator($canChange){
		if(!$this->CreatorID){
			$this->CreatorID = 0;
		}
		$creator = $this->CreatorID ? id_to_path($this->CreatorID, USER_TABLE, $this->DB_WE) : g_l('weClass', '[nobody]');
		if(!$canChange){
			return $creator;
		}

		$textname = 'wetmp_' . $this->Name . '_CreatorID';
		$idname = 'we_' . $this->Name . '_CreatorID';

		$inputFeld = we_html_tools::textInput($textname, $creator, '', ['readonly' => 'readonly'], '');
		$idfield = we_html_element::hidden($idname, $this->CreatorID);
		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_users_selector','" . $idname . "','" . $textname . "','user',document.we_form.elements['" . $idname . "'].value,'setHot')");

		return we_html_tools::formElementTable($inputFeld, g_l('weClass', '[maincreator]'), 'left', 'defaultfont', $idfield, $button);
	}

	private function formRestrictOwners($canChange){
		if($canChange){
			$n = 'we_' . $this->Name . '_RestrictOwners';
			$v = $this->RestrictOwners ? true : false;
			return we_html_forms::checkboxWithHidden($v ? true : false, $n, g_l('weClass', '[limitedAccess]'), false, '', "setScrollTo();we_cmd('setHot');we_cmd('reload_editpage');");
		}
		return '<table class="default"><tr><td><i class="fa fa-' . ($this->RestrictOwners ? 'check-' : '') . 'square-o wecheckIcon disabled"></i></td><td>' . g_l('weClass', '[limitedAccess]') . '</td></tr></table>';
	}

	private function formOwners(we_base_jsCmd $jsCmd, $canChange = true){
		$owners = array_filter(explode(',', $this->Owners));
		$ownersReadOnly = we_unserialize($this->OwnersReadOnly);

		$content = '<table class="default" style="margin:2px 0;">
<colgroup><col style="width:20px;"/><col style="width:351px;"/><col style="width:100px;"/><col style="width:26px;"/></colgroup>';
		if($owners){
			$this->DB_WE->query('SELECT ID,Path,(IF(IsFolder,"' . we_base_ContentTypes::USERGROUP . '",(IF(Alias>0,"' . we_base_ContentTypes::USERALIAS . '","' . we_base_ContentTypes::USER . '")))) AS ContentType FROM ' . USER_TABLE . ' WHERE ID IN(' . implode(',', $owners) . ')');
			while($this->DB_WE->next_record(MYSQLI_ASSOC)){
				$owner = $this->DB_WE->f('ID');
				$content .= '<tr><td class="userIcon" data-contenttype="' . $this->DB_WE->f('ContentType') . '"></td><td>' . $this->DB_WE->f('Path') . '</td><td>' .
					we_html_forms::checkboxWithHidden(!empty($ownersReadOnly[$owner]), 'we_owners_read_only[' . $owner . ']', g_l('weClass', '[readOnly]'), false, '', "we_cmd('setHot');", !$canChange) .
					'</td><td>' . ($canChange ? we_html_button::create_button(we_html_button::TRASH, "javascript:setScrollTo();we_cmd('setHot');;we_cmd('users_del_owner','" . $owner . "');") : '') . '</td></tr>';
			}
		} else {
			$content .= '<tr><td class="userIcon" data-contenttype="we/user"></td><td>' . g_l('weClass', '[onlyOwner]') . '</td><td></td><td></td></tr>';
		}
		$content .= '</table>';

		$textname = 'OwnerNameTmp';
		$idname = 'OwnerIDTmp';
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('users_del_all_owners','')", '', '', $this->Owners ? false : true);
		$addbut = $canChange ?
			we_html_element::hiddens([$idname => '', $textname => '']) . we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_users_selector',document.we_form.elements['" . $idname . "'].value,'" . $textname . "','','" . $idname . "','users_add_owner','','',1);") : "";

		$content = '<table class="default" style="width:500px;">
<tr><td>' . we_html_element::div(['class' => "multichooser"], $content) . '</td></tr>
' . ($canChange ? '<tr><td style="text-align:right;">' . $delallbut . $addbut . '</td></tr>' : "") . '</table>';
		$jsCmd->addCmd('setIconOfDocClass', 'userIcon');
		return we_html_tools::formElementTable($content, g_l('weClass', '[otherowners]'), 'left');
	}

	protected function formCreatorOwners(we_base_jsCmd $jsCmd){
		$canChange = ((!$this->ID) || we_users_util::isUserInUsers($_SESSION['user']['ID'], $GLOBALS['we_doc']->CreatorID));

		return '<table class="default">
<tr><td>' . $this->formCreator($canChange && we_base_permission::hasPerm(we_base_perm::USER_CHANGE_DOCUMENT_OWNER)) . '</td></tr>
<tr><td>' . $this->formRestrictOwners($canChange && we_base_permission::hasPerm(we_base_perm::USER_CHANGE_DOCUMENT_PERMISSION)) . '</td></tr>' .
			($this->RestrictOwners ?
			'<tr><td>' . $this->formOwners($jsCmd, $canChange && we_base_permission::hasPerm(we_base_perm::USER_CHANGE_DOCUMENT_PERMISSION)) . '</td></tr>' : '') .
			'</table>';
	}

	function del_all_owners(){
		$this->Owners = '';
	}

	function add_owner($id){
		$ids = array_filter(is_array($id) ? $id : explode(',', $id));
		$this->Owners = implode(',', array_unique(array_filter(array_merge(explode(',', $this->Owners), $ids)), SORT_NUMERIC));
	}

	function del_owner($id){
		$owners = array_filter(explode(',', $this->Owners));
		if(($pos = array_search($id, $owners, false)) === false){
			return;
		}

		unset($owners[$pos]);
		$this->Owners = implode(',', $owners);
	}

	/**
	 * @return bool
	 * @desc	checks if a document is restricted to several users and if
	  the user is one of the restricted users
	 */
	function userHasPerms(){
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) || !$this->RestrictOwners || we_users_util::isOwner($this->Owners) || we_users_util::isOwner($this->CreatorID)){
			return true;
		}
		return false;
	}

	function userIsCreator(){
		return (we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) || we_users_util::isOwner($this->CreatorID));
	}

	function userCanSave($ctConditionOk = false){
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
			return true;
		}
		if(!$ctConditionOk){ // check table condition in respective subclasses: eg in we_collection we check SAVE_COLLECTION
			if(($this->Table === OBJECT_FILES_TABLE)){
				if(!(we_base_permission::hasPerm([we_base_perm::OBJECTFILE_NEW_FOLDER, we_base_perm::OBJECTFILE_NEW]))){
					return false;
				}
			} elseif(!we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_SAVE_ALLOWED)){
				return false;
			}
		}
		if(!$this->RestrictOwners){
			return true;
		}
		if(!$this->userHasPerms()){
			return false;
		}
		$ownersReadOnly = we_unserialize($this->OwnersReadOnly);
		$readers = array_keys(array_filter($ownersReadOnly));
		return !we_users_util::isUserInUsers($_SESSION['user']['ID'], $readers);
	}

	public function formPath($disablePath = false, $notSetHot = false, $extra = ''){
		$disable = ( ($this->ContentType == we_base_ContentTypes::HTML || $this->ContentType == we_base_ContentTypes::WEDOCUMENT) && $this->Published);
		if($this->ContentType === we_base_ContentTypes::HTACCESS){
			$vals = we_base_ContentTypes::getExtension($this->ContentType, true);
			$this->Filename = $this->Filename ?: current($vals);
			$filenameinput = $this->formSelectFromArray('', 'Filename', array_combine($vals, $vals), g_l('weClass', '[filename]'));
		} else {
			$filenameinput = $this->formInputField('', 'Filename', g_l('weClass', '[filename]'), 0, 255, ['onchange' => "if(self.pathOfDocumentChanged){pathOfDocumentChanged(" . ($notSetHot ? 'false' : 'true') . ");}"]);
		}
		return $disable ? ($this->Path) : '
<table class="default">
	<tr>
		<td style="padding-bottom:4px;">' . $filenameinput . '</td>
		<td></td>
		<td>' . $this->formExtension2() . '</td>
	</tr>
	<tr><td colspan="3">' . $this->formDirChooser(0, '', 'ParentPath', 'ParentID', '', true, $disablePath) . '</td></tr>' .
			$extra . '
</table>';
	}

	protected function formExtension2(){
		return '';
	}

	function formCopyDocument(){
		$idname = 'we_' . $this->Name . '_CopyID';
		$but = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document', document.we_form.elements['" . $idname . "'].value,'" . $this->Table . "','" . $idname . "','','copyDocumentSelect','','0','" . $this->ContentType . "',1);");

		return we_html_element::hidden($idname, $this->CopyID) . $but;
	}

	# return html code for button and field to select user
	# ATTENTION !: You have to have we_cmd function in your file and browse_user section

	#
	function formUserChooser($old_userID = -1, $width = '', $in_textname = '', $in_idname = ''){
		$textname = $in_textname ?: 'we_' . $this->Name . '_UserName';
		$idname = $in_idname ?: 'we_' . $this->Name . '_UserID';

		$username = '';
		$userid = $old_userID;
		if(intval($userid) > 0){
			$username = f('SELECT username FROM ' . USER_TABLE . ' WHERE ID=' . intval($userid), 'username', $this->DB_WE);
		}

		return we_html_tools::formElementTable(self::htmlTextInput($textname, $username, '', ' readonly', 'text', $width, 0), 'User', 'left', '', we_html_element::hidden($idname, $userid), we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_users_selector',document.we_form.elements['" . $idname . "'].value,'" . $textname . "','user')"));
	}

	//FIXME: this should be a general selector
	protected function formDocChooser($width, $name, $type = 'txt', $selector = we_gui_suggest::DocSelector, $table = FILE_TABLE){
		$weSuggest = &we_gui_suggest::getInstance();
		$textname = $this->Name . '_' . $name;
		$idname = 'we_' . $this->Name . '_' . $type . '[' . $name . '#' . self::ETYPE_NUMBER . ']';
		$myid = $this->getElement($name, we_contents_base::ETYPE_NUMBER);
		$path = f('SELECT Path FROM ' . $this->DB_WE->escape($table) . ' WHERE ID=' . intval($myid), '', $this->DB_WE);

		$weSuggest->setAcId('TriggerID');
		$weSuggest->setContentType(we_base_ContentTypes::IMAGE /* $selector == weSuggest::DocSelector ? 'folder,' . we_base_ContentTypes::WEDOCUMENT : '' */);
		$weSuggest->setInput($textname, $path);
		$weSuggest->setLabel(g_l('weClass', '[' . $name . ']'));
		$weSuggest->setMaxResults(10);
		$weSuggest->setResult($idname, $myid);
		$weSuggest->setSelector($selector);
		$weSuggest->setTable($table);
		$weSuggest->setWidth($width);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_image',document.we_form.elements['" . $idname . "'].value,'" . $table . "','" . $idname . "','" . $textname . "','setHot','','','" . we_base_ContentTypes::IMAGE . "',1)"));
		$weSuggest->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value='';document.we_form.elements['" . $textname . "'].value='';WE().layout.weSuggest.checkRequired(window,'yuiAcInputTriggerID');we_cmd('setHot');"));
		return $weSuggest->getHTML();
	}

	function formTriggerDocument($isclass = false){
		$weSuggest = &we_gui_suggest::getInstance();
		$table = FILE_TABLE;
		$textname = 'we_' . $this->Name . '_TriggerName';
		if($isclass){
			$idname = 'we_' . $this->Name . '_DefaultTriggerID';
			$myid = $this->DefaultTriggerID ?: '';
		} else {
			$idname = 'we_' . $this->Name . '_TriggerID';
			$myid = $this->TriggerID ?: '';
		}
		$path = f('SELECT Path FROM ' . $this->DB_WE->escape($table) . ' WHERE ID=' . intval($myid), '', $this->DB_WE);

		$weSuggest->setAcId('TriggerID');
		$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::WEDOCUMENT]);
		$weSuggest->setInput($textname, $path);
		$weSuggest->setLabel(g_l('modules_object', '[seourltrigger]'));
		$weSuggest->setMaxResults(10);
		$weSuggest->setResult($idname, $myid);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setTable($table);
		$weSuggest->setWidth(0);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $idname . "'].value,'" . $table . "','" . $idname . "','" . $textname . "','setHot','','','" . we_base_ContentTypes::WEDOCUMENT . "',1)"));
		$weSuggest->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value='';document.we_form.elements['" . $textname . "'].value='';WE().layout.weSuggest.checkRequired(window,'yuiAcInputTriggerID');we_cmd('setHot');"));
		return $weSuggest->getHTML();
	}

	protected function formInputLangLink($headline, $langkey, $LDID = 0, $path = ''){
		$weSuggest = & we_gui_suggest::getInstance();
		$textname = 'we_' . $this->Name . '_LanguageDocName[' . $langkey . ']';
		$idname = 'we_' . $this->Name . '_LanguageDocID[' . $langkey . ']';
		$ackeyshort = 'LanguageDoc' . str_replace('_', '', $langkey);
		$myid = $LDID ?: '';
		$table = $this->IsFolder ? FILE_TABLE : $this->Table;
		$path = $path ?: ($LDID ? f('SELECT Path FROM ' . $this->DB_WE->escape($table) . ' WHERE ID=' . intval($myid), '', $this->DB_WE) : '');

		$rootDirID = $table === OBJECT_FILES_TABLE ? $this->rootDirID : 0;
		if($rootDirID && !$path){
			$path = f('SELECT Path FROM ' . $this->DB_WE->escape($table) . ' WHERE ID=' . intval($rootDirID), '', $this->DB_WE);
		}

		$weSuggest->setAcId($ackeyshort, $path);
		if($table === FILE_TABLE){
			$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::WEDOCUMENT]);
			$ctype = we_base_ContentTypes::WEDOCUMENT;
			$etype = FILE_TABLE;
		} else {
			$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::OBJECT_FILE]);
			$ctype = we_base_ContentTypes::OBJECT_FILE;
			$etype = OBJECT_FILES_TABLE;
		}

		if(!empty($this->DocType) && we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW)){
			$db = new DB_WE();
			$LDcoType = f('SELECT LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="tblDocTypes" AND DID=' . $this->DocType . ' AND Locale="' . $db->escape($langkey) . '"', '', $db);
			if($LDcoType){
				$weSuggest->setCreateButton(we_html_button::create_button('fa:add_doc,fa-plus,fa-lg fa-file-text-o', "javascript:top.we_cmd('new','" . FILE_TABLE . "','','" . we_base_ContentTypes::WEDOCUMENT . "','" . $LDcoType . "');"));
			}
		}
		$weSuggest->setInput($textname, $path, [], true);
		$weSuggest->setLabel($headline);
		$weSuggest->setMaxResults(10);
		$weSuggest->setResult($idname, $myid);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setTable($table);
		$weSuggest->setWidth(0);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $idname . "'].value,'" . $table . "','" . $idname . "','" . $textname . "','setHot','','" . $rootDirID . "','" . $ctype . "',1,0,0,'" . $langkey . "')"));
		$weSuggest->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value='-1';document.we_form.elements['" . $textname . "'].value='';WE().layout.weSuggest.checkRequired(window,'yuiAcInput" . $ackeyshort . "');we_cmd('setHot');"));
		$weSuggest->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:if(document.we_form.elements['" . $idname . "'].value){WE().layout.weEditorFrameController.openDocument('" . $etype . "',document.we_form.elements['" . $idname . "'].value,'" . $ctype . "');}"));
		//$weSuggest->setIsDropFromTree(true);//deactivated
		return $weSuggest->getHTML();
	}

	function formLangLinks($withHeadline = true){
		$defLang = self::getDefaultLanguage();
		$value = ($this->Language ?: $defLang);
		$inputName = 'we_' . $this->Name . '_Language';
		$languages = getWeFrontendLanguagesForBackend();
		$headline = ($withHeadline ? '<tr><td>' . g_l('weClass', '[language]') . '</td></tr>' : '');

		if(LANGLINK_SUPPORT){
			$ret = '';
			foreach($languages as $langkey => $lang){
				$divname = 'we_' . $this->Name . '_LanguageDocDiv[' . $langkey . ']';
				$LDID = !empty($this->LangLinks[$langkey]['id']) && $this->LangLinks[$langkey]['id'] !== -1 ? $this->LangLinks[$langkey]['id'] : 0;
				$path = $LDID ? $this->LangLinks[$langkey]['path'] : '';

				$ret .= we_html_element::div(['id' => $divname, 'style' => ($this->Language == $langkey ? 'display:none' : '')], $this->formInputLangLink($lang, $langkey, $LDID, $path));
				$langkeys[] = $langkey;
			}
			return '
<table class="default" style="margin-top:2px;">' .
				$headline . '
	<tr><td>' . we_html_tools::select($inputName, $languages, 1, $value, false, ['onblur' => "we_cmd('setHot');",
					'onchange' => "dieWerte='" . implode(',', $langkeys) . "';showhideLangLink('we_" . $this->Name . "_LanguageDocDiv',dieWerte,this.options[this.selectedIndex].value);we_cmd('setHot');"], "value") . '</td></tr>
	<tr><td style="text-align:left">' . g_l('weClass', '[languageLinks]') . '</td></tr>
</table>
<br/>' . $ret;
		}
		return '
<table class="default" style="margin-top:2px;">' .
			$headline . '
	<tr><td>' . we_html_tools::select($inputName, $languages, 1, $value, false, ['onblur' => "we_cmd('setHot');",
				'onchange' => "we_cmd('setHot');"], "value") . '</td></tr>
</table>';
	}

	//Function for getting and setting the $elements Array

	/* returns true if the element with the name $name is set */

	function issetElement($name){
		return isset($this->elements[$name]);
	}

	/* set the Data for an element */

	function setElement($name, $data, $type = 'txt', $key = self::ETYPE_TEXT){
		$this->elements[$name]['type'] = $type;
		switch($key){
			case self::ETYPE_NUMBER:
				if(isset($this->elements[$name][self::ETYPE_TEXT])){//remove dat if bdid is set
					unset($this->elements[$name][self::ETYPE_TEXT]);
				}
				$this->elements[$name][self::ETYPE_NUMBER] = intval($data);
				break;
			default:
			//FIXME this is not very nice. This is done for ff* (formfield)
			//t_e('unable to set key ' . $key, $name, $data, $type);
			case self::ETYPE_TEXT:
				if(isset($this->elements[$name][self::ETYPE_NUMBER])){//remove bdid if dat is set
					unset($this->elements[$name][self::ETYPE_NUMBER]);
				}
				$this->elements[$name][$key] = $data;
				break;
		}
	}

	function delElement($name){
		unset($this->elements[$name]);
	}

	/**
	 * get the data from an element
	 * @return mixed
	 */
	function getElement($name, $key = self::ETYPE_UNKNOWN, $default = '', $defaultOnEmpty = false){//FIXME should we bother bdid?
		switch($key){
			case self::ETYPE_UNKNOWN:
				//check bdid first
				return (!empty($this->elements[$name][self::ETYPE_NUMBER]) ?
					$this->elements[$name][self::ETYPE_NUMBER] :
					(isset($this->elements[$name][self::ETYPE_TEXT]) && (!$defaultOnEmpty || $this->elements[$name][self::ETYPE_TEXT]) ?
					$this->elements[$name][self::ETYPE_TEXT] :
					$default));
			default:
				return (isset($this->elements[$name][$key]) && (!$defaultOnEmpty || $this->elements[$name][$key]) ? $this->elements[$name][$key] : $default);
		}
	}

	public function getElementType($name){
		return (isset($this->elements[$name]['type']) ? $this->elements[$name]['type'] : '');
	}

	public function setElementType($name, $type){
		$this->elements[$name]['type'] = $type;
	}

	/** returns the keys of elements matching the criterion */
	function nextElement($type = 'txt'){
		if(!is_array($this->elements)){
			return [];
		}
		if(!$type){
			return array_keys($this->elements);
		}
		$ret = [];
		foreach($this->elements as $k => $arr){
			if(isset($arr['type']) && $arr['type'] == $type){
				$ret[] = $k;
			}
		}
		return $ret;
	}

	/* returns the JavaScript-Code which modifies the tree-menue */

	function getUpdateTreeScript($select = true, we_base_jsCmd $jsCmd = null, $asCmd = false){
		return we_tree_main::getJSUpdateTreeScript($this, $select, $jsCmd, $asCmd);
	}

	/** returns the Path dynamically (use it, when the class-variable Path is not set)  */
	public function getPath(){
		return rtrim($this->getParentPath(), '/') . '/' . ( isset($this->Filename) ? $this->Filename : '' ) . ( isset($this->Extension) ? $this->Extension : '' );
	}

	/** returns the Path dynamically (use it, when the class-variable Text is not set)  */
	function getText(){
		return $this->Text;
	}

	public function getEditorBodyAttributes($editor = 0){
		return [];
	}

	/** get the Path of the Parent-Object */
	function getParentPath(){
		return (!$this->ParentID) ? '/' : f('SELECT Path FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->ParentID), '', $this->DB_WE);
	}

	function constructPath(){
		if($this->ID){
			$pid = $this->ParentID;
			$p = '/' . $this->Text;
			$z = 0;
			while($pid && $z < 50){
				$hash = getHash('SELECT ParentID,Text FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($pid), $this->DB_WE);
				$pid = $hash['ParentID'];
				$text = $hash['Text'];
				$p = '/' . $text . $p;
				$z++;
			}
			if($z >= 50){
				return false;
			}
			return $p;
		}
		return false;
	}

	/* get the Real-Path of the Object (Server-Path)
	 * @$fileaccesss bool if true, a path valid for domain replacement is given
	 */

	public function getRealPath($old = false){
		return (($this->Table === FILE_TABLE) ? realpath(WEBEDITION_PATH . '..') : TEMPLATES_PATH) .
			($old ? $this->OldPath : $this->getPath());
	}

	/* get the Site-Path of the Object */

	public function getSitePath($old = false){
		return $_SERVER['DOCUMENT_ROOT'] . SITE_DIR . substr(($old ? $this->OldPath : $this->getPath()), 1);
	}

	/* get the HTTP-Path of the Object */

	function getHttpPath(){
		$urlReplace = we_folder::getUrlReplacements($GLOBALS['DB_WE'], true);
		$http = $this->getPath();
		if($urlReplace){
			$cnt = 0;
			$http = preg_replace($urlReplace, array_keys($urlReplace), $http, -1, $cnt);
			return ($cnt ? 'http:' : getServerUrl()) . $http;
		}

		return $http;
	}

	protected static function getDefaultLanguage(){
// get interface language of user
		list($userLanguage) = explode('_', isset($_SESSION['prefs']['Language']) ? $_SESSION['prefs']['Language'] : '');

// trying to get locale string out of interface language
		$key = array_search($userLanguage, getWELangs());

		$defLang = $GLOBALS['we']['lang']['DefaultFrontendLanguage'];

// if default language is not equal with frontend language
		if(substr($defLang, 0, strlen($key)) !== $key){
// get first language that fits
			foreach(array_keys(getWeFrontendLanguagesForBackend()) as $k){
				$parts = explode('_', $k);
				if($parts[0] === $key){
					$defLang = $k;
				}
			}
		}
		return $defLang;
	}

	function editor(we_base_jsCmd $jsCmd = null){

	}

	protected function getParentIDFromParentPath(){
		return 0;
	}

	function save($resave = 0, $skipHook = 0){
		return $this->we_save($resave, $skipHook);
	}

	public function we_new(){
		parent::we_new();
		$this->CreatorID = isset($_SESSION['user']['ID']) ? $_SESSION['user']['ID'] : 0;
		$this->ParentPath = $this->getParentPath();
	}

	public function we_load($from = self::LOAD_MAID_DB){
		parent::we_load($from);

		$this->getContentData();
		$this->getLangLinks();
		$this->OldPath = $this->Path;
	}

	public function we_save($resave = false, $skipHook = false){
		if($this->PublWhenSave){
			$this->Published = time();
		}
		if(!$resave){
			$this->ModDate = time();
			$this->ModifierID = empty($GLOBALS['we']['Scheduler_active']) && !empty($_SESSION['user']['ID']) ? $_SESSION['user']['ID'] : 0;
		}
		if(!parent::we_save($resave)){
			return false;
		}
		$this->update_filehash();
		$a = $this->saveContentDataInDB();
		if(!$resave && !$this->IsClassFolder){
			we_base_history::insertIntoHistory($this);
		}
		return $a;
	}

	/**
	 * resave weDocumentCustomerFilter
	 *
	 */
	function resaveWeDocumentCustomerFilter(){
		if(!empty($this->documentCustomerFilter)){
			we_customer_documentFilter::saveForModel($this);
		}
	}

	protected function getDefaultFilename(){
		return f('SELECT MAX(ID) FROM ' . $this->DB_WE->escape($this->Table), '', $this->DB_WE) + 1;
	}

	public function we_initSessDat($sessDat){//FIXME: use __wakeup
		if(!empty($sessDat)){
			foreach($this->persistent_slots as $cur){
				if(isset($sessDat[0][$cur])){
					$this->{$cur} = $sessDat[0][$cur];
				}
			}
			if(isset($sessDat[1])){
				$this->elements = $sessDat[1];
			}
		}
		$this->setElementsFromHTTP();
	}

	//FIXME: make this __wakeup - used also to read data from temporary table
	protected function initSerializedDat($sessDat){
		if(!is_array($sessDat)){
			$this->Name = md5(uniqid(__FUNCTION__, true));
			return false;
		}
		foreach($this->persistent_slots as $cur){
			if(isset($sessDat[0][$cur])){
				$this->{$cur} = $sessDat[0][$cur];
			}
		}
		if(isset($sessDat[1])){
			$this->elements = $sessDat[1];
		}
		$this->Name = md5(uniqid(__FUNCTION__, true));
		return true;
	}

	protected function setText(){
		$this->Text = $this->Filename;
	}

	protected function convertElemFromRequest($type, &$v, $k){
		switch($type){
			case 'internal'://pseudo-element for setElementsFromHTTP
				break;
			default:
				$v = we_base_util::rmHTML($v, true);
				break;
		}
	}

	protected function setElementsFromHTTP(){
		// do not set REQUEST VARS into the document
		switch(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0)){
			case 'switch_edit_page':
				if(we_base_request::_(we_base_request::STRING, 'we_cmd', false, 3)){
					return true;
				}
				break;
			case 'save_document':
				if(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 7) === 'save_document'){
					return true;
				}
				break;
		}

		if($_REQUEST){
			$this->setRequestData();
		}
		$this->Text = $this->getText();
		$this->ParentPath = $this->getParentPath();
		$this->Path = $this->getPath();
		return true;
	}

	private function setRequestData(){
		$schedDates = $regs = $dates = [];
		foreach($_REQUEST as $n => $v){
			if(preg_match('/^we_' . preg_quote($this->Name) . '_([^\[]+)$/', $n, $regs)){
				if(is_array($v)){
					$type = $regs[1];
					foreach($v as $name => $v2){
						$v2 = we_base_util::cleanNewLine($v2);
						if(preg_match('/(.+)#(.+)/', $name, $regs)){
							$name = $regs[1];
							$key = $regs[2];
						} else {
							$key = 'dat';
						}
						switch($type){
							case 'LanguageDocName':
							case 'LanguageDocID':
								$this->LangLinks[$name][$type === 'LanguageDocName' ? 'path' : 'id'] = $v2;
								break;
							case 'date':
								preg_match('|(.*)_(.*)|', $name, $regs);
								list(, $name, $what) = $regs;
								$dates[$name][$what] = $v2;
								break;
							case 'category'://from we:category
								$this->setElement($name, (is_array($v2) ? implode(',', $v2) : $v2));
								break;
							case 'checkAttrib':
								//attribs with checkbox, 0 must not be stored in db
								$type = 'attrib';
							case 'checkbox':
								if(empty($v2)){
									$this->delElement($name);
									break;
								}
								$this->setElement($name, $v2, $type, $key);
								break;
							case 'select':
								if(preg_match("|^[1-9][0-9]*$|", $v2)){
									$this->setElement($name, $v2, $type, self::ETYPE_NUMBER);
								} else {
									$this->setElement($name, $v2, $type, $key);
								}
								break;
							case 'link':
								if($key === self::ETYPE_NUMBER && empty($v2)){
									$this->delElement($name);
								} else {
									$this->setElement($name, $v2, $type, $key);
								}
								break;
							case 'float':
								$this->setElement($name, we_base_util::std_numberformat($v2), $type, $key);
								break;
							case 'int':
								$this->setElement($name, intval(str_replace(',', '.', $v2)), $type, $key);
								break;
							default:
								$this->setElement($name, $v2, $type, $key);
								break;
						}
					}
				} else {
					$this->setPersistentSlot($regs[1], $v);
				}
			} else if($n === 'we_owners_read_only'){
				$this->OwnersReadOnly = we_serialize($v, SERIALIZE_JSON);
			} elseif(preg_match('/^we_schedule_([^\[]+)$/', $n, $regs)){
				$rest = $regs[1];
				$sw = explode('_', $rest);
				$nr = end($sw);
				if(!isset($this->schedArr[$nr])){
					$this->schedArr[$nr] = [];
				}
				switch($sw[0]){
					case 'task':
					case 'active':
					case 'type':
					case 'doctypeAll':
						$this->schedArr[$nr][$sw[0]] = $v;
						break;
					case 'doctype':
						$this->schedArr[$nr]['DoctypeID'] = $v;
						break;
					case 'parentid':
						$this->schedArr[$nr]['ParentID'] = $v;
						break;
					case 'time':
						if((!isset($schedDates[$nr]) || !is_array($schedDates[$nr]))){
							$schedDates[$nr] = [];
						}
						$schedDates[$nr][$sw[1]] = $v;
						break;
					case we_schedule_scheduler::SUBMIT_DAYS:
						$this->schedArr[$nr]['days'] = $v;
						break;
					case we_schedule_scheduler::SUBMIT_MONTHS:
						$this->schedArr[$nr]['months'] = $v;
						break;
					case we_schedule_scheduler::SUBMIT_WDAYS:
						$this->schedArr[$nr]['weekdays'] = $v;
						break;
					default:
						//FIXME: this should not happen anymore
						t_e('old scheudle detected', $sw);
				}
			}
		}

		//normal date fields
		foreach($dates as $k => $v){
			if(!array_sum($dates[$k])){
				$this->setElement($k, 0, 'date', self::ETYPE_NUMBER);
			} else {
				$this->setElement($k, mktime(empty($dates[$k]['hour']) ? 0 : $dates[$k]['hour'], empty($dates[$k]['minute']) ? 0 : $dates[$k]['minute'], 0, empty($dates[$k]['month']) ? 1 : $dates[$k]['month'], empty($dates[$k]['day']) ? 1 : $dates[$k]['day'], empty($dates[$k]['year']) ? 1970 : $dates[$k]['year']), 'date', self::ETYPE_NUMBER);
			}
		}
//t_e($schedDates);
		//scheduler dates
		foreach($schedDates as $nr => $v){
			$this->schedArr[$nr]['time'] = mktime($schedDates[$nr]['hour'], $schedDates[$nr]['minute'], 0, $schedDates[$nr]['month'], $schedDates[$nr]['day'], $schedDates[$nr]['year']);
		}
	}

	protected function isElement($Name){
		return true; // overwrite
	}

	protected function getContentData(){
		//FIXME:if we need nHash we have to get it hex, since tblTemporaryDoc will have problems with bin data
		$this->DB_WE->query('SELECT ID,BDID,Dat,Type,Name,DocumentTable FROM ' . CONTENT_TABLE . ' c WHERE c.DID=' . intval($this->ID) . ' AND c.DocumentTable="' . $this->DB_WE->escape(stripTblPrefix($this->Table)) . '"');
		while($this->DB_WE->next_record(MYSQLI_ASSOC)){
			$rec = array_change_key_case($this->DB_WE->getRecord());
			$Name = $rec['name'];
			unset($rec['name']);

			switch($rec['type']){
				case 'formfield': // garbage fix!
					$dat = $rec['dat'];
					unset($rec['dat']);
					$this->elements[$Name] = array_merge(we_unserialize($dat), $rec);
					$this->elements[$Name]['table'] = CONTENT_TABLE;
					break;
				default:
					if($this->isElement($Name)){
						$this->elements[$Name] = array_change_key_case($rec);
						$this->elements[$Name]['table'] = CONTENT_TABLE;
					}
			}
		}
	}

	protected function getLangLinks(){
		$langkeys = array_keys(getWeFrontendLanguagesForBackend());
		if(LANGLINK_SUPPORT){
			$isObject = ($this->Table === OBJECT_FILES_TABLE || $this->Table === OBJECT_TABLE );
			$documentTable = $isObject && !$this->IsFolder ? stripTblPrefix(OBJECT_FILES_TABLE) : stripTblPrefix(FILE_TABLE);
			$this->DB_WE->query('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $documentTable . '" AND IsObject=' . intval($isObject) . ' AND DID=' . intval($this->ID) . ' AND Locale IN("' . implode('","', $langkeys) . '")');
			$tmpIDs = $this->DB_WE->getAllFirst(false);

			$tmpPaths = id_to_path($tmpIDs, $this->Table, $this->DB_WE, true);
			foreach($langkeys as $langkey){
				$this->LangLinks[$langkey] = empty($tmpIDs[$langkey]) || empty($tmpPaths[$tmpIDs[$langkey]]) ?
					['id' => 0, 'path' => ''] :
					['id' => $tmpIDs[$langkey], 'path' => $tmpPaths[$tmpIDs[$langkey]]];
			}
			return;
		}
		foreach($langkeys as $langkey){
			$this->LangLinks[$langkey] = ['id' => 0, 'path' => ''];
		}
	}

	private function getLinkReplaceArray(){
		$this->DB_WE->query('SELECT CONCAT_WS("_",Type,Name) AS Name,ID FROM ' . CONTENT_TABLE . ' WHERE DID=' . intval($this->ID) . ' AND DocumentTable="' . stripTblPrefix($this->Table) . '"');
		return $this->DB_WE->getAllFirst(false);
	}

	protected function saveContentDataInDB(){
		if(!is_array($this->elements)){
			return we_base_delete::deleteContentFromDB($this->ID, $this->Table, $this->DB_WE);
		}
		//don't stress index:
		$replace = $this->getLinkReplaceArray();
		$def = sql_function('DEFAULT');
		$data = $new = [];
		foreach($this->elements as $k => $v){
			if(!$this->isElement($k) ||
				//ignore fields which result in empty entry - they will be deleted
				((!isset($v['dat']) || $v['dat'] === '') && empty($v[self::ETYPE_NUMBER]) && empty($v['ffname'])) ||
				//don't set "vars" type
				(isset($v['type']) && $v['type'] === self::INTERNAL_VAR) ||
				//ignore binary data
				($k === 'data' && $this->isBinary())
			){
				continue;
			}
			$dat = isset($v[self::ETYPE_TEXT]) ? $v[self::ETYPE_TEXT] : '';
			$bdid = isset($v[self::ETYPE_NUMBER]) ? $v[self::ETYPE_NUMBER] : 0;

			$v['type'] = (empty($v['ffname']) ? (empty($v['type']) ? 'txt' : $v['type']) : 'formfield');

			switch($v['type']){
				case 'formfield':
					$copyV = $v;
					//remove empty fields on serialize
					unset($copyV['id'], $copyV['type']);
					$dat = we_serialize(array_filter($copyV), SERIALIZE_JSON);
					break;
				case 'date':
					$dat = sprintf('%016d', $dat);
					break;
			}

			if(!($bdid || $dat !== '')){
				//empty entry
				continue;
			}
			$key = $v['type'] . '_' . $k;
			if(isset($replace[$key])){
				$cid = $replace[$key];
				unset($replace[$key]);
			} else {
				$cid = 0;
				$new[] = 'x\'' . md5($k) . '\'';
			}

			$data[] = we_database_base::arraySetter([//be aware to change replace statement as well
					'ID' => $cid ?: $def,
					'Dat' => ($bdid ? sql_function('NULL') : (is_array($dat) ? we_serialize($dat) : $dat)),
					'BDID' => ($bdid ? intval($bdid) : sql_function('NULL')),
					'DID' => $this->ID,
					'Name' => $k,
					'Type' => $v['type'],
					'nHash' => sql_function('x\'' . md5($k) . '\''),
					'DocumentTable' => stripTblPrefix($this->Table)
					], ',', true);
		}
		//Update & Insert elements
		$ret = $data ?
			$this->DB_WE->query('REPLACE INTO ' . CONTENT_TABLE . ' (ID,Dat,BDID,DID,Name,Type,nHash,DocumentTable) VALUES ' . implode(',', $data)) :
			true;
		if(!$ret){
			return false;
		}
		//get ids of new elements
		if($new){
			$update = $this->DB_WE->getAllFirst('SELECT ID,Name FROM ' . CONTENT_TABLE . ' WHERE DocumentTable="' . stripTblPrefix($this->Table) . '" AND DID=' . $this->ID . ' AND nHash IN (' . implode(',', $new) . ')', false);
			foreach($update as $id => $name){
				$this->elements[$name]['id'] = $id;
			}
		}

		//delete obsolete elements
		if($replace){
			$this->DB_WE->query('DELETE FROM ' . CONTENT_TABLE . ' WHERE DID=' . $this->ID . ' AND DocumentTable="' . $this->DB_WE->escape(stripTblPrefix($this->Table)) . '" AND ID IN(' . implode(',', $replace) . ')');
		}
		return true;
	}

	protected function getPersistentSlotsFromDB($felder = '*'){
		parent::getPersistentSlotsFromDB($felder);
		$this->ParentPath = $this->getParentPath();
	}

	protected function isVariantNamesValid(){
		return true;
	}

	protected function isSaveDirinDir(){
		return true;
	}

	protected function isSameAsParent(){
		return false;
	}

	protected function isFilenameEmpty(){
		return ($this->Filename === '');
	}

	protected function isPathNotValid(){
		if(strpos($this->ParentPath, '..') !== false || ($this->ParentPath && $this->ParentPath{0} != '/')){
			return true;
		}
		if(($ws = get_ws($this->Table, true))){ //	doc has workspaces
			if(!(we_users_util::in_workspace($this->ParentID, $ws, $this->Table))){
				return true;
			}
		}
		return false;
	}

	protected function isFilenameNotValid(){
		return we_base_file::we_filenameNotValid($this->Filename);
	}

	protected function isFilenameNotAllowed(){
		if($this->Table === FILE_TABLE && $this->ParentID == 0 && strtolower($this->Filename . (isset($this->Extension) ? $this->Extension : '')) === 'webedition'){
			return true;
		}
		if(substr(strtolower($this->Filename . (isset($this->Extension) ? $this->Extension : '')), -1) === '.'){
			return true;
		}
		return false;
	}

	protected function isFileExtensionNotValid(){
		if(isset($this->Extension)){
			return !(preg_match('/^\.?[a-zA-Z0-9]+$/iD', $this->Extension) || !$this->Extension);
		}
		return false;
	}

	protected function isFilenameDouble(){
		return f('SELECT 1 FROM `' . $this->DB_WE->escape($this->Table) . '` WHERE ParentID=' . intval($this->ParentID) . ' AND Filename="' . $this->DB_WE->escape($this->Filename) . '" AND ID!=' . intval($this->ID), '', $this->DB_WE);
	}

	/* check if ParentPath is diffrent as ParentID, so we need to look what ParentID it is.
	  If it doesn't exists we have to create the folders (for auto Date-Folder Names)
	 */

	function checkPathDiffAndCreate(){
		if($this->getParentPath() != $this->ParentPath && $this->ParentPath != '' && $this->ParentPath != '/'){
			if(!$this->IsTextContentDoc || !$this->DocType){
				return false;
			}
			$doctype = new we_doctype_docTypes();
			$doctype->initByID($this->DocType, DOC_TYPES_TABLE);
			if(!$doctype->SubDir){
				return false;
			}
			$pathFirstPart = substr($this->getParentPath(), -1) === '/' ? '' : '/';
			$tail = '';
			switch($doctype->SubDir){
				case self::SUB_DIR_YEAR:
					$tail = $pathFirstPart . date('Y');
					break;
				case self::SUB_DIR_YEAR_MONTH:
					$tail = $pathFirstPart . date('Y') . '/' . date('m');
					break;
				case self::SUB_DIR_YEAR_MONTH_DAY:
					$tail = $pathFirstPart . date('Y') . '/' . date('m') . '/' . date('d');
					break;
			}
			if($this->getParentPath() . $tail != $this->ParentPath){
				return false;
			}

			$this->ParentID = $this->getParentIDFromParentPath();
			$this->Path = $this->getPath();
		}
		return ($this->ParentID != -1);
	}

	function correctDoublePath(){
		if($this->Filename){
			if(f('SELECT ID  FROM  ' . $this->DB_WE->escape($this->Table) . '  WHERE ID!=' . intval($this->ID) . ' AND Text="' . $this->DB_WE->escape($this->Filename . (isset($this->Extension) ? $this->Extension : '')) . '" AND ParentID=' . intval($this->ParentID), 'ID', $this->DB_WE)){
				$z = 0;
				$footext = $this->Filename . '_' . $z . (isset($this->Extension) ? $this->Extension : '');
				while(f('SELECT ID FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID!=' . intval($this->ID) . ' AND Text="' . $this->DB_WE->escape($footext) . '" AND ParentID=' . intval($this->ParentID), 'ID', $this->DB_WE)){
					$z++;
					$footext = $this->Filename . '_' . $z . (isset($this->Extension) ? $this->Extension : '');
				}
				$this->Text = $footext;
				$this->Filename = $this->Filename . '_' . $z;
				$this->Path = $this->getParentPath() . (($this->getParentPath() != '/') ? '/' : '') . $this->Text;
			}
			return;
		}
		if(f('SELECT ID  FROM  ' . $this->DB_WE->escape($this->Table) . '  WHERE ID!=' . intval($this->ID) . ' AND Text="' . $this->DB_WE->escape($this->Text) . '" AND ParentID=' . intval($this->ParentID), 'ID', $this->DB_WE)){
			$z = 0;
			$footext = $this->Text . '_' . $z;
			while(f('SELECT ID FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID!=' . intval($this->ID) . ' AND Text="' . $this->DB_WE->escape($footext) . '" AND ParentID=' . intval($this->ParentID), 'ID', $this->DB_WE)){
				$z++;
				$footext = $this->Text . '_' . $z;
			}
			$this->Text = $footext;
			$this->Path = $this->getParentPath() . (($this->getParentPath() != '/') ? '/' : '') . $this->Text;
		}
	}

	protected function check_requiredFields(){
		return ''; // overwrite
	}

	protected function scheduleToBeforeNow(){
		return false; // overwrite
	}

	function publInScheduleTable(){
		return false; // overwrite
	}

	protected function hasDoubbleFieldNames(){
		return false;
	}

	function we_resaveTemporaryTable(){
		return true;
	}

	function we_resaveMainTable(){
		$this->wasUpdate = true;
		return self::we_save(true, true);
	}

	public function we_rewrite(){
		return true;
	}

	protected function update_filehash(){

	}

	function parseTextareaFields(){

	}

	public function correctFields(){

	}

	/**
	 * returns if document can have variants the function returns true otherwise
	 * false
	 * if paramter checkField is true, this function checks also, if there are
	 * already fields selected for the variants.
	 *
	 * @param boolean $checkFields
	 * @return boolean
	 */
	function canHaveVariants($checkFields = false){
// overwrite
		return false;
	}

	protected function registerFileLinks(array $ids, $type, $isTemp = false){
		$sets = [];
		foreach($ids as $k => $remObj){
			$sets[] = we_database_base::arraySetter([
					$this->ID,
					stripTblPrefix($this->Table),
					$type,
					$remObj,
					stripTblPrefix(FILE_TABLE),
					sql_function('x\'' . md5(is_numeric($k) ? 'NULL' : $k) . '\''),
					0,
					$isTemp ? 1 : 0
					], ',', true);
		}
		if($sets){
			return (bool) $this->DB_WE->query('REPLACE INTO ' . FILELINK_TABLE . '(ID,DocumentTable,type,remObj,remTable,nHash,position,isTemp) VALUES ' . implode(',', $sets));
		}
		return true;
	}

	function registerMediaLinks($temp = false){
		$this->MediaLinks = array_filter($this->MediaLinks, function($v){
			return $v && is_numeric($v);
		});

		// filter MediaLinks by media contenttype
		$verifiedIDs = [];
		if(!empty($this->MediaLinks)){
			$whereType = 'AND ContentType IN ("' . we_base_ContentTypes::APPLICATION . '","' . we_base_ContentTypes::FLASH . '","' . we_base_ContentTypes::IMAGE . '","' . we_base_ContentTypes::VIDEO . '","' . we_base_ContentTypes::PDF . '")';
			$this->DB_WE->query('SELECT ID FROM ' . FILE_TABLE . ' WHERE ID IN (' . implode(',', array_unique(array_values($this->MediaLinks))) . ') ' . $whereType);
			while($this->DB_WE->next_record()){
				$verifiedIDs[] = $this->DB_WE->f('ID');
			}
		}
		$this->MediaLinks = array_intersect($this->MediaLinks, $verifiedIDs);

		if(empty($this->MediaLinks)){
			return true;
		}

		return $this->registerFileLinks($this->MediaLinks, 'media', $temp);
	}

	function unregisterMediaLinks($delPublished = true, $delTemp = true){
		if($delPublished){
			$this->DB_WE->query('DELETE FROM ' . FILELINK_TABLE . ' WHERE ID=' . intval($this->ID) . ' AND DocumentTable="' . stripTblPrefix($this->Table) . '" AND isTemp=0 AND type="media"');
		}
		if($delTemp){
			$this->DB_WE->query('DELETE FROM ' . FILELINK_TABLE . ' WHERE ID=' . intval($this->ID) . ' AND DocumentTable="' . stripTblPrefix($this->Table) . '" AND isTemp=1 AND type="media"');
		}
	}

	/**
	 * @return	int
	 * @desc	checks if the user can modify a document, or only read the doc (only preview tab).
	  returns	 1	if doc is not restricted any rules
	  -1	if doc is not in workspace of user
	  -2	if doc is restricted and user has nor rights
	  -3	if doc is locked by another user
	  -4	if user has not the right to save a file.
	 */
	public function userHasAccess(){
		$uid = $this->isLockedByUser();
		if($uid > 0 && $uid != $_SESSION['user']['ID'] && $this->ID){ // file is locked
			$this->LockUser = $uid;
			$this->saveInSession($_SESSION['weS']['we_data'][$GLOBALS['we_transaction']]);
			return self::FILE_LOCKED;
		}
		if($this->LockUser != 0 && $this->LockUser != $_SESSION['user']['ID'] && $uid == 0){
			$this->we_load(self::LOAD_TEMP_DB);
		}

		if(!$this->userHasPerms()){ //	File is restricted !
			return self::USER_NO_PERM;
		}

		if(!$this->userCanSave()){ //	user has no right to save.
			return self::USER_NO_SAVE;
		}

		if($this instanceof we_object){
			if($this->RestrictUsers && !(we_users_util::isOwner($this->CreatorID) || we_users_util::isOwner($this->Users))){ //	user is creator of doc - all is allowed.
				return self::USER_NO_PERM;
			}
		} elseif(we_users_util::isOwner($this->CreatorID) || we_users_util::isOwner($this->Owners)){ //	user is creator/owner of doc - all is allowed.
			$this->lockDocument();
			$this->saveInSession($_SESSION['weS']['we_data'][$GLOBALS['we_transaction']]);
			return self::USER_HASACCESS;
		}

		//	access to doc is not restricted, check workspaces of user
		if($this->ID){
			if(($ws = get_ws($this->Table, true))){ //	doc has workspaces
				if(!(we_users_util::in_workspace($this->ID, $ws, $this->Table))){
					return self::FILE_NOT_IN_USER_WORKSPACE;
				}
			}
		}
		$this->lockDocument();
		$this->saveInSession($_SESSION['weS']['we_data'][$GLOBALS['we_transaction']]);
		return self::USER_HASACCESS;
	}

	/**
	 * @return int
	 * @desc	checks if a file is locked by another user. returns that userID
	  or 0 when file is not locked
	 */
	public function isLockedByUser(){
		//select only own ID if not in same session
		return intval(f('SELECT UserID FROM ' . LOCK_TABLE . ' WHERE ID=' . intval($this->ID) . ' AND tbl="' . $this->DB_WE->escape(stripTblPrefix($this->Table)) . '" AND sessionID!=x\'' . we_base_sessionHandler::getCurrentHex() . '\' AND lockTime>NOW()', '', $this->DB_WE));
	}

	public function lockDocument(){
		if($_SESSION['user']['ID'] && $this->ID){ // only if user->id != 0
			//if lock is used by other user and time is up, update table
			$this->DB_WE->query('INSERT INTO ' . LOCK_TABLE . ' SET ID=' . intval($this->ID) . ',UserID=' . intval($_SESSION['user']['ID']) . ',tbl="' . $this->DB_WE->escape(stripTblPrefix($this->Table)) . '",sessionID=x\'' . we_base_sessionHandler::getCurrentHex() . '\',lockTime=NOW()+INTERVAL ' . (we_base_constants::PING_TIME + we_base_constants::PING_TOLERANZ) . ' SECOND
				ON DUPLICATE KEY UPDATE UserID=' . intval($_SESSION['user']['ID']) . ',sessionID=x\'' . we_base_sessionHandler::getCurrentHex() . '\',lockTime= NOW() + INTERVAL ' . (we_base_constants::PING_TIME + we_base_constants::PING_TOLERANZ) . ' SECOND,releaseRequestID=null,releaseRequestForce=null');
			$this->LockUser = intval($_SESSION['user']['ID']);
		}
	}

	/**
	 * Gets the navigation folders for the current document
	 *
	 * @return Array
	 */
	protected function getNavigationFoldersForDoc(){
		if($this->Table !== FILE_TABLE){
			return [];
		}
		$category = property_exists($this, 'Category') ? array_map('escape_sql_query', array_unique(array_filter(array_merge(explode(',', $this->Category), explode(',', $this->oldCategory))))) : '';
		$queries = ['( (Selection="' . we_navigation_navigation::SELECTION_STATIC . '" OR IsFolder=1) AND SelectionType="' . we_navigation_navigation::STYPE_DOCLINK . '" AND LinkID=' . intval($this->ID) . ')',];
		if(isset($this->DocType)){
			//FIXME: query should use ID, not parentID
			$queries[] = '(Selection="' . we_navigation_navigation::SELECTION_DYNAMIC . '" AND (DocTypeID="' . $this->DB_WE->escape($this->DocType) . '" OR FolderID=' . intval($this->ParentID) . '))';
		}
		if($category){
			//FIXME: query should use ID, not parentID
			$queries[] = '(Selection="' . we_navigation_navigation::SELECTION_DYNAMIC . '" AND DynamicSelection="' . we_navigation_navigation::DYN_DOCTYPE . '" AND (FIND_IN_SET("' . implode('",Categories) OR FIND_IN_SET("', $category) . '",Categories) ) )';
		}
		return $this->DB_WE->getAllq('SELECT DISTINCT ParentID FROM ' . NAVIGATION_TABLE . ' WHERE ' . implode(' OR ', $queries), true);
	}

	public function insertAtIndex(array $only = [], array $fieldTypes = []){

	}

	/**
	 * Rewrites the navigation cache files
	 *
	 */
	public function rewriteNavigation(){
		//FIXME: make this more efficient!
		// rewrite filter
		if(we_modules_info::isActive(we_modules_info::CUSTOMER) && !empty($this->documentCustomerFilter)){
			we_navigation_customerFilter::updateByFilter($this->documentCustomerFilter, $this->ID, $this->Table);
		}

		$folders = $this->getNavigationFoldersForDoc();
		foreach($folders as $f){
			we_navigation_cache::delNavigationTree($f);
		}
	}

	public function revert_published(){

	}

	public function isBinary(){
		return false;
	}

	protected function isMoved(){
		return ($this->wasMoved = ($this->OldPath && ($this->Path != $this->OldPath)));
	}

	public function wasMoved(){
		return $this->wasMoved;
	}

	public function showLockedWarning($userID){
		echo we_html_tools::getHtmlTop('', '', '', '', we_html_element::body(['class' => 'weDialogBody'], we_html_tools::dialogLayout('<p>' . sprintf(g_l('alert', '[temporaere_no_access_text]'), $this->Text, f('SELECT username FROM ' . USER_TABLE . ' WHERE ID=' . intval($userID))) . '</p>', g_l('alert', '[temporaere_no_access]')) .
//	For SEEM-Mode
				($_SESSION['weS']['we_mode'] == we_base_constants::MODE_SEE ?
					we_html_element::span(['style' => "text-decoration:none", 'onclick' => "WE().layout.weNavigationHistory.navigateReload()"], g_l('SEEM', '[try_doc_again]')) : '')
		));
		exit();
	}

//FIXME: make abstract
	public function getPropertyPage(we_base_jsCmd $jsCmd){

	}

	public function checkFieldsOnSave(){
		if($this->isPathNotValid()){
			return sprintf(g_l('weClass', '[notValidFolder]'), $this->Path);
		}
		if($this->isFilenameEmpty()){
			return g_l('weEditor', '[' . $this->ContentType . '][filename_empty]');
		}
		if(!$this->isSaveDirinDir()){
			return g_l('weEditor', '[pfolder_notsave]');
		}
		if($this->isSameAsParent()){
			return g_l('weEditor', '[folder_save_nok_parent_same]');
		}
		if($this->isFileExtensionNotValid()){
			return sprintf(g_l('weEditor', '[' . $this->ContentType . '][we_filename_notValid]', true) ?: g_l('weEditor', '[' . we_base_ContentTypes::APPLICATION . '][we_filename_notValid]'), $this->Path);
		}
		if($this->isFilenameNotValid()){
			return sprintf(g_l('weEditor', '[' . $this->ContentType . '][we_filename_notValid]', true) ?: g_l('weEditor', '[' . we_base_ContentTypes::APPLICATION . '][we_filename_notValid]'), $this->Path);
		}
		if($this->isFilenameNotAllowed()){
			return sprintf(g_l('weEditor', '[' . $this->ContentType . '][we_filename_notAllowed]', true) ?: g_l('weEditor', '[' . we_base_ContentTypes::APPLICATION . '][we_filename_notAllowed]'), $this->Path);
		}
		if($this->isFilenameDouble()){
			return sprintf(g_l('weEditor', '[' . $this->ContentType . '][response_path_exists]', true) ?: g_l('weEditor', '[' . we_base_ContentTypes::APPLICATION . '][response_path_exists]'), $this->Path);
		}
		if(!$this->checkPathDiffAndCreate()){
			return sprintf(g_l('weClass', '[notValidFolder]'), $this->Path);
		}
		if(($n = $this->check_requiredFields())){
			return sprintf(g_l('weEditor', '[required_field_alert]'), $n);
		}
		if($this->scheduleToBeforeNow()){
			return g_l('modules_schedule', '[toBeforeNow]');
		}
		if(($n = $this->hasDoubbleFieldNames())){
			return sprintf(g_l('weEditor', '[doubble_field_alert]'), $n);
		}
		if(!$this->isVariantNamesValid()){
			return g_l('weEditor', '[variantNameInvalid]');
		}
		return false;
	}

}
