<?php

/**
 * webEdition CMS
 *
 * $Rev: 15035 $
 * $Author: mokraemer $
 * $Date: 2018-03-12 16:26:53 +0100 (Mon, 12 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_objectFile extends we_document{
	const TYPE_BINARY = 'binary';
	const TYPE_CHECKBOX = 'checkbox';
	const TYPE_COLLECTION = 'collection';
	const TYPE_COUNTRY = 'country';
	const TYPE_DATE = 'date';
	const TYPE_FLASHMOVIE = 'flashmovie';
	const TYPE_FLOAT = 'float';
	const TYPE_HREF = 'href';
	const TYPE_IMG = 'img';
	const TYPE_INPUT = 'input';
	const TYPE_INT = 'int';
	const TYPE_LANGUAGE = 'language';
	const TYPE_LINK = 'link';
	const TYPE_META = 'meta';
	const TYPE_MULTIOBJECT = 'multiobject';
	const TYPE_OBJECT = 'object';
	const TYPE_SHOPCATEGORY = 'shopCategory';
	const TYPE_SHOPVAT = 'shopVat';
	const TYPE_TEXT = 'text';
	const ERROR_NOT_SAME_CLASS = -3;

	var $TableID = 0;
	var $rootDirID = 0;
	var $RootDirPath = '/';
	var $Workspaces = '';
	var $AllowedWorkspaces = [];
	var $AllowedClasses = '';
	var $Charset = '';
	var $Language = '';
	var $DefArray = [];
	var $documentCustomerFilter = ''; // DON'T SET TO NULL !!!!
	var $Url = '';
	var $TriggerID = 0;
	protected $classData = [];
	private $DefaultInit = false; // this flag is set when the document was first initialized with default values e.g. from Doc-Types

	/* Constructor */

	function __construct(){
		parent::__construct();
		$this->Table = OBJECT_FILES_TABLE;
		$this->ContentType = we_base_ContentTypes::OBJECT_FILE;
		$this->PublWhenSave = 0;
		$this->IsTextContentDoc = true;
		array_push($this->persistent_slots, 'CSS', 'DefArray', 'Text', 'AllowedClasses', 'Workspaces', 'RootDirPath', 'rootDirID', 'TableID', 'Category', 'IsSearchable', 'Charset', 'Language', 'Url', 'TriggerID', 'classData');
		if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
			array_push($this->persistent_slots, 'From', 'To');
		}
		if(!isset($GLOBALS['WE_IS_DYN'])){
			$ac = we_users_util::getAllowedClasses($this->DB_WE);
			$this->AllowedClasses = implode(',', $ac);
		}
		if(isWE()){
			if(we_modules_info::isActive(we_modules_info::CUSTOMER) && we_base_permission::hasPerm(we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER)){
				$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_WEBUSER;
			}
			$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PROPERTIES, we_base_constants::WE_EDITPAGE_INFO, we_base_constants::WE_EDITPAGE_CONTENT,
				we_base_constants::WE_EDITPAGE_WORKSPACE, we_base_constants::WE_EDITPAGE_PREVIEW, we_base_constants::WE_EDITPAGE_VARIANTS, we_base_constants::WE_EDITPAGE_VERSIONS,
				we_base_constants::WE_EDITPAGE_SCHEDULER]);
		}
		$this->CSS = '';
	}

	public static function initObject($classID, $formname = 'we_global_form', $categories = '', $parentid = 0, $objID = 0, $wewrite = false){
		$session = !empty($GLOBALS['WE_SESSION_START']);

		if(!(isset($GLOBALS['we_object']) && is_array($GLOBALS['we_object']))){
			$GLOBALS['we_object'] = [];
		}
		$GLOBALS['we_object'][$formname] = $wof = new we_objectFile();
		if((!$session) || (!isset($_SESSION['weS']['we_object_session_' . $formname])) || $wewrite){
			if($session){
				$_SESSION['weS']['we_object_session_' . $formname] = [];
			}
			$wof->we_new();
			if($objID){
				$wof->initByID($objID, OBJECT_FILES_TABLE);
				if(!$wof->TableID){
					return false;
				}
			} else {
				$wof->TableID = $classID;
				$wof->setRootDirID(true);
				$wof->resetParentID();
				$wof->restoreDefaults();
			}
			if(($wewrite || !$objID)){
				if($categories){
					$categories = makeIDsFromPathCVS($categories, CATEGORY_TABLE);
					$wof->Category = $categories;
				}
				if($parentid){
					// check if parentid is in correct folder ...
					$parentfolder = new we_class_folder();
					$parentfolder->initByID($parentid, OBJECT_FILES_TABLE);

					if(($wof->ParentPath == $parentfolder->Path) || strpos($parentfolder->Path . '/', $wof->ParentPath) === 0){
						$wof->ParentID = $parentfolder->ID;
						$wof->Path = $parentfolder->Path . '/' . $wof->Filename;
					}
				}
			}


			if($session){
				$wof->saveInSession($_SESSION['weS']['we_object_session_' . $formname]);
			}
		} else {
			if($objID){
				$wof->initByID($objID, OBJECT_FILES_TABLE);
			} elseif($session){
				$wof->we_initSessDat($_SESSION['weS']['we_object_session_' . $formname]);
			}
			if($classID && ($wof->TableID != $classID)){
				$wof->TableID = $classID;
			}
			if(strlen($categories)){
				$categories = makeIDsFromPathCVS($categories, CATEGORY_TABLE);
				$wof->Category = $categories;
			}
		}

		$wof->DefArray = $wof->DefArray ?: $wof->getDefaultValueArray(); //bug #7426

		if(($ret = we_base_request::_(we_base_request::URL, 'we_returnpage'))){
			$wof->setElement('we_returnpage', $ret);
		}

		if(isset($_REQUEST['we_ui_' . $formname]) && is_array($_REQUEST['we_ui_' . $formname])){
			we_base_util::convertDateInRequest($_REQUEST['we_ui_' . $formname], true);

			foreach($_REQUEST['we_ui_' . $formname] as $n => $v){
				$v = we_base_util::rmPhp($v);
				$wof->convertElemFromRequest('', $v, $n);
				$wof->setElement($n, $v);
			}
		}
		if(isset($_REQUEST['we_ui_' . $formname . '_categories'])){
			$cats = makeIDsFromPathCVS(we_base_request::_(we_base_request::WEFILELISTA, 'we_ui_' . $formname . '_categories'), CATEGORY_TABLE);
			$wof->Category = $cats;
		}
		if(isset($_REQUEST['we_ui_' . $formname . '_Category'])){
			$_REQUEST['we_ui_' . $formname . '_Category'] = (is_array($_REQUEST['we_ui_' . $formname . '_Category']) ?
				implode(',', $_REQUEST['we_ui_' . $formname . '_Category']) :
				implode(',', array_filter(explode(',', $_REQUEST['we_ui_' . $formname . '_Category']))));
		}
		foreach($wof->persistent_slots as $slotname){
			if($slotname != 'categories' && ($tmp = we_base_request::_(we_base_request::RAW, 'we_ui_' . $formname . '_' . $slotname)) !== false){
				$v = we_base_util::rmPhp($tmp);
				$wof->convertElemFromRequest('', $v, $slotname);
				$wof->{$slotname} = $v;
			}
		}

		we_imageDocument::checkAndPrepare($formname, 'we_object');
		we_flashDocument::checkAndPrepare($formname, 'we_object');
		we_otherDocument::checkAndPrepare($formname, 'we_object');

		if($session){
			$wof->saveInSession($_SESSION['weS']['we_object_session_' . $formname]);
		}
		return $wof;
	}

	function makeSameNew(array $keep = []){
		$this->DefaultInit = true;
		parent::makeSameNew(array_merge($keep, ['Category', 'TableID', 'rootDirID', 'RootDirPath', 'Workspaces', 'IsSearchable', 'Charset', 'Url', 'TriggerID']));
		$this->objectFileInit(true);
		$this->DefaultInit = false;
	}

	function we_rewrite(){
		$this->setLanguage();
		$this->setUrl();

		return ($this->DB_WE->query('UPDATE ' . $this->DB_WE->escape($this->Table) . ' SET Url="' . $this->DB_WE->escape($this->Url) . '" WHERE ID=' . intval($this->ID))) && parent::we_rewrite();
	}

	private static function getObjectRootPathOfObjectWorkspace($classDir, $classId, we_database_base $db){
		$rootID = f('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE TableID=' . $classId . ' AND IsClassFolder=1');
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
			return $rootID;
		}
		$ws = get_ws(OBJECT_FILES_TABLE, false);
		$lastPath = '';
		$lastID = $rootID;
		$lastLen = PHP_INT_MAX;
		$db->query('SELECT ID,Path FROM ' . OBJECT_FILES_TABLE . ' WHERE IsFolder=1 AND TableID=' . $classId . ' AND ID IN(' . $ws . ') ORDER BY LENGTH(Path)');
		while($db->next_record()){
			if($db->num_rows() === 1){
				return $db->f('ID');
			}
			if($lastPath){
				$len = strspn($lastPath ^ ($db->f('Path') . '/'), "\0");
				if($len < $lastLen){
					$lastLen = $len;
					$lastID = $db->f('ID');
					$lastPath = substr($lastPath, 0, $lastLen);
				}
			} else {
				$lastPath = $db->f('Path') . '/';
				$lastID = $db->f('ID');
				$lastLen = strlen($lastPath);
			}
		}
		if(!$lastPath){
			return $rootID;
		}
		//find last /, so we have the dir on top of all ws
		$lastPath = substr($lastPath, 0, strrpos($lastPath, '/'));

		if($lastPath === '/'){
			return $rootID;
		}
		return f('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE IsFolder=1 AND TableID=' . $classId . ' AND Path="' . $db->escape($lastPath) . '"');
	}

	function formCopyDocument(){
		$idname = 'we_' . $this->Name . '_CopyID';
		$rootDirId = self::getObjectRootPathOfObjectWorkspace($this->RootDirPath, $this->TableID, $this->DB_WE);
		$but = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.forms[0].elements['" . $idname . "'].value,'" . $this->Table . "','" . $idname . "','','copyDocumentSelect','','" . $rootDirId . "','" . $this->ContentType . "');");
		return we_html_element::hidden($idname, $this->CopyID) . $but;
	}

	function copyDoc($id){
		if(!$id){
			return false;
		}

		$doc = new we_objectFile();
		$doc->InitByID($id, $this->Table, self::LOAD_TEMP_DB);
		$doc->setRootDirID(true);
		if($this->ID == 0){
			foreach($this->persistent_slots as $name){
				$this->{$name} = isset($doc->{$name}) ? $doc->{$name} : '';
			}
			$this->CreationDate = time();
			$this->CreatorID = (isset($_SESSION['user']) ? $_SESSION['user']['ID'] : 0);
			$this->DefaultInit = true;
			$this->rootDirID = $doc->rootDirID;
			$this->RootDirPath = $doc->RootDirPath;
			$this->ID = 0;
			$this->OldPath = '';
			$this->Published = 0;
			$this->Text .= '_copy';
			$this->Path = $this->ParentPath . $this->Text;
			$this->OldPath = $this->Path;
		}
		$this->elements = $doc->elements;
		foreach(array_keys($this->elements) as $n){
			$this->elements[$n]['cid'] = 0;
		}
		$this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
		$this->Category = $doc->Category;
		$this->documentCustomerFilter = $doc->documentCustomerFilter;
		return true;
	}

	function restoreWorkspaces(){
		if(!$this->TableID){ // WORKARROUND for bug 4631
			$acs = we_users_util::getAllowedClasses($this->DB_WE);
			$this->TableID = $acs ? $ac[0] : 0;
			$ac = implode(',', $acs);
		}
		$foo = getHash('SELECT Workspaces,DefaultWorkspaces FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($this->TableID), $this->DB_WE);
		$defwsCSVArray = $foo ? makeArrayFromCSV($foo['DefaultWorkspaces']) : [];
		$owsCSVArray = $foo ? makeArrayFromCSV($foo['Workspaces']) : [];
		$this->Workspaces = [];

// loop throgh all default workspaces
		foreach($defwsCSVArray as $defWs){
// loop through each object workspace
			foreach($owsCSVArray as $ows){
				if(we_users_util::in_workspace($defWs, [$ows], FILE_TABLE)){ // if default workspace is within object workspace
					$this->Workspaces[] = $defWs;
				}
			}
		}

		$this->Workspaces = implode(',', $this->Workspaces);
	}

	function setRootDirID($doit = false){
		if($this->TableID && ($this->InWebEdition || $doit)){
			list($this->RootDirPath, $this->rootDirID) = getHash('SELECT o.Path,of.ID FROM ' . OBJECT_FILES_TABLE . ' of JOIN ' . OBJECT_TABLE . ' o ON o.ID=of.TableID WHERE of.IsClassFolder=1 AND of.ParentID=0 AND o.ID=' . intval($this->TableID), $this->DB_WE, MYSQLI_NUM);
		}
	}

	function resetParentID(){
		$len = strlen($this->RootDirPath . '/');
		if($this->ParentPath === '/' || (substr($this->ParentPath . '/', 0, $len) != substr($this->RootDirPath . '/', 0, $len))){
			$this->setParentID($this->rootDirID);
		}
// adjust to bug #376 regarding workspace
		$workspaceRootDirId = self::getObjectRootPathOfObjectWorkspace($this->RootDirPath, $this->TableID, $this->DB_WE);
		$this->ParentPath = id_to_path($workspaceRootDirId, OBJECT_FILES_TABLE, $this->DB_WE);
		$this->ParentID = $workspaceRootDirId;
	}

	function restoreDefaults($makeSameNewFlag = false){
		$this->DefaultInit = true;
		if(!$makeSameNewFlag){
			$this->resetParentID();
		}
		$this->Owners = '';
		$this->OwnersReadOnly = '';
		$this->RestrictOwners = '';
		$this->Category = '';
		$this->Text = '';
		$this->IsSearchable = 1;
		$this->Charset = '';
		$this->restoreWorkspaces();
		$this->elements = [];
		$hash = getHash('SELECT Users,UsersReadOnly,RestrictUsers,DefaultCategory,DefaultText,DefaultValues,DefaultTriggerID FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($this->TableID), $this->DB_WE);
		if($hash){
// fix - the class access permissions should not be applied

			$this->Category = $hash['DefaultCategory'] ?: '';
			if($hash['DefaultText']){
				$text = $hash['DefaultText'];
				$regs = [];
				if(preg_match('/%unique([^%]*)%/', $text, $regs)){
					$anz = ($regs[1] ? abs($regs[1]) : 16);
					$unique = substr(md5(uniqid(__FUNCTION__, true)), 0, min($anz, 32));
					$text = preg_replace('/%unique[^%]*%/', $unique, $text);
				}
				if(strpos($text, '%ID%') !== false){
//FIXME: this is NOT safe!!! Insert entry, and update afterwards
					$id = 1 + intval(f('SELECT max(ID) FROM ' . OBJECT_FILES_TABLE, '', $this->DB_WE));
					$text = str_replace('%ID%', $id, $text);
				}
				$this->Text = strtr($text, ['%d%' => date('d'),
					'%j%' => date('j'),
					'%m%' => date('m'),
					'%y%' => date('y'),
					'%Y%' => date('Y'),
					'%n%' => date('n'),
					'%h%' => date('H'),
					'%H%' => date('H'),
					'%g%' => date('G'),
					'%G%' => date('G'),
				]);
			}

			if($hash['DefaultValues']){
				$vals = we_unserialize($hash['DefaultValues']);
				if(isset($vals['WE_CSS_FOR_CLASS'])){
					$this->CSS = $vals['WE_CSS_FOR_CLASS'];
				}
				if(!empty($vals['elements']['Charset']['dat'])){
					$this->Charset = $vals['elements']['Charset']['dat'];
				}
				if(is_array($vals)){
					foreach($vals as $name => $field){
						if(is_array($field)){
							$foo = explode('_', $name);
							$type = $foo[0];
							unset($foo[0]);
							$name = implode('_', $foo);
							$n = ($type == self::TYPE_OBJECT ? 'we_object_' . $name : (isset($name) ? $name : ''));
							switch($type){
								case self::TYPE_DATE:
									if(!empty($field['defaultThumb'])){//creationDate
										continue 2;
									}
									break;
								case self::TYPE_MULTIOBJECT:
									$this->setElement($name, is_array($field['meta']) ? implode(',', $field['meta']) : '', 'multiobject');
									break;
							}
							$this->setElement($n, isset($field['default']) ? $field['default'] : '', $type, 'dat');
						}
					}
				}
			}
		}
		$this->setTypeAndLength();
	}

	protected function check_requiredFields(){
		foreach($this->DefArray as $n => $v){
			if(is_array($v) && !empty($v['required'])){
				list($type, $name) = explode('_', $n, 2);
				switch($type){
					case self::TYPE_OBJECT:
						$val = $this->getElement('we_object_' . $name);
						break;
					case self::TYPE_MULTIOBJECT:
						$temp = we_unserialize($this->getElement($name));
						$array = array_filter(isset($temp['objects']) ? $temp['objects'] : $temp);
						$val = !empty($array);
						break;
					case self::TYPE_CHECKBOX:
						$val = $this->getElement($name);
						break;
					case self::TYPE_META:
						$val = $this->getElement($name);
						break;
					default:
						$val = $this->getFieldValue($name, $type);
				}
				switch($val ? $type : '_empty_'){
					case self::TYPE_OBJECT:
						$name = ($val == '0' ? f('SELECT Text FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($name), '', $this->DB_WE) : $name);
					case self::TYPE_MULTIOBJECT:
					case self::TYPE_CHECKBOX:
					case self::TYPE_IMG:
						if($val != '0'){
							break;
						}
					//no break
					case '_empty_':
						return $name;
					default:
				}
			}
		}
		return '';
	}

	protected function isVariantNamesValid(){
		$variationFields = we_base_variants::getAllVariationFields($this);

		if(!empty($variationFields)){
			$i = 0;
			while($this->issetElement(we_base_variants::PREFIX . $i)){
				if(!trim($this->getElement(we_base_variants::PREFIX . $i++))){
					return false;
				}
			}
		}

		return true;
	}

	function getPath(){
		$ParentPath = $this->getParentPath();
		return $ParentPath . ($ParentPath != '/' ? '/' : '') . $this->Text;
	}

	/* must be called from the editor-script. Returns a filename which has to be included from the global-Script */

	function editor(we_base_jsCmd $jsCmd = null){
		switch($this->EditPageNr){
			default:
				$_SESSION['weS']['EditPageNr'] = $this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
			case we_base_constants::WE_EDITPAGE_PROPERTIES:
			case we_base_constants::WE_EDITPAGE_WORKSPACE:
				return new we_editor_properties($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_INFO:
				return new we_editor_info_objectFile($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_CONTENT:
				return new we_editor_contentobjectFile($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				return 'we_showObject.inc.php';
			case we_base_constants::WE_EDITPAGE_SCHEDULER:
				return new we_editor_schedpro($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VARIANTS:
				return new we_editor_variants($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_WEBUSER:
				return new we_editor_weDocumentCustomerFilter($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VERSIONS:
				return new we_editor_versions($this, $jsCmd);
		}
	}

	public function formPath($disablePath = false, $notSetHot = false, $extra = ''){
		$rootDirId = self::getObjectRootPathOfObjectWorkspace($this->RootDirPath, $this->TableID, $this->DB_WE);
		if(!$this->ParentID){
			$this->ParentID = $rootDirId;
			$this->ParentPath = id_to_path($rootDirId, OBJECT_FILES_TABLE);
		}
		$this->setUrl();
		return '<table class="default">
	<tr><td style="padding-bottom:4px;">' . $this->formInputField('', 'Text', g_l('modules_object', '[objectname]'), 0, 255, ['onchange' => "pathOfDocumentChanged(true);"]) . '</td><td></td><td></td></tr>
	<tr><td colspan="3" style="padding-bottom:4px;">' . $this->formDirChooser($rootDirId) . '</td></tr>
	<tr><td colspan="3" style="padding-bottom:4px;">
			<table class="default">
				<tr><td>' . $this->formIsSearchable() . '</td><td class="defaultfont">&nbsp;</td><td>&nbsp;</td></tr>
			</table></td></tr>
	<tr><td colspan="3" style="padding-bottom:4px;">
			<table class="default">
				<tr><td class="defaultfont">' . g_l('modules_object', '[seourl]') . ':</td><td class="defaultfont">&nbsp;</td><td class="defaultfont">&nbsp;' . $this->Url . '</td></tr>
			</table></td></tr>
	<tr><td colspan="3">' . $this->formTriggerDocument() . '</td></tr>
</table>';
	}

	protected function formIsSearchable(){
		return we_html_forms::checkboxWithHidden($this->IsSearchable, 'we_' . $this->Name . '_IsSearchable', g_l('weClass', '[IsSearchable]'), false, 'defaultfont', '_EditorFrame.setEditorIsHot(true);');
	}

	/**
	 * returns	a select menu within a html table. to ATTENTION this function is also used in classes object and objectFile !!!!
	 * 			when $withHeadline is true, a table with headline is returned, default is false
	 * @return	string select menue to determine charset
	 * @param	boolean
	 */
	function formCharset($withHeadline = false){
		$charsets = we_base_charsetHandler::inst()->getCharsetsForTagWizzard();
		$charsets[''] = '';
		asort($charsets);
		reset($charsets);

		$name = 'Charset';

		$inputName = 'we_' . $this->Name . '_Charset';

		$headline = ($withHeadline ? '<tr><td class="defaultfont">' . g_l('weClass', '[Charset]') . '</td></tr>' : '');
		return '
			<table class="default">
				' . $headline . '
				<tr><td>' . we_html_tools::textInput($inputName, $this->Charset, '', [], 'text', '14em') . '</td><td></td><td>' . we_html_tools::select('we_tmp_' . $this->Name . '_select[' . $name . ']', $charsets, 1, $this->Charset, false, [
				'onblur' => '_EditorFrame.setEditorIsHot(true);document.forms[0].elements[\'' . $inputName . '\'].value=this.options[this.selectedIndex].value;top.we_cmd(\'reload_editpage\');',
				'onchange' => '_EditorFrame.setEditorIsHot(true);document.forms[0].elements[\'' . $inputName . '\'].value=this.options[this.selectedIndex].value;top.we_cmd(\'reload_editpage\');'], 'value', 330) . '</td></tr>
			</table>';
	}

	public function formClass(){
		return ($this->ID ?
			'<span class="defaultfont">' . $this->classData['Text'] . '</span>' :
			$this->formSelect2(0, 'TableID', OBJECT_TABLE, 'ID,Text', '', 'IsFolder=0' . ($this->AllowedClasses ? ' AND ID IN(' . $this->AllowedClasses . ')' : '') . ' ORDER BY Path ', 1, $this->TableID, false, "if(_EditorFrame.getEditorDocumentId() != 0){we_cmd('reload_editpage');}else{we_cmd('restore_defaults');};_EditorFrame.setEditorIsHot(true);"));
	}

	public function formClassId(){
		return '<span class="defaultfont">' . $this->TableID . '</span>';
	}

	static function getSortedTableInfo($tableID, $contentOnly, we_database_base $db, $checkVariants = false){
		if(!$tableID){
			return [];
		}

		$tableInfo = $db->metadata(OBJECT_X_TABLE . $tableID);
		$tableInfo2 = [];
		foreach($tableInfo as $arr){
			$names = explode('_', $arr['name']);
			switch($names[0]){
				case 'variant':
					if($names[1] == we_base_variants::ELEMENT_NAME){
						break;
					}
				//no break
				case self::TYPE_INPUT:
				case self::TYPE_TEXT:
				case self::TYPE_INT:
				case self::TYPE_FLOAT:
				case self::TYPE_DATE:
				case self::TYPE_IMG:
				case we_object::QUERY_PREFIX:
				case self::TYPE_MULTIOBJECT:
				case self::TYPE_META:
					if($checkVariants){
						$variantdata = $arr;
					}
				default:
					$tableInfo2[] = $arr;
			}
		}

		if(!$contentOnly){
			return $tableInfo2;
		}

		if($checkVariants && isset($variantdata) && is_array($variantdata)){
			$tableInfo2[] = $variantdata;
		}

		return $tableInfo2;
	}

	function getFieldHTML(we_base_jsCmd $jsCmd, $name, $type, array $attribs, $editable = true, $variant = false){
		switch($type){
			default:
			case self::TYPE_INPUT:
				return $this->getInputFieldHTML($name, $attribs, $editable, $variant);
			case self::TYPE_COUNTRY:
				return $this->getCountryFieldHTML($name, $attribs, $editable, $variant);
			case self::TYPE_LANGUAGE:
				return $this->getLanguageFieldHTML($name, $attribs, $editable, $variant);
			case self::TYPE_HREF:
				return $this->getHrefFieldHTML($name, $attribs, $editable, $variant);
			case self::TYPE_LINK:
				return $this->htmlLinkInput($name, $attribs, $editable, $variant);
			case self::TYPE_TEXT:
				return $this->getTextareaHTML($name, $attribs, $editable, $variant);
			case self::TYPE_IMG:
				return $this->getImageHTML($name, $attribs, $editable, $variant);
			case self::TYPE_BINARY:
				return $this->getBinaryHTML($name, $attribs, $editable);
			case self::TYPE_FLASHMOVIE:
				return $this->getFlashmovieHTML($name, $attribs, $editable);
			case self::TYPE_DATE:
				return $this->getDateFieldHTML($name, $attribs, $editable, $variant);
			case self::TYPE_CHECKBOX:
				return $this->getCheckboxFieldHTML($name, $attribs, $editable);
			case self::TYPE_INT:
				return $this->getIntFieldHTML($name, $attribs, $editable, $variant);
			case self::TYPE_FLOAT:
				return $this->getFloatFieldHTML($name, $attribs, $editable, $variant);
			case self::TYPE_OBJECT:
				return $this->getObjectFieldHTML($jsCmd, $name, $attribs, $editable);
			case self::TYPE_MULTIOBJECT:
				return $this->getMultiObjectFieldHTML($jsCmd, $name, $attribs, $editable);
			case self::TYPE_COLLECTION:
				return $this->getCollectionFieldHTML($name, $attribs, $editable);
			case self::TYPE_META:
				return $this->getMetaFieldHTML($name, $attribs, $editable, $variant);
			case self::TYPE_SHOPVAT:
				return $this->getShopVatFieldHtml($name, $attribs, $editable);
			case self::TYPE_SHOPCATEGORY:
				return $this->getShopCategoryFieldHtml($name, $attribs, $editable);
			case 'OF':
				return '';
		}
	}

	public function getElementByType($name, $type, $attribs){
		switch($type){
			case self::TYPE_TEXT:
			case self::TYPE_INPUT:
			case self::TYPE_COUNTRY:
			case self::TYPE_LANGUAGE:
				return $this->getElement($name);
			case self::TYPE_HREF:
				return parent::getHrefByArray(we_unserialize($this->getElement($name)));
			case self::TYPE_LINK:
				return $this->htmlLinkInput($type, $name, $attribs, false, false);
			case self::TYPE_DATE:
				return $this->getElement($name);
			case self::TYPE_FLOAT:
			case self::TYPE_INT:
				return strlen($this->getElement($name)) ? $this->getElement($name) : '';
			case self::TYPE_META:
				return $this->getElement($name);
			default:
				return $this->getElement($name);
		}
	}

	function getFieldsHTML(we_base_jsCmd $jsCmd, $editable, $asString = false){
		if(empty($this->TableID)){
			return $asString ? '' : [];
		}
		$dv = we_unserialize($this->classData['DefaultValues']);

		$tableInfo_sorted = $this->getSortedTableInfo($this->TableID, true, $this->DB_WE);
		$fields = $regs = [];
		foreach($tableInfo_sorted as $cur){
			if($cur['name'] === 'OF_ID'){
				continue;
			}
			if(preg_match('/(.+?)_(.*)/', $cur['name'], $regs)){
				$fields[] = ['name' => $regs[2], 'type' => $regs[1]];
			}
		}

		$c = '';
		$parts = [];
		foreach($fields as $field){

			$realName = $field['type'] . '_' . $field['name'];
			$edMerk = $editable;
			if(!((!isset($dv[$realName]) || (isset($dv[$realName]) && empty($dv[$realName]['users']))) || we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) || we_users_util::isUserInUsers($_SESSION['user']['ID'], $dv[$realName]['users']))){
				$editable = false;
			}
			$c2 = $this->getFieldHTML($jsCmd, $field['name'], $field['type'], (isset($dv[$realName]) ? $dv[$realName] : []), $editable, false);
			if($asString){
				if($c2){
					$c .= $c2 . we_html_element::br() . we_html_element::br();
				}
			} else {
				$parts[] = [
					'headline' => '',
					'html' => $c2,
					'name' => $realName,
				];
			}

			$editable = $edMerk;
		}
		return $asString ? $c : $parts;
	}

	private function formatDescription($name){
		if(empty($this->DefArray[$name]['editdescription'])){
			return '';
		}
		$desc = $this->DefArray[$name]['editdescription'];
		return we_html_element::div(['class' => 'weObjectDescription'], (strpos($desc, '<script') === false ?
				str_replace("\n", we_html_element::br(), $desc) :
				$desc
		));
	}

	private function getHeadline($type, $name){
		return we_html_element::div(['class' => 'weObjectHeadline'], $name . (empty($this->DefArray[$type . '_' . $name]['required']) ? '' : '*' ));
	}

	private function getMetaFieldHTML($name, array $attribs, $editable = true, $variant = false){
		$vals = ($variant ? $attribs['meta'] : (empty($this->DefArray['meta_' . $name]['meta']) ? [] : $this->DefArray['meta_' . $name]['meta']));
		$element = $this->getElement($name);
		if(!$editable){
			return $this->getPreviewView($name, isset($vals[$element]) ? $vals[$element] : '');
		}
		return ($variant ?
			we_html_tools::select('we_' . $this->Name . '_meta[' . $name . ']', $vals, 1, [ $element ]) :
			$this->getHeadline('meta', $name) . we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('meta_' . $name) . $this->formSelectFromArray('meta', $name, $vals, '', 1, false, [
					'onchange' => '_EditorFrame.setEditorIsHot(true);']))
			);
	}

	private function getObjectFieldHTML(we_base_jsCmd $jsCmd, $ObjectID, array $attribs, $editable = true){
		$db = new DB_WE();
		//FIXME: this is bad matching text instead of id's
		$foo = getHash('SELECT of.Text,of.ID FROM ' . OBJECT_FILES_TABLE . ' of WHERE of.IsClassFolder=1 AND of.TableID=' . intval($ObjectID), $db);
		$name = isset($foo['Text']) ? $foo['Text'] : '';
		$pid = empty($foo['ID']) ? 0 : $foo['ID'];

		$textname = 'we_' . $this->Name . '_txt[we_object_' . $ObjectID . '_path]';
		$idname = 'we_' . $this->Name . '_object[we_object_' . $ObjectID . ']';
		$myid = $this->getElement('we_object_' . $ObjectID);
		$spath = $this->getElement('we_object_' . $ObjectID . '_path');
		list($path, $npubl) = getHash('SELECT Path,Published FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($myid), $db, MYSQLI_NUM) ?: [$spath, 1];
		if($path === ''){
			$myid = 0;
			$npubl = 1;
		}
		$ob = new we_objectFile();
		if($myid){
			$ob->initByID($myid, OBJECT_FILES_TABLE);
			$ob->DefArray = $ob->getDefaultValueArray();
		}

		$inputWidth = 443;

		if(!$editable){
			return we_html_multiIconBox::getHTML('', [
					[
						'headline' => '<span style="cursor: pointer;" class="weObjectHeadline">' . ($ob->Text ?: $name) . '</span>' . ($npubl ? '' : ' <span class="weObjectHeadline" style="color:red">' . g_l('modules_object', '[not_published]') . '</span>'),
						'rightHeadline' => true,
						'html' => ($myid ? $ob->getFieldsHTML($jsCmd, false, true) : ''),
						'fold' => true
					]
			]);
		}

		$cmd = 'object_change_objectlink,' . $GLOBALS['we_transaction'] . ',' . we_object::QUERY_PREFIX . $ObjectID;

		$weSuggest = &we_gui_suggest::getInstance();
		$weSuggest->setAcId($textname . we_base_file::getUniqueId(), '/' . $name);
		$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::OBJECT_FILE]);
		$weSuggest->setInput($textname, $path);
		$weSuggest->setMaxResults(10);
		$weSuggest->setResult($idname, $myid);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setTable(OBJECT_FILES_TABLE);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $idname . "'].value,'" . OBJECT_FILES_TABLE . "','" . $idname . "','" . $textname . "','" . $cmd . "','','" . $pid . "','" . we_base_ContentTypes::OBJECT_FILE . "'," . (we_base_permission::hasPerm(we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS) ? 0 : 1) . ')'));
		$weSuggest->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, ($myid ? "javascript:WE().layout.weEditorFrameController.openDocument('" . OBJECT_FILES_TABLE . "','" . $myid . "','" . we_base_ContentTypes::OBJECT_FILE . "');" : ''), '', '', ($myid ? false : true)));
		$weSuggest->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value=0;document.we_form.elements['" . $textname . "'].value='';top.we_cmd('object_reload_entry_at_object','" . $GLOBALS['we_transaction'] . "','" . we_object::QUERY_PREFIX . $ObjectID . "')"));
		$weSuggest->setWidth($inputWidth);

		return we_html_element::div(['class' => 'weObjectHeadline'], $name .
				(empty($this->DefArray[we_object::QUERY_PREFIX . $ObjectID]['required']) ? '' : '*') .
				($npubl ? '' : we_html_element::span(['style' => 'color:red'], ' ' . g_l('modules_object', '[not_published]')))) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription(we_object::QUERY_PREFIX . $ObjectID) .
				$weSuggest->getHTML()) .
			($_SESSION['weS']['we_mode'] === we_base_constants::MODE_SEE ?
			we_html_multiIconBox::getHTML('', [[
				'html' => ($myid ? $ob->getFieldsHTML($jsCmd, false, true) : ''),
				'fold' => true
			]]) : '');
	}

	private function getCollectionFieldHTML($name, array $attribs, $editable = true){
		$collectionID = $this->getElement($name);

		$db = new DB_WE();
		$collectionID = (($path = f('SELECT Path FROM ' . VFILE_TABLE . ' WHERE ID=' . intval($collectionID) . ' AND IsFolder=0', '', $db))) ? $collectionID : 0;

		$textname = 'we_' . $this->Name . '_txt[' . $name . '_path]';
		$idname = 'we_' . $this->Name . '_collection[' . $name . ']';

		if(!$editable){
			return $this->getPreviewView($name, $path . ' (ID: ' . $collectionID . ')');
		}

		$cmd = 'object_change_objectlink,' . $GLOBALS['we_transaction'] . ',' . we_object::QUERY_PREFIX . $name;

		$buttons = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $idname . "'].value,'" . VFILE_TABLE . "','" . $idname . "','" . $textname . "','" . $cmd . "','','" . 0 . "','" . we_base_ContentTypes::COLLECTION . "'," . (we_base_permission::hasPerm(we_base_perm::WORK_COLLECTIONS_CAN_SEE) ? 0 : 1) . ')') .
			(we_base_permission::hasPerm(we_base_perm::WORK_COLLECTION_NEW) ?
			we_html_button::create_button(we_html_button::ADD_COLLECTION, "javascript:top.we_cmd('edit_new_collection','write_back_to_opener," . $idname . "," . $textname . "','',-1,'" . VFILE_TABLE . "');", '', '', false, false) : ''
			) .
			we_html_button::create_button(we_html_button::OPEN_TO_EDIT, ("javascript:var cid=document.we_form.elements['" . $idname . "'].value;if(cid != '0'){WE().layout.weEditorFrameController.openDocument('" . VFILE_TABLE . "',cid,'" . we_base_ContentTypes::COLLECTION . "');}"), '', '', ($collectionID ? false : false)) . // FIXME: set disabled=true|false on select
			we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value=0;document.we_form.elements['" . $textname . "'].value='';top.we_cmd('object_reload_entry_at_object','" . $GLOBALS['we_transaction'] . "','" . we_object::QUERY_PREFIX . $collectionID . "')");

		$weSuggest = &we_gui_suggest::getInstance();
		$weSuggest->setNoAutoInit(true); // autosuggest is deactivated
		$weSuggest->setAcId($textname);
		$weSuggest->setContentType(we_base_ContentTypes::COLLECTION);
		$weSuggest->setInput($textname, $path);
		$weSuggest->setMaxResults(10);
		$weSuggest->setResult($idname, $collectionID);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setTable(VFILE_TABLE);
		$weSuggest->setWidth(396);

		return $this->getHeadline('collection', $name) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('collection_' . $name) . we_html_tools::formElementTable($weSuggest->getHTML(), '', "left", "defaultfont", $buttons));
	}

	private function getMultiObjectFieldHTML(we_base_jsCmd $jsCmd, $name, array $attribs, $editable = true){
		$temp = we_unserialize($this->getElement($name, 'dat'));
		$objects = isset($temp['objects']) ? $temp['objects'] : $temp;
		$max = empty($this->DefArray[self::TYPE_MULTIOBJECT . '_' . $name]['max']) ? 0 : intval($this->DefArray[self::TYPE_MULTIOBJECT . '_' . $name]['max']);
		$show = min(($max ?: PHP_INT_MAX), count($objects));
		$isSEEM = (isset($_SESSION['weS']['we_mode']) && $_SESSION['weS']['we_mode'] == we_base_constants::MODE_SEE);

		if(!$show && !$editable){
			return $this->getPreviewView($name, '');
		}

		$db = new DB_WE();
		$classid = $this->DefArray[self::TYPE_MULTIOBJECT . '_' . $name]['class'];

		if($editable){
			$f = 1;

			list($rootDir, $rootDirPath) = getHash('SELECT of.ID,of.Path FROM ' . OBJECT_FILES_TABLE . ' of WHERE of.IsClassFolder=1 AND of.TableID=' . intval($classid), $db, MYSQLI_NUM) ?: [
				0, '/'];

			$inputWidth = (true || $isSEEM ? 346 : 411);
			$editObjectButtonDis = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, '', '', '', true);

			$weSuggest = &we_gui_suggest::getInstance();
			$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::OBJECT_FILE]);
			$weSuggest->setMaxResults(10);
			$weSuggest->setSelector(we_gui_suggest::DocSelector);
			$weSuggest->setTable(OBJECT_FILES_TABLE);
			$weSuggest->setWidth($inputWidth);

			$upbutDis = we_html_button::create_button(we_html_button::DIRUP, "#", '', '', true);
			$downbutDis = we_html_button::create_button(we_html_button::DIRDOWN, "#", '', '', true);
			$plusbutDis = we_html_button::create_button(we_html_button::LIST_ADD, "#", '', '', true);
			$parts = [];
			for($f = 0; $f < $show; $f++){
				$textname = 'we_' . $this->Name . '_txt[' . $name . '_path' . $f . ']';
				$idname = 'we_' . $this->Name . '_' . self::TYPE_MULTIOBJECT . '[' . $name . '_default' . $f . ']';

				$path = $this->getElement('we_object_' . $name . '_path');
				if(($myid = intval($objects[$f]))){
					$path = $path ?: f('SELECT Path FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($myid), '', $db);
				}


				$buttontable = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $idname . "'].value,'" . OBJECT_FILES_TABLE . "','" . $idname . "','" . $textname . "','" . 'fieldMultiobject_selectMultiobject_callback,' . ($isSEEM ? 'isSEEM' : '') . ',' . self::TYPE_MULTIOBJECT . '_' . $name . "','','" . $rootDir . "','" . we_base_ContentTypes::OBJECT_FILE . "'," . (we_base_permission::hasPerm(we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS) ? 0 : 1) . ")") .
					($myid ?
					we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument('" . OBJECT_FILES_TABLE . "'," . intval($myid) . ",'" . we_base_ContentTypes::OBJECT_FILE . "');") :
					$editObjectButtonDis
					) .
					(empty($max) || (count($objects) < $max) ?
					we_html_button::create_button(we_html_button::LIST_ADD, "javascript:we_cmd('object_insert_meta_at_object','" . $GLOBALS['we_transaction'] . "','" . self::TYPE_MULTIOBJECT . '_' . $name . "','" . ($f) . "')") :
					$plusbutDis
					) .
					($f > 0 ?
					we_html_button::create_button(we_html_button::DIRUP, "javascript:we_cmd('object_up_meta_at_object','" . $GLOBALS['we_transaction'] . "','" . self::TYPE_MULTIOBJECT . '_' . $name . "','" . ($f) . "')") :
					$upbutDis
					) .
					($f < count($objects) - 1 ?
					we_html_button::create_button(we_html_button::DIRDOWN, "javascript:we_cmd('object_down_meta_at_object','" . $GLOBALS['we_transaction'] . "','" . self::TYPE_MULTIOBJECT . '_' . $name . "','" . ($f) . "')") :
					$downbutDis
					) .
					we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_delete_meta_at_object','" . $GLOBALS['we_transaction'] . "','" . self::TYPE_MULTIOBJECT . '_' . $name . "','" . ($f) . "')");

				$weSuggest->setAcId($textname . we_base_file::getUniqueId(), $rootDirPath);
				$weSuggest->setInput($textname, $path);
				$weSuggest->setResult($idname, $myid);

				$parts[] = [
					'html' => we_html_tools::formElementTable($weSuggest->getHTML(false), '', 'left', 'defaultfont', $buttontable),
					'noline' => true,
				];

				if($isSEEM && $myid){
					$ob = new we_objectFile();
					$ob->initByID($myid, OBJECT_FILES_TABLE);
					$ob->DefArray = $ob->getDefaultValueArray();
					$parts[] = [
						'html' => $ob->getFieldsHTML($jsCmd, false, true),
						'fold' => true,
						'noline' => true,
					];
				}
			}

			$this->setElement($name, implode(',', $objects), 'multiobject');

			return $this->getHeadline(self::TYPE_MULTIOBJECT, $name) . we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription(self::TYPE_MULTIOBJECT . '_' . $name) . we_html_multiIconBox::getHTML('', $parts) .
					(empty($max) || count($objects) < $max ?
					we_html_button::create_button(we_html_button::LIST_ADD, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('object_insert_meta_at_object','" . $GLOBALS['we_transaction'] . "','" . self::TYPE_MULTIOBJECT . '_' . $name . "','" . ($f - 1) . "')") :
					we_html_button::create_button(we_html_button::LIST_ADD, '#', '', '', true))
			);
		}

		$parts = [];
		for($f = 0; $f < $show; $f++){
			if($objects[$f]){
				$ob = new we_objectFile();
				$ob->initByID($objects[$f], OBJECT_FILES_TABLE);
				$ob->DefArray = $ob->getDefaultValueArray();
				$txt = $ob->Text;

				$parts[] = [
					'headline' => $txt,
					'html' => $ob->getFieldsHTML($jsCmd, false, true),
					'fold' => true,
				];
			}
		}

		$this->setElement($name, implode(',', $objects), 'multiobject');

		return we_html_multiIconBox::getHTML('', $parts);
	}

	private function getShopVatFieldHtml($name, array $attribs, $we_editmode = true){
		if($we_editmode && we_modules_info::isActive(we_modules_info::SHOP)){

			$shopVats = we_shop_vats::getAllShopVATs();

			$values = [];
			foreach($shopVats as $shopVat){
				$values[$shopVat->id] = $shopVat->vat . '% - ' . $shopVat->getNaturalizedText() . ' (' . $shopVat->territory . ')';
			}

			$val = $this->getElement($name) ?: $attribs['default'];

			return $this->getHeadline('shopVat', 'shopvat') . we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('shopVat_shopVat') . we_html_tools::select('we_' . $this->Name . '_shopVat[' . $name . ']', $values, 1, $val));
		}
		$val = $this->getElement($name);

		$weShopVat = we_shop_vats::getShopVATById($val) ?: we_shop_vats::getStandardShopVat();

		return $this->getPreviewView($name, $weShopVat->vat);
	}

	private function getShopCategoryFieldHtml($name, array $attribs, $we_editmode = true){
		if($we_editmode){
			$values = [];

			if($attribs['shopcatLimitChoice']){
				$values[] = we_category_category::we_getCatsFromIDs(intval($attribs['default']), ',', true, $this->DB_WE, '', 'Path');
				$input = we_html_tools::select('dummy', $values, 1, 0, false, ['disabled' => 'disabled']) .
					we_html_element::hidden('we_' . $this->Name . '_shopCategory[' . $name . ']', $attribs['default']);
			} else {
				$values = ['0' => ' '] + we_shop_category::getShopCatFieldsFromDir('Path', true); //Fix #9355 don't use array_merge() because numeric keys will be renumbered!
				$input = we_html_tools::select('we_' . $this->Name . '_shopCategory[' . $name . ']', $values, 1, ($this->getElement($name) ?: $attribs['default']));
			}

			return $this->getHeadline('_shopCategory', '_shopcategory') . we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('_shopCategory__shopCategory') . $input);
		}
		$val = we_shop_category::getShopCatFieldByID($this->getElement($name), $this->Category, $attribs['shopcatField'], ($attribs['shopcatShowPath'] == 'false' ? false : true), $attribs['shopcatRootdir'], true);

		return $this->getPreviewView($name, $val);
	}

	private function getHrefFieldHTML($name, array $attribs, $we_editmode = true, $variant = false){
		$hrefArr = we_unserialize($this->getElement($name));
		if(!is_array($hrefArr)){
			$hrefArr = [];
		}
		if(!$we_editmode){
			return $this->getPreviewView($name, parent::getHrefByArray($hrefArr));
		}

		$directory = (isset($attribs['hrefdirectory']) && $attribs['hrefdirectory'] === 'true') ? false : true;
		$file = (isset($attribs['hreffile']) && $attribs['hreffile'] === 'false') ? false : true;

		$attr = ' size="20" ';

		$int = isset($hrefArr['int']) ? $hrefArr['int'] : false;
		$intID = (!empty($hrefArr['intID']) ? $hrefArr['intID'] : '');
		$intPath = $intID ? id_to_path($intID) : '';
		$extPath = isset($hrefArr['extPath']) ? $hrefArr['extPath'] : '';
		$int_elem_Name = 'we_' . $this->Name . '_href[' . $name . we_base_link::MAGIC_INT_LINK . ']';
		$intPath_elem_Name = 'we_' . $this->Name . '_vars[' . $name . we_base_link::MAGIC_INT_LINK_PATH . ']';
		$intID_elem_Name = 'we_' . $this->Name . '_href[' . $name . we_base_link::MAGIC_INT_LINK_ID . ']';
		$ext_elem_Name = 'we_' . $this->Name . '_href[' . $name . we_base_link::MAGIC_INT_LINK_EXTPATH . ']';
		switch(isset($attribs['hreftype']) ? $attribs['hreftype'] : ''){
			case we_base_link::TYPE_INT:
				$out = self::hrefRow($intID_elem_Name, $intID, $intPath_elem_Name, $intPath, $attr, $int_elem_Name, false, true, '', $file, $directory);
				break;
			case we_base_link::TYPE_EXT:
				$out = self::hrefRow('', 0, $ext_elem_Name, $extPath, $attr, $int_elem_Name, false, true, '', $file, $directory);
				break;
			default:
				$out = self::hrefRow($intID_elem_Name, $intID, $intPath_elem_Name, $intPath, $attr, $int_elem_Name, true, $int, '', $file, $directory) .
					self::hrefRow('', 0, $ext_elem_Name, $extPath, $attr, $int_elem_Name, true, $int, '', $file, $directory);
		}
		return ($variant ? '' : $this->getHeadline('href', $name)) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('href_' . $name) . '<table>' . $out . '</table>');
	}

	public static function hrefRow($intID_elem_Name, $intID, $Path_elem_Name, $path, $attr, $int_elem_Name, $showRadio = false, $int = true, $extraCmd = '', $file = true, $directory = false){
		$checked = ($intID_elem_Name && $int) || ((!$intID_elem_Name) && (!$int));

		if($intID_elem_Name){
			$trashbut = we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $intID_elem_Name . "'].value='';document.we_form.elements['" . $Path_elem_Name . "'].value='';_EditorFrame.setEditorIsHot(true);");

			// Important: all extraCmds must call we_cmd('object_selectorHref_callback') at the beginning. actually no extraCmds are used
			$cmd = 'fieldHref_selectIntHref_callback,' . ($showRadio ? 1 : 0) . ',' . $int_elem_Name;
			$but = ( $file ?
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $intID_elem_Name . "'].value,'" . FILE_TABLE . "','" . $intID_elem_Name . "','" . $Path_elem_Name . "','" . $cmd . "','',0,''," . (we_base_permission::hasPerm(we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES) ? 0 : 1) . ",''," . ($directory ? 1 : 0) . ");") :
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',document.we_form.elements['" . $intID_elem_Name . "'].value,'" . FILE_TABLE . "','" . $intID_elem_Name . "','" . $Path_elem_Name . "','" . $cmd . "','',0);")
				);
			$weSuggest = &we_gui_suggest::getInstance();
			$weSuggest->setAcId($int_elem_Name . we_base_file::getUniqueId());
			$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::IMAGE, we_base_ContentTypes::HTML, we_base_ContentTypes::JS,
				we_base_ContentTypes::CSS, we_base_ContentTypes::APPLICATION, we_base_ContentTypes::PDF, we_base_ContentTypes::VIDEO]);
			$weSuggest->setInput($Path_elem_Name, $path, ['onchange' => ($showRadio ? "document.we_form.elements['" . $int_elem_Name . "'][0].checked=true;" : "")]);
			$weSuggest->setMaxResults(10);
			$weSuggest->setResult($intID_elem_Name, $intID);
			$weSuggest->setSelector(we_gui_suggest::DocSelector);
			$weSuggest->setTable(FILE_TABLE);
			$weSuggest->setWidth(200);
		} else {
			$trashbut = we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $Path_elem_Name . "'].value='';_EditorFrame.setEditorIsHot(true);");
			$cmd = 'fieldHref_selectExtHref_callback,' . ($showRadio ? 'showRadio' : '') . ',' . $int_elem_Name;
			$but = (we_base_permission::hasPerm(we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT) ? (
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server','" . $Path_elem_Name . "','" . (($directory && $file) ? 'filefolder' : ($file ? '' : we_base_ContentTypes::FOLDER)) . "',document.forms[0].elements['" . $Path_elem_Name . "'].value,'" . $cmd . "')")
				) : '');
		}

		return '<tr>' .
			($showRadio ?
			'<td>' . we_html_forms::radiobutton(($intID_elem_Name ? 1 : 0), $checked, $int_elem_Name, g_l('tags', (!$intID_elem_Name) ? '[ext_href]' : '[int_href]') . ':&nbsp;', true, 'defaultfont', '') . '</td><td>' :
			'<td><input type="hidden" name="' . $int_elem_Name . '" value="' . ($intID_elem_Name ? 1 : 0) . '" />'
			) .
			($intID_elem_Name ?
			$weSuggest->getHTML() :
			'<input' . ($showRadio ? ' onchange="this.form.elements[\'' . $int_elem_Name . '\'][' . ($intID_elem_Name ? 0 : 1) . '].checked=true;"' : '' ) . ' type="text" style="width:200px" name="' . $Path_elem_Name . '" value="' . $path . '" ' . $attr . ' />'
			) . '</td>
	<td>' . $but . '</td>
	<td>' . $trashbut . '</td>
</tr>';
	}

	private function htmlLinkInput($name, array $attribs, $we_editmode = true, $variant = true){
		$attribs['name'] = $name;
		$link = we_unserialize($this->getElement($name)) ?: ['ctype' => 'text', 'type' => we_base_link::TYPE_EXT, 'href' => '#', 'text' => g_l('global', '[new_link]')];

		$img = new we_imageDocument();

		$startTag = self::getLinkStartTag($link, [], $this->ParentID, $this->Path, $img);

		$editbut = we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:we_cmd('edit_link_at_object','" . $name . "')");
		$delbut = we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_delete_link_at_object','" . $GLOBALS['we_transaction'] . "', 'link_" . $name . "')");
		$buttons = $editbut . $delbut;
		$content = parent::getLinkContent($link, $this->ParentID, $this->Path, $img) ?: g_l('global', '[new_link]');

		return ($variant ? '' : $this->getHeadline('link', $name) ) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('link_' . $name) . ($startTag ? $startTag . $content . '</a>' : $content) . ($we_editmode ? ($buttons) : ''));
	}

	private function getPreviewView($name, $content){
		return '<div class="weObjectHeadline">' . $name . '</div>' .
			( ($content !== '') ? '<div class="weObjectContent">' . $content . '</div>' : '');
	}

	private function getInputFieldHTML($name, array $attribs, $editable = true, $variant = false){
		if(!$editable){
			return $this->getPreviewView($name, $this->getElement($name));
		}
		return ($variant ? '' : $this->getHeadline('input', $name) ) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('input_' . $name) . we_html_tools::textInput('we_' . $this->Name . "_input[$name]", $this->getElement($name), $this->getElement($name, "len"), [
					'onchange' => "_EditorFrame.setEditorIsHot(true);"], "text", 620));
	}

	private function getCountryFieldHTML($name, array $attribs, $editable = true, $variant = false){
		$lang = explode('_', $GLOBALS['WE_LANGUAGE']);
		$langcode = array_search($lang[0], getWELangs());

		if(!$editable){
			return '<div class="weObjectHeadline">' . $name . '</div>' .
				($this->getElement($name) != '--' || $this->getElement($name) ? '<div class="defaultfont">' . CheckAndConvertISObackend(we_base_country::getTranslation($this->getElement($name), we_base_country::TERRITORY, $langcode)) . '</div>' :
				'');
		}

		$countrycode = array_search($langcode, getWECountries());
		$countryselect = new we_html_select(['name' => 'we_' . $this->Name . '_language[' . $name . ']', 'style' => "width:620;", 'class' => "wetextinput", 'onchange' => "_EditorFrame.setEditorIsHot(true);"]);

		$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));

		foreach($topCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
		foreach($shownCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $langcode . '_' . $countrycode . '.UTF-8');
		asort($topCountries, SORT_LOCALE_STRING);
		asort($shownCountries, SORT_LOCALE_STRING);
		setlocale(LC_ALL, $oldLocale);

		if(WE_COUNTRIES_DEFAULT){
			$countryselect->addOption('--', CheckAndConvertISObackend(WE_COUNTRIES_DEFAULT));
		}
		foreach($topCountries as $countrykey => &$countryvalue){
			$countryselect->addOption($countrykey, CheckAndConvertISObackend($countryvalue));
		}
		unset($countryvalue);
		if(!empty($topCountries) && !empty($shownCountries)){
			$countryselect->addOption('-', '----', ['disabled' => 'disabled']);
		}

		foreach($shownCountries as $countrykey => &$countryvalue){
			$countryselect->addOption($countrykey, CheckAndConvertISObackend($countryvalue));
		}
		unset($countryvalue);
		$countryselect->selectOption($this->getElement($name));

		return ($variant ? '' : $this->getHeadline('country', $name) ) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('country_' . $name) . $countryselect->getHtml());
	}

	private function getLanguageFieldHTML($name, array $attribs, $editable = true, $variant = false){
		if(!$editable){
			return '<div class="weObjectPreviewHeadline">' . $name . '</div>' .
				($this->getElement($name) != '--' || $this->getElement($name) ? '<div class="defaultfont">' . CheckAndConvertISObackend(we_base_country::getTranslation($this->getElement($name), we_base_country::LANGUAGE, array_search($GLOBALS['WE_LANGUAGE'], getWELangs()))) . '</div>' :
				'');
		}
		$frontendL = $GLOBALS['we']['lang']['FrontendLanguages'];
		foreach($frontendL as &$lcvalue){
			$lccode = explode('_', $lcvalue);
			$lcvalue = $lccode[0];
		}
		$languageselect = new we_html_select(['name' => 'we_' . $this->Name . '_language[' . $name . ']', 'style' => "width:620;", 'class' => "wetextinput", 'onchange' => "_EditorFrame.setEditorIsHot(true);"]);
		if(!$this->DefArray['language_' . $name]['required']){
			$languageselect->addOption('--', '');
		}

		foreach(g_l('languages', '') as $languagekey => $languagevalue){
			if(in_array($languagekey, $frontendL)){
				$languageselect->addOption($languagekey, $languagevalue);
			}
		}
		$languageselect->selectOption($this->getElement($name));
		return ($variant ? '' : $this->getHeadline('language', $name) ) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('language_' . $name) . $languageselect->getHtml());
	}

	private function getCheckboxFieldHTML($name, array $attribs, $editable = true){
		if(!$editable){
			return $this->getPreviewView($name, g_l('global', ($this->getElement($name) ? '[yes]' : '[no]')));
		}
		return $this->getHeadline('checkbox', $name) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('checkbox_' . $name) . we_html_forms::checkboxWithHidden(($this->getElement($name) ? true : false), 'we_' . $this->Name . '_checkbox[' . $name . ']', '', false, 'defaultfont', "_EditorFrame.setEditorIsHot(true);"));
	}

	private function getIntFieldHTML($name, array $attribs, $editable = true, $variant = false){
		if(!$editable){
			return $this->getPreviewView($name, (strlen($this->getElement($name)) ? $this->getElement($name) : ''));
		}
		return ($variant ? '' : $this->getHeadline('int', $name) ) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('int_' . $name) . we_html_tools::textInput('we_' . $this->Name . '_int[' . $name . ']', $this->getElement($name), $this->getElement($name, 'len'), [
					'onchange' => "_EditorFrame.setEditorIsHot(true);"], 'number', 620));
	}

	private function getFloatFieldHTML($name, array $attribs, $editable = true, $variant = false){
		if(!$editable){
			return $this->getPreviewView($name, $this->getElement($name));
		}

		return ($variant ? '' : $this->getHeadline('float', $name)) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('float_' . $name) . we_html_tools::textInput('we_' . $this->Name . '_float[' . $name . ']', strlen($this->getElement($name)) ? $this->getElement($name) : '', $this->getElement($name, 'len'), [
					'onchange' => "_EditorFrame.setEditorIsHot(true);"], 'text', 620));
	}

	private function getDateFieldHTML($name, array $attribs, $editable = true, $variant = false){
		if(!$editable){
			return $this->getPreviewView($name, date(g_l('date', '[format][default]'), abs($this->getElement($name))));
		}
		$d = abs($this->getElement($name));
		return ($variant ? '' : $this->getHeadline('date', $name) ) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('date_' . $name) . we_html_tools::getDateInput('we_' . $this->Name . '_date[' . $name . ']', ($d ?: time()), true));
	}

	private function getTextareaHTML($name, array $attribs, $editable = true, $variant = false){
		if(!$editable){
			return $this->getPreviewView($name, $this->getFieldByVal($this->getElement($name), 'txt', $attribs));
		}
		//	send charset which might be determined in template
		$charset = (isset($this->Charset) ? $this->Charset : DEFAULT_CHARSET);

		$value = $this->getElement($name);
		$attr = array_merge($attribs, [
			'wysiwyg' => isset($attribs['dhtmledit']) ? $attribs['dhtmledit'] : false,
			'width' => isset($attribs['width']) ? $attribs['width'] : 620,
			'height' => isset($attribs['height']) ? $attribs['height'] : 200,
			'rows' => 10,
			'cols' => 60,
			'commands' => preg_replace('/ *, */', ',', isset($attribs['commands']) && $attribs['commands'] ? $attribs['commands'] : COMMANDS_DEFAULT),
			'tinyparams' => isset($attribs['tinyparams']) ? $attribs['tinyparams'] : '',
			'templates' => isset($attribs['templates']) ? $attribs['templates'] : '',
			'gallerytemplates' => isset($attribs['gallerytemplates']) ? $attribs['gallerytemplates'] : '',
			'class' => isset($attribs['class']) ? $attribs['class'] : '',
			'classes' => isset($attribs['cssClasses']) ? $attribs['cssClasses'] : ''
		]);
		unset($attr['meta']);

		$removefirstparagraph = ((!isset($attribs['removefirstparagraph'])) || ($attribs['removefirstparagraph'] === 'on')) ? true : false;
		$xml = (isset($attribs['xml']) && ($attribs['xml'] === 'on')) ? true : false;

		//$autobr = $this->getElement($name, 'autobr') ?: (isset($attribs['autobr']) ? $attribs['autobr'] : '');
		//$autobrName = 'we_' . $this->Name . '_text[' . $name . '#autobr]';

		return ($variant ? '' : $this->getHeadline('text', $name) ) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('text_' . $name) . we_html_forms::weTextarea('we_' . $this->Name . '_text[' . $name . ']', $value, $attr, false, 'autobrName', ((!empty($attribs['classes'])) || $this->getDocumentCss()) ? false : true, false, $xml, $removefirstparagraph, $charset, false, $name));
	}

	private function getImageHTML($name, array $attribs, $editable = true, $variant = false){
		$img = new we_imageDocument();
		$id = $this->getElement($name);
		if(!id_to_path($id)){
			$id = 0;
			$this->setElement($name, 0);
		}
		$img->initByID($id, FILE_TABLE, false);

// handling thumbnails for this image
// identifying default thumbnail of class:
		$defvals = $this->DefArray;
		$tmpName = ($variant ?
			preg_replace('|' . we_base_variants::PREFIX . '\d+_|', '', $name) :
			$name);

		$thumbID = isset($defvals['img_' . $tmpName]['defaultThumb']) ? $defvals['img_' . $tmpName]['defaultThumb'] : 0;
		$thumbID = $thumbID ? f('SELECT ID FROM ' . THUMBNAILS_TABLE . ' WHERE ID=' . $thumbID) : 0;

// creating thumbnail only if it really exists:
		if($thumbID){
			if($img->ID){
				$imgSrc = WEBEDITION_DIR . 'thumbnail.php?' . http_build_query([
						'id' => $id,
						'thumbID' => $thumbID,
						'm' => filemtime(WEBEDITION_PATH . '../' . $img->Path)
				]);
				$imgHeight = $imgWight = 0;
				$img->setElement('thumbnailid', $thumbID, 'thumb', self::ETYPE_NUMBER);
			} else {
				$imgSrc = ICON_DIR . 'no_image.gif';
				$imgHeight = $imgWight = 64;
			}
		}

		if(!$editable){
			return $this->getPreviewView($name, $img->getHtml());
		}
		$fname = 'we_' . $this->Name . '_img[' . $name . ']';
		$cmd = 'object_reload_entry_at_object,' . $GLOBALS['we_transaction'] . ',img_' . $name . ',setScrollTo';

		return ($variant ? '' : $this->getHeadline('img', $name)) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('img_' . $name) . '<input type=hidden name="' . $fname . '" value="' . $this->getElement($name) . '" />' .
// show thumbnail of image if there exists one:
				($thumbID ?
				'<img src="' . $imgSrc . '" ' . ($imgHeight ? 'style="height:' . $imgHeight . 'px;width:' . $imgWight . 'px"' : '') . '/>' :
				$img->getHtml()) .
				we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument('" . FILE_TABLE . "'," . ($id ?: 0) . ",'" . we_base_ContentTypes::IMAGE . "')", '', '', ($id ? false : true)) .
				we_html_button::create_button(we_html_button::IMAGE_SELECT, "javascript:we_cmd('we_selector_image','" . ($id ?: (isset($this->DefArray['img_' . $name]['defaultdir']) ? $this->DefArray['img_' . $name]['defaultdir'] : 0)) . "','" . FILE_TABLE . "','" . $fname . "','','" . $cmd . "','', " . (!empty($this->DefArray['img_' . $name]['rootdir']) ? $this->DefArray['img_' . $name]['rootdir'] : 0) . ",'" . we_base_ContentTypes::IMAGE . "')") .
				we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_remove_image_at_object','" . $GLOBALS['we_transaction'] . "','img_" . $name . "');setScrollTo();", '', '', ($id ? false : true))
		);
	}

	private function getBinaryHTML($name, array $attribs, $editable = true){
		$img = new we_otherDocument();
		$id = $this->getElement($name);
		$img->initByID($id, FILE_TABLE, false);

		if(!$editable){
			$content = $img->getHtml();
			return $this->getPreviewView($name, $content);
		}
		$fname = 'we_' . $this->Name . '_img[' . $name . ']';

		$cmd = 'object_reload_entry_at_object,' . $GLOBALS['we_transaction'] . ',binary_' . $name;
		$content = '<input type=hidden name="' . $fname . '" value="' . $this->getElement($name) . '" />' .
			$img->getHtml() .
			we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document','" . ($id ?: (isset($this->DefArray['binary_' . $name]['defaultdir']) ? $this->DefArray['binary_' . $name]['defaultdir'] : 0)) . "','" . FILE_TABLE . "','" . $fname . "','','" . $cmd . "','', " . (!empty($this->DefArray['binary_' . $name]['rootdir']) ? $this->DefArray['binary_' . $name]['rootdir'] : 0) . ",'" . we_base_ContentTypes::APPLICATION . "')") .
			we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_remove_image_at_object','" . $GLOBALS['we_transaction'] . "','binary_" . $name . "')");
		return $this->getHeadline('binary', $name) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('binary_' . $name) . $content);
	}

	private function getFlashmovieHTML($name, array $attribs, $editable = true){
		$img = new we_flashDocument();
		$id = $this->getElement($name);
		$img->initByID($id, FILE_TABLE, false);

		if(!$editable){
			return $this->getPreviewView($name, $img->getHtml());
		}
		$content = '';
		$fname = 'we_' . $this->Name . '_img[' . $name . ']';
		$content .= '<input type=hidden name="' . $fname . '" value="' . $this->getElement($name) . '" />' . $img->getHtml();

		$cmd = 'object_reload_entry_at_object,' . $GLOBALS['we_transaction'] . ',flashmovie_' . $name;
		$content .= we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document','" . ($id ?: (isset($this->DefArray['flashmovie_' . $name]['defaultdir']) ? $this->DefArray['flashmovie_' . $name]['defaultdir'] : 0)) . "','" . FILE_TABLE . "','" . $fname . "','','" . $cmd . "','', " . (!empty($this->DefArray['flashmovie_' . $name]['rootdir']) ? $this->DefArray['flashmovie_' . $name]['rootdir'] : 0) . ",'" . we_base_ContentTypes::FLASH . "')") .
			we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_remove_image_at_object','" . $GLOBALS['we_transaction'] . "','flashmovie_" . $name . "')");
		return $this->getHeadline('flashmovie', $name) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription('flashmovie_' . $name) . $content);
	}

	public function getDefaultValueArray(){
		if($this->TableID){
			if(empty($this->classData)){
				$this->classData = getHash('SELECT * FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($this->TableID), $this->DB_WE);
			}
			return is_array($this->classData['DefaultValues']) ? $this->classData['DefaultValues'] : ($this->classData['DefaultValues'] = we_unserialize($this->classData['DefaultValues']));
		}
		t_e('no table id');
		return [];
	}

	public function canMakeNew(){
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
			return true;
		}
		$ac = we_users_util::getAllowedClasses($this->DB_WE);
		return count($ac);
	}

	public static function getPossibleWorkspaces($ClassWs, $all = false){
		$wsArray = makeArrayFromCSV($ClassWs);
		$userWs = get_ws(FILE_TABLE, true);
// wenn User Admin ist oder keine Workspaces zugeteilt wurden
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) || ((!$userWs) && $all)){
// alle ws, welche in Klasse definiert wurden zurückgeben
			return $wsArray ? id_to_path($wsArray, FILE_TABLE, null, true) : [];
		}
// alle UserWs, welche sich in einem der ClassWs befinden zur�ckgeben
		$out = [];
		foreach($userWs as $ws){
			if(we_users_util::in_workspace($ws, $wsArray, FILE_TABLE)){
				$out[] = $ws;
			}
		}

		return $out ? id_to_path($out, FILE_TABLE, null, true) : [];
	}

	private function formWorkspaces(we_base_jsCmd $jsCmd){
		$classWsTmpl = $this->classData['WorkspacesTemplates'];

		$values = self::getPossibleWorkspaces($this->classData['Workspaces']);
		$ws = $this->Workspaces === '' ? [] : explode(',', $this->Workspaces);
//    remove not existing workspaces and templates
		$arr = id_to_path($ws, FILE_TABLE, $this->DB_WE, true);

		//only dirs which are in class
		$newArr = $arr ? array_intersect(array_keys($arr), array_keys($values)) : [];
		sort($newArr);

		$this->Workspaces = implode(',', $newArr);
		//list only dirs not already selected
		$values = array_diff_key($values, array_flip($newArr));

		if(empty($values)){
			$addbut = '';
		} else {
			$values = ['' => g_l('global', '[add_workspace]')] + $values;
			$addbut = we_html_tools::select(md5(uniqid(__FUNCTION__)), $values, 1, '', false, ['onchange' => '_EditorFrame.setEditorIsHot(true);we_cmd(\'object_add_workspace\',this.options[this.selectedIndex].value);']);
		}
		$obj = new we_chooser_multiDirAndTemplate(450, $newArr, 'object_del_workspace', $addbut, $classWsTmpl);

		$obj->isEditable = true;

		return $obj->get($jsCmd);
	}

	/* 	private function getTemplateFromWs($wsID){
	  $classWsTmpl = $this->classData['WorkspacesTemplates'];
	  if(isset($classWsTmpl[$wsID])){
	  return $classWsTmpl[$wsID];
	  }
	  $mwsp = id_to_path($wsID, FILE_TABLE, $this->DB_WE);
	  $wsp = id_to_path(array_keys($classWsTmpl), FILE_TABLE, $this->DB_WE, true);
	  foreach($wsp as $pos => $curWsp){
	  if(substr($mwsp, 0, strlen($wsp)) == $curWsp){
	  return $classWsTmpl[$pos];
	  }
	  }
	  return 0;
	  } */

	function add_workspace($id){
		$workspaces = makeArrayFromCSV($this->Workspaces);

		if(!in_array($id, $workspaces)){
			$workspaces[] = $id;
			$this->Workspaces = implode(',', $workspaces);
		}
	}

	function del_workspace($id){
		$workspaces = explode(',', trim($this->Workspaces));
		if(($pos = array_search($id, $workspaces)) !== false){
			unset($workspaces[$pos]);
		}

		$this->Workspaces = implode(',', $workspaces);
	}

	function ws_from_class(){
		$this->Workspaces = $this->classData['Workspaces'];
	}

	private function getTemplateFromWorkspace($parentID){
		$classWsTmpl = $this->classData['WorkspacesTemplates'];
		$wsArr = makeArrayFromCSV($this->Workspaces);

		if(empty($classWsTmpl) || empty($wsArr)){
			return 0;
		}
		//if the parent is set as workspace in object, this is fast
		if(isset($classWsTmpl[$parentID]) && in_array($parentID, $wsArr)){
			return $classWsTmpl[$parentID];
		}
		//search for set workspaces
		$ws = f('SELECT ID FROM ' . FILE_TABLE . ' AS parent WHERE ID IN (' . implode(',', $wsArr) . ') AND CONCAT(parent.Path,"/")=(
SELECT LEFT(Path,LENGTH(parent.Path)+1) FROM ' . FILE_TABLE . ' WHERE ID=' . intval($parentID) . ') ORDER BY Path DESC LIMIT 1');

		//return the template for found workspace, or for root if set - 0 otherwise
		return ($ws ? $classWsTmpl[$ws] : (isset($classWsTmpl[0]) ? $classWsTmpl[0] : 0));
	}

	function getTemplateID($parentID){
		$tid = $this->getTemplateFromWorkspace($parentID);
		//noting found, use first of class-Template
		return $tid ?: reset($this->classData['WorkspacesTemplates']);
	}

	function getFieldValue($t, $f){
		$elem = $this->getElement($t);
		switch($f){
			case self::TYPE_HREF:
				$hrefArr = we_unserialize($elem);
				if(!is_array($hrefArr)){
					$hrefArr = [];
				}
				return parent::getHrefByArray($hrefArr);
			case self::TYPE_LINK:
				$link = we_unserialize($elem);
				if(is_array($link)){
					$img = new we_imageDocument();
					return self::getLinkContent($link, 0, '', $img);
				}
				return '';
			case self::TYPE_META:
				if(!$this->DefArray || !is_array($this->DefArray)){
					$this->DefArray = $this->getDefaultValueArray();
				}
				$vals = $this->DefArray['meta_' . $t]['meta'];
				return empty($vals[$this->getElement($t)]) ? '' : $vals[$this->getElement($t)];
			default:
				return $elem;
		}
	}

	function setTitleAndDescription(){
		$fields = ['Description' => 'DefaultDesc', 'Title' => 'DefaultTitle', 'Keywords' => 'DefaultKeywords'];

		foreach($fields as $key => $field){
			if(!empty($this->classData[$field])){
				$regs = explode('_', $this->classData[$field], 2);
				if(isset($regs[0]) && $regs[0] !== '' && isset($regs[1]) && $regs[1] !== ''){
					$elem = $this->getFieldValue($regs[1], $regs[0]);
					$this->setElement($key, $elem);
				}
			}
		}
	}

	function setUrl(){
		$max = 3;
		$urlfield = [];
		if(empty($this->classData['DefaultUrl'])){
			$this->Url = '';
		}
		$regs = [];
		$text = $this->classData['DefaultUrl'];
		for($i = 0; $i <= $max; ++$i){
			$cur = '';
			if(!empty($this->classData['DefaultUrlfield' . $i])){
				preg_match('/(.+?)_(.*)/', $this->classData['DefaultUrlfield' . $i], $regs);
				$cur = $urlfield[$i] = (isset($regs[1]) && $regs[1] !== '' && isset($regs[2]) && $regs[2] !== '' ?
					$this->getFieldValue($regs[2], $regs[1]) : '');
			}
			if($i > 0 && preg_match('/%urlfield' . $i . '([^%]*)%/', $text, $regs)){
				$anz = (!$regs[1] ? 64 : abs($regs[1]));
				$text = preg_replace('/%urlfield' . $i . '[^%]*%/', substr($cur, 0, $anz), $text);
			}
		}
		if(!isset($urlfield[0]) || $urlfield[0] == ''){
			$urlfield[0] = time();
		}

		if(preg_match('/%urlunique([^%]*)%/', $text, $regs)){
			$anz = (!$regs[1] ? 16 : abs($regs[1]));
			$unique = substr(md5(uniqid(__FUNCTION__, true)), 0, min($anz, 32));
			$text = preg_replace('/%urlunique[^%]*%/', $unique, $text);
		}

		$text = strtr($text, [
			'%ID%' => $this->ID,
			'%locale%' => $this->Language,
			'%language%' => substr($this->Language, 0, 2),
			'%country%' => substr($this->Language, 4, 2),
			'%d%' => date('d', $this->CreationDate),
			'%j%' => date('j', $this->CreationDate),
			'%m%' => date('m', $this->CreationDate),
			'%y%' => date('y', $this->CreationDate),
			'%Y%' => date('Y', $this->CreationDate),
			'%n%' => date('n', $this->CreationDate),
			'%g%' => date('G', $this->CreationDate),
			'%G%' => date('G', $this->CreationDate),
			'%h%' => date('H', $this->CreationDate),
			'%H%' => date('H', $this->CreationDate),
			'%Md%' => date('d', $this->ModDate),
			'%Mj%' => date('j', $this->ModDate),
			'%Mm%' => date('m', $this->ModDate),
			'%My%' => date('y', $this->ModDate),
			'%MY%' => date('Y', $this->ModDate),
			'%Mn%' => date('n', $this->ModDate),
			'%Mg%' => date('G', $this->ModDate),
			'%MG%' => date('G', $this->ModDate),
			'%Mh%' => date('H', $this->ModDate),
			'%MH%' => date('H', $this->ModDate),
			'%Fd%' => date('d', $urlfield[0]),
			'%Fj%' => date('j', $urlfield[0]),
			'%Fm%' => date('m', $urlfield[0]),
			'%Fy%' => date('y', $urlfield[0]),
			'%FY%' => date('Y', $urlfield[0]),
			'%Fn%' => date('n', $urlfield[0]),
			'%Fg%' => date('G', $urlfield[0]),
			'%FG%' => date('G', $urlfield[0]),
			'%Fh%' => date('H', $urlfield[0]),
			'%FH%' => date('H', $urlfield[0]),
			'%DirSep%' => '/'
			]
		);

		if(strpos($text, '%Parent%') !== false){
			$fooo = getHash('SELECT Text FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($this->ParentID), $this->DB_WE);
			if(!empty($fooo['Text'])){
				$text = str_replace('%Parent%', $fooo['Text'], $text);
			}
		}
		if(strpos($text, '%PathIncC%') !== false){
			$zwtext = ltrim(str_replace($this->Text, '', $this->Path), '/');
			$text = str_replace('%PathIncC%', $zwtext, $text);
		}
		if(strpos($text, '%PathNoC%') !== false){
			$zwtext = str_replace($this->Text, '', $this->Path);
			$classN = $this->classData['Path'];
			$zwtext = ltrim(str_replace($classN, '', $zwtext), '/');
			$text = str_replace('%PathNoC%', $zwtext, $text);
		}
		//remove duplicate "//" which will produce errors and transform URL to lowercase
		$text = preg_replace('|\.+$|', '', strtr((OBJECTSEOURLS_LOWERCASE ? mb_convert_case($text, MB_CASE_LOWER, $this->Charset) : $text), [' ' => '-', '//' => '/']));
		$text = (URLENCODE_OBJECTSEOURLS ?
			strtr(urlencode(str_replace(we_base_util::SEO_FILTER_CHARS, '', $text)), ['%2F' => '/']) :
			we_base_util::convertDisallowedChars($text, $this->Charset));
		$this->Url = substr($text, 0, 256);
	}

	public function insertAtIndex(array $only = [], array $fieldTypes = []){
		if(!($this->IsSearchable && $this->Published)){
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=' . $this->TableID . ' AND ID=' . intval($this->ID));
			return true;
		}

		$this->setTitleAndDescription();
		$text = '';
		foreach($this->nextElement('') as $k){
			$v = $this->getElement($k);
			if(!empty($v)){
				switch($this->getElementType($k)){
					default:
					case self::TYPE_OBJECT:
					case self::TYPE_MULTIOBJECT:
					case self::TYPE_LANGUAGE:
					case self::TYPE_HREF:
						//not handled
						break;
					case self::TYPE_DATE:
						$text .= ' ' . date(g_l('date', '[format][default]'), $v);
						break;
					case self::TYPE_INT:
						$text .= ' ' . intval($v);
						break;
					case self::TYPE_FLOAT:
						$text .= ' ' . floatval($v);
						break;

					case self::TYPE_META://FIXME: meta returns the key not the value
					case self::TYPE_INPUT:
					case 'txt':
					case self::TYPE_TEXT:
						if(strpos($v, 'a:') === 0 || $v{0} === '{'){
							//link/href
							$tmp = we_unserialize($v);
							if(isset($tmp['text'])){
								$text .= ' ' . $tmp['text'];
							}
						} else {
							$text .= ' ' . $v;
						}
						break;
				}
			}
		}
		$maxDB = 65535;
		$text = substr(preg_replace(["/\n+/", '/  +/'], ' ', trim(strip_tags($text))), 0, $maxDB);

		if(!$text){
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=' . $this->TableID . ' AND ID=' . intval($this->ID));
			//no need to keep an entry without relevant data in the index
			return true;
		}

		$ws = array_unique(explode(',', $this->Workspaces));

		if(!$ws){
			return $this->DB_WE->query('REPLACE INTO ' . INDEX_TABLE . ' SET ' . we_database_base::arraySetter([
						'ID' => $this->ID,
						'OID' => $this->ID,
						'Text' => $text,
						'WorkspaceID' => 0,
						'Category' => $this->Category,
						'ClassID' => $this->TableID,
						'Title' => $this->getElement('Title'),
						'Description' => $this->getElement('Description'),
						'Language' => $this->Language
			]));
		}

		//$ws = id_to_path($ws, FILE_TABLE, $this->DB_WE);

		foreach($ws as $w){
			if(!$this->DB_WE->query('REPLACE INTO ' . INDEX_TABLE . ' SET ' . we_database_base::arraySetter([
						'ID' => $this->ID,
						'OID' => $this->ID,
						'Text' => $text,
						'WorkspaceID' => $w,
						'Category' => $this->Category,
						'ClassID' => $this->TableID,
						'Title' => $this->getElement('Title'),
						'Description' => $this->getElement('Description'),
						'Language' => $this->Language
				]))){
				return false;
			}
		}
		return true;
	}

	function setLanguage($language = ''){
		$this->Language = $language ?: $this->Language;
	}

	private function setPublishTime($time){
		$this->Published = $time;
		return
			$this->DB_WE->query('UPDATE ' . OBJECT_FILES_TABLE . ' SET Published=' . $time . ' WHERE ID=' . $this->ID);
	}

	function markAsPublished(){
		return $this->setPublishTime(time());
	}

	function markAsUnPublished(){
		return $this->setPublishTime(0);
	}

	protected function convertElemFromRequest($type, &$v, $k){
		if(!$type){
			foreach(array_keys($this->DefArray) as $n){
				$regs = explode('_', $n, 2);
				$testtype = $regs[0];
				if(isset($regs[1])){
					$fieldname = $regs[1];
					if($k == $fieldname){
						$type = $testtype;
						break;
					}
				}
			}
		}
		switch($type){
			case 'text':
				$v = we_base_util::rmPhp($v);
				break;
			case 'input':
				$v = !empty($this->DefArray[$type . '_' . $k]['forbidhtml']) && $this->DefArray[$type . '_' . $k]['forbidhtml'] === 'on' ? we_base_util::rmHTML($v, true) : we_base_util::rmPhp($v);
				break;
			default:
				$v = we_base_util::rmHTML($v, true);
				break;
		}
	}

	public function we_initSessDat($sessDat){
		parent::we_initSessDat($sessDat);
		$this->DefArray = $this->getDefaultValueArray();
		$this->objectFileInit();
	}

	function we_ImportSave(){
		if(!parent::we_save(true)){
			return false;
		}
		$this->wasUpdate = true;
		return $this->saveTmp();
	}

	function correctWorkspaces(){
		if($this->Workspaces){
			$ws = makeArrayFromCSV($this->Workspaces);
			$newWs = [];
			foreach($ws as $wsID){
				if(f('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ID=' . intval($wsID) . ' AND IsFolder=1', '', $this->DB_WE)){
					$newWs[] = $wsID;
				} else if($wsID == 0 && strlen($wsID) == 1){
					$newWs[] = $wsID;
				}
			}
			$this->Workspaces = implode(',', $newWs);
		}
	}

	protected function isPathNotValid(){
		return parent::isPathNotValid() || $this->ParentID == 0 || $this->ParentPath === '/' || strpos($this->Path, $this->RootDirPath) !== 0;
	}

	function parseTextareaFields($rebuildMode = 'default'){
		foreach($this->elements as $name => $element){
			if($element['type'] === 'text'){
				$this->MediaLinks = array_merge($this->MediaLinks, we_wysiwyg_editor::reparseInternalLinks($element['dat'], false, $name));
			}
		}
	}

	public function we_save($resave = false, $skipHook = false){
		if(intval($this->TableID) == 0 || $this->IsFolder){
			return false;
		}
		$this->errMsg = '';

		if($this->isPathNotValid()){
			return false;
		}
		if(empty($this->classData)){
			$this->getDefaultValueArray();
		}
		$dv = we_unserialize($this->classData['DefaultValues']);
		foreach($this->elements as $n => $elem){
			if(isset($elem['type']) && $elem['type'] == self::TYPE_TEXT){
				if(isset($dv["text_$n"]['xml']) && $dv["text_$n"]['xml'] === "on"){
					$this->elements[$n] = $elem; //FIXME: what do we do here?
				}
			}
		}

		if($this->canHaveVariants()){
			we_base_variants::correctModelFields($this);
		}
		if(!$this->TriggerID){
			$this->TriggerID = f('SELECT TriggerID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($this->ParentID), '', $this->DB_WE);
			if(!$this->TriggerID){
				$this->TriggerID = $this->classData['DefaultTriggerID'];
			}
		}
		$resaveWeDocumentCustomerFilter = true;
		$this->correctWorkspaces();
		$this->correctMultiObject();

		if(!$skipHook){
			$hook = new we_hook_base('preSave', '', [$this, 'resave' => $resave]);
//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}
		}

		if((!$this->ID || $resave)){
			$resaveWeDocumentCustomerFilter = false;
			if((!parent::we_save($resave, true)) || ($resave) || (!$this->we_republish())){
				return false;
			}
		}
		$this->ModDate = time();
		$this->ModifierID = !isset($GLOBALS['we']['Scheduler_active']) && isset($_SESSION['user']['ID']) ? $_SESSION['user']['ID'] : 0;
		$this->wasUpdate = true;
		$this->setUrl();

		$this->unregisterMediaLinks(false);
		$this->parseTextareaFields('temp');
		$this->registerMediaLinks(true);

		if(!$resave && $resaveWeDocumentCustomerFilter){
			$this->resaveWeDocumentCustomerFilter();
		}

		if(!$this->Published){
			if(!we_contents_root::we_save(true)){
				return false;
			}
			if(!$resave && we_contents_temporaryDocument::isInTempDB($this->ID, $this->Table, $this->DB_WE)){
				we_contents_temporaryDocument::delete($this->ID, $this->Table, $this->DB_WE);
			}
		}
		$a = $this->saveTmp();
// version
		if($this->ContentType === we_base_ContentTypes::OBJECT_FILE && defined('VERSIONING_OBJECT') && VERSIONING_OBJECT){
			$version = new we_versions_version();
			$version->save($this);
		}
		if(LANGLINK_SUPPORT){
			if(!is_array($this->LangLinks)){
				$this->LangLinks = [];
			}
			$this->setLanguageLink($this->LangLinks, OBJECT_FILES_TABLE, false, true);
		} else {
			//if language changed, we must delete eventually existing entries in tblLangLink, even if !LANGLINK_SUPPORT!
			$this->checkRemoteLanguage($this->Table, false);
		}

// hook
		if(!$skipHook){
			$hook = new we_hook_base('save', '', [$this, 'resave' => $resave]);
//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}
		}

		return $a;
	}

	function registerMediaLinks($temp = false, $linksReady = false){
		//register media in fields type link
		if(!$linksReady){
			$dv = $this->getDefaultValueArray();
			foreach($dv as $k => $v){
				if(strpos($k, 'link_') === 0){
					$name = str_replace('link_', '', $k);
					$link = $this->getElement($name);
					$link = is_array($link) ? $link : we_unserialize($link, [], true);
					if(isset($link['type']) && isset($link['id']) && isset($link['img_id'])){ //FIXME: $link should be an object so we can check class
						if($link['type'] === 'int' && $link['id']){
							$this->MediaLinks['link[name=' . $name . ']'] = $link['id'];
						}
						if($link['img_id']){
							$this->MediaLinks['link[name=' . $name . ']'] = $link['img_id'];
						}
					}
				}
			}
		}

		return parent::registerMediaLinks($temp, $linksReady);
	}

	function ModifyPathInformation($parentID){
		$this->setParentID($parentID);
		$this->Path = $this->getPath();
		$this->wasUpdate = true;
		$this->savePersistentSlotsToDB(['Text', 'Path', 'ParentID']);
		$this->saveTmp();
		$this->insertAtIndex();
		$this->modifyChildrenPath(); // only on folders, because on other classes this function is empty
	}

	function hasWorkspaces(){
		return $this->classData['Workspaces'] !== '';
	}

	function setTypeAndLength(){
		if(!$this->TableID || $this->IsFolder){
			return;
		}
		$db = $this->DB_WE;
		$tableInfo = $db->metadata(OBJECT_X_TABLE . intval($this->TableID));
		$regs = [];
		foreach($tableInfo as $cur){
			if(preg_match('/(.+?)_(.*)/', $cur['name'], $regs)){
				if($regs[1] != 'OF'){
					$name = $regs[2];
					$this->setElement($name, $cur['len'], $regs[1], 'len');
				}
			}
		}
	}

	public function we_load($from = self::LOAD_MAID_DB){
		switch($from){
			case self::LOAD_SCHEDULE_DB:
				if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
					$sessDat = f('SELECT SerializedData FROM ' . SCHEDULE_TABLE . ' WHERE DID=' . intval($this->ID) . ' AND documentTable="' . stripTblPrefix($this->Table) . '" AND task="' . we_schedule_scheduler::SCHEDULE_FROM . '"', '', $this->DB_WE);
					if($sessDat){
						$this->getPersistentSlotsFromDB();
						if($this->initSerializedDat(we_unserialize($sessDat))){

							//make sure at least TableID is set from db
							//and Published as well #5742
							$this->getPersistentSlotsFromDB('TableID,Published,Text,Path,ParentID');
							$this->getUniqueIDsAndFixNames();
							$this->setTypeAndLength();
							break;
						}
					}
				}
				$from = self::LOAD_TEMP_DB;
//no break;
			case self::LOAD_REVERT_DB: //we_temporaryDocument::revert gibst nicht mehr siehe #5789
			case self::LOAD_TEMP_DB:
				$sessDat = we_contents_temporaryDocument::load($this->ID, $this->Table, $this->DB_WE);
				if($sessDat){
//fixed: at least TableID must be fetched
					$this->getPersistentSlotsFromDB();
//overwrite with new data
					$this->initSerializedDat($sessDat);
//make sure at least TableID is set from db
//and Published as well #5742
					$this->getPersistentSlotsFromDB('TableID,Published,Text,Path,ParentID');
					$this->getUniqueIDsAndFixNames();
					$this->setTypeAndLength();
					break;
				}
				$from = self::LOAD_MAID_DB;
//no break;
			case self::LOAD_MAID_DB:
				parent::we_load($from);
				break;
		}
		$this->classData = getHash('SELECT * FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($this->TableID), $this->DB_WE);
		$this->classData['WorkspacesTemplates'] = (!isset($this->classData['Workspaces']) || $this->classData['Workspaces'] === '' || !isset($this->classData['Templates']) || $this->classData['Templates'] === '' ? [
			] : array_combine(explode(',', $this->classData['Workspaces']), explode(',', $this->classData['Templates'])));
		$this->loadSchedule();
		$this->setTitleAndDescription();
		$this->getLinkedObjects();
		$this->initVariantDataFromDb();
// init Customer Filter !!!!
		$this->initWeDocumentCustomerFilterFromDB();
	}

	private function getUniqueIDsAndFixNames(){
		if(!is_array($this->DefArray) || empty($this->DefArray)){
			return;
		}
		$newDefArr = $this->getDefaultValueArray();
		foreach($newDefArr as $n => $v){
			if(is_array($v) && isset($v['uniqueID'])){
				if(($oldName = $this->DefArrayNameNotEqual($n, $v['uniqueID']))){
					$foo = explode('_', $n);
					unset($foo[0]);
					$nn = implode('_', $foo);
					$foo = explode('_', $oldName);
					unset($foo[0]);
					$no = implode('_', $foo);
					$this->elements[$nn] = isset($this->elements[$no]) ? $this->elements[$no] : '';
					unset($this->elements[$no]);
				}
			}
		}
	}

	private function DefArrayNameNotEqual($name, $uniqueID){
		foreach($this->DefArray as $n => $v){
			if(is_array($v) && isset($v['uniqueID'])){
				if($v['uniqueID'] == $uniqueID){
					return ($n == $name) ? '' : $n;
				}
			}
		}
		return '';
	}

	public function we_publish($DoNotMark = false, $saveinMainDB = true, $skipHook = false){
		if(!$skipHook){
			$hook = new we_hook_base('prePublish', '', [$this]);
//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}
		}
		$old = $this->Published;
		$this->oldCategory = f('SELECT Category FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->ID), '', $this->DB_WE);

		if(!($saveinMainDB ? we_contents_root::we_save(true) : $this->we_save($DoNotMark))){
			return false;
		}
		if($DoNotMark){
			$this->unregisterMediaLinks(true, false);
			$this->parseTextareaFields('main');
			// TODO: we should try to throw out obsolete elements from temporary! but this affects static docs only!
			// TODO: when doing rebuild media link test all elements against template!
			$this->registerMediaLinks(); // last param: when rebuilding static docs do not delete temp entries!
		} else {
			if(!$this->markAsPublished()){
				return false;
			}

			$this->unregisterMediaLinks();
			$this->registerMediaLinks(false, true);
		}
		//hook
		if(!$skipHook){
			$hook = new we_hook_base('publish', '', [$this, 'prePublishTime' => $old]);
//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}
		}
		if(!$DoNotMark){
			we_contents_temporaryDocument::delete($this->ID, $this->Table, $this->DB_WE);
		}
		//if($oldUrl != $this->Url || !$wasPublished || $this->oldCategory != $this->Category){
		//FIXME: changes of customerFilter are missing here
		$this->rewriteNavigation();
		//}
//clear navigation cache to see change if object in navigation #6916

		return $this->insertAtIndex();
	}

	public function we_unpublish($skipHook = 0){
		$oldPublished = $this->Published;
		if(!$this->ID || !$this->markAsUnPublished()){
			return false;
		}

		/* version */
		if($this->ContentType === we_base_ContentTypes::OBJECT_FILE && defined('VERSIONING_OBJECT') && VERSIONING_OBJECT){
			$version = new we_versions_version();
			$version->save($this, 'unpublished');
		}
		/* hook */
		if(!$skipHook){
			$hook = new we_hook_base('unpublish', '', [$this]);
//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}
		}
//clear navigation cache to see change if object in navigation #6916
		$this->rewriteNavigation();

		$ret = $this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=' . $this->TableID . ' AND ID=' . intval($this->ID));

		// if document was modified before unpublishing, the actual version is in tblTemporaryDoc: we unregister temp=0
		// otherwise we have nothing to do
		if($ret && $oldPublished && ($this->ModDate > $oldPublished)){
			$this->unregisterMediaLinks(true, false);
		}

		return $ret;
	}

	public function we_republish($rebuildMain = true){
		return ($this->Published && $this->ModDate <= $this->Published ?
			$this->we_publish(true, $rebuildMain) :
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=' . $this->TableID . ' AND ID=' . intval($this->ID))
			);
	}

	private function objectFileInit($makeSameNewFlag = false){
		if($this->ID){
			$this->setRootDirID();
			$oldTableID = f('SELECT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($this->ID), '', $this->DB_WE);
			if($oldTableID != $this->TableID){
				$this->resetParentID();
			}
			if(($def = f('SELECT DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($this->TableID), '', $this->DB_WE))){
				$vals = we_unserialize($def);
				if(isset($vals['WE_CSS_FOR_CLASS'])){
					$this->CSS = $vals['WE_CSS_FOR_CLASS'];
				}
			}
		} else if(!empty($GLOBALS['we_EDITOR']) && (!$this->ID)){
			if($this->DefaultInit == true){
				if(!$this->TableID){
					$ac = we_users_util::getAllowedClasses($this->DB_WE);
					$this->AllowedClasses = implode(',', $ac);
					$this->TableID = $ac[0];
				}
				if($this->TableID){
					$this->setRootDirID();
					if(!$makeSameNewFlag){
						$this->resetParentID();
					}
					$this->restoreDefaults($makeSameNewFlag);
				}
			} else {
				$initWeDocumentCustomerFilter = ($this->ParentID ? false : true);

				if(!$this->Charset && $this->DefArray['elements']['Charset']){
					$this->Charset = $this->DefArray['elements']['Charset']['dat'];
				}

				$this->setRootDirID();
				/*
				  if(!isset($this->ParentID)) {
				  $this->resetParentID();
				  }
				 */
				$this->checkAndCorrectParent();
				if($initWeDocumentCustomerFilter){
// get customerFilter of parent Folder
					$tmpFolder = new we_class_folder();
					$tmpFolder->initByID($this->rootDirID, $this->Table);
					$this->documentCustomerFilter = $tmpFolder->documentCustomerFilter;
					unset($tmpFolder);
				}
			}
		}
	}

	function getLinkedObjects(){
		if(!$this->TableID || $this->IsFolder){
			return;
		}
		static $recursiveObjects = [];
		if(empty($recursiveObjects)){
			$recursiveObjects[] = $this->ID;
		}

		$linkObjects = [];
		$tableInfo = $this->getSortedTableInfo($this->TableID, false, $this->DB_WE);
		$regs = [];
		foreach($tableInfo as $cur){
			if($cur['name'] === 'OF_ID'){
				continue;
			}
			if(preg_match('/(.+?)_(.*)/', $cur['name'], $regs)){
				if($regs[1] == self::TYPE_OBJECT){
					$id = $this->getElement('we_' . $cur['name']);
					if($id){
						$linkObjects[] = $id;
					}
				}
			}
		}
		foreach($linkObjects as $id){
			if(!in_array($id, $recursiveObjects)){
				$recursiveObjects[] = $id;
				$tmpObj = new we_objectFile();
				$tmpObj->initByID($id, OBJECT_FILES_TABLE, self::LOAD_MAID_DB);
				array_pop($recursiveObjects);
				foreach($tmpObj->elements as $n => $elem){
					if($elem['type'] != self::TYPE_OBJECT && $n != 'Title' && $n != 'Description'){
						if(!isset($this->elements[$n])){
							$this->elements[$n] = $elem;
						}
					}
				}
			}
		}
	}

	protected function getContentData(){
		if(!$this->TableID || $this->IsFolder){
			return;
		}
		$db = $this->DB_WE;
		$tableInfo = $this->getSortedTableInfo($this->TableID, false, $db);

		$db->query('SELECT * FROM ' . OBJECT_X_TABLE . intval($this->TableID) . ' WHERE OF_ID=' . intval($this->ID));
		if($db->next_record()){
			foreach($tableInfo as $cur){
				if($cur['name'] === 'OF_ID'){
					continue;
				}
				$regs = explode('_', $cur['name'], 2);
				if(count($regs) > 1){

					$name = ($regs[0] == self::TYPE_OBJECT ? 'we_object_' : '') . $regs[1];
					switch($regs[0]){
//						case self::TYPE_HREF:
						case self::TYPE_IMG:
							$key = self::ETYPE_NUMBER;
							break;
						default:
							$key = self::ETYPE_TEXT;
					}

					$this->elements[$name] = [
						$key => $db->f($cur['name']),
						'type' => $regs[0],
						'len' => $cur['len']
					];
//						if($regs[0] == "multiobject"){
//							$this->elements[$name]['class'] = $db->f($tableInfo[$i]['name']);
//						}
				}
			}
// add variant data if available
			$fieldname = 'variant_' . we_base_variants::ELEMENT_NAME;
			$elementName = we_base_variants::ELEMENT_NAME;

			if($db->f($fieldname)){
				$this->elements[$elementName] = [
					'dat' => $db->f($fieldname),
					'type' => 'variant',
					'len' => strlen($db->f($fieldname))
				];
			}
		}
	}

	protected function setText(){
// do nothing here!
	}

	protected function isFilenameEmpty(){
		return ($this->Text === '');
	}

	protected function isFilenameNotValid(){
		return preg_match('/[^a-z0-9\._\-]/i', $this->Text);
	}

	protected function isFilenameNotAllowed(){
		return false;
	}

	protected function isFilenameDouble(){
		return f('SELECT 1 FROM ' . OBJECT_FILES_TABLE . ' WHERE ParentID=' . intval($this->ParentID) . ' AND Text="' . $this->DB_WE->escape($this->Text) . '" AND ID!=' . intval($this->ID), '', $this->DB_WE);
	}

	public function checkFieldsOnSave(){
		if($this->urlDouble()){
			return sprintf(g_l('weEditor', '[' . $this->ContentType . '][we_objecturl_exists]'), $this->Url);
		}
		return parent::checkFieldsOnSave();
	}

	private function urlDouble(){
		$this->setUrl();
		$db = new DB_WE();

		return ($this->Url ? f('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Url="' . $db->escape($this->Url) . '" AND ID!=' . intval($this->ID), '', $db) : false);
	}

	function checkPathDiffAndCreate(){
		return true;
	}

	protected function scheduleToBeforeNow(){
		//FIXME: check schedarray!
		return false; //(we_modules_info::isActive(we_modules_info::SCHEDULER) && ($this->To < time()));
	}

	function publInScheduleTable(){
		return (we_modules_info::isActive(we_modules_info::SCHEDULER) ?
			we_schedule_scheduler::publInScheduleTable($this, $this->DB_WE) :
			false);
	}

	protected function writeDocument(){
		return true; // do nothing;
	}

	function getContentDataFromTemporaryDocs($ObjectID/* , $loadBinary = 0 */){
		$DocumentObject = f('SELECT DocumentObject FROM ' . TEMPORARY_DOC_TABLE . ' WHERE DocumentID=' . intval($ObjectID) . ' AND Active=1 AND  DocTable="' . stripTblPrefix(OBJECT_FILES_TABLE) . '"', '', $this->DB_WE);
		if($DocumentObject){
			$DocumentObject = we_unserialize($DocumentObject);
			if(isset($DocumentObject[0]['elements']) && is_array($DocumentObject[0]['elements'])){
				$this->elements = $DocumentObject[0]['elements'];
			}
		}
	}

	protected function saveContentDataInDB(){
		if(intval($this->TableID) == 0){
			return false;
		}

		$tableInfo = $this->DB_WE->metadata(OBJECT_X_TABLE . intval($this->TableID));

		if(!$this->wasUpdate){
			$this->CreatorID = $this->CreatorID ?: (isset($_SESSION['user']['ID']) ? $_SESSION['user']['ID'] : 0);
		}

		$data = $regs = [];
		foreach($tableInfo as $cur){
			$regs = explode('_', $cur['name'], 2);
			if(count($regs) > 1){
				$name = $regs[1];
				if($regs[0] === 'OF'){
					$data[$cur['name']] = (isset($this->$name) ? $this->$name : '');
				} else {
					$name = ($regs[0] == self::TYPE_OBJECT ? ('we_object_' . $name) : $name);
					$val = $this->getElement($name);
					$data[$cur['name']] = is_array($val) ? we_serialize($val, SERIALIZE_JSON) : $val;
				}
			}
		}
		$where = ($this->wasUpdate) ? ' WHERE OF_ID=' . intval($this->ID) : '';
		$ret = (bool) ($this->DB_WE->query(($this->wasUpdate ? 'UPDATE ' : 'INSERT INTO ') . OBJECT_X_TABLE . intval($this->TableID) . ' SET ' . we_database_base::arraySetter($data) . $where));
		return $ret;
	}

	private function saveTmp(){
		$saveArr = [];
		$this->saveInSession($saveArr);
		if(($this->ModDate > $this->Published) && $this->Published){
			if(!we_contents_temporaryDocument::save($this->ID, $this->Table, $saveArr, $this->DB_WE)){
				return false;
			}
		}
		return $this->savePersistentSlotsToDB(['Path', 'Text', 'ParentID', 'CreatorID', 'ModifierID', 'RestrictOwners', 'Owners', 'OwnersReadOnly', 'Published', 'ModDate',
				'IsSearchable', 'Charset', 'Url', 'TriggerID']);
	}

	function getDocumentData($includepath = ''){
		extract($GLOBALS, EXTR_SKIP); // globalen Namensraum herstellen.
		if(isset($GLOBALS['we_doc'])){
			$backupdoc = $GLOBALS['we_doc'];
		}
		$backObj = isset($GLOBALS['we_obj']) ? $GLOBALS['we_obj'] : 0;
		$GLOBALS['we_obj'] = $this;

		$GLOBALS['we_doc'] = new we_webEditionDocument();
		$GLOBALS['we_doc']->initByObj($this);

		$GLOBALS['we_doc']->InWebEdition = false;
		$we_include = $includepath ?: $GLOBALS['we_doc']->TemplatePath;
		ob_start();
		include($we_include);
		$contents = ob_get_clean();
		if(isset($backupdoc)){
			$GLOBALS['we_doc'] = $backupdoc;
		} else {
			unset($GLOBALS['we_doc']);
		}
		if($backObj){
			$GLOBALS['we_obj'] = $backObj;
		} else {
			unset($GLOBALS['we_obj']);
		}

		return $contents;
	}

	protected function setElementsFromHTTP(){
		parent::setElementsFromHTTP();
		if($_REQUEST){
			$regs = [];
			$hrefFields = false;
			$multiobjectFields = false;
			$imgFields = false;

			foreach(array_keys($_REQUEST) as $n){
				if(preg_match('/^we_' . $this->Name . '_(' . self::TYPE_HREF . '|' . self::TYPE_MULTIOBJECT . '|' . self::TYPE_IMG . ')$/', $n, $regs)){
					${$regs[1] . 'Fields'} |= true;
				}
			}
			if($hrefFields){
				$empty = ['int' => 1, 'intID' => '', 'extPath' => ''];
				$hrefs = $match = [];
				foreach($_REQUEST['we_' . $this->Name . '_' . self::TYPE_HREF] as $k => $val){
					if(preg_match('|^(.+)' . we_base_link::MAGIC_INFIX . '(.+)$|', $k, $match)){
						$hrefs[$match[1]][$match[2]] = $val;
					}
				}

				foreach($hrefs as $k => $v){
					//remove path
					$href = array_merge($empty, $v);
					unset($href['intPath']);
					$this->setElement($k, we_serialize($href, SERIALIZE_JSON), self::TYPE_HREF);
				}
			}

			if($imgFields){
				foreach($_REQUEST['we_' . $this->Name . '_' . self::TYPE_IMG] as $k => $val){
					$this->setElement($k, $val, self::TYPE_IMG, self::ETYPE_NUMBER);
				}
			}

			if($multiobjectFields){
				$multiobjects = [];
				foreach($this->nextElement(self::TYPE_MULTIOBJECT) as $k){
					$multiobjects[$k] = [];
				}

				$match = [];
				foreach($_REQUEST['we_' . $this->Name . '_' . self::TYPE_MULTIOBJECT] as $k => $val){
					if(preg_match('|^(.+)_default(.+)$|', $k, $match)){
						$multiobjects[$match[1]][$match[2]] = $val;
					}
				}

				foreach($multiobjects as $realName => $data){
					ksort($data, SORT_NUMERIC);
					$this->setElement($realName, implode(',', $data), 'multiobject');
				}
			}
		}
	}

	function userCanSave($ctConditionOk = false){
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
			return true;
		}
		if(!we_base_permission::hasPerm(we_base_perm::OBJECTFILES_CAN_SEE)){
			return false;
		}
		if(!$this->RestrictOwners){
			return true;
		}

		$readers = array_keys(array_filter(we_unserialize($this->OwnersReadOnly)));
		return !we_users_util::isUserInUsers($_SESSION['user']['ID'], $readers);
	}

	/**
	 * @return bool
	 * @desc	checks if the user has the right to see an objectfile
	 */
	function userHasPerms(){
		return (we_base_permission::hasPerm(we_base_perm::OBJECTFILES_CAN_SEE) || (!$this->RestrictOwners) || we_users_util::isOwner($this->Owners) || we_users_util::isOwner($this->CreatorID));
	}

	/**
	 * checks if this object can have variants
	 *
	 * if paramter checkField is true, this function checks also, if there are
	 * already fields selected for the variants.
	 *
	 * @return boolean
	 */
	function canHaveVariants($checkFields = false){
		if($this->IsFolder){
			return false;
		}
		$object = new we_object();
		$object->initByID($this->TableID, OBJECT_TABLE);

		return ($checkFields ?
			$object->canHaveVariants() && !empty($object->getVariantFields()) :
			$object->canHaveVariants());
	}

	public function initByID($we_ID, $we_Table = OBJECT_FILES_TABLE, $from = self::LOAD_MAID_DB){
		parent::initByID(intval($we_ID), OBJECT_FILES_TABLE, $from);
		if($this->issetElement('Charset')){
			$this->Charset = $this->getElement('Charset', 'dat');
			$this->delElement('Charset');
		}

// Fix for added field OF_IsSearchable
		if($this->IsSearchable != 1 && $this->IsSearchable != 0){
			$this->IsSearchable = true;
		}
	}

	function initVariantDataFromDb(){
		if(($dat = $this->getElement(we_base_variants::ELEMENT_NAME))){
			if($dat && !is_array($dat)){
// unserialize the variant data when loading the model
				$this->setElement(we_base_variants::ELEMENT_NAME, we_unserialize($dat), 'variant');
			}
			we_base_variants::setVariantDataForModel($this);
		}
	}

	/**
	 * @return	array with the filed names as keys and attributes as values
	 */
	function getVariantFields(){
		if($this->IsFolder){
			return [];
		}
		$object = new we_object();
		$object->initByID($this->TableID, OBJECT_TABLE);
		return $object->getVariantFields();
	}

	function downMetaAtObject($name, $i){
		$old = we_unserialize($this->getElement($name));
		$objects = isset($old['objects']) ? $old['objects'] : $old;
		$temp = $objects[($i + 1)];
		$objects[($i + 1)] = $objects[$i];
		$objects[$i] = $temp;

		$this->setElement($name, implode(',', $objects), 'multiobject');
	}

	function upMetaAtObject($name, $i){
		$old = we_unserialize($this->getElement($name));
		$objects = isset($old['objects']) ? $old['objects'] : $old;
		$temp = $objects[($i - 1)];
		$objects[($i - 1)] = $objects[$i];
		$objects[$i] = $temp;
		$this->setElement($name, implode(',', $objects), 'multiobject');
	}

	function addMetaToObject($name, $pos){
		$amount = 1;
		$old = we_unserialize($this->getElement($name));
		$objects = isset($old['objects']) ? $old['objects'] : $old;
		for($i = count($objects) + $amount - 1; 0 <= $i; $i--){
			if(($pos + $amount) < $i){
				$objects[$i] = $objects[($i - $amount)];
			} else if($pos < $i && $i <= ($pos + $amount)){
				$objects[$i] = 0;
			}
		}
		$this->setElement($name, implode(',', $objects), 'multiobject');
	}

	function removeMetaFromObject($name, $nr){
		$old = we_unserialize($this->getElement($name));
		$objects = isset($old['objects']) ? $old['objects'] : $old;
		for($i = 0; $i < count($objects) - 1; $i++){
			if($i >= $nr){
				$objects[$i] = $objects[($i + 1)];
			}
		}
		unset($objects[$i]);
		$this->setElement($name, implode(',', $objects), 'multiobject');
	}

	function checkAndCorrectParent(){
		if(!isset($this->ParentID) || $this->ParentID == ''){
			$this->resetParentID();
		}
		$len = strlen($this->RootDirPath . '/');
		if(substr($this->ParentPath . '/', 0, $len) != substr($this->RootDirPath . '/', 0, $len)){
			$this->resetParentID();
		}
	}

	protected function updateRemoteLang($db, $id, $lang, $type){
		$hash = getHash('SELECT Language,TableID FROM ' . $db->escape($this->Table) . ' WHERE ID=' . intval($id), $db);
		$oldLang = $hash['Language'];
		if($oldLang == $lang){
			return;
		}
		$tid = $hash['TableID'];
//update Lang of doc
		$db->query('UPDATE ' . $db->escape($this->Table) . ' SET Language="' . $db->escape($lang) . '" WHERE ID=' . intval($id));
		$db->query('UPDATE ' . LANGLINK_TABLE . ' SET DLocale="' . $db->escape($lang) . '" WHERE DID=' . intval($id) . ' AND DocumentTable="' . $db->escape($type) . '"');
//drop invalid entries => is this safe???
		$db->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DID=' . intval($id) . ' AND DocumentTable="' . $db->escape($type) . '" AND Locale!="' . $db->escape($lang) . '"');
	}

	protected function getNavigationFoldersForDoc(){
		$category = array_map('escape_sql_query', array_unique(array_filter(array_merge(explode(',', $this->Category), explode(',', $this->oldCategory)))));

		$queries = ['( (Selection="' . we_navigation_navigation::SELECTION_STATIC . '" OR IsFolder=1) AND SelectionType="' . we_navigation_navigation::STYPE_OBJLINK . '"  AND LinkID=' . intval($this->ID) . ')',
			//FIXME: query should use ID, not parentID
			'(Selection="' . we_navigation_navigation::SELECTION_DYNAMIC . '" AND DynamicSelection="' . we_navigation_navigation::DYN_CLASS . '" AND ClassID=' . $this->TableID . ')'
		];
		if($category){
			//FIXME: query should use ID, not parentID
			$queries[] = '( Selection="' . we_navigation_navigation::SELECTION_DYNAMIC . '" AND DynamicSelection="' . we_navigation_navigation::DYN_CLASS . '" AND (FIND_IN_SET("' . implode('",Categories) OR FIND_IN_SET("', $category) . '",Categories) ) )';
		}

		$this->DB_WE->query('SELECT DISTINCT ParentID FROM ' . NAVIGATION_TABLE . ' WHERE ' . implode(' OR ', $queries));
		return $this->DB_WE->getAll(true);
	}

	public static function getObjectHref($id, $pid, $path = '', $hidedirindex = false, $objectseourls = false){
		if(!$id){
			return '';
		}

		$path = $path ?: getScriptName();

		$foo = getHash('SELECT of.Published,of.Workspaces,of.TriggerID,f.Published AS fPub FROM ' . OBJECT_FILES_TABLE . ' of LEFT JOIN ' . FILE_TABLE . ' f ON (of.TriggerID=f.ID AND f.IsDynamic=1) WHERE of.ID=' . intval($id));

		if(!$foo){
			return '';
		}
		if(!$foo['fPub']){
			//trigger document is not published - we have to find another one.
			$foo['TriggerID'] = 0;
		}

// check if object is published.
		if(empty($GLOBALS['we_doc']) || (!$GLOBALS['we_doc']->InWebEdition && !$foo['Published'])){
			$GLOBALS['we_link_not_published'] = 1;
			return '';
		}

		$showLink = false;
		//note: /=0, so "0" is a valid wsp
		if($foo['Workspaces'] !== ''){
			$wsp = explode(',', $foo['Workspaces']);
			if(we_users_util::in_workspace(($foo['TriggerID'] ?: $pid), $wsp, FILE_TABLE)){
				$showLink = true;
			}
		}
		if($showLink){
			$path = ($foo['TriggerID'] ? id_to_path($foo['TriggerID']) : self::getNextDynDoc($path, $pid, $foo['Workspaces']));
			if(!$path){
				return '';
			}
			$pidstr = ($pid ? '?pid=' . intval($pid) : '');

			if($objectseourls && show_SeoLinks()){
				$objectdaten = getHash('SELECT Url,TriggerID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($id) . ' LIMIT 1');
				if($objectdaten['TriggerID']){
					$path_parts = pathinfo(id_to_path($objectdaten['TriggerID']));

					if($objectdaten['Url']){
						return ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' .
							($hidedirindex && seoIndexHide($path_parts['basename']) ?
							'' :
							$path_parts['filename'] . '/' ) .
							$objectdaten['Url'] . $pidstr;
					}
				}
			}
			if($hidedirindex && !(!empty($GLOBALS['we_editmode']) || !empty($GLOBALS['WE_MAIN_EDITMODE']) )){
				$path_parts = pathinfo($path);
				if(seoIndexHide($path_parts['basename'])){
					$path = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
				}
			}
			return $path . '?' . WEOBJ_PARAM . '=' . intval($id) . str_replace('?', '&amp;', $pidstr);
		}
		if($foo['Workspaces']){
			$path = self::getNextDynDoc('', $pid, $foo['Workspaces']);
			/* $fooArr = makeArrayFromCSV($foo['Workspaces']);
			  $path = f('SELECT Path FROM ' . FILE_TABLE . ' WHERE Published>0 AND ContentType="' . we_base_ContentTypes::WEDOCUMENT . '" AND IsDynamic=1 AND Path LIKE "' . $DB_WE->escape(id_to_path($fooArr[0], FILE_TABLE, $DB_WE)) . '%" LIMIT 1', '', $DB_WE); */
			return ($path ? $path . '?' . WEOBJ_PARAM . '=' . intval($id) . '&pid=' . intval($pid) : '');
		}

		return '';
	}

	//Fix: #10219 leave this public while using in redirectSEOurls.php!
	public static function getNextDynDoc($path, $pid, $ws1){
		if($path && f('SELECT IsDynamic FROM ' . FILE_TABLE . ' WHERE Published>0 AND ContentType="' . we_base_ContentTypes::WEDOCUMENT . '" AND Path="' . $GLOBALS['DB_WE']->escape($path) . '" LIMIT 1')){
			return $path;
		}

		return (we_users_util::in_workspace($pid, explode(',', $ws1), FILE_TABLE) ?
			f('SELECT Path FROM ' . FILE_TABLE . ' WHERE Published>0 AND ContentType="' . we_base_ContentTypes::WEDOCUMENT . '" AND IsDynamic=1 AND Path LIKE "' . id_to_path(intval($pid), FILE_TABLE) . '%" ORDER BY CHAR_LENGTH(Path) LIMIT 1') :
			'');
	}

	//FIMXE: remove, but needed, since objects still serialize links
	function changeLink($name){
		$this->setElement($name, we_serialize($_SESSION['weS']['WE_LINK']));
		unset($_SESSION['weS']['WE_LINK']);
	}

	public function getDocumentCss(){
		return id_to_path($this->CSS, FILE_TABLE, null, true);
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd){
		if($this->EditPageNr != we_base_constants::WE_EDITPAGE_WORKSPACE){
			$parts = [
				['headline' => g_l('weClass', '[path]'),
					'html' => $this->formPath(),
					'icon' => we_html_multiIconBox::PROP_PATH
				],
				['headline' => g_l('modules_object', '[class]'),
					'html' => $this->formClass(),
					'noline' => true,
					'icon' => we_html_multiIconBox::PROP_CLASS,
					'fold' => true,
					// No link to class in normal mode
					'hide' => $_SESSION['weS']['we_mode'] !== we_base_constants::MODE_SEE || we_base_permission::hasPerm(we_base_perm::OBJECTS_CAN_SEE)
				],
				['headline' => g_l('modules_object', '[class]'),
					'html' => '<a href="javascript:WE().layout.weEditorFrameController.openDocument(\'' . OBJECT_TABLE . '\',' . $this->TableID . ',\'object\');">' . $this->formClass() . '</a> (' . g_l('modules_object', '[class_id]') . ': ' . $this->formClassId() . ')',
					'rightHeadline' => true,
					'icon' => we_html_multiIconBox::PROP_CLASS,
					'hide' => $_SESSION['weS']['we_mode'] !== we_base_constants::MODE_NORMAL, //	Link to class in normal mode
				//'fold' => true
				], [
					'headline' => g_l('weClass', '[language]'),
					'html' => $this->formLangLinks(),
					'icon' => we_html_multiIconBox::PROP_LANG,
					'fold' => true
				],
				(we_modules_info::isActive(we_modules_info::CATEGORY) ? [
				'headline' => g_l('global', '[categorys]'),
				'html' => $this->formCategory($jsCmd),
				'icon' => we_html_multiIconBox::PROP_CATEGORIES,
				'fold' => true
				] : null), [
					'headline' => g_l('modules_object', '[copyObject]'),
					'html' => $this->formCopyDocument(),
					'icon' => we_html_multiIconBox::PROP_COPY
				], [
					'headline' => g_l('weClass', '[owners]'),
					'html' => $this->formCreatorOwners($jsCmd),
					'icon' => we_html_multiIconBox::PROP_USER,
					'fold' => true
				], [
					'headline' => g_l('weClass', '[Charset]'),
					'html' => $this->formCharset(),
					'icon' => we_html_multiIconBox::PROP_CHARSET
				]
			];
		} elseif($this->hasWorkspaces()){ //	Show workspaces
			$parts = [[
				'headline' => g_l('weClass', '[workspaces]'),
				'html' => $this->formWorkspaces($jsCmd),
				'noline' => true,
				'icon' => we_html_multiIconBox::PROP_WORKSPACE
				],
				['headline' => '',
					'html' => we_html_button::create_button('ws_from_class', "javascript:we_cmd('object_ws_from_class');_EditorFrame.setEditorIsHot(true);"),
				]
			];
		} else { //	No workspaces defined
			$parts = [[
				'headline' => '',
				'html' => g_l('modules_object', '[no_workspace_defined]'),
				]
			];
		}
		return we_html_multiIconBox::getHTML('PropertyPage', $parts, true);
	}

	private function correctMultiObject(){
		foreach($this->nextElement(self::TYPE_MULTIOBJECT) as $k){
			$old = we_unserialize($this->getElement($k));
			$objects = isset($old['objects']) ? $old['objects'] : $old;
			$this->setElement($k, implode(',', $objects), 'multiobject');
		}
	}

	public static function makePIDTail($pid, $cid, we_database_base $db, $table = FILE_TABLE){
		if($table !== FILE_TABLE){
			return '1';
		}
		static $wsFlag = [];
		$parentIDs = [];
		$pid = intval($pid);
		$cid = intval($cid);

		$parentIDs[] = $pid;
		while($pid != 0){
			$pid = f('SELECT ParentID FROM ' . FILE_TABLE . ' WHERE ID=' . intval($pid), '', $db);
			$parentIDs[] = $pid;
		}
		if(isset($wsFlag[$cid])){
			$flag = $wsFlag[$cid];
		} else {
			$fooArr = we_unserialize(f('SELECT DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . $cid, '', $db));
			$wsFlag[$cid] = $flag = (isset($fooArr['WorkspaceFlag']) ? $fooArr['WorkspaceFlag'] : 1);
		}
		$pid_tail = [];
		if($flag){
			$pid_tail[] = 'of.Workspaces=""';
		}
		foreach($parentIDs as $pid){
			$pid_tail[] = 'FIND_IN_SET(' . intval($pid) . ',of.Workspaces)';
		}
		return ($pid_tail ? ' (' . implode(' OR ', $pid_tail) . ') ' : 1);
	}

}
