<?php

/**
 * webEdition CMS
 *
 * $Rev: 14930 $
 * $Author: lukasimhof $
 * $Date: 2018-02-20 12:34:12 +0100 (Tue, 20 Feb 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
/*  a class for handling flashDocuments. */

class we_otherDocument extends we_binaryDocument{

	function __construct(){
		parent::__construct();
		if(isWE()){
			$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_PREVIEW;
		}
		$this->ContentType = we_base_ContentTypes::APPLICATION;
		$this->uploadableCTs = [we_base_ContentTypes::HTML, we_base_ContentTypes::JS, we_base_ContentTypes::CSS];
	}

	/* must be called from the editor-script. Returns a filename which has to be included from the global-Script */

	function editor(we_base_jsCmd $jsCmd = null){
		switch($this->EditPageNr){
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				return new we_editor_other_preview($this, $jsCmd);
			default:
				return parent::editor($jsCmd);
		}
	}

	/* gets the HTML for including in HTML-Docs */

	function getHtml($dyn = false){
		$data = $this->getElement('data');
		$this->html = ($this->ID || ($data && !is_dir($data) && is_readable($data)) ?
			'<p><b>Datei</b>: ' . $this->Text . '</p>' :
			g_l('global', '[no_file_uploaded]'));

		return $this->html;
	}

	protected function formExtension2(){
		return we_html_tools::formElementTable(we_html_tools::textInput('we_' . $this->Name . '_Extension', $this->Extension, '', ['onchange' => "_EditorFrame.setEditorIsHot(true);",
					'style' => "width:92px"]), g_l('weClass', '[extension]'));
	}

	public function we_save($resave = false, $skipHook = false){
		return parent::we_save($resave, $skipHook);
	}

	/**
	 * create instance of weMetaData to access metadata functionality:
	 */
	protected function getMetaDataReader($force = false){
		return false;
	}

	public function insertAtIndex(array $only = [], array $fieldTypes = []){
		if(!($this->IsSearchable && $this->Published)){
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . intval($this->ID));
			return true;
		}

		$text = '';
		foreach($this->nextElement('txt') as $k){
			$v = trim($this->getElement($k, 'dat'));
			if($v){
				$text .= ' ' . $v;
			}
		}
		$text = trim(strip_tags($text));

		switch($this->Extension){
			case '.doc':
			case '.xls':
			case '.pps':
			case '.ppt':
				$content = '';
				break;
			case '.pdf':
			case '.rtf':
				$content = $this->getDocumentData(1000000);
				break;
			case '.odt':
			case '.ods':
			case '.odf':
			case '.odp':
			case '.odg':
			case '.ott':
			case '.ots':
			case '.otf':
			case '.otp':
			case '.otg':
				$name = $this->getElement('data');
				if(class_exists('ZipArchive', true) && file_exists($name)){
					$zip = new ZipArchive;
					if($zip->open($name) === TRUE){
						$content = CheckAndConvertISOfrontend(strip_tags(preg_replace(['|</text[^>]*>|', '|<text[^/>]*/>|'], ' ', str_replace(['&#x0d;', '&#x0a;'], ' ', $zip->getFromName('content.xml')))));
						$zip->close();
						break;
					}
				}
				$content = '';
				break;
			default:
				$content = '';
		}

		$content = preg_replace('/[\x00-\x1F]/', '', $content);
		$text .= ' ' . trim($content);

		$maxDB = 65535;

		$set = [
			'ID' => intval($this->ID),
			'DID' => intval($this->ID),
			'Text' => substr(preg_replace(["/\n+/", '/  +/'], ' ', $text), 0, $maxDB),
			'WorkspaceID' => intval($this->ParentID),
			'Category' => $this->Category,
			'Doctype' => '',
			'Title' => $this->getElement('Title'),
			'Description' => $this->getElement('Description'),
		];
		return $this->DB_WE->query('REPLACE INTO ' . INDEX_TABLE . ' SET ' . we_database_base::arraySetter($set));
	}

	//FIXME: move to we_contents_pdf
	public function setMetaDataFromFile($file){
		if($this->Extension === '.pdf' && file_exists($file)){
			t_e('wrong documenttype passed!');
		}
	}

	static function checkAndPrepare($formname, $key = 'we_document'){
		$reqName = 'we_ui_' . $formname;
		// check to see if there is an image to create or to change
		if(empty($_FILES[$reqName]) || empty($_FILES[$reqName]['name']) || !is_array($_FILES[$reqName]['name'])){
			return;
		}
		foreach($_FILES[$reqName]['name'] as $binaryName => $filename){
			$binaryDataId = we_base_request::_(we_base_request::STRING, 'WE_UI_BINARY_DATA_ID_' . $binaryName);

			if($binaryDataId !== false && isset($_SESSION[$binaryDataId])){
				$_SESSION[$binaryDataId]['doDelete'] = false;

				if(we_base_request::_(we_base_request::BOOL, 'WE_UI_DEL_CHECKBOX_' . $binaryName)){
					$_SESSION[$binaryDataId]['doDelete'] = true;
				} elseif($filename){
					// file is selected, check to see if it is an image
					$ct = getContentTypeFromFile($filename);
					if($ct == we_base_ContentTypes::APPLICATION){
						$binaryId = intval($GLOBALS[$key][$formname]->getElement($binaryName));

						// move document from upload location to tmp dir
						$_SESSION[$binaryDataId]['serverPath'] = TEMP_PATH . we_base_file::getUniqueId();
						we_base_file::insertIntoCleanUp($_SESSION[$binaryDataId]['serverPath'], 3600);
						move_uploaded_file($_FILES[$reqName]['tmp_name'][$binaryName], $_SESSION[$binaryDataId]['serverPath']);
						$unique = we_base_file::getUniqueId();
						$tmp_Filename = $binaryName . '_' . $unique . '_' . preg_replace('/[^A-Za-z0-9._-]/', '', $_FILES[$reqName]['name'][$binaryName]);
						if($binaryId){
							$_SESSION[$binaryDataId]['id'] = $binaryId;
						}

						$_SESSION[$binaryDataId]['fileName'] = preg_replace('#^(.+)\..+$#', '${1}', $tmp_Filename);
						$_SESSION[$binaryDataId]['extension'] = (strpos($tmp_Filename, '.') > 0) ? preg_replace('#^.+(\..+)$#', '${1}', $tmp_Filename) : '';
						$_SESSION[$binaryDataId]['text'] = $_SESSION[$binaryDataId]['fileName'] . $_SESSION[$binaryDataId]['extension'];
						$_SESSION[$binaryDataId]['type'] = $_FILES[$reqName]['type'][$binaryName];
						$_SESSION[$binaryDataId]['size'] = $_FILES[$reqName]['size'][$binaryName];
						$_SESSION[$binaryDataId]['unique'] = $unique;
					}
				}
			}
		}
	}

}
