<?php

/**
 * webEdition CMS
 *
 * $Rev: 14912 $
 * $Author: mokraemer $
 * $Date: 2018-02-16 09:34:24 +0100 (Fri, 16 Feb 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_textContentDocument extends we_textDocument{
	/* Doc-Type of the document */
	public $DocType = '';

	/* these fields are never read from temporary tables */
	const primaryDBFiels = 'Path,Text,Filename,Extension,ParentID,Published,ModDate,CreatorID,ModifierID,Owners,RestrictOwners,WebUserID,Language,temp_template_id,DocType,TemplateID,OwnersReadOnly,temp_category,urlMap,viewType,IsProtected,CreationDate';

	function __construct(){
		parent::__construct();

		$this->persistent_slots[] = 'DocType';
		$this->PublWhenSave = 0;
		$this->IsTextContentDoc = true;
		if(isWE()){
			if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
				array_push($this->persistent_slots, 'From', 'To');
			}
			$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PREVIEW, we_base_constants::WE_EDITPAGE_SCHEDULER]);
		}
	}

	function editor(we_base_jsCmd $jsCmd = null){
		switch($this->EditPageNr){
			case we_base_constants::WE_EDITPAGE_SCHEDULER:
				return new we_editor_schedpro($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VALIDATION:
				return new we_editor_validateDocument($this, $jsCmd);
			default:
				return parent::editor($jsCmd);
		}
	}

	public function makeSameNew(array $keep = []){
		parent::makeSameNew(array_merge($keep, ['Category', 'ContentType', 'DocType', 'IsSearchable', 'Extension']));
	}

	public function insertAtIndex(array $only = [], array $fieldTypes = []){
		if(!($this->IsSearchable && $this->Published)){
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . intval($this->ID));
			return true;
		}
		$text = '';

		if($only){
			foreach($only as $cur){
				if($this->getElementType($cur) === 'txt'){
					$dat = $this->getElement($cur, 'dat');
					//skip vars + serialized data
					if($dat && $dat[0] !== '{' && $dat[0] !== '[' && $dat[0] !== '$' && !preg_match('-^[asO]:\d+:|^[{\[].*[}\]]$-', $dat)){
						$text .= ' ' . $this->getElement($cur, 'dat');
					}
				}
			}
		} else {
			foreach($this->nextElement('txt') as $k){
				$dat = $this->getElement($k, 'dat');
				if($k[0] === '$' || (isset($k[1]) && $k[1] === '$') || empty($dat)){
					//skip elements whose names are variables or if element is empty
					continue;
				}

				if($dat[0] !== '{' && $dat[0] !== '[' && !preg_match('-^[asO]:\d+:|^[{\[].*[}\]]$-', $dat)){
					$text .= ' ' . $dat;
				}
			}
			//variants are initialized, so nothing special to do
		}

		$maxDB = 65535;
		return $this->DB_WE->query('REPLACE INTO ' . INDEX_TABLE . ' SET ' . we_database_base::arraySetter([
					'ID' => intval($this->ID),
					'DID' => intval($this->ID),
					'Text' => substr(preg_replace(['/(&#160;|&nbsp;)/', "/ *[\r\n]+/", '/  +/'], ' ', trim(strip_tags($text))), 0, $maxDB),
					'WorkspaceID' => intval($this->ParentID),
					'Category' => $this->Category,
					'Doctype' => $this->DocType,
					'Title' => $this->getElement('Title'),
					'Description' => $this->getElement('Description'),
					'Language' => $this->Language
		]));
	}

	/* publish a document */

	function getMetas($code){
		$regs = [];
		$title = (preg_match('|< ?title[^>]*>(.*)< ?/ ?title[^>]*>|i', $code, $regs) ? $regs[1] : '');
		$tempname = we_base_file::saveTemp($code);
		$metas = get_meta_tags($tempname);
		unlink($tempname);
		$metas['title'] = $title;
		return $metas;
	}

	public function changeDoctype($dt = '', $force = false){
		if((!$this->ID) || $force){
			if($dt){
				$this->DocType = $dt;
			}
			if($this->DocType && ($rec = getHash('SELECT dt.*,dtf.Path AS ParentFilePath FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID WHERE dt.ID=' . intval($this->DocType), new DB_WE()))){
				$this->Extension = $rec['Extension'];
				if($rec['ParentFileID']){
					$this->ParentPath = $rec['ParentFilePath'];
					$this->ParentID = $rec['ParentFileID'];
				}
				if($this->ContentType == we_base_ContentTypes::WEDOCUMENT){
					// only switch template, when current template is not in Templates
					$templates = explode(',', $rec['Templates']);
					if(!in_array($this->TemplateID, $templates)){
						$this->setTemplateID($rec['TemplateID']);
					}
					$this->IsDynamic = $rec['IsDynamic'];
				}
				$this->IsSearchable = $rec['IsSearchable'];
				$this->Category = $rec['Category'];
				$this->Language = $rec['Language'];
				$pathFirstPart = substr($this->ParentPath, -1) === '/' ? '' : '/';
				switch($rec['SubDir']){
					case self::SUB_DIR_YEAR:
						$this->ParentPath .= $pathFirstPart . date('Y');
						break;
					case self::SUB_DIR_YEAR_MONTH:
						$this->ParentPath .= $pathFirstPart . date('Y') . '/' . date('m');
						break;
					case self::SUB_DIR_YEAR_MONTH_DAY:
						$this->ParentPath .= $pathFirstPart . date('Y') . '/' . date('m') . '/' . date('d');
						break;
				}
				$this->checkPathDiffAndCreate();
				$this->Text = $this->Filename . $this->Extension;

				// get Customerfilter of parent
				if(we_modules_info::isActive(we_modules_info::CUSTOMER) && isset($this->documentCustomerFilter)){
					$tmpFolder = new we_folder();
					$tmpFolder->initByID($this->ParentID, $this->Table);
					$this->documentCustomerFilter = $tmpFolder->documentCustomerFilter;
					unset($tmpFolder);
				}
			}
		}
	}

	protected function formDocType($disable = false){
		if($disable){
			$name = ($this->DocType ? f('SELECT DocType FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($this->DocType), 'DocType', $this->DB_WE) : g_l('weClass', '[nodoctype]'));
			return g_l('weClass', '[doctype]') . ': ' . $name;
		}
		$dtq = we_doctype_docTypes::getDoctypeQuery($this->DB_WE);

		return $this->formSelect2(0, 'DocType', DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'], 'dt.ID,dt.DocType', g_l('weClass', '[doctype]'), $dtq['where'], 1, $this->DocType, false, (($this->DocType !== '') ?
				"WE().util.showConfirm(window, '', '" . g_l('weClass', '[doctype_changed_question]') . "',['doctype_changed']);" :
				"we_cmd('doctype_changed');"), [], 'left', '', '', we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:top.we_cmd('doctype_edit')", '', '', (!we_base_permission::hasPerm(we_base_perm::WORK_DOCTYPE_EDIT))), ((we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_NO_DOCTYPE_ALLOWED) || ($this->ID && empty($this->DocType)) ) ) ? [
				'', g_l('weClass', '[nodoctype]')] : '');
	}

	function formDocTypeTempl(){
		return '
<table class="default">
	<tr><td style="text-align:left;">' . $this->formDocType($this->Published) . '</td></tr>
	<tr><td>' . $this->formIsSearchable() . '</td><td>' . $this->formInGlossar() . '</td></tr>
</table>';
	}

	public function we_new(){
		parent::we_new();
		$this->Filename = $this->getDefaultFilename();
	}

	public function we_load($from = self::LOAD_MAID_DB){
		switch($from){
			case self::LOAD_MAID_DB:
				parent::we_load($from);
				break;
			case self::LOAD_REVERT_DB: //we_temporaryDocument::revert gibst nicht mehr siehe #5789
				t_e('old call revert_db');
			case self::LOAD_TEMP_DB:
				$sessDat = we_contents_temporaryDocument::load($this->ID, $this->Table, $this->DB_WE);
				if($sessDat){
					$this->initSerializedDat($sessDat);
					$this->getPersistentSlotsFromDB(/* self::primaryDBFiels */);
					if(property_exists($this, 'temp_category')){
						$this->Category = $this->temp_category;
						$this->TemplateID = $this->temp_template_id;
					}
					$this->OldPath = $this->Path;
				} else {
					$this->we_load(self::LOAD_MAID_DB);
				}
				break;
			case self::LOAD_SCHEDULE_DB:
				if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
					$sessDat = we_unserialize(f('SELECT SerializedData FROM ' . SCHEDULE_TABLE . ' WHERE DID=' . intval($this->ID) . ' AND documentTable="' . stripTblPrefix($this->Table) . '" AND task="' . we_schedule_scheduler::SCHEDULE_FROM . '"', '', $this->DB_WE));
					if($sessDat && $this->initSerializedDat($sessDat)){
						$this->getPersistentSlotsFromDB(/* self::primaryDBFiels */);
						$this->OldPath = $this->Path;
						if(property_exists($this, 'temp_category')){
							$this->Category = $this->temp_category;
							$this->TemplateID = $this->temp_template_id;
						}

						break;
					}// take tmp db, when doc not in schedule db
				}
				$this->we_load(self::LOAD_TEMP_DB);

				break;
		}
		$this->OldPath = $this->Path;
		if(isWE()){//no need to load schedule data, if not in editmode
			$this->loadSchedule();
		}
	}

	public function we_save($resave = false, $skipHook = false){
		$this->errMsg = '';
		$this->setText();
		if(!$skipHook){
			$hook = new we_hook_base('preSave', '', [$this, 'resave' => $resave]);
			//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}
		}

		if(!$this->ID && !we_contents_root::we_save(false)){ // when no ID, then allways save before in main table
			return false;
		}
		if(!$resave){
			$this->ModifierID = empty($GLOBALS['we']['Scheduler_active']) && isset($_SESSION['user']['ID']) ? $_SESSION['user']['ID'] : 0;
			$this->ModDate = time();
			$this->wasUpdate = true;
			we_base_history::insertIntoHistory($this);
			$this->resaveWeDocumentCustomerFilter();
		}

		/* version */
		$version = new we_versions_version();

		// allways store in temp-table
		$ret = $this->saveTmp(!$resave);
		$this->OldPath = $this->Path;

		if(($this->ContentType == we_base_ContentTypes::WEDOCUMENT && defined('VERSIONING_TEXT_WEBEDITION') && VERSIONING_TEXT_WEBEDITION) || ($this->ContentType == we_base_ContentTypes::HTML && defined('VERSIONING_TEXT_HTML') && VERSIONING_TEXT_HTML)){
			$version->save($this);
		}

		/* hook */
		if(!$skipHook){
			$hook = new we_hook_base('save', '', [$this, 'resave' => $resave]);
			//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}
		}

		return $ret;
	}

	public function we_publish($DoNotMark = false, $saveinMainDB = true, $skipHook = false){
		if(!$skipHook){
			$hook = new we_hook_base('prePublish', '', [$this]);
			//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}
		}
		$this->oldCategory = f('SELECT Category FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->ID), '', $this->DB_WE);

		if(!($saveinMainDB ? we_contents_root::we_save(true) : $this->we_save($DoNotMark))){
			return false; // calls the root function, so the document will be saved in main-db but it will not be written!
		}

		$oldPublished = $this->Published;

		$this->Published = time();

		if(!$this->writeDocWhenPubl()){
			$this->Published = $oldPublished;
			return false;
		}

		if(!$DoNotMark){
			if(!$this->DB_WE->query('UPDATE ' . $this->DB_WE->escape($this->Table) . ' SET Published=' . intval($this->Published) . ' WHERE ID=' . intval($this->ID))){
				return false; // mark the document as published;
			}
		}

		//Bug #5505
//		if($oldPublished == 0 || $this->isMoved() || $this->Category != $this->oldCategory || $oldDocType != $this->DocType){
		//FIXME: changes of customerFilter are missing here
		$this->rewriteNavigation();
		//	}
		if(!empty($_SESSION['weS']['versions']['fromScheduler']) && (($this->ContentType == we_base_ContentTypes::WEDOCUMENT && defined('VERSIONING_TEXT_WEBEDITION') && VERSIONING_TEXT_WEBEDITION) || ($this->ContentType == we_base_ContentTypes::HTML && defined('VERSIONING_TEXT_HTML') && VERSIONING_TEXT_HTML))){
			$version = new we_versions_version();
			$version->save($this, 'published');
		}
		/* hook */
		if(!$skipHook){
			$hook = new we_hook_base('publish', '', [$this, 'prePublishTime' => $oldPublished]);
			//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}
		}
		we_contents_temporaryDocument::delete($this->ID, $this->Table, $this->DB_WE);
		return $this->insertAtIndex();
	}

	public function we_unpublish($skipHook = 0){
		if(!$this->ID || (file_exists($this->getRealPath(true)) && !we_base_file::deleteLocalFile($this->getRealPath(!$this->isMoved())))){
			return false;
		}
		if(!$this->DB_WE->query('UPDATE ' . $this->DB_WE->escape($this->Table) . ' SET Published=0 WHERE ID=' . intval($this->ID))){
			return false;
		}

		$this->Published = 0;
		$this->rewriteNavigation();

		/* version */
		if((VERSIONING_TEXT_WEBEDITION && $this->ContentType == we_base_ContentTypes::WEDOCUMENT ) || (VERSIONING_TEXT_HTML && $this->ContentType == we_base_ContentTypes::HTML)){
			$version = new we_versions_version();
			$version->save($this, 'unpublished');
		}
		/* hook */
		if(!$skipHook){
			$hook = new we_hook_base('unpublish', '', [$this]);
			//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}
		}

		$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . intval($this->ID));

		return true;
	}

	public function we_republish($rebuildMain = true){
		return ($this->Published ?
			$this->we_publish(true, $rebuildMain) :
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . intval($this->ID))
			);
	}

	function we_resaveTemporaryTable(){
		$saveArr = [];
		$this->saveInSession($saveArr, true);
		if(($this->ModDate > $this->Published) && $this->Published){
			return (!we_contents_temporaryDocument::isInTempDB($this->ID, $this->Table, $this->DB_WE) ?
				we_contents_temporaryDocument::save($this->ID, $this->Table, $saveArr, $this->DB_WE) :
				we_contents_temporaryDocument::resave($this->ID, $this->Table, $saveArr, $this->DB_WE)
				);
		}
		return true;
	}

	function ModifyPathInformation($parentID){
		$this->setParentID($parentID);
		$this->Path = $this->getPath();
		$this->wasUpdate = true;
		$this->savePersistentSlotsToDB([/* 'Filename', 'Extension', */ 'Text', 'Path', 'ParentID']);
		//$this->we_resaveTemporaryTable();
		$this->insertAtIndex();
		$this->modifyChildrenPath(); // only on folders, because on other classes this function is empty
	}

	private function saveTmp($write = true){
		$saveArr = [];
		$this->saveInSession($saveArr, true);
		if(!we_contents_temporaryDocument::save($this->ID, $this->Table, $saveArr, $this->DB_WE)){
			return false;
		}
		if(!$this->savePersistentSlotsToDB(['Path', 'Text', 'Filename', 'Extension', 'ParentID', 'CreatorID', 'ModifierID', 'RestrictOwners', 'Owners', 'Published', 'ModDate',
				'temp_template_id', 'temp_category', 'DocType', 'WebUserID', 'Language'])){
			return false;
		}
		return ($write ? $this->writeDocument() : true);
	}

	protected function writeMainDir($doc){
		return true; // do nothing!
	}

	private function writeDocWhenPubl(){
		if(!$this->ID){
			return false;
		}
		$realPath = $this->getRealPath();
		$parent = str_replace('\\', '/', dirname($realPath));
		$cf = [];
		while(!we_base_file::checkAndMakeFolder($parent, true)){
			$cf[] = $parent;
			$parent = str_replace('\\', '/', dirname($parent));
		}
		for($i = (count($cf) - 1); $i >= 0; $i--){
			we_base_file::createLocalFolderByPath($cf[$i]);
		}
		$doc = $this->getDocumentToSave();
		return parent::writeMainDir($doc);
	}

	public function revert_published(){
		we_contents_temporaryDocument::delete($this->ID, $this->Table, $this->DB_WE);
		$this->initByID($this->ID);
		$this->ModDate = $this->Published;
		$this->we_save();
		$this->we_publish();
		if(we_modules_info::isActive(we_modules_info::WORKFLOW) && $this->ContentType == we_base_ContentTypes::WEDOCUMENT){
			if(we_workflow_utility::inWorkflow($this->ID, $this->Table)){
				we_workflow_utility::removeDocFromWorkflow($this->ID, $this->Table, $_SESSION['user']['ID'], '');
			}
		}
	}

	function publInScheduleTable(){
		return (we_modules_info::isActive(we_modules_info::SCHEDULER) ?
			we_schedule_scheduler::publInScheduleTable($this, $this->DB_WE) :
			false);
	}

}
