<?php

/**
 * webEdition CMS
 *
 * $Rev: 14928 $
 * $Author: mokraemer $
 * $Date: 2018-02-20 00:02:47 +0100 (Tue, 20 Feb 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_dialog_image extends we_dialog_base{

	//private $weFileupload = null;

	protected function __construct($noInternals = true){
		parent::__construct($noInternals);
		$this->changeableArgs = ['type', 'extSrc', 'fileID', 'src', 'fileSrc', 'width', 'height', 'hspace', 'vspace', 'border', 'alt', 'align', 'name', 'thumbnail', 'ratiow',
			'class', 'title', 'longdescid', 'longdescsrc', 'longdesc'];
		$this->dialogTitle = g_l('wysiwyg', '[insert_image]');
		$this->bodyId = 'weImageDialog';
		$this->initByHttp();
	}

	function initBySrc($src, $width = 0, $height = 0, $hspace = 0, $vspace = 0, $border = 0, $alt = '', $align = '', $name = '', $class = '', $title = '', $longdesc = ''){
		if($src){
			$this->args['src'] = $src;
			$tokkens = explode('?', $src);
			$id = false;
			$thumb = 0;

			if(count($tokkens) == 2){
				parse_str($tokkens[1], $foo);
				if(isset($foo['id'])){
					$id = $foo['id'];
					$fileScr = $tokkens[0];
				} else if(isset($foo['thumb'])){
					$foo = explode(',', $foo['thumb']);
					$id = $foo[0];
					$thumb = $foo[1];
					$fileScr = id_to_path($id);
				}
			}

			if($id !== false){
				$this->args['type'] = we_base_link::TYPE_INT;
				$this->args['fileID'] = $id;
				$this->args['fileSrc'] = $id == 0 ? '' : $fileScr;
				$this->args['thumbnail'] = $thumb;
			} else {
				$this->args["type"] = we_base_link::TYPE_EXT;
				$this->args['extSrc'] = preg_replace(['|^https?://' . $_SERVER['SERVER_NAME'] . '(/.*)$|i',
					'|^' . WEBEDITION_DIR . 'we_cmd.php[^"\'#]+(#.*)$|',
					'|^' . WEBEDITION_DIR . '|',
					], ['${1}', '${1}', ''], $this->args['src']);
				$this->args['fileID'] = '';
				$this->args['fileSrc'] = '';
				$this->args['thumbnail'] = 0;
			}
		} else {
			$this->args['type'] = we_base_link::TYPE_INT;
			$this->args['extSrc'] = '';
			$this->args['fileSrc'] = '';
		}
		$this->initAttributes($width, $height, $hspace, $vspace, $border, $alt, $align, $name, $class, $title, $longdesc);
	}

	function initAttributes($width = 0, $height = 0, $hspace = 0, $vspace = 0, $border = 0, $alt = '', $align = '', $name = '', $class = '', $title = '', $longdesc = ''){
		$tokkens = explode('?', $longdesc);
		$longdescid = '';
		if(count($tokkens) == 2){
			$foo = explode('=', $tokkens[1]);
			if(count($foo) == 2){
				if($foo[0] === 'id'){
					$longdescid = $foo[1];
				}
			}
		}

		$this->args['width'] = $width;
		$this->args['height'] = $height;
		$this->args['hspace'] = $hspace;
		$this->args['vspace'] = $vspace;
		$this->args['border'] = $border;
		$this->args['alt'] = $alt;
		$this->args['align'] = $align;
		$this->args['name'] = $name;
		$this->args['class'] = $class;
		$this->args['title'] = $title;
		$this->args['longdesc'] = $longdesc;
		$this->args['longdescid'] = $longdescid;
		$this->args['longdescsrc'] = ($longdesc ? $tokkens[0] : '');
		$this->args['ratio'] = we_base_request::_(we_base_request::INT, 'we_dialog_args', 1, 'ratio');
	}

	function initByFileID($fileID, $width = 0, $height = 0, $hspace = 0, $vspace = 0, $border = 0, $alt = '', $align = '', $name = '', $thumb = '', $class = '', $title = '', $longdesc = ''){
		if($fileID){
			$this->args['type'] = we_base_link::TYPE_INT;
			$this->args['extSrc'] = '';
			$this->args['fileID'] = $fileID;
			if($thumb){
				$thumbObj = new we_contents_thumbnail();
				$thumbObj->initByImageIDAndThumbID($fileID, $thumb);
				$thumbpath = $thumbObj->getOutputPath();
				$this->args['thumbnail'] = $thumb;
				$this->args['fileSrc'] = id_to_path($fileID);
				$this->args['src'] = $thumbpath . '?thumb=' . $fileID . ',' . $thumb;
				$width = ''; //$thumbObj->getOutputWidth();
				$height = ''; //$thumbObj->getOutputHeight();
				unset($thumbObj);
			} else {
				$this->args['thumbnail'] = '';
				$this->args['fileSrc'] = f('SELECT Path FROM ' . FILE_TABLE . ' WHERE ID=' . intval($this->args['fileID']), '', $this->db);
				$this->args['src'] = $this->args['fileSrc'] . '?id=' . $fileID;
			}
			$this->args['ratio'] = 1;
		}
		$this->initAttributes($width, $height, $hspace, $vspace, $border, $alt, $align, $name, $class, $title, $longdesc);
	}

	protected function initByHttp(){
		parent::initByHttp();
		$src = $this->getHttpVar(we_base_request::URL_FIX, 'src');
		$width = $this->getHttpVar(we_base_request::UNIT, 'width');
		$height = $this->getHttpVar(we_base_request::UNIT, 'height');
		$hspace = $this->getHttpVar(we_base_request::INT, 'hspace');
		$vspace = $this->getHttpVar(we_base_request::INT, 'vspace');
		$class = $this->getHttpVar(we_base_request::STRING, 'class');
		$title = $this->getHttpVar(we_base_request::STRING, 'title');
		$longdescsrc = $this->getHttpVar(we_base_request::URL_FIX, 'longdescsrc');
		$longdescid = $this->getHttpVar(we_base_request::INT, 'longdescid');
		$longdesc = ($longdescsrc && $longdescid ? $longdescsrc . '?id=' . $longdescid : $this->getHttpVar(we_base_request::STRING, 'longdesc'));
		$border = $this->getHttpVar(we_base_request::UNIT, 'border');
		$alt = $this->getHttpVar(we_base_request::STRING, 'alt');
		$align = $this->getHttpVar(we_base_request::STRING, 'align');
		$name = $this->getHttpVar(we_base_request::STRING, 'name');
		$isPresetFromDnD = $this->getHttpVar(we_base_request::INT, 'isPresetFromDnD', 0);
		$type = $isPresetFromDnD ? we_base_link::TYPE_INT : ($this->getHttpVar(we_base_request::STRING, 'type') ? $this->getHttpVar(we_base_request::STRING, 'type') : we_base_link::TYPE_INT);
		$thumbnail = $this->getHttpVar(we_base_request::INT, 'thumbnail');

		if($src && !$thumbnail){
			$this->initBySrc($src, $width, $height, $hspace, $vspace, $border, $alt, $align, $name, $class, $title, $longdesc);
		} else if($type){
			$fileID = $this->getHttpVar(we_base_request::INT, 'fileID');

			switch($type){
				case we_base_link::TYPE_EXT:
					$extSrc = $this->getHttpVar(we_base_request::URL_FIX, 'extSrc', '') ?: we_base_link::EMPTY_EXT;
					$this->initBySrc($extSrc, $width, $height, $hspace, $vspace, $border, $alt, $align, $name, $class, $title, $longdesc);
					break;
				case we_base_link::TYPE_INT:
					if((we_base_request::_(we_base_request::BOOL, 'imgChangedCmd') || $isPresetFromDnD) && $fileID){
						$imgpath = $_SERVER['DOCUMENT_ROOT'] . id_to_path($fileID);
						$imgObj = new we_imageDocument();
						$imgObj->initByID($fileID);

						$preserveData = (we_base_request::_(we_base_request::BOOL, 'wasThumbnailChange') || we_base_request::_(we_base_request::BOOL, 'isTinyMCEInitialization'));
						$width = $imgObj->getElement('width', we_contents_base::ETYPE_NUMBER);
						$height = $imgObj->getElement('height', we_contents_base::ETYPE_NUMBER);
						$alt = $preserveData ? $alt : $imgObj->getElement('alt');
						$hspace = $preserveData ? $hspace : $imgObj->getElement('hspace');
						$vspace = $preserveData ? $vspace : $imgObj->getElement('vspace');
						$title = $preserveData ? $title : ($imgObj->getElement('useMetaTitle') && $imgObj->getElement('Title') ? $imgObj->getElement('Title') : $imgObj->getElement('title'));
						$name = $preserveData ? $name : $imgObj->getElement('name');
						$align = $preserveData ? $align : $imgObj->getElement('align');
						$border = $preserveData ? $border : $imgObj->getElement('border');
						$longdesc = $preserveData ? $longdesc : ($imgObj->getElement('longdescid', we_contents_base::ETYPE_NUMBER) ? (id_to_path($imgObj->getElement('longdescid', we_contents_base::ETYPE_NUMBER)) . '?id=' . $imgObj->getElement('longdescid', we_contents_base::ETYPE_NUMBER)) : $longdesc);
						$alt = $preserveData ? $alt : f('SELECT c.Dat FROM ' . CONTENT_TABLE . ' c WHERE c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND c.DID=' . intval($fileID) . ' AND c.nHash=x\'' . md5("alt") . '\'', '', $this->db);
					}
					$this->initByFileID($fileID, $width, $height, $hspace, $vspace, $border, $alt, $align, $name, $thumbnail, $class, $title, $longdesc);
					break;
			}
		} else {
			$this->defaultInit();
		}
	}

	function defaultInit(){
		$this->args['src'] = '';
		$this->args['width'] = 0;
		$this->args['height'] = 0;
		$this->args['hspace'] = 0;
		$this->args['vspace'] = 0;
		$this->args['class'] = '';
		$this->args['title'] = '';
		$this->args['longdesc'] = '';
		$this->args['border'] = 0;
		$this->args['alt'] = '';
		$this->args['align'] = '';
		$this->args['name'] = '';
		$this->args['type'] = we_base_link::TYPE_INT;
		$this->args['ratio'] = 1;
		$this->args['isPresetByDnD'] = 0;
	}

	/* protected function getHeaderHTML($additionals = ''){
	  return parent::getHeaderHTML(/* $this->weFileupload->getJs() . $this->weFileupload->getCss() );
	  } */

	protected function getDialogContentHTML(){
		$weSuggest = & we_gui_suggest::getInstance();
		if($this->noInternals || (isset($this->args['outsideWE']) && $this->args['outsideWE'] == 1)){
			$this->args['type'] = we_base_link::TYPE_EXT;
			$extSrc = we_html_tools::formElementTable(we_html_tools::textInput("we_dialog_args[extSrc]", (isset($this->args['extSrc']) ? str_replace('%20', ' ', $this->args['extSrc']) : ""), '', [
						], "text", 410), '', "left", "defaultfont", '', '', '', '', '', 0);
			$radioButtonExt = we_html_element::hidden('we_dialog_args[type]', we_base_link::TYPE_EXT);
			$intSrc = '';
			$thumbnails = '';
			$longdesc = we_html_tools::formElementTable(we_html_tools::textInput('we_dialog_args[longdesc]', str_replace('"', '&quot;', (isset($this->args['longdesc']) ? $this->args['longdesc'] : "")), '', [
						], "text", 520), g_l('weClass', '[longdesc_text]'));
		} else {
			/**
			 * input for external image files
			 */
			$btnExt = we_base_permission::hasPerm(we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT) ?
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server','we_dialog_args[extSrc]','',document.we_form.elements['we_dialog_args[extSrc]'].value,'dialog_extSrcOnchange')") : '';
			$radioButtonExt = we_html_forms::radiobutton(we_base_link::TYPE_EXT, (isset($this->args['type']) && $this->args['type'] === we_base_link::TYPE_EXT), "we_dialog_args[type]", g_l('wysiwyg', '[external_image]'), true, "defaultfont", "top.we_cmd('dialog_setType')");
			$btnOpen = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:top.openExtSource('extHref');", '', '', (empty($this->args['extSrc']) || $this->args['extSrc'] === we_base_link::EMPTY_EXT ? true : false), false, '_ext', false, g_l('wysiwyg', '[openNewWindow]'));
			$textInput = we_html_tools::textInput('we_dialog_args[extSrc]', (isset($this->args['extSrc']) ? $this->args['extSrc'] : ''), '', ['onchange' => "top.extSrc_doOnchange;"], 'text', 315);
			$extSrc = we_html_tools::formElementTable($textInput, '', "left", "defaultfont", $btnExt, $btnOpen, '', '', '', 0);

			/**
			 * input for webedition internal image files
			 */
			$startID = $this->args['selectorStartID'] ?: (IMAGESTARTID_DEFAULT ?: 0);
			$radioButtonInt = we_html_forms::radiobutton(we_base_link::TYPE_INT, (!isset($this->args['type']) || $this->args['type'] !== we_base_link::TYPE_EXT), "we_dialog_args[type]", g_l('wysiwyg', '[internal_image]'), true, "defaultfont", "top.we_cmd('dialog_setType')");

			$weSuggest->setAcId("Image");
			$weSuggest->setContentType(we_base_ContentTypes::IMAGE);
			$weSuggest->setInput("we_dialog_args[fileSrc]", str_replace('"', '&quot;', (isset($this->args['fileSrc']) ? $this->args['fileSrc'] : "")));
			$weSuggest->setJsCommandOnItemSelect('dialog_imageChanged');
			$weSuggest->setLabel('');
			$weSuggest->setMaxResults(10);
			$weSuggest->setResult("we_dialog_args[fileID]", str_replace('"', '&quot;', (isset($this->args['fileID']) ? $this->args['fileID'] : "")));
			$weSuggest->setSelector(we_gui_suggest::DocSelector);
			$weSuggest->setWidth(315);
			$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_image',document.we_form.elements['we_dialog_args[fileID]'].value,'" . FILE_TABLE . "','we_dialog_args[fileID]','we_dialog_args[fileSrc]','','','" . we_base_ContentTypes::IMAGE . "'," . (we_base_permission::hasPerm(we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES) ? 0 : 1) . ");"));
			$weSuggest->setOpenButton(we_gui_suggest::BTN_EDIT);
			$weSuggest->setAdditionalButton(we_html_button::create_button(we_html_button::ADD_IMAGE, "javascript:we_cmd('we_fileupload_editor', '" . we_base_ContentTypes::IMAGE . "', 1, 0, 0, 0, 'we_suggest_processExternalSelection,Image')"));
			$weSuggest->setIsDropFromTree(true);
			$weSuggest->setIsDropFromExt(true);
			$intSrc = $weSuggest->getHTML();

			/**
			 * thumbnail select list
			 */
			$thumbdata = (empty($this->args['thumbnail']) ? '' : $this->args['thumbnail']);
			$thumbnails = '<select id="selectThumbnail" name="we_dialog_args[thumbnail]" onchange="imageChanged(true);"' . ($this->getDisplayThumbsSel() === 'none' ? ' disabled="disabled"' : '') . '>';
			$thumbnails .= '<option value="0"' . (($thumbdata == 0) ? ' selected="selected"' : '') . '>' . g_l('wysiwyg', '[nothumb]') . '</option>';

			$this->db->query('SELECT ID,Text,description FROM ' . THUMBNAILS_TABLE . ' ORDER BY Text');
			while($this->db->next_record()){
				$thumbnails .= '<option title="' . $this->db->f('description') . '" value="' . $this->db->f('ID') . '"' . (($thumbdata == $this->db->f("ID")) ? (' selected="selected"') : "") . '>' . $this->db->f('Text') . '</option>';
			}
			$thumbnails .= '</select>';

			/**
			 * longdesc file chooser
			 */
			$weSuggest->setAcId("Longdesc");
			$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::HTML]);
			$weSuggest->setInput("we_dialog_args[longdescsrc]", str_replace('"', '&quot;', (isset($this->args['longdescsrc']) ? $this->args['longdescsrc'] : "")));
			$weSuggest->setLabel(g_l('weClass', '[longdesc_text]'));
			$weSuggest->setJsCommandOnItemSelect('');
			$weSuggest->setMaxResults(7);
			$weSuggest->setResult("we_dialog_args[longdescid]", (isset($this->args['longdescid']) ? $this->args['longdescid'] : ""));
			$weSuggest->setSelector(we_gui_suggest::DocSelector);
			$weSuggest->setWidth(315);
			$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['we_dialog_args[longdescid]'].value,'" . FILE_TABLE . "','we_dialog_args[longdescid]','we_dialog_args[longdescsrc]','','','',''," . (we_base_permission::hasPerm(we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES) ? 0 : 1) . ");"));
			$weSuggest->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('dialog_emptyLongdesc');"));
			$weSuggest->setAdditionalButton('');

			$longdesc = $weSuggest->getHTML();
		}

		$height = we_html_tools::formElementTable(we_html_tools::textInput('we_dialog_args[height]', (isset($this->args['height']) ? $this->args['height'] : ""), '', [
					'onkeypress' => "return WE().util.IsDigitPercent(event);", 'onkeyup' => "return checkWidthHeight(this);"], 'text', 140, 0, '', !empty($this->args['thumbnail'])), g_l('wysiwyg', '[height]'));
		$width = we_html_tools::formElementTable(we_html_tools::textInput('we_dialog_args[width]', (isset($this->args['width']) ? $this->args['width'] : ""), '', ['onkeypress' => "return WE().util.IsDigitPercent(event);",
					'onkeyup' => "return checkWidthHeight(this);"], 'text', 140, 0, '', !empty($this->args['thumbnail'])), g_l('wysiwyg', '[width]'));
		$onclick = "checkWidthHeight(document.we_form.elements['we_dialog_args[width]']);";
		$ratio = we_html_forms::checkboxWithHidden((isset($this->args['ratio']) ? $this->args['ratio'] : false), 'we_dialog_args[ratio]', g_l('thumbnails', '[ratio]'), false, "defaultfont", $onclick);
		$hspace = we_html_tools::formElementTable(we_html_tools::textInput('we_dialog_args[hspace]', (isset($this->args['hspace']) ? $this->args['hspace'] : ""), '', [
					'onkeypress' => "return WE().util.IsDigit(event);"], 'number', 140), g_l('wysiwyg', '[hspace]'));
		$vspace = we_html_tools::formElementTable(we_html_tools::textInput('we_dialog_args[vspace]', (isset($this->args['vspace']) ? $this->args['vspace'] : ""), '', [
					'onkeypress' => "return WE().util.IsDigit(event);"], 'number', 140), g_l('wysiwyg', '[vspace]'));
		$border = we_html_tools::formElementTable(we_html_tools::textInput('we_dialog_args[border]', (isset($this->args['border']) ? $this->args['border'] : ""), '', [
					'onkeypress' => "return WE().util.IsDigit(event);"], 'number', 140), g_l('wysiwyg', '[border]'));
		//$name = we_html_tools::formElementTable(we_html_tools::textInput('we_dialog_args[name]', (isset($this->args['name']) ? $this->args['name'] : ""), '', '', "number", 315), "Name");
		$alt = we_html_tools::formElementTable(we_html_tools::textInput('we_dialog_args[alt]', (isset($this->args['alt']) ? $this->args['alt'] : ""), '', [], "text", 315), g_l('wysiwyg', '[altText]'));
		$title = we_html_tools::formElementTable(we_html_tools::textInput('we_dialog_args[title]', (isset($this->args['title']) ? $this->args['title'] : ""), '', [], "text", 315), g_l('global', '[title]'));

		$align = we_html_tools::formElementTable(we_html_tools::select('we_dialog_args[align]', [
					'' => g_l('global', '[default]'),
					'top' => 'Top',
					'middle' => 'Middle',
					'bottom' => 'Bottom',
					'left' => 'Left',
					'right' => 'Right',
					'texttop' => 'Text Top',
					'absmiddle' => 'Abs Middle',
					'baseline' => 'Baseline',
					'absbottom' => 'Abs Bottom'
					], 1, $this->args['align']), g_l('wysiwyg', '[alignment]'), 'left', 'defaultfont', '', '', '', '', '', '');

		$classSelect = we_html_tools::formElementTable($this->getClassSelect('width: 140px;'), g_l('wysiwyg', '[css_style]'), 'left', 'defaultfont', '', '', '', '', '', '');

		$parts = [];
		$parts[] = ['html' => '<table class="default" style="margin-bottom:4px;">
<tr><td style="width:200px"><div style="display:inline;float:left">' . ($intSrc ? $radioButtonInt : '') . '</div></td><td><div style="display:inline;float:right">' . $radioButtonExt . '</div></td></tr>' .
			'</table>
<table class="default" style="margin-bottom:4px;">
<tr><td><div id="imageExt" style="margin-top:4px;' . (isset($this->args['type']) && $this->args['type'] === we_base_link::TYPE_EXT ? '' : 'display:none;') . '">' . $extSrc . '</div></td></tr>' .
			($intSrc ?
			'<tr><td><div id="imageInt" style="margin:2px 0 2px 0;' . (!isset($this->args['type']) || $this->args['type'] !== we_base_link::TYPE_EXT ? '' : 'display:none;') . '">' . $intSrc . '</div></td></tr>' :
			'') .
			'</table>',
			'noline' => true
		];

		$parts[] = ['headline' => g_l('wysiwyg', '[image][formatting]'),
			'html' => '<table class="default" style="width:530px; margin-bottom:1em;">
					<tr>
						<td>' . we_html_tools::formElementTable($thumbnails, g_l('wysiwyg', '[thumbnail]'), 'left', 'defaultfont', '', '', '', '', '', '') . '</td>
						<td>' . $classSelect . '</td>
						<td>' . $align . '</td>
					</tr>
				</table>'
		];
		$parts[] = ['headline' => g_l('global', '[attributes]'),
			'html' => '<table class="default" style="width:560px">
					<tr><td style="padding-bottom:15px;">' . $width . '</td><td style="padding-bottom:15px;">' . $height . '</td><td style="padding-bottom:15px;">' . $ratio . '</td></tr>
					<tr><td style="padding-bottom:15px;">' . $border . '</td><td style="padding-bottom:15px;">' . $hspace . '</td><td style="padding-bottom:15px;">' . $vspace . '</td></tr>
					<tr><td colspan="3" style="padding-bottom:15px;">' . $alt . '</td></tr>
					<tr><td colspan="3" style="padding-bottom:15px;">' . $title . '</td></tr>
					<tr><td colspan="3" style="padding-bottom:15px;">' . $longdesc . '</td></tr>
				</table>
				<div></div>' .
			we_html_element::hiddens(['we_dialog_args[name]' => (isset($this->args['name']) ? $this->args['name'] : ''),
				'we_dialog_args[rendered_width]' => 0,
				'we_dialog_args[rendered_height]' => 0,
				'imgChangedCmd' => 0,
				'wasThumbnailChange' => 0,
				'isTinyMCEInitialization' => 0,
				'tinyMCEInitRatioH' => 0,
				'tinyMCEInitRatioW' => 0
			]) .
			we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'plugins/weimage/js/weimage.js')
		];
		return $parts;
	}

	private function getDisplayThumbsSel(){
		$p = (isset($this->args['fileSrc']) ? $this->args['fileSrc'] : "");
		$tmp = $p ? explode('.', $p) : [];
		$extension = count($tmp) > 1 ? '.' . $tmp[count($tmp) - 1] : '';
		unset($p);

		return (we_base_imageEdit::gd_version() > 0 && we_base_imageEdit::is_imagetype_supported(isset(we_base_imageEdit::$GDIMAGE_TYPE[strtolower($extension)]) ? we_base_imageEdit::$GDIMAGE_TYPE[strtolower($extension)] : "") && isset($this->args['type']) && $this->args['type'] == we_base_link::TYPE_INT) ? "block" : "none";
	}

	protected function cmdFunction(array $args){
		switch(isset($this->we_cmd[0]) ? $this->we_cmd[0] : ''){
			case 'update_dialog_image':
				$payload = ['args' => $args, 'displayThumbnailSel' => $this->getDisplayThumbsSel()];
				$js = we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_cmdFrame.js', '', [
						'id' => 'loadVarDialog_cmdFrame',
						'data-payload' => setDynamicVar($payload)
				]);
				$this->jsCmd->addCmd('update_dialog_image');
				break;
			default:
				if(!empty($args['thumbnail']) && $args['fileID']){
					$thumbObj = new we_contents_thumbnail();
					$thumbObj->initByImageIDAndThumbID($args['fileID'], $args['thumbnail']);
					if(!file_exists($thumbObj->getOutputPath(true))){
						$thumbObj->createThumb();
					}
				}

				$attribs = we_base_request::_(we_base_request::BOOL, 'imgChangedCmd') && !we_base_request::_(we_base_request::BOOL, 'wasThumbnailChange') ? we_base_request::_(we_base_request::STRING, 'we_dialog_args') : $args;
				$attribs['width'] = empty($attribs['width']) || intval($attribs['width']) === 0 ? '' : $attribs['width'];
				$attribs['height'] = empty($attribs['height']) || intval($attribs['height']) === 0 ? '' : $attribs['height'];
				$attribs['src'] = isset($args['src']) ? $args['src'] : '';
				$attribs['longdesc'] = intval($attribs['longdescid']) ? $attribs['longdescsrc'] . '?id=' . intval($attribs['longdescid']) : '';
				$payload = ['attributes' => $attribs];

				$js = we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_cmdFrame.js', "we_cmd('image_writeback')", [
						'id' => 'loadVarDialog_cmdFrame',
						'data-payload' => setDynamicVar($payload)
				]);
		}

		return we_html_tools::getHtmlTop('', '', '', $js . $this->jsCmd->getCmds(), we_html_element::body());
	}

	protected function getJs(){
		//$weSuggest = & weSuggest::getInstance();
		//$css = !empty($this->args['cssClasses']) ? explode(',', $this->args['cssClasses']) : [];

		return parent::getJs() .
			we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_image.js');
	}

}
