<?php

/**
 * webEdition CMS
 *
 * $Rev: 15030 $
 * $Author: mokraemer $
 * $Date: 2018-03-11 13:19:45 +0100 (Sun, 11 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_fileupload_ui_editor extends we_fileupload_ui_base{
	protected $isExternalBtnUpload = true;
	protected $parentID = [
		'setField' => false,
		'preset' => IMAGESTARTID_DEFAULT,
		'setFixed' => false,
	];

	public function __construct($name, array $acceptedCT = [], array $acceptedExt = [], array $forbiddenCT = [], array $forbiddenExt = [], $doImport = true){
		$this->doImport = $doImport;
		if($this->doImport){
			$this->noPermsCT = $this->getForbiddenCtsFromPerms();
		}

		parent::__construct('we_File', $acceptedCT, $acceptedExt, $forbiddenCT, $forbiddenExt);

		$this->type = 'editor';
		$this->doCommitFile = true;

		$this->controls = [
			'showControls' => self::CONTROLS_SHOW_TRUE,
			'showControlsInitialState' => self::CONTROLS_SHOW_TRUE,
			'showBtnsUploadCancel' => self::CONTROLS_SHOW_FALSE,
			'showEditOptions' => $this->isPermToUpload('editables') ? self::CONTROLS_SHOW_TRUE : self::CONTROLS_SHOW_FALSE,
			'showImportMeta' => self::CONTROLS_SHOW_FALSE
		];

		$this->doImport = $doImport;
		$this->responseClass = $this->doImport ? 'we_fileupload_resp_import' : $this->responseClass = 'we_fileupload_resp_base';
	}

	public function setFieldParentID($parentID = []){
		$this->parentID = array_merge($this->parentID, $parentID);
	}

	// move to fileupload.scss and dete
	public function getCss(){
		return parent::getCss() . we_html_element::cssElement('
			.paddingTop{
				padding-top: 12px;
			}
		');
	}

	public function getHtml($returnRows = false){
		$formUploader = we_html_element::div(['id' => 'imageUpload', 'class' => 'fileuploadEditor', 'style' => 'display:block;width:384px;'], $this->getHiddens() .
				we_html_element::div(['style' => 'width:542px'], we_html_element::div(['id' => 'div_fileupload_right', 'style' => "position:relative;"], $this->getFormUploader($this->getDropZoneContent())
					)
				)
		);

		$moreHiddens = [];

		$parts = [
			$this->makeMultiIconRow('', $formUploader, '', false, false, true, ''),
			$this->getFormParentID(),
			$this->getFormSameName(),
		];

		if($this->doImport){
			$parts[] = $this->getFormIsSearchable();

			if($this->isPermToUpload('metaImportables')){
				$parts[] = $this->getFormImportMeta();
			} else {
				$moreHiddens['fu_doc_importMetadata'] = '';
			}

			if(we_base_permission::hasPerm(we_base_perm::WORK_CATEGORY_EDIT) && we_modules_info::isActive(we_modules_info::CATEGORY)){
				$parts[] = $this->getFormCategories();
			} else {
				$this->moreHiddens['fu_doc_categories'] = '';
			}

			if($this->isPermToUpload('images')){
				$parts[] = $this->getFormImageAttributes();
				$parts[] = $this->getFormThumbnails();
			} else {
				$moreHiddens['fu_doc_alt'] = '';
				$moreHiddens['fu_doc_title'] = '';
				$moreHiddens['fu_doc_thumbs'] = '';
			}
		} else {
			$moreHiddens['fu_doc_isSearchable'] = '';
			$moreHiddens['fu_doc_categories'] = '';
			$moreHiddens['fu_doc_alt'] = '';
			$moreHiddens['fu_doc_title'] = '';
			$moreHiddens['fu_doc_thumbs'] = '';
			$moreHiddens['fu_doc_importMetadata'] = '';
		}

		$box = we_html_multiIconBox::getHTML('', $parts, true);

		return we_html_element::jsScript(JS_DIR . 'multiIconBox.js') . we_html_element::hiddens($moreHiddens) . $box;
	}

	public function getHtmlFooter(){
		return we_html_element::div(['class' => 'editfooter fileuploadEditor_footer', 'style' => ''], we_html_element::div(['class' => 'containerBtn'], we_html_element::div([
						'class' => 'elemBtnFooter'], we_html_button::create_button(we_html_button::UPLOAD, 'javascript:weFileUpload_instance.startUpload();', '', '', true, false, '_btn_footer', true, '', 'noMarginLeft')) .
					we_html_element::div(['class' => 'elemBtnFooter hidden'], we_html_button::create_button(we_html_button::CANCEL, 'javascript:weFileUpload_instance.cancelUpload();', '', '', true, false, '_btn_footer')) .
					we_html_element::div(['class' => 'elemBtnFooter'], we_html_button::create_button(we_html_button::CLOSE, 'javascript:top.close()', '', '', false, false, '_btn_footer'))
				)
		);
	}

	public function getFormImageAttributes(){
		$html = we_html_element::div([], we_html_element::label([], 'Alternativ Text') . '<br/>' . we_html_tools::textInput('fu_doc_alt', '', '', [], 'text', 378)) .
			we_html_element::div([], we_html_element::label([], 'Titel') . '<div>' . we_html_tools::textInput('fu_doc_title', '', '', [], 'text', 378) . '</div>');
		$headline = 'Attribute';

		return $this->makeMultiIconRow($headline, $html, we_html_multiIconBox::PROP_ATTRIB, true);
	}

	function getFormCategories(){
		$table = new we_html_table([
			'id' => 'CategoriesBlock',
			'style' => 'display: block;',
			'class' => 'default withSpace'
			], 2, 1);

		$table->setColContent(0, 0, we_html_element::div([
				'id' => 'categoriesDiv',
				'class' => 'blockWrapper',
				'style' => 'width:378px; height: 80px; border: #AAAAAA solid 1px;'
		]));
		$table->setCol(1, 0, ['colspan' => 2, 'style' => 'text-align:right'], we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:removeAllCats();") . we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',-1,'" . CATEGORY_TABLE . "','','','addCat')")
		);

		$html = $table->getHtml() . we_html_element::hiddens([
				'fu_doc_categories' => 0
		]);
		$headline = g_l('global', '[categorys]');

		return $this->makeMultiIconRow($headline, $html, we_html_multiIconBox::PROP_CATEGORIES, true);
	}

	public function getFormIsSearchable($isMultiIconBox = true){
		$html = we_html_element::div([], we_html_forms::checkboxWithHidden(($this->imageEditProps['isSearchable'] ? true : false), 'fu_doc_isSearchable', g_l('importFiles', '[imgsSearchable_label]'), false, 'defaultfont', ''));
		$headline = g_l('importFiles', '[imgsSearchable]');

		return $this->makeMultiIconRow($headline, $html, '', false, true, true);
	}

	public function getFormImportMeta($hide = false){
		$html = we_html_element::div(['id' => 'div_importMeta', 'class' => 'importMeta' . ($hide ? ' hidden' : '')], we_html_forms::checkboxWithHidden(($this->imageEditProps['importMetadata'] ? true : false), 'fu_doc_importMetadata', g_l('importFiles', '[import_metadata]'), false, 'defaultfont', ''));
		$headline = g_l('importFiles', '[metadata]');

		return $this->makeMultiIconRow($headline, $html, '', false, false, true);
	}

	public function getFormSameName(){
		$html = we_html_element::div(['style' => 'margin:10px 0 0 0;'], we_html_element::div(['style' => 'margin-top:10px'], we_html_forms::radiobutton('overwrite', false, "sameName", g_l('importFiles', '[sameName_overwrite]'), false, "defaultfont", 'document.we_form.fu_file_sameName.value=this.value;') .
					we_html_forms::radiobutton('rename', true, "sameName", g_l('importFiles', '[sameName_rename]'), false, "defaultfont", 'document.we_form.fu_file_sameName.value=this.value;') .
					we_html_forms::radiobutton('nothing', false, "sameName", g_l('importFiles', '[sameName_nothing]'), false, "defaultfont", 'document.we_form.fu_file_sameName.value=this.value;')
				) .
				we_html_element::hidden('fu_file_sameName', 'rename')
		);

		$headline = g_l('importFiles', '[sameName_headline]');

		return $this->makeMultiIconRow($headline, $html, '', false, true, true);
	}

	public function getFormParentID($formName = 'we_form'){
		if(!$this->doImport && $this->parentID['preset'] && !is_numeric($this->parentID['preset']) && !path_to_id($this->parentID['preset'])){
			// in sselect we when uploading to ext dir we have parentDir in $this->parentID['preset' with no id to path!
			$parentDir = $this->parentID['preset'];
			$parentID = -1;
		} else {
			$parentDir = '';
			$parentID = $this->parentID['preset'] ? (is_numeric($this->parentID['preset']) ? $this->parentID['preset'] : path_to_id($this->parentID['preset'])) : ($this->imageEditProps['parentID'] ?: (IMAGESTARTID_DEFAULT ?: 0));
		}

		$DB_WE = new DB_WE();
		$parentID = ($parentID && f('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ID=' . intval($parentID) . ' AND IsFolder=1', '', $DB_WE) ? $parentID : 0);

		if(($ws = get_ws(FILE_TABLE, true))){
			if(!(we_users_util::in_workspace($parentID, $ws, FILE_TABLE))){
				if($parentID && $this->parentID['setFixed']){ //only warn, if a parentID was ever set.
					t_e('wrong uploader configuration: fixed parentID not in user\'s ws', $ws, $this->parentID['preset']);
				}
				$parentID = intval(reset($ws));
			}
		}

		if(!$this->parentID['setFixed']){
			$weSuggest = &we_gui_suggest::getInstance();
			$weSuggest->setAcId("fu_file_parentID");
			$weSuggest->setContentType(we_base_ContentTypes::FOLDER);
			$weSuggest->setInput("fu_file_parentDir", $parentID ? id_to_path($parentID, FILE_TABLE) : '/', [], false);
			$weSuggest->setMaxResults(10);
			$weSuggest->setResult("fu_file_parentID", $parentID);
			$weSuggest->setSelector(we_gui_suggest::DirSelector);
			$weSuggest->setWidth(326);
			$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',$parentID,'" . FILE_TABLE . "','fu_file_parentID','fu_file_parentDir','','',0,'" . we_base_ContentTypes::FOLDER . "'," . (we_base_permission::hasPerm(we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES) ? 0 : 1) . ");"));

			$html = $weSuggest->getHTML();
		} else {
			$parentPath = $parentDir ?: id_to_path($parentID);
			$html = we_html_element::input(['value' => $parentPath, 'disabled' => 'disabled']) .
				we_html_button::create_button(we_html_button::SELECT, '', '', '', true) .
				we_html_element::hiddens(['fu_file_parentID' => $parentID,
					'fu_file_parentDir' => $parentDir,
			]);
		}

		$headline = g_l('importFiles', '[destination_dir]');

		return $this->makeMultiIconRow($headline, $html, we_html_multiIconBox::PROP_PATH, false, true);
	}

	public function getFormThumbnails(){
		$thumbnails = new we_html_select([// FIXME: use we_cmd in onchange!
			'multiple' => 'multiple',
			'name' => 'thumbnails_tmp',
			'id' => 'thumbnails_tmp',
			'class' => 'defaultfont',
			'size' => 6,
			'style' => 'width: 378px;',
			'onchange' => "this.form.fu_doc_thumbs.value='';for(var i=0;i<this.options.length;i++){if(this.options[i].selected){this.form.fu_doc_thumbs.value +=(this.options[i].value + ',');}};this.form.fu_doc_thumbs.value=this.form.fu_doc_thumbs.value.replace(/^(.+),$/,'$1');"
		]);
		$DB_WE = new DB_WE();
		$DB_WE->query('SELECT ID,Text,description FROM ' . THUMBNAILS_TABLE . ' ORDER BY Text');

		$thumbsArr = explode(',', trim($this->imageEditProps['thumbnails'], ' ,'));
		while($DB_WE->next_record()){
			$attribs = ['title' => $DB_WE->f('description'),
				(in_array($DB_WE->f('ID'), $thumbsArr) ? 'selected' : '') => 'selected'
			];
			$thumbnails->addOption($DB_WE->f('ID'), $DB_WE->f('Text'), $attribs);
		}

		$html = $thumbnails->getHtml() . we_html_element::hidden('fu_doc_thumbs', $this->imageEditProps['thumbnails']);
		$headline = g_l('thumbnails', '[create_thumbnails]');

		return $this->makeMultiIconRow($headline, $html, we_html_multiIconBox::PROP_IMAGE, true);
	}

	protected function makeMultiIconRow($headline, $html, $icon = '', $fold = false, $noline = false, $rightHeadline = false, $space = we_html_multiIconBox::SPACE_MED){
		$row = [
			'headline' => $headline,
			'html' => $html,
			'class' => '',
			'icon' => $icon,
			'rightHeadline' => $rightHeadline,
			'space' => $space,
			'fold' => $fold,
		];

		return $noline ? array_merge($row, ['noline' => true]) : $row;
	}

	public function getJs(){
		return self::getJSFileuploader() . self::getJSUtils() . $this->getJSDialog() . $this->getJSInit();
	}

	public function getJSDialog(){
		return self::getJSFileuploader() .
			we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_fileupload.js', '', ['id' => 'loadVarFileupload_ui_editor', 'data-editor' => setDynamicVar([
					'trashButton' => we_html_button::create_button(we_html_button::TRASH, "javascript:" . we_base_constants::PLACEHOLDER),
					'categoriesDivSize' => 368,
					'variantCats' => ($this->imageEditProps['categories'] ? id_to_path($this->imageEditProps['categories'], CATEGORY_TABLE) : [])
		])]);
	}

	public static function showFrameset(){
		$acceptedCT = explode(',', stripslashes(we_base_request::_(we_base_request::CMD, 'we_cmd', '', 1)));
		$doImport = boolval(we_base_request::_(we_base_request::CMD, 'we_cmd', true, 2));
		$importToID = we_base_request::_(we_base_request::CMD, 'we_cmd', 0, 3);
		$setFixedImportTo = we_base_request::_(we_base_request::CMD, 'we_cmd', 0, 4);
		$isPreset = boolval(we_base_request::_(we_base_request::CMD, 'we_cmd', false, 5));
		$cmdUploadFinished = we_base_request::_(we_base_request::STRING, 'we_cmd', '', 6);

		$fileUpload = new self('we_File', $acceptedCT, [], [], [], $doImport);
		$fileUpload->setIsPreset($isPreset);
		$fileUpload->setFieldParentID(['setField' => true, 'preset' => $importToID, 'setFixed' => $setFixedImportTo]);
		$fileUpload->setCmd('cmdUploadFinished', $cmdUploadFinished);

		echo we_html_tools::getHtmlTop('fileupload', '', '', $fileUpload->getJs() .
			$fileUpload->getCss(), we_html_element::body(['class' => 'weDialogBody'], we_html_element::form([
					], we_html_element::div([
						'id' => 'we_fileupload_editor', 'class' => 'weDialogBody', 'style' => 'position:absolute;top:0;bottom:40px;left:0;right:0;overflow: auto;'], $fileUpload->getHtml()) .
					we_html_element::div(['id' => 'we_fileupload_footer', 'class' => '', 'style' => 'position:absolute;height:40px;bottom:0;left:0;right:0;overflow: hidden;'], $fileUpload->getHtmlFooter())
				)
			)
		);
	}

}
