<?php
/**
 * webEdition CMS
 *
 * $Rev: 15035 $
 * $Author: mokraemer $
 * $Date: 2018-03-12 16:26:53 +0100 (Mon, 12 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * @desc    class for tag <we:listview type="shopVariants">
 *
 */
class we_listview_variants extends we_listview_base{
	private $VariantData = [];
	var $Position = 0;
	public $id = 0;
	var $ObjectId = 0;
	var $DefaultName = 'default';
	var $Model = null;
	var $IsObjectFile = false;
	var $hidedirindex = false;
	var $objectseourls = false;
	private $firstKey = false;

	function __construct($name, $rows, $defaultname, $documentid, $objectid, $offset, $hidedirindex, $objectseourls, $triggerID){
		parent::__construct($name, $rows, $offset);

		$this->DefaultName = $defaultname;
		$this->hidedirindex = $hidedirindex;
		$this->objectseourls = $objectseourls;
		$this->triggerID = $triggerID;

		// we have to init a new document and look for the given field
		// get id of given document and check if it is a document or an objectfile
		if($documentid){
			$this->id = $documentid;

			$this->Model = new we_webEditionDocument();
			$this->Model->initByID($this->id);
		} else if($objectid && we_modules_info::isActive(we_modules_info::OBJECT)){
			$this->IsObjectFile = true;
			$this->id = $objectid;
			$this->Model = new we_objectFile();
			$this->Model->initByID($this->id, OBJECT_FILES_TABLE);
			// check if its a document or a objectFile
		} elseif(isset($GLOBALS['we_doc']->TableID)){ // is an objectFile can this happen??!
			$this->id = $GLOBALS['we_doc']->ID;
			$this->IsObjectFile = true;
			$this->Model = $GLOBALS['we_doc'];
		} elseif(isset($GLOBALS['we_obj'])){
			$this->id = $GLOBALS['we_obj']->ID;
			$this->IsObjectFile = true;
			$this->Model = $GLOBALS['we_obj'];
		} else {
			$this->id = $GLOBALS['we_doc']->ID;
			$this->Model = $GLOBALS['we_doc'];
		}

		// store model in listview object
		$this->VariantData['Record'] = we_base_variants::getVariantData($this->Model, $this->DefaultName);
		$this->anz_all = count($this->VariantData['Record']);
		$this->anz = min($this->rows, $this->anz_all);
	}

	function next_record(){
		$this->Position = ($this->count + $this->start);
		if(!isset($this->VariantData['Record'][$this->Position])){
			$this->Record = [];
			return false;
		}
		$this->Record = $this->VariantData['Record'][$this->Position];

		$vardata = reset($this->Record);
		$key = key($this->Record);
		$this->firstKey = $key;
		foreach($vardata as $name => $value){

			$this->Record[strtolower($name)] = (isset($value['type']) && $value['type'] === 'img' ?
				// there is a difference between objects and webEdition Documents
				isset($value[we_contents_base::ETYPE_NUMBER]) ? $value[we_contents_base::ETYPE_NUMBER] : $value[we_contents_base::ETYPE_TEXT] :
				(isset($value[we_contents_base::ETYPE_TEXT]) ? $value[we_contents_base::ETYPE_TEXT] : '')
				);
		}

		$this->Record[self::VARIANT_FIELD . '_name'] = $key;

		if($key != $this->DefaultName){
			$varUrl = we_base_variants::REQUEST . '=' . $key;
			$this->Record[self::VARIANT_FIELD] = $key;
		} else {
			$varUrl = $this->Record[self::VARIANT_FIELD] = '';
		}

		$this->Record[self::ID_FIELD] = $this->id;
		$path_parts = pathinfo($this->IsObjectFile ? $GLOBALS['we_doc']->Path : $this->Model->Path);
		if($this->IsObjectFile){ // objectFile
			if($this->objectseourls && show_SeoLinks()){
				$Url = f('SELECT Url FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $this->id, '', $this->DB_WE);
				if($Url != ''){
					$this->Record[self::PATH_FIELD] = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') .
						( $this->hidedirindex && seoIndexHide($path_parts['basename']) ?
						'' : '/' . $path_parts['filename']
						) . '/' . $Url . ($varUrl ? "?$varUrl" : '');
				} else {
					$this->Record[self::PATH_FIELD] = ($this->hidedirindex && seoIndexHide($path_parts['basename']) ?
						($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' . '?' . WEOBJ_PARAM . '=' . $this->id . ($varUrl ? "&amp;$varUrl" : '') :
						$GLOBALS['we_doc']->Path . '?' . WEOBJ_PARAM . '=' . $this->id . ($varUrl ? '&amp;' . $varUrl : '')
						);
				}
			} elseif($this->hidedirindex && seoIndexHide($path_parts['basename'])){
				$this->Record[self::PATH_FIELD] = $GLOBALS['we_doc']->Path = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' . ($varUrl ? "?$varUrl" : '');
			} else {
				$this->Record[self::PATH_FIELD] = $GLOBALS['we_doc']->Path . '?' . WEOBJ_PARAM . '=' . $this->id . ($varUrl ? "&amp;$varUrl" : '');
			}
		} else // webEdition Document

		if($this->hidedirindex && seoIndexHide($path_parts['basename'])){
			$this->Record[self::PATH_FIELD] = $this->Model->Path = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' . ($varUrl ? '?' . $varUrl : '');
		} else {
			$this->Record[self::PATH_FIELD] = $this->Model->Path . ($varUrl ? '?' . $varUrl : '');
		}

		$this->count++;
		return true;
	}

	public function f($key, $stripWE = false){
		return parent::f($key, $stripWE);
	}

	function getFirstKey(){
		return $this->firstKey;
	}

}
