<?php

/**
 * webEdition CMS
 *
 * $Rev: 14923 $
 * $Author: mokraemer $
 * $Date: 2018-02-16 18:52:26 +0100 (Fri, 16 Feb 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_main_headermenu{

	/**
	 * creates a new messageConsole
	 *
	 * @param string $consoleName
	 * @return string
	 */
	public static function createMessageConsole($consoleName = 'NoName'){
		return '
<div id="messageConsole" onclick="_console_.openMessageConsole();">
<table><tr>
	<td style="vertical-align:middle"><div class="small messageConsoleMessage" id="messageConsoleMessage' . $consoleName . '">--</div></td>
	<td><div class="navigation" id="messageConsoleImageDiv"><i id="messageConsoleImage' . $consoleName . '" class="fa fa-lg fa-bell"></i></div></td>
	</tr></table>
</div>';
	}

	static function css(){
		return
			we_html_element::cssLink(WEBEDITION_DIR . 'css/menu.css') .
			we_html_element::jsScript(JS_DIR . 'menu/clickMenu.js');
	}

	//FIXME: this should not be executed directly, this should be a command which loads the data
	static function getMenuReloadCode($location = 'top.opener.', $asCmd = false){
		$menu = self::getMenu();
		return $asCmd ? ['updateMenu', $menu->getHTML()] :
			$location . 'document.getElementById("nav").parentNode.innerHTML="' . str_replace("\n", '"+"', addslashes($menu->getHTML())) . '";';
	}

	private static function getMenu(){
		$we_menu = include(WE_INCLUDES_PATH . 'menu/we_base_menu.inc.php');

		if(// menu for normalmode
			isset($_SESSION['weS']['we_mode']) && $_SESSION['weS']['we_mode'] == we_base_constants::MODE_NORMAL){
			$jmenu = new we_gui_menu($we_menu);
		} else { // menu for seemode
			if(!we_base_permission::hasPerm(we_base_perm::SEEM_CAN_SEE_MENUE)){
				return null;
			}
			$jmenu = new we_gui_menu($we_menu);
		}

		return $jmenu;
	}

	static function pbody(){
// all available elements
		$jmenu = self::getMenu();

		if(isset($_SESSION['weS']['we_mode']) && $_SESSION['weS']['we_mode'] == we_base_constants::MODE_NORMAL){
			$navigationButtons = [];
// menu for normalmode
		} else if(we_base_permission::hasPerm(we_base_perm::SEEM_CAN_SEE_MENUE)){
			$navigationButtons = [];
// menu for seemode
		} else {
//  no menu in this case !
			$navigationButtons = [[
				'onclick' => "top.we_cmd('dologout');",
				'i' => "close",
				'text' => g_l('menu_global', '[close]')
			]];
		}
		$navigationButtons = array_merge($navigationButtons, [
			['onclick' => 'WE().layout.weNavigationHistory.navigateReload();', 'i' => 'refresh', 'text' => g_l('menu_global', '[reload]')],
			['onclick' => 'WE().layout.weNavigationHistory.navigateBack();', 'i' => 'caret-left', 'text' => g_l('menu_global', '[back]')],
			['onclick' => 'WE().layout.weNavigationHistory.navigateNext();', 'i' => 'caret-right', 'text' => g_l('menu_global', '[next]')],
			]
		);
		?>
		<div>
			<div id="home" class="navigation" onclick="top.we_cmd('start_multi_editor');"><i class="fa fa-home" title="<?= g_l('menu_global', '[home]'); ?>"></i></div>
			<div id="weMainMenu">
				<?php
				if($jmenu){
					echo $jmenu->getHTML();
				}
				?>
			</div>
			<div id="navigationHistory">
				<?php
				if($navigationButtons){
					foreach($navigationButtons as $button){
						echo '<div class="navigation" onclick="' . $button['onclick'] . '"><i class="fa fa-' . $button['i'] . '" title="' . $button['text'] . '"></i></div>';
					}
				}
				?></div>
			<div id="weHeaderRight"><?php
				if(($versionInfo = we_base_util::updateAvailable())){
					?>
					<div id="newUpdate" class="navigation">
						<i class="fa fa-lg fa-exclamation-circle" onclick="<?= (we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) ? "top.we_cmd('update');" : ""); ?>" title="<?php printf(g_l('sysinfo', '[newWEAvailable]'), $versionInfo['dotted'] . ' (svn ' . $versionInfo['svnrevision'] . ')', $versionInfo['date']); ?>"></i>
					</div>
					<?php
				}

				echo self::createMessageConsole('mainWindow');
				?>
				<div id="logout" class="navigation" onclick="top.we_cmd('dologout');"><i class="fa fa-power-off fa-lg"></i></div>
			</div>
		</div>
		<?php
	}

}
