<?php
/**
 * webEdition CMS
 *
 * $Rev: 14885 $
 * $Author: mokraemer $
 * $Date: 2018-02-12 14:22:05 +0100 (Mon, 12 Feb 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * base class for rpc commands
 *
 * @package none
 * @abstract
 */
abstract class we_rpc_cmd{
	const STATUS_OK = 0;
	const STATUS_NO_PERMISSION = 1;
	const STATUS_NOT_ALLOWED_VIEW = 2;
	const STATUS_LOGIN_FAILED = 3;
	const STATUS_REQUEST_MALFORMED = 4;
	const STATUS_NO_CMD = 5;
	const STATUS_NO_CIEW = 6;
	const STATUS_NO_SESSION = 7;
	const STATUS_NO_VIEW = 8;

	protected $CmdShell;
	protected $ExtraViews = [];
	protected $Permissions = [];
	protected $Status = self::STATUS_OK;
	protected $Parameters = [];

	function executeRpcCmd($shell){
		$this->checkSession();
		$this->checkParameters();

		foreach($this->Permissions as $perm){
			if(!we_base_permission::hasPerm($perm)){
				$this->Status = self::STATUS_NO_PERMISSION;
			}
		}

		$this->CmdShell = $shell;
	}

	function execute(){
		return new we_rpc_response();
	}

	public function cleanUp(){
		//overwrite
	}

	function checkSession(){
		if(!isset($_SESSION['user']['ID']) || !$_SESSION['user']['ID']){
			$this->Status = self::STATUS_NO_SESSION;
			return false;
		}

		return true;
	}

	function checkParameters(){
		foreach($this->Parameters as $par){
			if(we_base_request::_(we_base_request::STRING, $par) === false){
				$this->Status = self::STATUS_REQUEST_MALFORMED;
				return false;
			}
		}
		return true;
	}

	public function getStatus(){
		return $this->Status;
	}

}
