<?php

/**
 * webEdition CMS
 *
 * $Rev: 14853 $
 * $Author: mokraemer $
 * $Date: 2018-02-07 21:28:53 +0100 (Wed, 07 Feb 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_widget_sct extends we_widget_base{
	private $sc = '';

	public function __construct($curID = '', $aProps = []){
		$aCols = explode(';', $aProps ? $aProps[3] : we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0));
		$disableNew = true;
		$cmdNew = "javascript:top.we_cmd('new','" . FILE_TABLE . "','','" . we_base_ContentTypes::WEDOCUMENT . "');";
		if(we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW)){
			if(we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_NO_DOCTYPE_ALLOWED)){
				$disableNew = false;
			} else {
				$dtq = we_doctype_docTypes::getDoctypeQuery($GLOBALS['DB_WE']);
				$id = f('SELECT dt.ID FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'] . ' LIMIT 1');
				if($id){
					$disableNew = false;
					$cmdNew = "javascript:top.we_cmd('new','" . FILE_TABLE . "','','" . we_base_ContentTypes::WEDOCUMENT . "','" . $id . "')";
				} else {
					$disableNew = true;
				}
			}
		} else {
			$disableNew = true;
		}

		$disableObjects = false;
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$allClasses = we_users_util::getAllowedClasses($GLOBALS['DB_WE']);
			if(empty($allClasses)){
				$disableObjects = true;
			}
		}

		$js = [];

		if(we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENTS_CAN_SEE)){
			$js["open_document"] = "top.we_cmd('open_document');";
		}
		if(we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE, we_base_perm::WORK_PROPERTIES_CAN_SEE], false) && !$disableNew){
			$js["new_document"] = $cmdNew;
		}
		if(we_base_permission::hasPerm(we_base_perm::WORK_TEMPLATE_NEW) && $_SESSION['weS']['we_mode'] == we_base_constants::MODE_NORMAL){
			$js["new_template"] = "top.we_cmd('new','" . TEMPLATES_TABLE . "','','" . we_base_ContentTypes::TEMPLATE . "');";
		}
		if(we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_FOLDER_NEW)){
			$js["new_directory"] = "top.we_cmd('new','" . FILE_TABLE . "','','" . we_base_ContentTypes::FOLDER . "')";
		}
		if(we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENTS_CAN_SEE)){
			$js["unpublished_pages"] = "top.we_cmd('openUnpublishedPages');";
		}
		if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm(we_base_perm::OBJECTFILES_CAN_SEE) && !$disableObjects){
			$js["unpublished_objects"] = "top.we_cmd('openUnpublishedObjects');";
		}
		if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILE_NEW, we_base_perm::WORK_PROPERTIES_CAN_SEE], false) && !$disableObjects){
			$js["new_object"] = "top.we_cmd('new_objectFile');";
		}
		if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm(we_base_perm::OBJECT_NEW) && $_SESSION['weS']['we_mode'] == we_base_constants::MODE_NORMAL){
			$js["new_class"] = "top.we_cmd('new_object');";
		}
		if(we_base_permission::hasPerm(we_base_perm::USER_EDIT_SETTINGS)){
			$js["preferences"] = "top.we_cmd('openPreferences');";
		}
		if(we_base_permission::hasPerm(we_base_perm::WORK_IMAGE_NEW)){
			$js['btn_add_image'] = "top.we_cmd('new','tblFile','','image/*')";
		}


		$shortcuts = [];
		foreach($aCols as $sCol){
			$shortcuts[] = explode(',', $sCol);
		}

		$sSctOut = '';
		$col = 0;

		foreach($shortcuts as $sctCol){
			$sSctOut .= '<div class="sct_row" style="display: block; margin-right: 1em; float: left;"><table class="default" style="width:100%;">';
			$iCurrSctRow = 0;
			foreach($sctCol as $label){
				if(isset($js[$label])){
					$icon = '';
					switch($label){
						case 'new_directory':
							$icon = we_base_ContentTypes::FOLDER;
							break;
						case 'unpublished_pages':
						case 'open_document':
						case 'new_document':
							$icon = we_base_ContentTypes::WEDOCUMENT;
							break;
						case 'unpublished_objects':
						case 'new_object':
							$icon = we_base_ContentTypes::OBJECT_FILE;
							break;
						case 'new_template':
							$icon = we_base_ContentTypes::TEMPLATE;
							break;
						case 'new_class':
							$icon = we_base_ContentTypes::OBJECT;
							break;
						case 'btn_add_image':
							$icon = we_base_ContentTypes::IMAGE;
							break;
						case 'preferences':
							$icon = 'settings';
							break;
					}

					$sSctOut .= '<tr onclick="' . $js[$label] . '"><td class="sctFileIcon" data-contenttype="' . $icon . '"></td><td class="middlefont sctText">' . g_l('button', '[' . $label . '][value]') . '</tr>';
				}
				$iCurrSctRow++;
			}
			$sSctOut .= '</table></div>';
			$col++;
		}

		$this->sc = $sSctOut;
	}

	public function getInsertDiv($iCurrId, $aProps, we_base_jsCmd $jsCmd){
		$cfg = self::getDefaultConfig();
		$oTblDiv = we_html_element::div(['id' => "m_" . $iCurrId . "_inline",
				'style' => "width:100%;height:" . ($cfg["height"] - 25) . "px;overflow:auto;"
				], $this->sc);
		$aLang = [g_l('cockpit', '[shortcuts]'), ''];
		$jsCmd->addCmd('setIconOfDocClass', 'sctFileIcon');
		return [$oTblDiv, $aLang];
	}

	public static function getDefaultConfig(){
		return
			[
				'width' => self::WIDTH_SMALL,
				'expanded' => 0,
				'height' => 210,
				'res' => 0,
				'cls' => 'red',
				'csv' => ';',
				'dlgHeight' => 435,
				'isResizable' => 1
		];
	}

	public static function showDialog(){
		list($jsFile, $oSelCls) = self::getDialogPrefs();
		$disableNew = true;
		$cmdNew = "javascript:top.we_cmd('new','" . FILE_TABLE . "','','" . we_base_ContentTypes::WEDOCUMENT . "');";
		if(we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW)){
			if(we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_NO_DOCTYPE_ALLOWED)){
				$disableNew = false;
			} else {
				$dtq = we_doctype_docTypes::getDoctypeQuery($GLOBALS['DB_WE']);
				$id = f('SELECT dt.ID FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'] . ' LIMIT 1');

				if($id){
					$disableNew = false;
					$cmdNew = "javascript:top.we_cmd('new','" . FILE_TABLE . "','','" . we_base_ContentTypes::WEDOCUMENT . "','" . $id . "')";
				} else {
					$disableNew = true;
				}
			}
		} else {
			$disableNew = true;
		}

		$disableObjects = false;
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$allClasses = we_users_util::getAllowedClasses($GLOBALS['DB_WE']);
			if(empty($allClasses)){
				$disableObjects = true;
			}
		}

		$shortcuts = array_filter([
			'open_document' => (we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENTS_CAN_SEE) ? g_l('button', '[open_document][value]') : ''),
			'new_document' => (we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENTS_CAN_SEE) && !$disableNew ? g_l('button', '[new_document][value]') : ''),
			'new_template' => (we_base_permission::hasPerm(we_base_perm::WORK_TEMPLATE_NEW) ? g_l('button', '[new_template][value]') : ''),
			'new_directory' => (we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_FOLDER_NEW) ? g_l('button', '[new_directory][value]') : ''),
			'unpublished_pages' => (we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENTS_CAN_SEE) ? g_l('button', '[unpublished_pages][value]') : ''),
			'unpublished_objects' => (we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm(we_base_perm::OBJECTFILES_CAN_SEE) && !$disableObjects ? g_l('button', '[unpublished_objects][value]') : ''),
			'new_object' => (we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm(we_base_perm::OBJECTFILE_NEW) && !$disableObjects ? g_l('button', '[new_object][value]') : ''),
			'new_class' => (we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm(we_base_perm::OBJECT_NEW) ? g_l('button', '[new_class][value]') : ''),
			'preferences' => (we_base_permission::hasPerm(we_base_perm::USER_EDIT_SETTINGS) ? g_l('button', '[preferences][value]') : ''),
			'btn_add_image' => (we_base_permission::hasPerm(we_base_perm::WORK_IMAGE_NEW) ? g_l('button', '[btn_add_image][alt]') : '')
		]);

		$oSctPool = new we_html_select([
			'name' => "sct_pool",
			'class' => 'defaultfont',
			'onchange' => "addBtn(_fo.list11,this.options[this.selectedIndex].text,this.options[this.selectedIndex].value,true);this.options[0].selected=true;"
			]
		);
		$oSctPool->insertOption(0, " ", "", true);
		$iCurrOpt = 1;
		foreach($shortcuts as $key => $value){
			$oSctPool->insertOption($iCurrOpt, $key, $value, true);
			$iCurrOpt++;
		}

		$oSctList11 = new we_html_select(['multiple' => 'multiple',
			'name' => "list11",
			"size" => 10,
			'style' => "width:200px;",
			'class' => 'defaultfont',
			"onDblClick" => "moveSelectedOptions(this.form.list11,this.form.list21,false);"
		]);
		$oSctList21 = new we_html_select(['multiple' => 'multiple',
			'name' => "list21",
			"size" => 10,
			'style' => "width:200px;",
			'class' => 'defaultfont',
			"onDblClick" => "moveSelectedOptions(this.form.list21,this.form.list11,false);"
		]);

		$oBtnDelete = we_html_button::create_button(we_html_button::DELETE, "javascript:removeOption(document.forms[0].list11);removeOption(document.forms[0].list21);", false, "", false, false);
		$oShortcutsRem = we_html_tools::alertAttentionBox(g_l('cockpit', '[sct_rem]'), we_html_tools::TYPE_INFO, 420);

		$oPool = new we_html_table(['width' => 420, 'class' => 'default'], 3, 3);
		$oPool->setColContent(0, 0, $oSctList11->getHTML());
		$oPool->setCol(0, 1, ['style' => 'text-align:center;vertical-align:middle;'], we_html_element::a(['href' => '#',
				'onclick' => "moveOptionUp(document.forms[0].list11);moveOptionUp(document.forms[0].list21);return false;"
				], '<i class="fa fa-lg fa-caret-up"></i>') .
			we_html_element::br() . we_html_element::br() .
			we_html_element::a(['href' => '#',
				'onclick' => "moveSelectedOptions(document.forms[0].list11,document.forms[0].list21,false);return false;"
				], '<i class="fa fa-lg fa-caret-right"></i>') .
			we_html_element::br() . we_html_element::br() .
			we_html_element::a(['href' => '#',
				'onclick' => "moveSelectedOptions(document.forms[0].list21,document.forms[0].list11,false);return false;"
				], '<i class="fa fa-lg fa-caret-left"></i>') .
			we_html_element::br() . we_html_element::br() .
			we_html_element::a(['href' => '#',
				'onclick' => "moveOptionDown(document.forms[0].list11);moveOptionDown(document.forms[0].list21);return false;"
				], '<i class="fa fa-lg fa-caret-down"></i>'
		));
		$oPool->setColContent(0, 2, $oSctList21->getHTML());
		$oPool->setCol(1, 0, ['colspan' => 3, 'style' => 'text-align:center;padding-top:5px;'], $oBtnDelete);

		$content = $oShortcutsRem . we_html_element::br() . we_html_tools::formElementTable(
				$oSctPool->getHTML(), g_l('cockpit', '[select_buttons]'), "left", "defaultfont") . we_html_element::br() . $oPool->getHTML();

		$parts = [
			['headline' => '', 'html' => $content,
			],
			['headline' => '', 'html' => $oSelCls->getHTML(),
			]
		];

		$save_button = we_html_button::create_button(we_html_button::SAVE, "javascript:save();");
		$preview_button = we_html_button::create_button(we_html_button::PREVIEW, "javascript:preview();");
		$cancel_button = we_html_button::create_button(we_html_button::CLOSE, "javascript:exit_close();");
		$buttons = we_html_button::position_yes_no_cancel($save_button, $preview_button, $cancel_button);

		$sTblWidget = we_html_element::jsScript(JS_DIR . 'multiIconBox.js') . we_html_multiIconBox::getHTML('sctProps', $parts, true, $buttons, -1, "", "", "", g_l('cockpit', '[shortcuts]'));

		echo we_html_tools::getHtmlTop(g_l('cockpit', '[shortcuts]'), '', '', $jsFile .
			we_html_element::jsScript(JS_DIR . 'widgets/sct.js', '', ['id' => 'loadVarWidget', 'data-widget' => setDynamicVar([
					'aLang' => $shortcuts,
				])]
			), we_html_element::body(
				['class' => 'weDialogBody', 'onload' => "init();"
				], we_html_element::form([], $sTblWidget)));
	}

	public function showPreview(){
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addCmd('initPreview', [
			'id' => we_base_request::_(we_base_request::STRING, 'we_cmd', '', 5),
			'type' => 'sct',
			'tb' => g_l('cockpit', '[shortcuts]'),
		]);
		$jsCmd->addCmd('setIconOfDocClass', 'sctFileIcon');
		echo we_html_tools::getHtmlTop(g_l('cockpit', '[shortcuts]'), '', '', $jsCmd->getCmds()
			, we_html_element::body(
				['style' => 'margin:10px 15px;',
				], we_html_element::div(['id' => "sct"
					], $this->sc)));
	}

}
