<?php
/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Code Snipptes are used in templates inside webEdition
 *
 * @see Parser.php
 * @see dtd:http://docs.oasis-open.org/dita/v1.0.1/dtd/topic.dtd
 *
 */
class we_wizard_codeSnippet{
	/**
	 * Name of the Snippet
	 *
	 * @var string
	 */
	private $title = '';

	/**
	 * Description of the snippet
	 *
	 * @var string
	 */
	private $shortdesc = '';

	/**
	 * Author of the snippet
	 *
	 * @var string
	 */
	private $author = '';

	/**
	 * Snippet code
	 *
	 * @var string
	 */
	private $code = '';

	/**
	 * initialize the snippet from an xml file
	 *
	 * @param string $file
	 */
	function __construct($file){
		if(preg_match('-^.*\.xml$-', $file)){
			$this->readXML($file);
		} else {
			$this->readJSON($file);
		}
	}

	private function readJSON($file){
		$data = we_unserialize(we_base_file::load($file));
		foreach($data as $key => $d){
			$this->{$key} = is_array($d) ? g_l($d[0], $d[1]) : $d;
		}
	}

	private function readXML($file){
		$Parser = new we_xml_parser($file);

		// set the title
		if($Parser->execMethod_count("/topic[1]/title[1]", 'g_l') > 0){
			$this->title = $Parser->getData("/topic[1]/title[1]/g_l[1]");
			$this->title = g_l('snippet', $this->title);
		}

		// set the short description
		if($Parser->execMethod_count("/topic[1]/shortdesc[1]", 'g_l') > 0){
			$this->shortdesc = $Parser->getData("/topic[1]/shortdesc[1]/g_l[1]");
			$this->shortdesc = g_l('snippet', $this->shortdesc);
		}

		// set the author
		if($Parser->execMethod_count("/topic[1]/prolog[1]", "author") > 0){
			$this->author = $Parser->getData("/topic[1]/prolog[1]/author[1]");
		}

		// set the code
		if($Parser->execMethod_count("/topic[1]/body[1]", "p") > 0){
			$this->code = $Parser->getData("/topic[1]/body[1]/p[1]");
		}
	}

	private function updateCode(){
		$ret = $this->code;
		$matches = [];
		if(preg_match_all('|__GL\(([^)]+)\)__|', $ret, $matches)){
			foreach($matches[1] as $match){
				$ret = str_replace('__GL(' . $match . ')__', g_l('snippet', $match), $ret);
			}
		}
		return $ret;
	}

	private static function changeCharset($string, $charset = ''){
		$charset = $charset ?: $GLOBALS['we_doc']->getElement('Charset', 'dat', $GLOBALS['we']['Charset']['Backend'], true);

		if($charset != 'UTF-8' && $charset){
			$string = mb_convert_encoding($string, $charset, 'UTF-8');
		}

		return $string;
	}

	/**
	 * get the snippet name
	 *
	 * @return string
	 */
	function getName($charset = ''){
		return self::changeCharset($this->title, $charset);
	}

	/**
	 * get the snippet description
	 *
	 * @return string
	 */
	function getDescription($charset = ''){
		return self::changeCharset($this->shortdesc, $charset);
	}

	/**
	 * get the snippet author
	 *
	 * @return string
	 */
	function getAuthor($charset = ''){
		return self::changeCharset($this->author, $charset);
	}

	/**
	 * get the snippet code
	 *
	 * @return string
	 */
	function getCode($charset = ''){
		return self::changeCharset($this->updateCode(), $charset);
	}

}
