<?php
/**
 * webEdition CMS
 *
 * $Rev: 14892 $
 * $Author: mokraemer $
 * $Date: 2018-02-12 18:29:55 +0100 (Mon, 12 Feb 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
/**
 * Class we_error_handler
 *
 * Provides a error handler for webEdition.
 */
//essential includes, use these to allow it to be called without we "running"
require_once(WE_INCLUDES_PATH . 'we_db_tools.inc.php');

abstract class we_error_handler{
	private static $initialized = false;
	private static $handling = [
		'notice' => false,
		'deprecated' => false,
		'warning' => false,
		'error' => true,
		'sql' => true,
		'display' => false,
		'log' => true,
		'send' => false,
		'shutdown' => 'we',
	];

	const FILTER_SQL_FUNCTIONS = ['trigger_error'];
	const FILTER_ERR_FUNCTIONS = [];
	const E_SQL = -1;
	const E_JS = -2;

	public static function start($in_webEdition = true){
		if(self::$initialized){
			return;
		}
		self::$initialized = true;
		self::$handling['notice'] = defined('WE_ERROR_NOTICES') ? (WE_ERROR_NOTICES == 1 ? true : false) : false;
		self::$handling['deprecated'] = defined('WE_ERROR_DEPRECATED') ? (WE_ERROR_DEPRECATED == 1 ? true : false) : false;
		self::$handling['warning'] = defined('WE_ERROR_WARNINGS') ? (WE_ERROR_WARNINGS == 1 ? true : false) : false;
		self::$handling['log'] = defined('WE_ERROR_LOG') ? (WE_ERROR_LOG == 1 ? true : false) : true;
		self::$handling['send'] = (defined('WE_ERROR_MAIL') && defined('WE_ERROR_MAIL_ADDRESS')) ? (WE_ERROR_MAIL == 1 ? true : false) : false;
		self::$handling['log'] = defined('WE_ERROR_LOG') ? (WE_ERROR_LOG == 1 ? true : false) : true;
		self::$handling['send'] = (defined('WE_ERROR_MAIL') && defined('WE_ERROR_MAIL_ADDRESS')) ? (WE_ERROR_MAIL == 1 ? true : false) : false;

		// Get way of how to show errors
		if($in_webEdition){
			ini_set('display_errors', false);
			if(!defined('WE_ERROR_LOG')){
				define('WE_ERROR_LOG', 1);
			}
			if(!defined('WE_ERROR_HANDLER')){
				define('WE_ERROR_HANDLER', 1);
			}
			//we want all errors inside WE
			//we_error_setHandleAll();
		} else {
			self::$handling['display'] = defined('WE_ERROR_SHOW') ? (WE_ERROR_SHOW == 1 ? true : false) : true;
		}

		if((defined('WE_ERROR_HANDLER') && WE_ERROR_HANDLER)){
			$error_level = 0 +
				(self::$handling['deprecated'] ? E_DEPRECATED | E_USER_DEPRECATED | E_STRICT : 0) +
				(self::$handling['notice'] ? E_NOTICE | E_USER_NOTICE : 0) +
				(self::$handling['warning'] ? E_WARNING | E_CORE_WARNING | E_COMPILE_WARNING | E_USER_WARNING : 0) +
				(self::$handling['error'] ? E_ERROR | E_PARSE | E_CORE_ERROR | E_COMPILE_ERROR | E_USER_ERROR | E_RECOVERABLE_ERROR : 0);
			error_reporting($error_level);
			ini_set('error_reporting', $error_level);

			set_error_handler('we_error_handler::errorHandler', $error_level);
			register_shutdown_function('we_error_handler::shutdownHandler');
			set_exception_handler('we_error_handler::exceptionHandler');
		} else {
			//disable strict & deprecated errors

			$cur_error = error_reporting();
			if(($cur_error & (E_DEPRECATED | E_STRICT) ) > 0){
				$new_error = $cur_error & ~(E_DEPRECATED | E_STRICT);
				error_reporting($new_error);
			}
		}
	}

	public static function getBacktrace(array $skip = []){
		$detailedError = $caller = $file = $line = '';
		$backtrace = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
		$cnt = 0;
		$found = false;
		//error handler called directly caused by an error
		foreach($backtrace as $no => $arr){
			if($arr['function'] === 't_e'){
				unset($backtrace[$no - 1]);
				$found = true;

				break;
			}
		}
		if(!$found){
			$pos = array_search('error_handler', $skip);
			unset($skip[$pos]);
		}

		foreach($backtrace as $no => $arr){
			//NOTE: error_handler holds line no & filename of the callee if not called by t_e
			if((!empty($arr['class']) && $arr['class'] == __CLASS__ && $arr['function'] != 'errorHandler') || in_array($arr['function'], $skip)){
				continue;
			} else if($cnt == 0){ //this is the caller
				$caller = $arr['function'];
				$file = (isset($arr['file']) ? str_replace([realpath($_SERVER['DOCUMENT_ROOT']) . '/', $_SERVER['DOCUMENT_ROOT'] . '/', $_SERVER['DOCUMENT_ROOT']], '', $arr['file']) : '');
				$line = (isset($arr['line']) ? $arr['line'] : '');
			}
			$detailedError .= '#' . ($cnt++) . ' ' . (empty($arr['class']) ? '' : $arr['class'] . $arr['type']) . $arr['function'] . (empty($arr['args']) ? '' : '(' . implode(',', $arr['args']) . ')') . ' called at [' . (isset($arr['file']) ? str_replace([
					realpath($_SERVER['DOCUMENT_ROOT']) . '/', $_SERVER['DOCUMENT_ROOT'] . '/',
					$_SERVER['DOCUMENT_ROOT']], '', $arr['file']) : '') . ':' . (isset($arr['line']) ? $arr['line'] : '') . "]\n";
		}
		return [$detailedError, $caller, $file, $line];
	}

	public static function setHandleAll(){
		self::$handling = [
			'notice' => true,
			'deprecated' => true,
			'warning' => true,
			'error' => true,
			'sql' => true,
			'display' => false,
			'log' => true,
			'send' => (defined('WE_ERROR_MAIL') && defined('WE_ERROR_MAIL_ADDRESS')) ? (WE_ERROR_MAIL == 1 ? true : false) : false,
			'shutdown' => 'we',
		];
	}

	public static function setHandling($type, $value){
		self::$handling[$type] = $value;
	}

	public static function getHandling($type){
		return self::$handling[$type];
	}

	private static function translateType($type){
		switch($type){
			case E_RECOVERABLE_ERROR:
			case E_ERROR:
				return 'Error';
			case E_WARNING:
				return 'Warning';
			case E_PARSE:
				return 'Parse error';
			case E_NOTICE:
				return 'Notice';
			case E_CORE_ERROR:
				return 'Core error';
			case E_CORE_WARNING:
				return 'Core warning';
			case E_COMPILE_ERROR:
				return 'Compile error';
			case E_COMPILE_WARNING:
				return 'Compile warning';
			case E_USER_ERROR:
				return 'User error';
			case E_USER_WARNING:
				return 'User warning';
			case E_USER_NOTICE:
				return 'User notice';
			case E_DEPRECATED:
				return 'Deprecated notice';
			case E_STRICT:
				return 'Strict Error';
			case E_USER_DEPRECATED:
				return 'User deprecated notice';
			case self::E_SQL:
				return 'SQL Error';
			case self::E_JS:
				return 'JS Error';
			default:
				return 'unknown Error(' . $type . ')';
		}
	}

	private static function showDevice($type, $message, $file, $line, $skip = false){
		// Display error?
		if(self::$handling['display']){
			self::display($type, $message, $file, $line, $skip);
		}

		// Log error?
		if(self::$handling['log']){
			$insertID = self::log($type, $message, $file, $line, $skip);
		}

		// Mail error?
		if(!empty(self::$handling['send'])){
			self::mail($type, $message, $file, $line, $skip, isset($insertID) ? $insertID : false);
		}
	}

	private static function mail($type, $message, $file, $line, $skipBT = false, $insertID = false){
		static $max = 15;
		if(--$max < 0){
			//don't mail more than this
			return;
		}
		$detailedError = $caller = '-';
		if($skipBT === false){
			list($detailedError, $caller, $file, $line) = self::getBacktrace(($type == self::E_SQL ? self::FILTER_SQL_FUNCTIONS : self::FILTER_ERR_FUNCTIONS));
		} else if(is_array($skipBT)){
			list($detailedError, $caller, $file, $line) = $skipBT;
		}

		$ttype = self::translateType($type);

		// Build the error table
		$detailedError = "An $ttype occurred while executing a script in webEdition.\n\n\n" .
			($insertID && function_exists('getServerUrl') ?
			getServerUrl() . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=showerrorlog&function=pos&ID=' . $insertID . "\n\n" : '') .
// Domain
			'webEdition address: ' . $_SERVER['SERVER_NAME'] . ',

URI: ' . $_SERVER['REQUEST_URI'] . '
Host: ' . gethostname() . '
Error type: ' . $ttype . "
Error message: " . str_replace($_SERVER['DOCUMENT_ROOT'], 'SECURITY_REPL_DOC_ROOT', $message) . '
Script name: ' . str_replace($_SERVER['DOCUMENT_ROOT'], 'SECURITY_REPL_DOC_ROOT', $file) . '
Line number: ' . $line . '
Caller: ' . $caller . '
Backtrace: ' . $detailedError;

		// Log the error
		if(defined('WE_ERROR_MAIL_ADDRESS')){
			if(!mail(WE_ERROR_MAIL_ADDRESS, $ttype . ': ' . $_SERVER['SERVER_NAME'] . '(webEdition)', $detailedError)){
				if(in_array($type, ['E_ERROR', 'E_CORE_ERROR', 'E_COMPILE_ERROR', 'E_USER_ERROR'])){
					echo 'Cannot log error! Could not send e-mail: <pre>' . $detailedError . '</pre>';
				}
			}
		} else {
			if(in_array($type, ['E_ERROR', 'E_CORE_ERROR', 'E_COMPILE_ERROR', 'E_USER_ERROR'])){
				echo 'Cannot log error! Could not send e-mail due to no known recipient: <pre>' . $detailedError . '</pre>';
			}
		}
	}

	private static function display($type, $message, $file, $line, $skipBT = false){
		$detailedError = $caller = '-';
		if($skipBT === false){
			list($detailedError, $caller, $file, $line) = self::getBacktrace(($type == self::E_SQL ? ['error_showDevice', 'trigger_error', 'error_handler', 'getBacktrace',
					'display_error_message'] : [
					'error_showDevice', 'error_handler', 'getBacktrace', 'display_error_message']));
		} else if(is_string($skipBT)){
			$detailedError = $skipBT;
		}

		// Build the error table
		echo '<br/><table bgcolor="#FFFFFF" cellpadding="4" style="text-align:center;border: 1px solid #265da6;width:95%"><colgroup><col sytle="width:10%"/><col style="width:90%" /></colgroup>
		<tr bgcolor="#f7f7f7" style="vertical-align:top">
			<td colspan="2" style="border-bottom: 1px solid #265da6;">An error occurred while executing this script.</td>
		</tr>
	<tr style="vertical-align:top">
		<td style="white-space:nowrap;border-bottom: 1px solid #265da6; border-right: 1px solid #265da6;font-weight:bold;">Error type:</td>
		<td style="border-bottom: 1px solid #265da6;"><i>' . self::translateType($type) . '</i></td>
	</tr>
	<tr style="vertical-align:top">
			<td style="white-space:nowrap;border-bottom: 1px solid #265da6; border-right: 1px solid #265da6;font-weight:bold;">Error message:</td>
			<td style="border-bottom: 1px solid #265da6;"><i><pre>' . str_replace($_SERVER['DOCUMENT_ROOT'], "", $message) . '</pre></i></td>
	</tr>
	<tr style="vertical-align:top">
			<td style="white-space:nowrap;border-bottom: 1px solid #265da6; border-right: 1px solid #265da6;font-weight:bold;">Script name:</td>
			<td style="border-bottom: 1px solid #265da6;"><i>' . str_replace($_SERVER['DOCUMENT_ROOT'], "", $file) . '</i></td>
	</tr>
	<tr style="vertical-align:top">
			<td style="white-space:nowrap;border-bottom: 1px solid #265da6; border-right: 1px solid #265da6;font-weight:bold;">Line number:</td>
			<td style="border-bottom: 1px solid #265da6;"><i>' . $line . '</i></td>
	</tr>
	<tr style="vertical-align:top">
			<td style="white-space:nowrap;border-right: 1px solid #265da6;font-weight:bold;">Backtrace</td>
			<td>' . str_replace(["\r", "\n"], '', nl2br($detailedError)) . '</td>
	</tr>
	</table><br/>';
	}

	public static function log($type, $message, $file, $line, $skipBT = false){
		static $max = 500;
		if(--$max < 0){
			//don't log more messages per request
			return 0;
		}
		require_once($_SERVER['DOCUMENT_ROOT'] . '/webEdition/we/include/we_db_tools.inc.php');
		$detailedError = $caller = '-';
		if($skipBT === false){
			list($detailedError, $caller, $file, $line) = self::getBacktrace(($type == self::E_SQL ? ['error_showDevice', 'trigger_error', 'error_handler', 'getBacktrace',
					'log_error_message'] : [
					'error_showDevice', 'error_handler', 'getBacktrace', 'log_error_message']));
		} else if(is_string($skipBT)){
			$detailedError = $skipBT;
		}

		// Log the error
		if(defined('DB_HOST') && defined('DB_USER') && defined('DB_PASSWORD') && defined('DB_DATABASE')){
			$logVars = ['Request', 'Session', 'Server'];
			$_SERVER['SERVER_HOST'] = gethostname();
			$tbl = defined('ERROR_LOG_TABLE') ? ERROR_LOG_TABLE : TBL_PREFIX . 'tblErrorLog';
			$query = 'INSERT INTO ' . $tbl . ' SET Type="' . escape_sql_query(self::translateType($type)) . '",
			`Function`="' . escape_sql_query($caller) . '",
			File="' . escape_sql_query(str_replace([realpath($_SERVER['DOCUMENT_ROOT']) . '/', $_SERVER['DOCUMENT_ROOT'] . '/', $_SERVER['DOCUMENT_ROOT']], 'SECURITY_REPL_DOC_ROOT/', $file)) . '",
			Line=' . intval($line) . ',
			Text="' . escape_sql_query(str_replace($_SERVER['DOCUMENT_ROOT'], 'SECURITY_REPL_DOC_ROOT', $message)) . '",
			Backtrace="' . escape_sql_query($detailedError) . '"';
			if(isset($GLOBALS['DB_WE'])){
				$db = new DB_WE();
				if(!$db->query($query)){
					self::mail($type, 'Cannot log error! Query failed: ' . $message, $file, $line, $skipBT);
				} else {
					$id = $db->getInsertId();
					foreach($logVars as $var){
						$db->query('UPDATE IGNORE ' . $tbl . ' SET ' . self::getDBVariableMax($var) . ' WHERE ID=' . $id);
					}
				}
			} else {
				($link = mysqli_connect(DB_HOST, DB_USER, DB_PASSWORD, DB_DATABASE)) or exit('Cannot log error! Could not connect: ' . mysqli_error($link));
				if(mysqli_query($link, $query) === FALSE){
					self::mail($type, 'Cannot log error! Query failed: ' . $message, $file, $line, $skipBT);
					//exit('Cannot log error! Query failed: ' . mysql_error());
				} else {
					$id = mysqli_insert_id($link);
					foreach($logVars as $var){
						mysqli_query($link, 'UPDATE ' . $tbl . ' SET ' . self::getDBVariableMax($var) . ' WHERE ID=' . $id);
					}
				}
				mysqli_close($link);
			}
		} else {
			self::mail($type, 'Cannot log error! Database connection not known: ' . $message, $file, $line, $skipBT);
			//exit('Cannot log error! Database connection not known.');
		}
		return (isset($id)) ? $id : false;
	}

	private static function getDBVariableMax($var){
		static $max = 65500; //max lenght of text-col in mysql - this is enough debug-data, leave some space...
		switch($var){
			case 'Request':
				$ret = (isset($_REQUEST) ? self::prettyPrintArray(print_r(array_diff_key($_REQUEST, [
						'user' => '',
						'username' => '',
						'pass' => '',
						'password' => '',
						's' => '',
						'WE_LOGIN_password' => '',
						'WE_LOGIN_username' => '',
						'Password' => '',
						'Password2' => '',
							]), true), 1) : ' - ');
				break;
			case 'Session':
				if(!isset($_SESSION)){
					$ret = ' - ';
					break;
				}
				$ret = '';
				//FIXME: clone will be reduced to unsetting weS+webuser if all vars have moved
				if(!empty($_SESSION['webuser']['registered'])){
					$ret .= 'webUser: ' .
						self::prettyPrintArray(print_r(['ID' => $_SESSION['webuser']['ID'], 'Username' => $_SESSION['webuser']['Username'] . '(' . $_SESSION['webuser']['Forename'] . ' ' . $_SESSION['webuser']['Surname'] . ')'], true)) .
						"----------------------------------------\n";
				}
				if(!empty($_SESSION['user']['ID'])){
					$ret .= 'webEdition-User: ' .
						self::prettyPrintArray(print_r(['ID' => $_SESSION['user']['ID'], 'Username' => $_SESSION['user']['Username']], true)) .
						"----------------------------------------\n";
				}

				if(isset($_SESSION['weS'])){
					$ret .= "Internal data:\n" .
						self::prettyPrintArray(print_r(array_diff_key($_SESSION['weS'], [
							'versions' => '',
							'prefs' => '',
							'we_data' => '',
							'perms' => '',
							'webuser' => ''
								]), true), 1) .
						"----------------------------------------\n";
				}

				if(isset($_SESSION['perms'])){
					$ret .= "Effective Permissions:\n" . self::prettyPrintArray(print_r(array_filter($_SESSION['perms']), true)) .
						"\n------------------------------------\n";
				}
				$ret .= self::prettyPrintArray(print_r(array_diff_key($_SESSION, [
						'prefs' => '',
						'perms' => '',
						'webuser' => '',
						'weS' => ''
							]), true), 1);

				break;
			case 'Global':
				if(!isset($GLOBALS)){
					$ret = ' - ';
					break;
				}
				$ignore = ['GLOBALS', '_GET', '_POST', '_REQUEST', '_COOKIE', '_FILES', '_SERVER', '_SESSION',
					'we', 'DB_WE', 'we_doc', 'WE_MAIN_DOC', 'loader', 'WE_MAIN_DOC_REF'];
				$clone = [];
				foreach($GLOBALS as $key => $val){
					if(!in_array($key, $ignore)){
						$clone[] = $val;
					}
				}
				$ret .= print_r($clone, true);

				break;
			case 'Server':
				$ret = (isset($_SERVER) ? self::prettyPrintArray(print_r($_SERVER, true)) : ' - ');
				break;
			default:
				$ret = '';
		}

		if(strlen($ret) > $max){
			$ret = substr($ret, 0, $max) . "\n[...]";
		}
		return $var . '="' . escape_sql_query($ret) . '"';
	}

	private static function prettyPrintArray($var, $unindent = ''){
		return preg_replace(['|Array\n\(|', '|\n\)$|', '|\n(    )' . ($unindent ? '{' . $unindent . '}' : '+' ) . '|', '|\n(    )|'], ['', '', "\n", "\n\t"], $var);
	}

	public static function errorHandler($type, $message, $file, $line, $context){
		// Don't respond to the error if it was suppressed with a '@'
		if(error_reporting() == 0){
			return true;
		}
		if(strpos($message, 'MYSQL-ERROR') === 0 || strpos($message, 'MYSQL-WARNING') === 0){
			$type = self::E_SQL;
			if(!self::$handling['sql']){
				//sql-handling disabled
				return true;
			}
		}

		switch($type){
			case E_NOTICE:
			case E_USER_NOTICE:
				if(self::$handling['notice']){
					self::showDevice($type, $message, $file, $line);
				}
				break;

			case self::E_SQL:
			case E_WARNING:
			case E_CORE_WARNING:
			case E_COMPILE_WARNING:
			case E_USER_WARNING:
				if(self::$handling['warning']){
					self::showDevice($type, $message, $file, $line);
				}

				break;

			case E_ERROR:
			case E_PARSE:
			case E_CORE_ERROR:
			case E_COMPILE_ERROR:
			case E_USER_ERROR:
			case E_RECOVERABLE_ERROR:
				if(self::$handling['error']){
					self::showDevice($type, $message, $file, $line, true);
					//try to close the session!
					if(defined('SYSTEM_WE_SESSION') && SYSTEM_WE_SESSION && isset($_SESSION)){
						session_write_close();
					}
				}

				// Stop execution
				return false;
			case E_DEPRECATED:
			case E_USER_DEPRECATED:
				if(self::$handling['deprecated']){
					self::showDevice($type, $message, $file, $line);
				}
				break;
			default:
		}
		//Error handled
		return true;
	}

	public static function shutdownHandler(){
		if(self::$handling['shutdown'] != 'we'){
			return;
		}
		$error = error_get_last();
		if(is_array($error)){
			switch($error['type']){
				case E_ERROR:
				case E_PARSE:
				case E_CORE_ERROR:
				case E_COMPILE_ERROR:
				case E_USER_ERROR:
				case E_RECOVERABLE_ERROR:
					self::errorHandler($error['type'], $error['message'] . "\n" . print_r($error, true), $error['file'], $error['line'], null);
				default:
			}
		}
	}

	public static function exceptionHandler($exception){
		$type = E_ERROR;
		$message = $exception->getMessage();
		$file = $exception->getFile();
		$line = $exception->getLine();
		$bt = $exception->getTraceAsString();

		self::showDevice($type, $message, $file, $line, $bt);
	}

}
