<?php

/**
 * webEdition CMS
 *
 * $Rev: 14972 $
 * $Author: mokraemer $
 * $Date: 2018-02-28 14:32:48 +0100 (Wed, 28 Feb 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_glossary_view extends we_modules_view{
	/**
	 * Glossary Instance
	 * @var object
	 */
	private $page = 0;

	/**
	 */
	public function __construct(){
		parent::__construct();
		$this->Model = new we_glossary_glossary();
	}

	public function getTitle(){
		switch($this->Model->Type){
			case we_glossary_glossary::TYPE_ABBREVATION:
				return g_l('modules_glossary', '[abbreviation]');
			case we_glossary_glossary::TYPE_ACRONYM:
				return g_l('modules_glossary', '[acronym]');
			case we_glossary_glossary::TYPE_FOREIGNWORD:
				return g_l('modules_glossary', '[foreignword]');
			case we_glossary_glossary::TYPE_LINK:
				return g_l('modules_glossary', '[link]');
			case we_glossary_glossary::TYPE_TEXTREPLACE:
				return g_l('modules_glossary', '[textreplacement]');
		}
		return '';
	}

	function getCommonHiddens($cmds = []){
		return
			parent::getCommonHiddens($cmds) .
			we_html_element::hidden("IsFolder", (isset($this->Model->IsFolder) ? $this->Model->IsFolder : '0'));
	}

	function getJSTop(){
		$modData = we_modules_info::getModuleField(we_modules_info::GLOSSARY, 'text');
		$title = ($modData ? 'webEdition ' . g_l('global', '[modules]') . ' - ' . $modData : '');
		return
			parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'glossary/glossary_view.js', "parent.document.title='" . $title . "';");
	}

	function getJSProperty(array $jsVars = []){
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'glossary/glossary_view_prop.js');
	}

	public function processCommands(we_base_jsCmd $jscmd){
		$cmdid = we_base_request::_(we_base_request::STRING, "cmdid");
		switch(($cmd = we_base_request::_(we_base_request::STRING, "cmd"))){

			case 'new_glossary_acronym':
			case 'new_glossary_abbreviation':
			case 'new_glossary_foreignword':
			case 'new_glossary_link':
			case 'new_glossary_textreplacement':
				if(!we_base_permission::hasPerm(we_base_perm::GLOSSARY_NEW)){
					$jscmd->addMsg(g_l('modules_glossary', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$this->Model = new we_glossary_glossary();
				$this->Model->Type = array_pop(explode('_', $cmd, 4));
				$jscmd->addCmd('reloadHeaderFooter', $this->Model->Text);
				break;

			case "glossary_edit_acronym":
			case "glossary_edit_abbreviation":
			case "glossary_edit_foreignword":
			case "glossary_edit_link":
			case "glossary_edit_textreplacement":
				if(!we_base_permission::hasPerm(we_base_perm::GLOSSARY_EDIT)){
					$jscmd->addMsg(g_l('modules_glossary', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					$_REQUEST['home'] = 1;
					$_REQUEST['pnt'] = 'edbody';
					break;
				}
				$this->Model = new we_glossary_glossary($cmdid);
				$jscmd->addCmd('reloadHeaderFooter', $this->Model->Text);
				break;

			case 'populateWorkspaces':
				$objectLinkID = we_base_request::_(we_base_request::INT, 'link', 0, 'Attributes', 'ObjectLinkID');
				$values = we_navigation_dynList::getWorkspacesForObject($objectLinkID);

				if($values){
					$jscmd->addCmd('we_cmd', ['doPopulateWorkspaces', 'values', $values]);
				} elseif(we_navigation_dynList::getWorkspaceFlag($objectLinkID)){
					$jscmd->addCmd('doPopulateWorkspaces', 'workspace');
				} else {
					$jscmd->addCmd('doPopulateWorkspaces', 'noWorkspace');
				}
				break;

			case 'save_exception':
				if(!$cmdid || !($exception = we_base_request::_(we_base_request::STRING, 'Exception'))){
					break;
				}

				$language = substr($cmdid, 0, 5);

				we_glossary_glossary::editException($language, $exception);

				$jscmd->addMsg(g_l('modules_glossary', '[save_ok]'), we_base_util::WE_MESSAGE_NOTICE);

				break;

			case "save_glossary":
				if(($exception = we_base_request::_(we_base_request::STRING, 'Exception'))){
					$language = substr($cmdid, 0, 5);

					we_glossary_glossary::editException($language, $exception);
					break;
				}
				$type = we_base_request::_(we_base_request::STRING, 'Type');
				$this->Model->Text = we_base_request::_(we_base_request::STRING, $type, '', 'Text');
				if($this->Model->Type != we_glossary_glossary::TYPE_FOREIGNWORD){
					$this->Model->Title = we_base_request::_(we_base_request::STRING, $type, '', 'Title');
				}
				$this->Model->Attributes = we_base_request::_(we_base_request::STRING, $type, '', 'Attributes');

				if(!we_base_permission::hasPerm([we_base_perm::GLOSSARY_NEW, we_base_perm::GLOSSARY_EDIT])){
					$jscmd->addMsg(g_l('modules_glossary', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if(!trim($this->Model->Text)){
					$jscmd->addMsg(g_l('modules_glossary', '[name_empty]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if($this->Model->checkFieldText($this->Model->Text)){
					$jscmd->addMsg(g_l('modules_glossary', '[text_notValid]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if($this->Model->checkFieldText($this->Model->Title)){
					$jscmd->addMsg(g_l('modules_glossary', '[title_notValid]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				//$oldpath = $this->Glossary->Path;
				// set the path and check it
				$this->Model->setPath();

				if($this->Model->pathExists($this->Model->Path)){
					$jscmd->addMsg(g_l('modules_glossary', '[name_exists]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if($this->Model->isSelf()){
					$jscmd->addMsg(g_l('modules_glossary', '[path_nok]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}


				$StateBefore = ($this->Model->ID ?
					f('SELECT Published FROM ' . GLOSSARY_TABLE . " WHERE ID = " . intval($this->Model->ID)) :
					0);

				$isNew = $this->Model->ID == 0;

				if($this->Model->save()){
					$this->Model->Text = htmlentities($this->Model->Text, ENT_QUOTES);
					$this->Model->Title = htmlentities($this->Model->Title, ENT_QUOTES);

					if($isNew){
						$jscmd->addCmd('makeTreeEntry', [
							'id' => $this->Model->ID,
							'parentid' => $this->Model->Language . '_' . $this->Model->Type,
							'text' => $this->Model->Text,
							'open' => false,
							'contenttype' => ($this->Model->IsFolder ? we_base_ContentTypes::FOLDER : we_base_ContentTypes::GLOSSAR),
							'table' => GLOSSARY_TABLE,
							'published' => ($this->Model->Published > 0 ? 1 : 0)
						]);
						$jscmd->addCmd('drawTree');
					} else {
						$jscmd->addCmd('updateTreeEntry', [
							'id' => $this->Model->ID,
							'parentid' => $this->Model->Language . '_' . $this->Model->Type,
							'text' => $this->Model->Text,
							'published' => ($this->Model->Published > 0 ? 1 : 0)
						]);
					}

					$this->Model->Text = html_entity_decode($this->Model->Text, ENT_QUOTES);
					$this->Model->Title = html_entity_decode($this->Model->Title, ENT_QUOTES);

					$message = "";
					$pub = we_base_request::_(we_base_request::BOOL, 'Published');
					// Replacment of item is activated
					if($StateBefore == 0 && $pub){
						$message .= sprintf(g_l('modules_glossary', '[replace_activated]'), $this->Model->Text) . "\\n";

						// Replacement of item is deactivated
					} else if($StateBefore > 0 && !$pub){
						$message .= sprintf(g_l('modules_glossary', '[replace_deactivated]'), $this->Model->Text) . "\\n";
					}
					$message .= sprintf(g_l('modules_glossary', '[item_saved]'), $this->Model->Text);

					$jscmd->addMsg($message, we_base_util::WE_MESSAGE_NOTICE);
					$jscmd->addCmd('doAfterSave', $this->Model->Type, $this->Model->Language);

					// --> Save to Cache

					$Cache = new we_glossary_cache($this->Model->Language);
					$Cache->write();
					unset($Cache);

					// --> Save to Cache End
				}
				break;

			case "delete_glossary":

				if(!we_base_permission::hasPerm(we_base_perm::GLOSSARY_DELETE)){
					$jscmd->addMsg(g_l('modules_glossary', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				if($this->Model->delete()){
					$jscmd->addCmd('deleteTreeEntry', $this->Model->ID);
					$jscmd->addMsg(g_l('modules_glossary', ($this->Model->IsFolder == 1 ? '[group_deleted]' : '[item_deleted]')), we_base_util::WE_MESSAGE_NOTICE);

					// --> Save to Cache

					$Cache = new we_glossary_cache($this->Model->Language);
					$Cache->write();
					unset($Cache);

					// --> Save to Cache End

					$this->Model = new we_glossary_glossary();
					$_REQUEST['home'] = 1;
					$_REQUEST['pnt'] = 'edbody';
				} else {
					$jscmd->addMsg(g_l('modules_glossary', '[nothing_to_delete]'), we_base_util::WE_MESSAGE_ERROR);
				}
				break;

			case "switchPage":
			default:
				break;
		}

		$_SESSION['weS']['glossary_session'] = $this->Model;
	}

	function processVariables(){
		if(isset($_SESSION['weS']['glossary_session'])){
			$this->Model = $_SESSION['weS']['glossary_session'];
		}
		$isPublished = we_base_request::_(we_base_request::BOOL, 'Published');
		if(is_array($this->Model->persistent_slots)){
			foreach($this->Model->persistent_slots as $val){
				if(isset($_REQUEST[$val])){
					if($val === 'Published'){
						if($this->Model->Published == 0 && $isPublished){
							$this->Model->Published = time();
						} elseif(!$isPublished){
							$this->Model->Published = 0;
						}
					} else {
						$this->Model->$val = $_REQUEST[$val];
					}
				}
			}
		}
		$this->page = we_base_request::_(we_base_request::INT, 'page', $this->page);
	}

	public function getHomeScreen(){

		$hidden = ['cmd' => 'home',
			'pnt' => 'edbody',
			'name' => 'home',
			'value' => 0,
		];

		$form = ['name' => 'we_form',];

		$createAbbreviation = we_html_button::create_button('new_glossary_abbreviation', "javascript:we_cmd('new_glossary_abbreviation');", '', "", !we_base_permission::hasPerm(we_base_perm::GLOSSARY_NEW));
		$createAcronym = we_html_button::create_button('new_glossary_acronym', "javascript:we_cmd('new_glossary_acronym');", '', "", !we_base_permission::hasPerm(we_base_perm::GLOSSARY_NEW));
		$createForeignWord = we_html_button::create_button('new_glossary_foreignword', "javascript:we_cmd('new_glossary_foreignword');", '', "", !we_base_permission::hasPerm(we_base_perm::GLOSSARY_NEW));
		$createLink = we_html_button::create_button('new_glossary_link', "javascript:top.opener.we_cmd('new_glossary_link');", '', "", !we_base_permission::hasPerm(we_base_perm::GLOSSARY_NEW));
		$createTextReplacement = we_html_button::create_button('new_glossary_textreplacement', "javascript:we_cmd('new_glossary_textreplacement');", '', "", !we_base_permission::hasPerm(we_base_perm::GLOSSARY_NEW));

		$content = $createAbbreviation . '<br/>' .
			$createAcronym . '<br/>' .
			$createForeignWord . '<br/>' .
			$createLink . '<br/>' .
			$createTextReplacement;

		return parent::getActualHomeScreen("glossary", $content, we_html_element::form($form, $this->getCommonHiddens($hidden)));
	}

}
