<?php

/**
 * webEdition CMS
 *
 * $Rev: 14972 $
 * $Author: mokraemer $
 * $Date: 2018-02-28 14:32:48 +0100 (Wed, 28 Feb 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* the parent class of storagable webEdition classes */
class we_voting_view extends we_modules_view{

	function __construct(){
		parent::__construct();
		$this->Model = new we_voting_voting();
	}

	function getCommonHiddens($cmds = []){
		return
			parent::getCommonHiddens($cmds) .
			we_html_element::hiddens(["vernr" => (isset($cmds['vernr']) ? $cmds['vernr'] : 0),
				"IsFolder" => (isset($this->Model->IsFolder) ? $this->Model->IsFolder : '0')
		]);
	}

	function getJSTop(){
		$modData = we_modules_info::getModuleField(we_modules_info::VOTING, 'text');
		$title = ($modData ? 'webEdition ' . g_l('global', '[modules]') . ' - ' . $modData : '');

		return
			parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/voting/voting_top.js', "parent.document.title='" . $title . "'");
	}

	function getJSProperty(array $jsVars = []){
		$this->Model->Owners = (is_array($this->Model->Owners) ? array_filter($this->Model->Owners) : []);
		$owners = ($this->Model->Owners ?
			$this->db->getAllFirstq('SELECT ID,Path,IsFolder FROM ' . USER_TABLE . ' WHERE ID IN (' . implode(',', $this->Model->Owners) . ')', true, MYSQLI_ASSOC) :
			[]);
		return we_html_element::jsScript(WE_JS_MODULES_DIR . '/voting/voting_prop.js', '', ['id' => 'loadVarVoting', 'data-voting' => setDynamicVar([
					'isFolder' => $this->Model->IsFolder,
					'delBut' => we_html_button::create_button(we_html_button::TRASH, 'javascript:top . content . setHot();' . we_base_constants::PLACEHOLDER),
					'delBut1' => we_html_button::create_button(we_html_button::TRASH, 'javascript:top.content.setHot();if(answers_edit.itemCount>answers_edit.minCount){ ' . we_base_constants::PLACEHOLDER . '; }else{ callAnswerLimit();}'),
					'answerCount' => $this->Model->QASet ? count($this->Model->QASet[0]['answers']) : 0,
					'QASet' => $this->Model->QASet,
					'QASetAdditions' => $this->Model->QASetAdditions,
					'showVariant' => we_base_request::_(we_base_request::INT, 'vernr', 0),
					'allow' => [
						'freeText' => $this->Model->AllowFreeText,
						'images' => $this->Model->AllowImages,
						'media' => $this->Model->AllowMedia,
						'successor' => $this->Model->AllowSuccessors
					],
					'owners' => $owners,
					'blackList' => is_array($this->Model->BlackList) ? $this->Model->BlackList : []
		])]);
	}

	function processCommands(we_base_jsCmd $jscmd){
		switch(we_base_request::_(we_base_request::STRING, "cmd")){
			case "resetscores":
				foreach($this->Model->arr_Scores as &$val){
					$val = 0;
				}
				break;
			case "new_voting":
			case "new_voting_group":
				if(!we_base_permission::hasPerm(we_base_perm::VOTING_NEW)){
					$jscmd->addMsg(g_l('modules_voting', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$this->Model = new we_voting_voting();
				$this->Model->IsFolder = we_base_request::_(we_base_request::STRING, "cmd") === 'new_voting_group' ? 1 : 0;
				$jscmd->addCmd('loadHeaderFooter', $this->Model->Text);
				break;
			case "voting_edit":
				if(!we_base_permission::hasPerm(we_base_perm::VOTING_EDIT)){
					$jscmd->addMsg(g_l('modules_voting', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					$_REQUEST['home'] = '1';
					$_REQUEST['pnt'] = 'edbody';
					break;
				}

				$this->Model = new we_voting_voting(we_base_request::_(we_base_request::INT, "cmdid"));

				if(!$this->Model->isAllowedForUser()){
					$jscmd->addMsg(g_l('modules_voting', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					$this->Model = new we_voting_voting();
					$_REQUEST['home'] = true;
					break;
				}
				$jscmd->addCmd('loadHeaderFooter', $this->Model->Text);
				break;
			case "save_voting":
				if(!we_base_permission::hasPerm([we_base_perm::VOTING_NEW, we_base_perm::VOTING_EDIT])){
					$jscmd->addMsg(g_l('modules_voting', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if(we_voting_voting::filenameNotValid($this->Model->Text)){
					$jscmd->addMsg(g_l('modules_voting', '[wrongtext]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if(!trim($this->Model->Text)){
					$jscmd->addMsg(g_l('modules_voting', '[name_empty]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if($this->Model->Active == 1 && $this->Model->ActiveTime && $this->Model->Valid < time()){
					$jscmd->addMsg(g_l('modules_voting', '[not_active]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				$oldpath = $this->Model->Path;
				// set the path and check it
				$this->Model->setPath();

				if($this->Model->pathExists($this->Model->Path)){
					$jscmd->addMsg(g_l('modules_voting', '[name_exists]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				if($this->Model->isSelf()){
					$jscmd->addMsg(g_l('modules_voting', '[path_nok]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				$error = false;
				$q_empty = true;
				$a_empty = true;
				if(!$this->Model->IsFolder && !empty($this->Model->QASet)){
					foreach($this->Model->QASet as $set){
						$q = trim($set['question']);
						if($q === ''){
							$q_empty = true;
							break;
						}
						$q_empty = false;

						foreach($set['answers'] as $ans){
							$q = trim($ans);
							if($q === ''){
								$a_empty = true;
								break;
							}
							$a_empty = false;
						}
					}

					if($q_empty){
						$error = true;
						$jscmd->addMsg(g_l('modules_voting', '[question_empty]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					} else if($a_empty){
						$error = true;
						$jscmd->addMsg(g_l('modules_voting', '[answer_empty]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					}
				}

				if($this->Model->ParentID > 0){
					$folder = f('SELECT IsFolder FROM ' . VOTING_TABLE . ' WHERE ID=' . intval($this->Model->ParentID));
					if(empty($folder)){
						$jscmd->addMsg(g_l('modules_voting', '[path_nok]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					}
				}
				if(!$error){
					$newone = ($this->Model->ID == 0);

					$this->Model->save(we_base_request::_(we_base_request::BOOL, 'scores_changed'));

					if($this->Model->IsFolder && $oldpath != '' && $oldpath != '/' && $oldpath != $this->Model->Path){
						$db_tmp = new DB_WE();
						$this->db->query('SELECT ID FROM ' . VOTING_TABLE . ' WHERE Path LIKE "' . $db_tmp->escape($oldpath) . '%" AND ID!=' . intval($this->Model->ID));
						while($this->db->next_record()){
							$db_tmp->query('UPDATE ' . VOTING_TABLE . ' SET Path="' . $this->Model->evalPath($this->db->f('ID'), $db_tmp) . '" WHERE ID=' . $this->db->f('ID'));
						}
					}

					if($newone){
						$jscmd->addCmd('makeTreeEntry', [
							'id' => $this->Model->ID,
							'parentid' => $this->Model->ParentID,
							'text' => $this->Model->Text,
							'open' => false,
							'contenttype' => ($this->Model->IsFolder ? we_base_ContentTypes::FOLDER : we_base_ContentTypes::VOTING),
							'table' => VOTING_TABLE,
							'published' => ($this->Model->isActive() ? 1 : 0)
						]);
						$jscmd->addCmd('drawTree');
					} else {
						$jscmd->addCmd('updateTreeEntry', [
							'id' => $this->Model->ID,
							'parentid' => $this->Model->ParentID,
							'text' => $this->Model->Text,
							'published' => ($this->Model->isActive() ? 1 : 0)
						]);
					}
					$jscmd->addCmd('loadHeaderFooter', $this->Model->Text);
					$jscmd->addMsg(g_l('modules_voting', ($this->Model->IsFolder ? '[save_group_ok]' : '[save_ok]')), we_base_util::WE_MESSAGE_NOTICE);
				}
				break;
			case 'delete_voting':
				if(!we_base_permission::hasPerm(we_base_perm::VOTING_DELETE)){
					$jscmd->addMsg(g_l('modules_voting', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				if($this->Model->delete()){
					$jscmd->addCmd('deleteTreeEntry', $this->Model->ID);
					$jscmd->addMsg(g_l('modules_voting', ($this->Model->IsFolder == 1 ? '[group_deleted]' : '[voting_deleted]')), we_base_util::WE_MESSAGE_NOTICE);
					$this->Model = new we_voting_voting();
					$_REQUEST['home'] = 1;
					$_REQUEST['pnt'] = 'edbody';
				} else {
					$jscmd->addMsg(g_l('modules_voting', '[nothing_to_delete]'), we_base_util::WE_MESSAGE_ERROR);
				}

				break;
			case "switchPage":
				break;
			case "export_csv":
				$fname = rtrim(we_base_request::_(we_base_request::FILE, 'csv_dir'), '/') . '/voting_' . $this->Model->ID . '_export_' . time() . '.csv';

				$enclose = we_base_request::_(we_base_request::STRING, 'csv_enclose', '"');
				$enclose = $enclose == 0 ? '"' : '\'';
				$delimiter = we_base_request::_(we_base_request::STRING, ';');
				$delimiter = ($delimiter === '\t' ? "\t" : $delimiter);
				switch(we_base_request::_(we_base_request::STRING, 'csv_lineend')){
					default:
					case 'windows':
						$lineend = "\r\n";
						break;
					case 'unix':
						$lineend = "\n";
						break;
					case 'mac':
						$lineend = "\r";
						break;
				}

				$content = [];
				$questName = we_base_request::_(we_base_request::STRING, 'question_name');
				if($questName && ($data = we_base_request::_(we_base_request::STRING, $questName . '_item0'))){
					$content[] = $enclose . addslashes($data) . $enclose . $delimiter;
				}
				$answerName = we_base_request::_(we_base_request::STRING, 'answers_name');
				$cnt = we_base_request::_(we_base_request::INT, 'item_count');
				if($answerName && $cnt){
					for($i = 0; $i < $cnt; $i++){
						if(($data = we_base_request::_(we_base_request::STRING, $answerName . '_item' . $i))){
							$content[] = $enclose . addslashes($data) . $enclose . $delimiter . $this->Model->Scores[$i];
						}
					}
				}
				we_base_file::save($_SERVER['DOCUMENT_ROOT'] . $fname, implode($lineend, $content));
				$_REQUEST['lnk'] = $fname;
				break;
			case 'exportGroup_csv':
				$fname = '/' . ltrim(we_base_request::_(we_base_request::FILE, 'csv_dir') . '/votingGroup_' . $this->Model->ID . '_export_' . time() . '.csv', '/');

				$enclose = we_base_request::_(we_base_request::STRING, 'csv_enclose', '"');
				$enclose = $enclose == 0 ? '"' : $enclose;
				$delimiter = we_base_request::_(we_base_request::STRING, 'csv_delimiter', ';');
				$delimiter = $delimiter === '\t' ? "\t" : $delimiter;
				switch(we_base_request::_(we_base_request::STRING, 'csv_lineend')){
					default:
					case 'windows':
						$lineend = "\r\n";
						break;
					case 'unix':
						$lineend = "\n";
						break;
					case 'mac':
						$lineend = "\r";
						break;
				}

				$allData = $this->Model->loadDB();
				$CSV_Charset = we_base_request::_(we_base_request::STRING, 'the_charset', 'UTF-8');
				$content = [mb_convert_encoding($enclose . trim(g_l('modules_voting', '[voting-session]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[voting-id]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[time]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[ip]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[user_agent]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[cookie]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[log_fallback]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[status]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[answerID]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[answerText]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[voting-successor]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[voting-additionalfields]')) . $enclose . $delimiter, $CSV_Charset, DEFAULT_CHARSET
					)
				];

				foreach($allData as $key => $data){
					$cookie = g_l('modules_voting', $data['cookie'] ? '[enabled]' : '[disabled]');
					$fallback = g_l('global', $data['fallback'] ? '[yes]' : '[no]');

					if($data['status'] != we_voting_voting::SUCCESS){
						switch($data['status']){
							case we_voting_voting::ERROR :
								$mess = g_l('modules_voting', '[log_error]');
								break;
							case we_voting_voting::ERROR_ACTIVE :
								$mess = g_l('modules_voting', '[log_error_active]');
								break;
							case we_voting_voting::ERROR_REVOTE :
								$mess = g_l('modules_voting', '[log_error_revote]');
								break;
							case we_voting_voting::ERROR_BLACKIP :
								$mess = g_l('modules_voting', '[log_error_blackip]');
								break;
							default:
								$mess = g_l('modules_voting', '[log_error]');
						}
					} else {
						$mess = g_l('modules_voting', '[log_success]');
					}

					$lineFields = [
						$data['votingsession'],
						$data['voting'],
						date(g_l('weEditorInfo', '[date_format]'), $data['time']),
						$data['ip'],
						$data['agent'],
						$cookie,
						$fallback,
						$mess,
						$data['answer'],
						$data['answertext'],
						$data['successor'],
					];

					if($data['additionalfields'] != ''){
						$addData = we_unserialize($data['additionalfields']);

						if(is_array($addData) && !empty($addData)){
							foreach($addData as $values){
								$lineFields[] = $values;
							}
						} else {
							$lineFields[] = '-';
						}
					} else {
						$lineFields[] = '-';
					}
					$content[] = $enclose . mb_convert_encoding(implode($enclose . $delimiter . $enclose, array_map('trim', $lineFields)), $CSV_Charset, DEFAULT_CHARSET) . $enclose . $delimiter;
				}

				we_base_file::save($_SERVER['DOCUMENT_ROOT'] . $fname, implode($lineend, $content));
				$_REQUEST['lnk'] = $fname;
				break;

			default:
		}

		$_SESSION['weS']['voting_session'] = $this->Model;
	}

	function processVariables(){

		if(isset($_SESSION['weS']['voting_session'])){
			$this->Model = $_SESSION['weS']['voting_session'];
		}

		if(is_array($this->Model->persistent_slots)){
			foreach($this->Model->persistent_slots as $key => $type){
				if(($v = we_base_request::_($type, $key, '__UNSET__')) !== '__UNSET__'){
					$this->Model->{$key} = $v;
				}
			}
		}

		if(isset($_REQUEST['page'])){
			$this->page = $_REQUEST['page'];
		}

		$qaset = $qaADDset = [];
		$qname = we_base_request::_(we_base_request::STRING, 'question_name');
		$vcount = we_base_request::_(we_base_request::INT, 'variant_count');
		$aname = we_base_request::_(we_base_request::STRING, 'answers_name');
		$icount = we_base_request::_(we_base_request::INT, 'item_count');
		if($qname && $vcount && $aname && $icount){
			for($i = 0; $i < $vcount; $i++){
				if(($quest = we_base_request::_(we_base_request::STRING, $qname . '_variant' . $i . '_' . $qname . '_item0')) !== false){
					$set = ['question' => addslashes($quest),
						'answers' => [],
					];

					$an = $aname . '_variant' . $i . '_' . $aname . '_item';
					$anImage = $an . 'ImageID';
					$anMedia = $an . 'MediaID';
					$anSuccessor = $an . 'SuccessorID';
					$addset = [];
					for($j = 0; $j < $icount; $j++){
						if(($tmp = we_base_request::_(we_base_request::STRING, $an . $j)) !== false){
							$set['answers'][] = addslashes($tmp);
						}
						if(($tmp = we_base_request::_(we_base_request::STRING, $anImage . $j)) !== false){
							$addset['imageID'][] = ($tmp != 'Array' ? addslashes($tmp) : 0);
						}
						if(($tmp = we_base_request::_(we_base_request::STRING, $anMedia . $j)) !== false){
							$addset['mediaID'][] = ($tmp != 'Array' ? addslashes($tmp) : 0);
						}
						if(($tmp = we_base_request::_(we_base_request::STRING, $anSuccessor . $j)) !== false){
							$addset['successorID'][] = ($tmp != 'Array' ? addslashes($tmp) : 0);
						}
					}
					$qaset[] = $set;
					$qaADDset[] = $addset;
				}
			}
		}

		$this->Model->QASet = $qaset;
		$this->Model->QASetAdditions = $qaADDset;

		/* FIXME: this doesn't work! multi_edit assumes the value is the same as the displayed item, so you get an image-tag & textual user here which is really not what we need. To fix this, multi_edit.js must distinguish between "label" & value!
		 * if(($on = we_base_request::_(we_base_request::STRING, 'owners_name')) && ($oc = we_base_request::_(we_base_request::INT, 'owners_count'))){
		  $this->voting->Owners = [];
		  $an = $on . '_variant0_' . $on . '_item';
		  for($i = 0; $i < $oc; $i++){
		  if(($tmp = we_base_request::_(we_base_request::STRING, $an . $i))){
		  $up = str_replace(array(stripslashes($this->item_pattern), stripslashes($this->group_pattern)), '', $tmp);
		  $this->voting->Owners[] = path_to_id($up, USER_TABLE);
		  }
		  }
		  $this->voting->Owners = array_unique($this->voting->Owners);
		  }
		 */

		$ipset = [];
		if(($in = we_base_request::_(we_base_request::STRING, 'iptable_name')) && ($ic = we_base_request::_(we_base_request::INT, 'iptable_count'))){
			$in = $in . '_variant0_' . $in . '_item';
			for($i = 0; $i < $ic; $i++){
				if(($tmp = we_base_request::_(we_base_request::STRING, $in . $i)) !== false){
					$ipset[] = addslashes($tmp);
				}
			}
			$this->Model->BlackList = $ipset;
		}


		if(($day = we_base_request::_(we_base_request::INT, 'PublishDate_day'))){
			$this->Model->PublishDate = mktime(we_base_request::_(we_base_request::INT, 'PublishDate_hour'), we_base_request::_(we_base_request::INT, 'PublishDate_minute'), 0, we_base_request::_(we_base_request::INT, 'PublishDate_month'), $day, we_base_request::_(we_base_request::INT, 'PublishDate_year'));
		}

		if(($day = we_base_request::_(we_base_request::INT, 'Valid_day'))){
			$this->Model->Valid = mktime(we_base_request::_(we_base_request::INT, 'Valid_hour'), we_base_request::_(we_base_request::INT, 'Valid_minute'), 0, we_base_request::_(we_base_request::INT, 'Valid_month'), $day, we_base_request::_(we_base_request::INT, 'Valid_year'));
		}

		if(we_base_request::_(we_base_request::FLOAT, 'scores_0') !== false && ($ic = we_base_request::_(we_base_request::INT, 'item_count')) && we_base_request::_(we_base_request::BOOL, 'scores_changed')){
			$this->Model->Scores = [];
			for($j = 0; $j < $ic; $j++){
				if(($tmp = we_base_request::_(we_base_request::FLOAT, 'scores_' . $j))){
					$this->Model->Scores[] = $tmp;
				}
			}
		}
	}

	public function getHomeScreen(){
		$hiddens = ['cmd' => "home",
			'pnt' => 'edbody'
		];
		$content = we_html_button::create_button('new_voting', "javascript:we_cmd('new_voting');", '', "", !we_base_permission::hasPerm(we_base_perm::VOTING_NEW)) .
			'<br/>' .
			we_html_button::create_button('new_voting_group', "javascript:we_cmd('new_voting_group');", '', "", !we_base_permission::hasPerm(we_base_perm::VOTING_NEW));

		return parent::getActualHomeScreen("voting", $content, we_html_element::form(['name' => 'we_form'], $this->getCommonHiddens($hiddens) . we_html_element::hidden("home", 0)));
	}

}
