<?php

/**
 * webEdition CMS
 *
 * $Rev: 14882 $
 * $Author: mokraemer $
 * $Date: 2018-02-12 03:12:56 +0100 (Mon, 12 Feb 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
function we_include_tag_file($filename){
	// as default: all tag_functions are in this file.
	if(function_exists($filename)){
		// do noting
		return true;
	}
	if(file_exists(WE_INCLUDES_PATH . 'we_tags/' . $filename . '.inc.php')){
		require_once (WE_INCLUDES_PATH . 'we_tags/' . $filename . '.inc.php');
		return true;
	}
	//error check is only required for custom tags
	if(file_exists(WE_INCLUDES_PATH . 'we_tags/custom_tags/' . $filename . '.inc.php')){
		require_once (WE_INCLUDES_PATH . 'we_tags/custom_tags/' . $filename . '.inc.php');
		return function_exists($filename) ? true : parseError(sprintf(g_l('parser', '[tag_not_known]'), trim($filename)));
	}

	/* $toolinc = '';
	  if(we_tool_lookup::getToolTag($filename, $toolinc, true)){
	  require_once ($toolinc);
	  return function_exists($filename) ? true : parseError(sprintf(g_l('parser', '[tag_not_known]'), trim($filename)));
	  } */
	if(strpos(trim($filename), 'if') === 0){ // this ifTag does not exist
		echo parseError(sprintf(g_l('parser', '[tag_not_known]'), trim($filename)));
		return false;
	}
	return parseError(sprintf(g_l('parser', '[tag_not_known]'), trim($filename)));
}

/**
 * get the full name of an Attribute with applied postTagName if set
 * @param string $var
 * @return string
 */
function we_tag_getPostName($var){
	if($var && isset($GLOBALS['postTagName'])){
		return $var . $GLOBALS['postTagName'];
	}
	return $var;
}

function we_profiler($start = true){
	if($start){
		if(isset($_SERVER['REQUEST_TIME_FLOAT'])){
			define('WE_PROFILER', (float) $_SERVER['REQUEST_TIME_FLOAT']);
		} else {
			define('WE_PROFILER', microtime(true));
		}
		$GLOBALS['we_profile'] = [];
	} else {
		echo implode(',', array_keys($GLOBALS['we_profile'][0])) . "<br/>\n";
		foreach($GLOBALS['we_profile'] as $line){
			echo implode(',', $line) . "<br/>\n";
		}
	}
}

function we_tag($name, $attribs = [], $content = '', $internal = false){
	//keep track of editmode
	$edMerk = isset($GLOBALS['we_editmode']) ? $GLOBALS['we_editmode'] : '';
	//FIXME: do we support this????
	$user = weTag_getAttribute('user', $attribs, [], we_base_request::STRING_LIST);
	if(defined('WE_PROFILER')){
		static $last = WE_PROFILER;
		$bt = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 1);
		$cur = microtime(true);
		$prof = [
			'tag' => $name,
			'line' => $bt[0]['line'],
			'file' => $bt[0]['file'],
			'relTime' => round(($cur - $last) * 100, 2) . ' ms',
			'absTime' => round(($cur - WE_PROFILER) * 100, 2) . ' ms',
			'mem' => round(((memory_get_usage() / 1024))) . ' kB',
			''
		];
		$last = $cur;
		$GLOBALS['we_profile'][] = $prof;
		//annotate tag, if possible
		$attribs['data-time'] = $prof['relTime'];
		$attribs['data-mem'] = $prof['mem'];
		$attribs['data-line'] = $prof['line'];
		$attribs['data-file'] = $prof['file'];
	}
	//make sure comment attribute is never shown
	switch($name){
		case 'setVar':
		case 'xmlnode'://special handling inside tag setVar and xmlnode
			$attribs = removeAttribs($attribs, ['cachelifetime', 'comment', 'user']);
			$nameTo = '';
			$to = 'screen';
			break;
		default:
			$to = weTag_getAttribute('to', $attribs, 'screen');
			$nameTo = weTag_getAttribute('nameto', $attribs, isset($attribs['name']) ? $attribs['name'] : '');
			$attribs = removeAttribs($attribs, ['cachelifetime', 'comment', 'to', 'nameto', 'user']);

			/* if to attribute is set, output of the tag is redirected to a variable
			 * this makes only sense if tag output is equal to non-editmode */
			if($to != 'screen'){
				$GLOBALS['we_editmode'] = false;
			}
	}

	//make a copy of the name - this copy is never touched even not inside blocks/listviews etc.
	if(isset($attribs['name'])){
		$attribs['_name_orig'] = $attribs['name'];
		$attribs['name'] = we_tag_getPostName($attribs['name']);
		if(!empty($GLOBALS['we_editmode']) && ($GLOBALS['we_doc'] instanceof we_webEditionDocument)){
			$GLOBALS['we_doc']->addUsedElement($name, $attribs['name']);
		}
	}

	if($edMerk && $user && (!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR))){
		if(!in_array($_SESSION['user']['Username'], $user)){
			$GLOBALS['we_editmode'] = false;
		}
	}

	$fn = 'we_tag_' . $name;
	if(($foo = we_include_tag_file($fn)) !== true){
		return $foo;
	}

	switch($name){
		case 'setVar':
			we_tag_setVar($attribs);
			//nothing more to do don't waste time
			return '';
		default:
			$foo = $fn($attribs, $content, $internal);
			$GLOBALS['we_editmode'] = $edMerk;
			return we_tag_tag::redirectTagOutput($foo, $nameTo, $to);
	}
}

//tag utility functions

/**
 *
 * @param string $code
 * @deprecated since version 7.1.0
 * @return void
 */
function printElement($code){
	if($code === ''){//tag calculate can return 0, we need to write this.
		return;
	}
	t_e('deprecated', 'we-tag contained php code which needs evaluation, this is deprecated, use we_tag_parse_ instead', $code);
	if(strpos($code, '<?') === FALSE){
		echo $code;
		return;
	}
	eval('?>' . str_replace(['<?php', '<?=', '?>'], ['<?php ', '<?= ', ' ?>'], $code));
}

function getArrayValue($var, $name, $arrayIndex, $isset = false){
	$arr_matches = preg_split('/\]\[|\[|\]/', $arrayIndex);
	if(count($arr_matches) > 1){
		unset($arr_matches[count($arr_matches) - 1]);
	}
	if($name !== null){
		$arr_matches[0] = $name;
	}
	foreach($arr_matches as $cur){
		if(!isset($var[$cur])){
			return ($isset ? false : '');
		}
		$var = $var[$cur];
	}
	return ($isset ? true : $var);
}

/**
 * get an attribute from $attribs, and return its value according to default
 * @param string $name attributes name
 * @param array $attribs array containg the attributes
 * @param mixed $default default value
 * @return mixed returns the attributes value or default if not set
 */
function weTag_getParserAttribute($name, $attribs, $default = '', $type = we_base_request::RAW){
	return weTag_getAttribute($name, $attribs, $default, $type, false);
}

/**
 * get an attribute from $attribs, and return its value according to default
 * @param string $name attributes name
 * @param array $attribs array containg the attributes
 * @param mixed $default default value
 * @param bool $useGlobal check if attribute value is a php-variable and is found in $GLOBALS
 * @return mixed returns the attributes value or default if not set
 */
function weTag_getAttribute($name, $attribs, $default = '', $type = we_base_request::RAW, $useGlobal = true){
	//FIXME: add an array holding attributes accessed for removal!
	$value = isset($attribs[$name]) ? $attribs[$name] : $default;
	$regs = [];
	if($useGlobal && !is_array($value) && preg_match('|^\\\\?\$([^\[]+)(\[.*\])?|', $value, $regs)){
		$value = (isset($regs[2]) ?
			getArrayValue($GLOBALS, $regs[1], $regs[2]) :
			(isset($GLOBALS[$regs[1]]) ? $GLOBALS[$regs[1]] : ''));
	}

	$value = we_base_request::filterVar($value, (is_bool($type) ? we_base_request::BOOL : $type), $default);

	return is_array($value) || is_bool($value) ? $value : htmlspecialchars_decode($value);
}

function we_tag_path_hasIndex($path, $indexArray){
	foreach($indexArray as $index){
		if(file_exists($path . $index)){
			return true;
		}
	}
	return false;
}

function cutSimpleText($text, $len){
	if($len >= strlen($text)){
		return $text;
	}
	$text = substr($text, 0, $len);
	//cut to last whitespace, if any.
	return substr($text, 0, max([
			strrpos($text, ' '),
			strrpos($text, '.'),
			strrpos($text, ','),
			strrpos($text, '/'),
			strrpos($text, "\n"),
			strrpos($text, "\t"),
		]) ?: $len
	);
}

function cutText($text, $max = 0, $striphtml = false){
	$text = $striphtml ? strip_tags($text) : $text;
	if((!$max) || (strlen($text) <= $max)){
		return $text;
	}
	//no tags, simple cut off
	if($striphtml || strpos($text, '<') === false){
		return cutSimpleText($text, $max) . ($striphtml ? ' ...' : ' &hellip;');
	}

	$ret = '';
	$tags = $foo = [];
	//split text on tags, entities and "rest"
	preg_match_all('%(&#?[[:alnum:]]+;)|([^<&]*)|<(/?)([[:alnum:]]+)([ \t\r\n]+[[:alnum:]]+[ \t\r\n]*=[ \t\r\n]*"[^"]*")*[ \t\r\n]*(/?)>%sm', $text, $foo, PREG_SET_ORDER);

	foreach($foo as $cur){
		switch(count($cur)){
			case 2://entity
				if($max > 0){
					$ret .= $cur[0];
					$max -= 1;
				}
				break;
			case 3://text
				if($max > 0){
					$len = strlen($cur[0]);
					$ret .= ($len > $max ? cutSimpleText($cur[0], $max) : $cur[0]);
					$max -= $len;
					if($max <= 0){
						$ret .= ($striphtml ? ' ...' : ' &hellip;');
					}
				}
				break;
			case 7://tags
				if($max > 0){
					$ret .= $cur[0];
					if(!$cur[6]){//!selfclosing
						if($cur[3]){//close
							array_pop($tags);
						} else {
							$tags[] = $cur[4];
						}
					}
				}
				break;
		}
	}

//close open tags
	while($tags){
		$ret .= '</' . array_pop($tags) . '>';
	}

	return $ret;
}

function we_getDocForTag($docAttr, $maindefault = false){
	switch($docAttr){
		case 'self' :
			return $GLOBALS['we_doc'];
		case 'top' :
			return $GLOBALS['WE_MAIN_DOC'];
		default :
			return ($maindefault ?
				$GLOBALS['WE_MAIN_DOC'] :
				$GLOBALS['we_doc']);
	}
}

function modulFehltError($modul, $tag){
	return parseError(sprintf(g_l('parser', '[module_missing]'), $modul, str_replace(['we_tag_', 'we_parse_tag_'], '', $tag)));
}

function parseError($text, $extra = ''){
	t_e('warning', html_entity_decode($text, ENT_QUOTES, $GLOBALS['we']['Charset']['Backend']), g_l('weClass', '[template]') . ': ' . we_tag_tagParser::$curFile, $extra);
	return '<b>' . g_l('parser', '[error_in_template]') . ':</b>' . $text . "<br/>\n" . g_l('weClass', '[template]') . ': ' . we_tag_tagParser::$curFile;
}

function attributFehltError($attribs, $attrs, $tag, $canBeEmpty = false){
	$tag = str_replace(['we_tag_', 'we_parse_tag_'], '', $tag);
	if(!is_array($attrs)){
		$attrs = [$attrs => $canBeEmpty];
	}
	foreach($attrs as $attr => $canBeEmpty){
		if($canBeEmpty){
			if(!isset($attribs[$attr])){
				return parseError(sprintf(g_l('parser', '[attrib_missing2]'), $attr, $tag));
			}
		} elseif(!isset($attribs[$attr]) || $attribs[$attr] === ''){
			return parseError(sprintf(g_l('parser', '[attrib_missing]'), $attr, $tag));
		}
	}
	return '';
}

/**
 * @return array
 * @param array $atts
 * @param array $ignore
 * @desc Removes all empty values from assoc array without the in $ignore given
 */
function removeEmptyAttribs($atts, $ignore = []){
	foreach($atts as $k => $v){
		if($v === '' && !in_array($k, $ignore)){
			unset($atts[$k]);
		}
	}
	return $atts;
}

/**
 * @return array
 * @param array $atts
 * @param array $ignore
 * @desc only uses the attribs given in the array use
 */
function useAttribs(array $atts, array $use){
	return array_intersect_key($atts, array_flip($use));
}

function we_getInputRadioField($name, $value, $itsValue, $atts){
	//  This function replaced fnc: we_getRadioField
	$atts['type'] = 'radio';
	$atts['name'] = $name;
	$atts['value'] = oldHtmlspecialchars($itsValue, -1, 'ISO-8859-1', false);
	if($value == $itsValue){
		$atts['checked'] = 'checked';
	}
	return getHtmlTag('input', $atts);
}

function we_getTextareaField($name, $value, array $atts){
	$atts['name'] = $name;
	return getHtmlTag('textarea', $atts, oldHtmlspecialchars($value), true);
}

function we_getInputTextInputField($name, $value, array $atts){
	$atts['type'] = 'text';
	$atts['name'] = $name;
	$atts['value'] = oldHtmlspecialchars($value);
	return getHtmlTag('input', $atts);
}

function we_getInputCheckboxField($name, $value, array $attr){
	//  returns a checkbox with associated hidden-field
	$tmpname = md5(uniqid(__FUNCTION__, true)); // #6590, changed from: uniqid(time())
	if($value){
		$attr['checked'] = 'checked';
	}
	$attr['type'] = 'checkbox';
	$attr['value'] = 1;
	$attr['name'] = $tmpname;
	$attr['onclick'] = 'this.form.elements[\'' . $name . '\'].value=(this.checked) ? 1 : 0';

	return getHtmlTag('input', $attr) . getHtmlTag('input', ['type' => 'hidden', 'name' => $name, 'value' => oldHtmlspecialchars($value)]);
}

function we_getSelectField($name, $value, array $values, array $attribs = [], $addMissing = true){
	$attribs['name'] = $name;
	$content = '';
	$isin = 0;
	foreach($values as $option){
		$opt = oldHtmlspecialchars($option, -1, 'ISO-8859-1', false);
		if($option == $value){
			$content .= getHtmlTag('option', ['value' => $opt, 'selected' => 'selected'], $opt, true);
			$isin = 1;
		} else {
			$content .= getHtmlTag('option', ['value' => $opt], $opt, true);
		}
	}
	if((!$isin) && $addMissing && $value != ''){
		$content .= getHtmlTag('option', ['value' => oldHtmlspecialchars($value), 'selected' => 'selected'
			], oldHtmlspecialchars($value), true);
	}
	return getHtmlTag('select', $attribs, $content, true);
}

function we_pre_tag_listview($newLV){
	//prevent error if $GLOBALS['we_lv_array'] is no array
	if(empty($GLOBALS['we_lv_array'])){
		$GLOBALS['we_lv_array'] = [];
	}

	//add current lv to the stack
	if(!empty($GLOBALS['lv'])){
		$GLOBALS['we_lv_array'][] = $GLOBALS['lv'];
	}
	//set new lv
	$GLOBALS['lv'] = $newLV;
}

//this function is used by all tags adding elements to we_lv_array
function we_post_tag_listview(){
	if(!empty($GLOBALS['lv'])){
		unset($GLOBALS['lv']);
	}
	if(empty($GLOBALS['we_lv_array'])){
		$GLOBALS['we_lv_array'] = [];
		return;
	}
	//get the last lv
	$GLOBALS['lv'] = array_pop($GLOBALS['we_lv_array']);
}

function getFieldOutLang(array $attribs, $fallBackBackend = false){
	$lang = weTag_getAttribute('outputlanguage', $attribs, '', we_base_request::STRING);
	if(!$lang){
		$doc = we_getDocForTag(weTag_getAttribute('doc', $attribs, 'self', we_base_request::STRING));
		$lang = $doc->Language;
	}
	if(!$lang){
		$lang = ($fallBackBackend ? array_search($GLOBALS['WE_LANGUAGE'], getWELangs()) : $GLOBALS['we']['lang']['DefaultFrontendLanguage']);
	}
	$langcode = substr($lang, 0, 2);
	return [$lang, $langcode];
}
