<?php

/**
 * webEdition CMS
 *
 * $Rev: 14983 $
 * $Author: mokraemer $
 * $Date: 2018-03-01 19:00:51 +0100 (Thu, 01 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
function we_tag_formfield(array $attribs){
	if(($foo = attributFehltError($attribs, 'name', __FUNCTION__))){
		return $foo;
	}
	$name = weTag_getAttribute('name', $attribs, '', we_base_request::STRING);

	$types = weTag_getAttribute('type', $attribs, 'textinput', we_base_request::STRING_LIST);
	$attrs = weTag_getAttribute('attribs', $attribs, '', we_base_request::STRING_LIST);

	$type_sel = $GLOBALS['we_doc']->getElement($name, 'fftype') ?: ($types ? $types[0] : 'textinput');

	$xml = weTag_getAttribute('xml', $attribs, XHTML_DEFAULT, we_base_request::BOOL);
	$ff = [];

	// here add some mandatory fields
	$mandatoryFields = ($xml ? ['textarea_cols', 'textarea_rows'] : []);
	$m = [];
	$removeAttribs = ['doc', 'type', 'attribs'];
	$attrs = array_merge($attrs, $mandatoryFields);

	//check tag attributes as defaults
	foreach($attribs as $k => $v){
		if(preg_match('/^([^_]+)_([^_]+)$/', $k, $m)){
			$removeAttribs[] = $k;
			if($m[1] == $type_sel){
				if(in_array($k, $attrs)){
					$ff[$m[2]] = [
						'value' => $GLOBALS['we_doc']->getElement($name, 'ff_' . $type_sel . '_' . $m[2]),
						'default' => $attribs[$k],
						'change' => 0
					];
				} else {
					$ff[$m[2]] = [
						'change' => 0,
						'default' => $attribs[$k]
					];
				}
			}
		}
	}

	//check attribs attribute for changeable attributes
	foreach($attrs as $a){
		if(preg_match('/^([^_]+)_([^_]+)$/', $a, $m) && ($m[1] == $type_sel)){
			$ff[$m[2]]['change'] = 1;
			if($t = $GLOBALS['we_doc']->getElement($name, 'ff_' . $type_sel . '_' . $m[2])){
				$ff[$m[2]]['value'] = $t;
			}
		}
	}

	if(!empty($GLOBALS['we_editmode'])){
		return we_editTag_formfield($name, $types, $type_sel, $ff);
	}

	$tagAtts = removeAttribs($attribs, $removeAttribs);

	foreach(array_keys($ff) as $f){
		if(!((($f === 'value') && ($type_sel === 'textarea')) || $f === 'type')){
			if(($val = $GLOBALS['we_doc']->getElement($name, 'ff_' . $type_sel . '_' . $f))){
				$tagAtts[$f] = oldHtmlspecialchars($val);
			}
		}
	}
	$ffname = $tagAtts['name'] = oldHtmlspecialchars($GLOBALS['we_doc']->getElement($attribs['name'], 'ffname'));

	$tagContent = '';
	$tagEndTag = false;

	$ret = '';
	switch($type_sel){
		case 'checkbox':
		case 'radio':
			$tagName = 'input';
			$tagAtts['type'] = $type_sel;
			if($GLOBALS['we_doc']->getElement($name, 'ffchecked')){
				$tagAtts['checked'] = 'checked';
			}
			break;
		case 'textinput': // correct input type="text"
			$tagName = 'input';
			$tagAtts['type'] = 'text';
			break;
		case 'textarea':
			$tagName = 'textarea';
			$tagEndTag = true;
			if(isset($ff['value'])){
				$tagContent = oldHtmlspecialchars($ff['value']['value']);
			}
			if(!array_key_exists('cols', $tagAtts)){
				$tagAtts['cols'] = 20;
			}
			if(!array_key_exists('rows', $tagAtts)){
				$tagAtts['rows'] = 5;
			}
			break;
		case 'select':
			$tagName = 'select';
			$selected = $GLOBALS['we_doc']->getElement($name, 'ffdefault');
			$foo = explode('<_BR_>', strtr($GLOBALS['we_doc']->getElement($name, 'ffvalues'), ["\r\n" => '<_BR_>', "\r" => '<_BR_>', "\n" => '<_BR_>',]));
			foreach($foo as $v){
				$atts = ['value' => oldHtmlspecialchars(trim($v))
				];
				if($selected == $v){
					$atts['selected'] = 'selected';
				}
				$tagContent .= getHtmlTag('option', $atts, oldHtmlspecialchars($v));
			}
			break;
		case 'country':
			$tagName = 'select';
			$orgVal = $GLOBALS['we_doc']->getElement($name, 'ffdefault');
			$docAttr = weTag_getAttribute("doc", $attribs, "self", we_base_request::STRING);
			$doc = we_getDocForTag($docAttr);
			$lang = $doc->Language;
			$langcode = ($lang ? substr($lang, 0, 2) : array_search($GLOBALS['WE_LANGUAGE'], getWELangs()) );

			$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
			foreach($topCountries as $countrykey => &$countryvalue){
				$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
			}
			unset($countryvalue);
			$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
			foreach($shownCountries as $countrykey => &$countryvalue){
				$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
			}
			unset($countryvalue);
			$oldLocale = setlocale(LC_ALL, NULL);
			setlocale(LC_ALL, $lang . '.UTF-8');
			asort($topCountries, SORT_LOCALE_STRING);
			asort($shownCountries, SORT_LOCALE_STRING);
			setlocale(LC_ALL, $oldLocale);

			$tagContent = (WE_COUNTRIES_DEFAULT == '' ? '' : '<option value="--" ' . ($orgVal === '--' ? ' selected="selected">' : '>') . WE_COUNTRIES_DEFAULT . '</option>');

			foreach($topCountries as $countrykey => $countryvalue){
				$tagContent .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($countryvalue) . '</option>';
			}
			$tagContent .= '<option value="-" disabled="disabled">----</option>';
			foreach($shownCountries as $countrykey => $countryvalue){
				$tagContent .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($countryvalue) . '</option>';
			}
			$tagAtts['size'] = (isset($attrs['size']) ? $attrs['size'] : 1);
			//  $newAtts['name'] = $name;
			break;
		case 'language':
			$tagName = 'select';
			$orgVal = $GLOBALS['we_doc']->getElement($name, 'ffdefault');
			$docAttr = weTag_getAttribute('doc', $attribs, 'self', we_base_request::STRING);
			$doc = we_getDocForTag($docAttr);
			$lang = $doc->Language;
			$langcode = ($lang ? substr($lang, 0, 2) : array_search($GLOBALS['WE_LANGUAGE'], getWELangs()));
			$frontendL = $GLOBALS['we']['lang']['FrontendLanguages'];
			foreach($frontendL as &$lcvalue){
				$lccode = explode('_', $lcvalue);
				$lcvalue = $lccode[0];
			}
			unset($lcvalue);
			foreach($frontendL as $lcvalue){
				$frontendLL[$lcvalue] = we_base_country::getTranslation($lcvalue, we_base_country::LANGUAGE, $langcode);
			}

			$oldLocale = setlocale(LC_ALL, NULL);
			setlocale(LC_ALL, $lang . '.UTF-8');
			asort($frontendLL, SORT_LOCALE_STRING);
			setlocale(LC_ALL, $oldLocale);

			$tagContent = '';
			foreach($frontendLL as $langkey => $langvalue){
				$tagContent .= '<option value="' . $langkey . '" ' . ($orgVal == $langkey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($langvalue) . '</option>';
			}
			$tagAtts['size'] = (isset($attrs['size']) ? $attrs['size'] : 1);

			break;
		case 'file':
			return getHtmlTag('input', [
				'type' => 'hidden',
				'name' => 'MAX_FILE_SIZE',
				'value' => oldHtmlspecialchars($GLOBALS['we_doc']->getElement($name, 'ffmaxfilesize')),
				'xml' => $xml
			]);
	}
	return getHtmlTag($tagName, $tagAtts, $tagContent, $tagEndTag) . $ret;
}

function we_editTag_formfield($name, $types, $type_sel, array $ff){
	static $availTypes = false;
	$availTypes = $availTypes ?: [
		'checkbox' => g_l('modules_object', '[checkbox_field]'),
		'radio' => 'Radio',
		'textinput' => g_l('modules_object', '[input_field]'),
		'textarea' => g_l('modules_object', '[textarea_field]'),
		'select' => g_l('modules_object', '[meta_field]'),
		'country' => g_l('modules_object', '[country_field]'),
		'language' => g_l('modules_object', '[language_field]'),
		'file' => g_l('modules_object', '[binary_field]'),
	];
	$ffname = $GLOBALS['we_doc']->getElement($name, 'ffname');
	$nameprefix = 'we_' . $GLOBALS['we_doc']->Name . '_txt[' . $name . '#';

	$tmp_select = '';
	foreach($types as $k){
		if(isset($availTypes[$k])){
			$tmp_select .= '<option value="' . $k . '"' . (($k == $type_sel) ? ' selected="selected"' : '') . '>' . $availTypes[$k] . '</option>';
		}
	}

	$tbl = '<table style="padding:4px;border:0; color: black;" class="weEditTable weEditmodeStyle">
<colgroup>
	<col style="width:8em;color: black;padding-right:1ex;"/>
	<col style="width:15em;"/>
</colgroup>
	<tr>
		<td>' . g_l('global', '[name]') . ':</td>
		<td><input type="text" name="' . $nameprefix . 'ffname]" style="width:20em" value="' . $ffname . '" required="required" pattern="[^\[\]. ]+" onblur="checkFormFields(this);"/></td>
	</tr>
	<tr>
		<td>' . g_l('global', '[type]') . ':</td>
		<td><select name="' . $nameprefix . 'fftype]" onchange="setScrollTo();we_cmd(\'reload_editpage\');">' . $tmp_select . '</select></td>
	</tr>';

	if($ff){
		$tbl .= '	<tr>
		<td>' . g_l('global', '[attributes]') . ':</td>
		<td><table class="weEditTable spacing0 border0">';

		foreach($ff as $f => $m){
			$tbl .= '<tr><td style="color: black; margin-left:5px;"><span class="small bold">' . g_l('weTag', '[formfield][' . $f . ']', true) . '(' . $f . ')' . ':</span></td><td>';
			$val = isset($m['value']) ? $m['value'] : '';

			$default = isset($m['default']) ? makeArrayFromCSV($m['default']) : [];

			if($m['change'] == 1){
				if(count($default) > 1){
					$valselect = '<select name="' . $name . 'tmp" onchange="this.form.elements[\'' . $nameprefix . 'ff_' . $type_sel . '_' . $f . ']\'].value=this.options[this.selectedIndex].value;">' .
						'<option value=""></option>';
					foreach($default as $v){
						$valselect .= '<option value="' . $v . '">' . $v . '</option>';
					}
					$valselect .= '</select>';
				} else {
					$valselect = '';
				}
				if((!isset($m['value'])) && count($default) == 1){
					$val = $default[0];
				}
				$tbl .= '<input type="text" name="' . $nameprefix . 'ff_' . $type_sel . '_' . $f . ']" style="width:20em" ' . ($val ? ' value="' . $val . '"' : '') . ' />' . $valselect;
			} else {
				if(count($default) > 1){
					$val = $GLOBALS['we_doc']->getElement($name, 'ff_' . $type_sel . '_' . $f);
					if(count($default) > 1){
						$valselect = '<select name="' . $nameprefix . 'ff_' . $type_sel . '_' . $f . ']">';
						foreach($default as $v){
							$valselect .= '<option value="' . $v . '"' . (($v == $val) ? " selected" : "") . '>' . $v . '</option>';
						}
						$valselect .= '</select>';
						$tbl .= $valselect;
					}
				} else {
					$foo = empty($default) ? '' : $default[0];
					$tbl .= $foo . '<input type="hidden" name="' . $nameprefix . 'ff_' . $type_sel . '_' . $f . ']" value="' . $foo . '" />';
				}
			}
			$tbl .= '</td></tr>';
		}
		$tbl .= '</table>
		</td>
	</tr>';
	}

	switch($type_sel){
		case 'select':
			$tbl .= '	<tr>
		<td>' . g_l('global', '[values]') . ':</td>
		<td><textarea name="' . $nameprefix . 'ffvalues]" style="width:20em;height:5ex;">' . $GLOBALS['we_doc']->getElement($name, 'ffvalues') . '</textarea></td>
	</tr>
	<tr>
		<td>' . g_l('global', '[default]') . ':</td>
		<td><input type="text" name="' . $nameprefix . 'ffdefault]" style="width:20em" value="' . $GLOBALS['we_doc']->getElement($name, 'ffdefault') . '" /></td>
	</tr>';
			break;
		case 'file':
			$tbl .= '	<tr>
		<td>' . g_l('global', '[max_file_size]') . ':</td>
		<td><input type="text" name="' . $nameprefix . 'ffmaxfilesize]" value="' . $GLOBALS['we_doc']->getElement($name, 'ffmaxfilesize') . '" /></td>
	</tr>';
			break;
		case 'radio':
		case 'checkbox':
			$tbl .= '	<tr>
		<td>' . g_l('global', '[checked]') . ':</td>
		<td><select name="' . $nameprefix . 'ffchecked]"><option value="0"' . ($GLOBALS['we_doc']->getElement($name, 'ffchecked') ? "" : " selected") . '>' . g_l('global', '[no]') . '</option><option value="1"' . ($GLOBALS['we_doc']->getElement($name, 'ffchecked') ? " selected" : "") . '>' . g_l('global', '[yes]') . '</option></select></td>
	</tr>';
			break;
	}
	$tbl .= '</table>';

	return $tbl;
}
