<?php

/**
 * webEdition CMS
 *
 * $Rev: 15014 $
 * $Author: mokraemer $
 * $Date: 2018-03-08 01:03:26 +0100 (Thu, 08 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
function we_tag_href(array $attribs){
	if(($foo = attributFehltError($attribs, 'name', __FUNCTION__))){
		return $foo;
	}
	$name = weTag_getAttribute('name', $attribs, '', we_base_request::STRING);
	$type = weTag_getAttribute('type', $attribs, we_base_link::TYPE_ALL, we_base_request::STRING);
	$hidedirindex = weTag_getAttribute('hidedirindex', $attribs, TAGLINKS_DIRECTORYINDEX_HIDE, we_base_request::BOOL);
	$include = weTag_getAttribute('include', $attribs, false, we_base_request::BOOL);
	$reload = weTag_getAttribute('reload', $attribs, false, we_base_request::BOOL);
	$rootdir = weTag_getAttribute('rootdir', $attribs, '/', we_base_request::FILE);
//	$seeMode = weTag_getAttribute((isset($attribs['seem']) ? 'seem' : 'seeMode'), $attribs, true, true);
	$file = weTag_getAttribute('file', $attribs, true, we_base_request::BOOL);
	$directory = weTag_getAttribute('directory', $attribs, false, we_base_request::BOOL);
	$attribs = removeAttribs($attribs, ['rootdir', 'file', 'directory']);

	if(isset($GLOBALS['we_doc']->OF_ID)){
		$hrefArr = we_unserialize($GLOBALS['we_doc']->getElement($name));
		return we_document::getHrefByArray($hrefArr);
	}

	$nint = $name . we_base_link::MAGIC_INT_LINK;
	$nintID = $name . we_base_link::MAGIC_INT_LINK_ID;
	$nintPath = $name . we_base_link::MAGIC_INT_LINK_PATH;
	$extPath = '';
	$int = ($type == we_base_link::TYPE_EXT ? false : ($type == we_base_link::TYPE_INT || $GLOBALS['we_doc']->getElement($nint)) ? $GLOBALS['we_doc']->getElement($nint) : false);

	if($int){
		$intID = intval($GLOBALS['we_doc']->getElement($nintID, we_contents_base::ETYPE_NUMBER)) ?: intval(weTag_getAttribute('startid', $attribs, 0, we_base_request::INT));
		if($intID && ($foo = getHash('SELECT Path,ContentType,(IsFolder=1||Published>0) AS isPublished FROM ' . FILE_TABLE . ' WHERE ID=' . $intID, null, MYSQLI_NUM))){
			list($intPath, $ct, $isPublished) = $foo;
		} else {
			$isPublished = 0;
			$intPath = '';
		}
		$href = $intPath;
		$include_path = $href ? WEBEDITION_PATH . '..' . $href : ''; //(symlink) webEdition always points to the REAL DOC-Root!
		$path_parts = pathinfo($href);
		if($hidedirindex && seoIndexHide($path_parts['basename'])){
			$href = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
		}
	} else {
		$intID = 0;
		$intPath = $ct = '';
		$isPublished = false;
// we have to use a html_entity_decode first in case a user has set &amp, &uuml; by himself
		$href = $extPath = oldHtmlspecialchars(html_entity_decode($GLOBALS['we_doc']->getElement($name)));
		$include_path = $href ? WEBEDITION_PATH . '..' . $href : ''; //(symlink) webEdition always points to the REAL DOC-Root!
	}

	if(!$GLOBALS['we_editmode']){
		if($int && we_modules_info::isActive(we_modules_info::CUSTOMER) && $intID && weTag_getAttribute('cfilter', $attribs, true, we_base_request::BOOL)){
			$filter = we_customer_documentFilter::getFilterByIdAndTable($intID, FILE_TABLE, $GLOBALS['DB_WE']);

			if(is_object($filter)){
				if($filter->accessForVisitor($intID, $ct, true) != we_customer_documentFilter::ACCESS){
					return '';
				}
			}
		}

		switch(weTag_getAttribute('only', $attribs, '', we_base_request::STRING)){
			case 'id':
				return $isPublished ? intval($intID) : 0;
		}
		if($int && !$isPublished){
			return '';
		}
		if($int && $intID && !$include && !isset($attribs['isInternal'])){
			$urlReplace = we_folder::getUrlReplacements($GLOBALS['DB_WE'], true);
			if($urlReplace){
				return preg_replace($urlReplace, array_keys($urlReplace), $href);
			}
		}
		if($include){
			if($include_path && file_exists($include_path)){
				include($include_path);
			}
			return '';
		}

		return $href;
	}

	if($rootdir[0] != '/'){
		$rootdirid = $rootdir;
		$rootdir = id_to_path($rootdir, FILE_TABLE);
	} else {
		$rootdir = ($rootdir === '/' ? $rootdir : rtrim($rootdir, '/'));
		$rootdirid = path_to_id($rootdir, FILE_TABLE);
	}
// Bug Fix #7045
	if($rootdir === '/'){
		$rootdir = '';
	}

	$int_elem_Name = 'we_' . $GLOBALS['we_doc']->Name . '_href[' . $nint . '#' . we_contents_base::ETYPE_NUMBER . ']';
	$intPath_elem_Name = 'we_' . $GLOBALS['we_doc']->Name . '_' . we_contents_base::INTERNAL_VAR . '[' . $nintPath . ']';
	$intID_elem_Name = 'we_' . $GLOBALS['we_doc']->Name . '_href[' . $nintID . '#' . we_contents_base::ETYPE_NUMBER . ']';
	$ext_elem_Name = 'we_' . $GLOBALS['we_doc']->Name . '_txt[' . $name . ']';

	$span = '<span class="defaultfont">';
	$size = 5 * intval(weTag_getAttribute('size', $attribs, 20, we_base_request::INT));

	$cmd1 = "document.we_form.elements['" . $intID_elem_Name . "'].value";
	$cmdInt = 'tag_weHref_selectorCallback,' . we_base_link::TYPE_INT . ',' . $type . ',' . $int_elem_Name . ',' . ($include || $reload ? 1 : 0);
	$cmdExt = 'tag_weHref_selectorCallback,' . we_base_link::TYPE_EXT . ',' . $type . ',' . $int_elem_Name . ',0';
	if(($directory && $file) || $file){
		$btnInt = we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:we_cmd('we_selector_document', " . $cmd1 . ", '" . FILE_TABLE . "','" . $intID_elem_Name . "','" . $intPath_elem_Name . "','" . $cmdInt . "','', '" . $rootdirid . "', '', " . (we_base_permission::hasPerm(we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES) ? 0 : 1) . ",''," . ($directory ? 1 : 0) . ");");
		$btnExt = we_base_permission::hasPerm(we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT) ? we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:we_cmd('browse_server', document.we_form.elements['" . $ext_elem_Name . "'].value, '" . (($directory && $file) ? "filefolder" : '') . "', document.forms[0].elements['" . $ext_elem_Name . "'].value, '" . $cmdExt . "','" . $rootdir . "')") : '';
	} else {
		$btnInt = we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:we_cmd('we_selector_directory', " . $cmd1 . ", '" . FILE_TABLE . "','" . $intID_elem_Name . "','" . $intPath_elem_Name . "','" . $cmdInt . "','', '" . $rootdirid . "');");
		$btnExt = we_base_permission::hasPerm(we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT) ? we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:we_cmd('browse_server', document.we_form.elements['" . $ext_elem_Name . "'].value, '" . we_base_ContentTypes::FOLDER . "', document.forms[0].elements['" . $ext_elem_Name . "'].value, '" . $cmdExt . "','" . $rootdir . "')") : '';
	}
	$btnEdit = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:we_cmd('tag_weHref_openDocument','" . $intID_elem_Name . "');");
	$btnTrashInt = we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('tag_weHref_trash','" . we_base_link::TYPE_INT . "','" . $intID_elem_Name . "','" . $intPath_elem_Name . "'," . ($include || $reload ? 1 : 0) . ");");
	$btnTrashExt = we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('tag_weHref_trash','" . we_base_link::TYPE_EXT . "','" . $ext_elem_Name . "');");

	switch($type){
		case we_base_link::TYPE_ALL:
		case we_base_link::TYPE_INT:
			$weSuggest = &we_gui_suggest::getInstance();
			$weSuggest->setAcId($name . we_base_file::getUniqueId(), $rootdir);
			$weSuggest->setContentType($directory ? we_base_ContentTypes::FOLDER : [we_base_ContentTypes::FOLDER, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::IMAGE,
					we_base_ContentTypes::HTML, we_base_ContentTypes::JS,
					we_base_ContentTypes::CSS, we_base_ContentTypes::APPLICATION, we_base_ContentTypes::VIDEO, we_base_ContentTypes::AUDIO, we_base_ContentTypes::PDF]);
			$weSuggest->setInput($intPath_elem_Name, $intPath);
			$weSuggest->setMaxResults(10);
			$weSuggest->setResult($intID_elem_Name, $intID);
			$weSuggest->setSelector($directory ? we_gui_suggest::DirSelector : we_gui_suggest::DocSelector);
			$weSuggest->setTable(FILE_TABLE);
			$weSuggest->setWidth($size);
	}

	ob_start();
	if($include && $include_path && file_exists($include_path)){
		include($include_path);
	}

	return
		'<table class="weEditTable">' .
		($type == we_base_link::TYPE_ALL || $type == we_base_link::TYPE_INT ? '
<tr>
	<td>' . ($type == we_base_link::TYPE_ALL ? we_html_forms::radiobutton(1, $int, $int_elem_Name,  g_l('tags', '[int_href]') . ':') : g_l('tags', '[int_href]') . ':<input type="hidden" name="' . $int_elem_Name . '" value="1" />' ) . '</td>
	<td>' . $weSuggest->getHTML() . '</td>
	<td>' . $btnInt . '</td>
	<td>' . $btnTrashInt . '</td>
	<td>' . $btnEdit . '</td>
	</tr>' : '') .
		($type == we_base_link::TYPE_ALL || $type == we_base_link::TYPE_EXT ? '
<tr>
	<td>' . ($type == we_base_link::TYPE_ALL ? we_html_forms::radiobutton(0, !$int, $int_elem_Name, $span . g_l('tags', '[ext_href]') . ':</span>') : $span . g_l('tags', '[ext_href]') . ':</span><input type="hidden" name="' . $int_elem_Name . '" value="0" />') . '</td>
	<td>' .
		getHtmlTag('input', array_merge(removeAttribs($attribs, ['onkeydown', 'onkeydown']), ['style' => 'width:' . ($size) . 'px;',
			'onchange' => ($type == we_base_link::TYPE_ALL ? 'this.form.elements[\'' . $int_elem_Name . '\'][1].checked=true;' : ''),
			'type' => 'text',
			'name' => 'we_' . $GLOBALS['we_doc']->Name . '_txt[' . $name . ']',
			'placeholder' => "http://example.org",
			'value' => $extPath
		]))
		. '</td>
	<td>' . $btnExt . '</td>
	<td>' . $btnTrashExt . '</td>
</tr>' : '') . '
</table>' . ob_get_clean();
}
