<?php

/**
 * webEdition CMS
 *
 * $Rev: 14956 $
 * $Author: mokraemer $
 * $Date: 2018-02-27 11:38:55 +0100 (Tue, 27 Feb 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
function we_tag_ifEmpty(array $attribs){
	if(!empty($GLOBALS['we_editmode'])){
		return false;
	}

	$docAttr = weTag_getAttribute('doc', $attribs, '', we_base_request::STRING);
	$orgName = weTag_getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
	$name = $orgName ?: we_tag_getPostName(weTag_getAttribute('match', $attribs, '', we_base_request::STRING));
	$doc = we_getDocForTag($docAttr, false);
	$regs = [];

	if(!$name && ($foo = attributFehltError($attribs, 'name', __FUNCTION__))){
		echo $foo;
		return false;
	}

	if(preg_match('|^\\\\?\$([^\[]+)(\[.*\])?|', $name, $regs)){
		$name = (isset($regs[2]) ?
			getArrayValue($GLOBALS, $regs[1], $regs[2]) :
			(isset($GLOBALS[$regs[1]]) ? $GLOBALS[$regs[1]] : ''));
	}

	switch(weTag_getAttribute('type', $attribs, '', we_base_request::STRING)){
		case 'checkbox':
		case 'object':
			$val = $doc->getElement($name);
			return empty($val);
		case 'binary' :
		case 'img':
		case 'flashmovie' :
			$val = $doc->getElement($name, we_contents_base::ETYPE_NUMBER);
			return empty($val);
		case 'href':
			if(isset($doc->OF_ID)){
				$hreftmp = $doc->getElement($name);
				if(!$hreftmp || !is_array($hreftmp)){
					return true;
				}
				$hreftmp = trim(we_document::getHrefByArray($hreftmp));

				return ($hreftmp && $hreftmp{0} === '/' ?
					(!file_exists(WEBEDITION_PATH . '../' . $hreftmp)) :
					($hreftmp ? false : true)
					);
			}
			if(intval($doc->getElement($name . we_base_link::MAGIC_INT_LINK))){ // for type = href int
				$intID = $doc->getElement($name . we_base_link::MAGIC_INT_LINK_ID, we_contents_base::ETYPE_NUMBER);
				return !($intID && id_to_path([$intID]));
			}
			$hreftmp = $doc->getElement($name);
			if($hreftmp && $hreftmp{0} === '/'){
				return (!file_exists(WEBEDITION_PATH . '../' . $hreftmp));
			}

		default:
			$dat = $doc->getElement($name, we_contents_base::ETYPE_TEXT);
			//   #3938 added this - some php version crashed, when unserialize started with a ?,?,?
			if($dat && (substr($dat, 0, 2) === 'a:' || $dat{0} === '{')){ //  only unserialize, when $match could be an array
				// Added @-operator in front of the unserialze function because there
				// were some PHP notices that had no effect on the output of the function
				// remark holeg: when it is a serialized array, the function looks if it is not empty
				if(is_array($arr = we_unserialize($dat))){
					return empty($arr);
				}
			}

		//   end of #3938
	}

	$val1 = $doc->getElement($name);
	$val2 = $doc->getElement($name, we_contents_base::ETYPE_NUMBER);
	return empty($val1) && empty($val2);
}
