<?php

/**
 * webEdition CMS
 *
 * $Rev: 15033 $
 * $Author: mokraemer $
 * $Date: 2018-03-12 09:32:10 +0100 (Mon, 12 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
function we_tag_ifFieldEmpty($attribs){
	if(attributFehltError($attribs, 'match', __FUNCTION__) && ($foo = attributFehltError($attribs, 'name', __FUNCTION__))){
		echo $foo;
		return false;
	}
	$type = weTag_getAttribute('type', $attribs, '', we_base_request::STRING);
	$orig_name = weTag_getAttribute('_name_orig', $attribs, '', we_base_request::STRING) ?: weTag_getAttribute('match', $attribs, '', we_base_request::STRING);

	if(empty($GLOBALS['lv'])){
		echo parseError(g_l('parser', '[field_not_in_lv]'));
		return true;
	}

	$name = ($GLOBALS['lv']->f($orig_name) ? $orig_name : we_tag_getPostName($orig_name));

	switch($type){
		case 'calendar' :
			$cal = $GLOBALS['lv']->getCalendarStruct();
			if(!empty($cal)){
				if($cal['date'] < 0 || empty($cal['storage'])){
					return true;
				}
				switch($orig_name){
					case 'day':
						$sd = mktime(0, 0, 0, $cal['month_human'], $cal['day_human'], $cal['year_human']);
						$ed = mktime(23, 59, 59, $cal['month_human'], $cal['day_human'], $cal['year_human']);
						break;
					case 'month':
						$sd = mktime(0, 0, 0, $cal['month_human'], 1, $cal['year_human']);
						$ed = mktime(23, 59, 59, $cal['month_human'], $cal['numofentries'], $cal['year_human']);
						break;
					case 'year':
						$sd = mktime(0, 0, 0, 1, 1, $cal['year_human']);
						$ed = mktime(23, 59, 59, 12, 31, $cal['year_human']);
						break;
				}
				if(isset($sd) && isset($ed)){
					foreach($cal['storage'] as $entry){
						if($sd < $entry && $ed > $entry){
							return false;
						}
					}
				}
			}
			return true;
		case 'multiobject':
			$data = we_unserialize((!empty($GLOBALS['lv']) ?
				$GLOBALS['lv']->f($orig_name) :
				$GLOBALS['we_doc']->getElement($orig_name)));
			$objects = array_filter(isset($data['objects']) ? $data['objects'] : $data);
			return empty($objects);
		case 'object' : //Bug 3837: erstmal die Klasse rausfinden um auf den Eintrag object_X zu kommen
			if($GLOBALS['lv'] instanceof we_listview_document){ // listview/document with objects included using we:object
				return empty($GLOBALS['lv']->f($name));
			}
			$name = strpos($orig_name, '/') === false ? $orig_name : substr(strrchr($orig_name, '/'), 1);
			$objectid = f('SELECT ID FROM ' . OBJECT_TABLE . ' WHERE Text="' . $GLOBALS['DB_WE']->escape($name) . '"');
			return empty($GLOBALS['lv']->f('object_' . $objectid));
		case 'checkbox' :
		case 'binary' :
		case 'img' :
		case 'flashmovie' :
		case 'collection':
			return empty($GLOBALS['lv']->f($name));
		case 'float':
			return floatval($GLOBALS['lv']->f($name)) === floatval(0);
		case 'int':
			return intval($GLOBALS['lv']->f($name)) === 0;
		case 'href' :
			if($GLOBALS['lv'] instanceof we_listview_objectBase){
				if(!($hrefArr = $GLOBALS['lv']->f($name) ? we_unserialize($GLOBALS['lv']->f($name)) : [])){
					return true;
				}
				$hreftmp = trim(we_document::getHrefByArray($hrefArr));
				/**
				 * TODO: file_exists only work when the choosen webEdition document is in the correct DOCUMENT_ROOT
				 * but within multi domains we have cross DOCUMENT_ROOT references!!!
				 */
				if(!$hreftmp || $hreftmp === '/' || ($hreftmp{0} === '/' && (!file_exists(WEBEDITION_PATH . '../' . $hreftmp)))){
					return true;
				}
				return false;
			}

			// we must check $match . we_base_link::MAGIC_INT_LINK for block-Postfix instead of $match (which exists only for href type = ext): #6422
			$isInBlock = ( $GLOBALS['lv']->f($orig_name . we_base_link::MAGIC_INT_LINK) || $GLOBALS['lv']->f($orig_name) ) ? false : true;
			$name = $isInBlock ? we_tag_getPostName($orig_name) : $orig_name;

			$int = ($GLOBALS['lv']->f($name . we_base_link::MAGIC_INT_LINK) == '') ? 0 : $GLOBALS['lv']->f($name . we_base_link::MAGIC_INT_LINK);
			if($int){ // for type = href int
				$intID = $GLOBALS['lv']->f($name . we_base_link::MAGIC_INT_LINK_ID);
				return !($intID && id_to_path($intID));
			}
			$hreftmp = $GLOBALS['lv']->f($name);
			if($hreftmp{0} === '/' && (!file_exists(WEBEDITION_PATH . '../' . $hreftmp))){
				return true;
			}

			break; //see return of function
		default :
			$tmp = we_unserialize($GLOBALS['lv']->f($name), '', true);
			if(is_array($tmp)){
				return empty($tmp);
			}
		//no break;
	}
	return empty($GLOBALS['lv']->f($name));
}
