<?php
/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
we_modules_info::isActive(we_modules_info::SHOP);

function we_tag_showShopItemNumber(array $attribs){
	if(($foo = attributFehltError($attribs, 'shopname', __FUNCTION__))){
		return $foo;
	}

	$shopname = weTag_getAttribute('shopname', $attribs, '', we_base_request::STRING);
	$type = weTag_getAttribute('option', $attribs, false, we_base_request::BOOL) ? 'select' : '';
	$type = $type ?: (weTag_getAttribute('inputfield', $attribs, false, we_base_request::BOOL) ? 'textinput' : '');
	$type = $type ?: (weTag_getAttribute('type', $attribs, '', we_base_request::STRING));
	$xml = weTag_getAttribute('xml', $attribs, XHTML_DEFAULT, we_base_request::BOOL);
	$num_format = weTag_getAttribute('num_format', $attribs, '', we_base_request::STRING);
	$floatquantities = weTag_getAttribute('floatquantities', $attribs, false, we_base_request::BOOL);

	if(!isset($GLOBALS[$shopname]) || empty($GLOBALS[$shopname])){
		return parseError(sprintf(g_l('parser', '[missing_createShop]'), 'showShopItemNumber'));
	}

	$attr = removeAttribs($attribs, ['option', 'inputfield', 'type', 'start', 'stop', 'shopname', 'nameto', 'to', 'floatquantities', 'num_format']);

	// $type of the field
	$articleType = ($GLOBALS['lv']->f('OF_ID') ? we_shop_shop::OBJECT : we_shop_shop::DOCUMENT);

	$itemQuantity = (!empty($GLOBALS['lv']) && ($GLOBALS['lv'] instanceof we_listview_shopItems) ?
		$GLOBALS[$shopname]->getItemQuantity($GLOBALS['lv']->getShoppingCartKey()) :
		0);

	switch($type){
		case 'select':
			$start = weTag_getAttribute('start', $attribs, 0, we_base_request::INT);
			$stop = weTag_getAttribute('stop', $attribs, 10, we_base_request::INT);
			$step = weTag_getAttribute('step', $attribs, 1, we_base_request::INT);
			$stop = ($floatquantities ?
				( $stop > $itemQuantity ? $stop : $itemQuantity) :
				( intval($stop) > intval($itemQuantity) ? $stop : $itemQuantity));

			$attr['name'] = 'shop_cart_id[' . $GLOBALS['lv']->getShoppingCartKey() . ']';
			$attr['size'] = 1;
			$attr['xml'] = $xml;

			$out = '';
			while($start <= $stop){
				$out .= ($itemQuantity == $start ?
					getHtmlTag('option', ['xml' => $xml, 'value' => $start, 'selected' => 'selected'], $start) :
					getHtmlTag('option', ['xml' => $xml, 'value' => $start], $start));

				$start = $start + $step;
			}
			return getHtmlTag('select', $attr, $out, true) . getHtmlTag('input', ['type' => 'hidden', 'name' => 't', 'value' => time()]);

		case 'textinput':
			$itemQuantity = ($floatquantities ? we_base_util::formatNumber($itemQuantity, $num_format, 2) : intval($itemQuantity));
			$attr = array_merge($attr, ['type' => 'text', 'name' => 'shop_cart_id[' . $GLOBALS['lv']->getShoppingCartKey() . ']', 'size' => 2, 'value' => $itemQuantity]);
			return getHtmlTag('input', $attr) . getHtmlTag('input', ['type' => 'hidden', 'name' => 't', 'value' => time()]);

		default:
			return ($floatquantities ? we_base_util::formatNumber($itemQuantity, $num_format, 2) : intval($itemQuantity));
	}
}
