<?php

/**
 * webEdition CMS
 *
 * $Rev: 15014 $
 * $Author: mokraemer $
 * $Date: 2018-03-08 01:03:26 +0100 (Thu, 08 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
function we_tag_userInput(array $attribs, $content){
	if(($foo = attributFehltError($attribs, 'name', __FUNCTION__))){
		return $foo;
	}

	$name = weTag_getAttribute('name', $attribs, '', we_base_request::STRING);
	$type = weTag_getAttribute('type', $attribs, '', we_base_request::STRING);
	$property = weTag_getAttribute('property', $attribs, false, we_base_request::BOOL);
	$format = weTag_getAttribute('format', $attribs, '', we_base_request::STRING);
	$checked = weTag_getAttribute('checked', $attribs, false, we_base_request::BOOL);
	$inlineedit = weTag_getAttribute('inlineedit', $attribs, true, we_base_request::BOOL);
	$value = weTag_getAttribute('value', $attribs, '', we_base_request::RAW);
	$editable = weTag_getAttribute('editable', $attribs, true, we_base_request::BOOL);
	//$autobrAttr = weTag_getAttribute('autobr', $attribs, true, we_base_request::BOOL);
	$hidden = weTag_getAttribute('hidden', $attribs, false, we_base_request::BOOL);
	$size = weTag_getAttribute('size', $attribs, '', we_base_request::UNIT);
	$values = weTag_getAttribute('values', $attribs, [], we_base_request::STRING_LIST);
	$xml = weTag_getAttribute('xml', $attribs, XHTML_DEFAULT, we_base_request::BOOL);
	$removeFirstParagraph = weTag_getAttribute('removefirstparagraph', $attribs, defined('REMOVEFIRSTPARAGRAPH_DEFAULT') ? REMOVEFIRSTPARAGRAPH_DEFAULT : true, we_base_request::BOOL);

	if($hidden && ($type != 'date')){
		$type = 'hidden';
	}

	$object = (isset($GLOBALS['WE_FORM']) ?
		(isset($GLOBALS['we_object'][$GLOBALS['WE_FORM']]) ?
		$GLOBALS['we_object'][$GLOBALS['WE_FORM']] :
		(isset($GLOBALS['we_document'][$GLOBALS['WE_FORM']]) ?
		$GLOBALS['we_document'][$GLOBALS['WE_FORM']] :
		(isset($GLOBALS['we_doc']) ?
		$GLOBALS['we_doc'] :
		false))) :
		'');

	if($object){
		$tableID = '';
		if($object instanceof we_objectFile){
			$name = weTag_getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
			$tableID = $object->TableID;
		}
		if($property){
			$isset = isset($object->{$name});
			$orgVal = $isset ? $object->{$name} : $value;
		} else {
			$isset = (!$object->ID && $object->getElement($name) === '' ?
				false :
				$object->issetElement($name));

			$orgVal = $isset ? $object->getElement($name) : $value;
		}

		$content = $object->getFieldByVal($orgVal, $type, $attribs, true, $object->ParentID, $object->Path, $GLOBALS['DB_WE'], $tableID);
	} else {
		$orgVal = $value;
		$isset = false;
	}

	$fieldname = 'we_ui_' . (isset($GLOBALS['WE_FORM']) ? $GLOBALS['WE_FORM'] : '') .
		($property ? ( '_' . $name) : ('[' . $name . ']') );


	if(!$editable && !$hidden){
		switch($type){
			case 'img':
			case 'binary':
			case 'flashmovie':
				break;
			default:
				$hidden = getHtmlTag('input', [
					'type' => 'hidden',
					'name' => $fieldname,
					'value' => oldHtmlspecialchars($orgVal),
					'xml' => $xml
				]);
				return (($type != 'hidden') ? $content : '') . $hidden;
		}
	}
	switch($type){
		case 'img' :
			$imgDataId = we_base_request::_(we_base_request::HTML, 'WE_UI_IMG_DATA_ID_' . $name, md5(uniqid(__FUNCTION__, true)));

			if($editable){
				if(($foo = attributFehltError($attribs, 'parentid', __FUNCTION__))){
					return $foo;
				}

				if(!isset($_SESSION[$imgDataId])){
					$_SESSION[$imgDataId] = [];
				}
				$_SESSION[$imgDataId]['parentid'] = weTag_getAttribute('parentid', $attribs, 0, we_base_request::INT);
				//$_SESSION[$imgDataId]['maxfilesize'] = weTag_getAttribute('maxfilesize',$attribs);
				$_SESSION[$imgDataId]['width'] = weTag_getAttribute('width', $attribs, 0, we_base_request::INT);
				$_SESSION[$imgDataId]['height'] = weTag_getAttribute('height', $attribs, 0, we_base_request::INT);
				$_SESSION[$imgDataId]['quality'] = weTag_getAttribute('quality', $attribs, 8, we_base_request::INT);
				$_SESSION[$imgDataId]['keepratio'] = weTag_getAttribute('keepratio', $attribs, true, we_base_request::BOOL);
				$_SESSION[$imgDataId]['maximize'] = weTag_getAttribute('maximize', $attribs, false, we_base_request::BOOL);
				$_SESSION[$imgDataId]['id'] = $orgVal ?: '';

				$bordercolor = weTag_getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
				$checkboxstyle = weTag_getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
				$inputstyle = weTag_getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
				$checkboxclass = weTag_getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
				$inputclass = weTag_getAttribute('inputclass', $attribs, '', we_base_request::STRING);
				$checkboxtext = weTag_getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::STRING);

				if($_SESSION[$imgDataId]['id']){
					$attribs['id'] = $_SESSION[$imgDataId]['id'];
				}

				if(isset($_SESSION[$imgDataId]['serverPath']) && strpos($_SESSION[$imgDataId]['serverPath'], TEMP_PATH) === false){
					$src = '/' . ltrim(substr($_SESSION[$imgDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');

					$imgTag = '<img src="' . $src . '" alt="" width="' . $_SESSION[$imgDataId]["imgwidth"] . '" height="' . $_SESSION[$imgDataId]["imgheight"] . '" />';
				} else {
					unset($attribs['width'], $attribs['height']);
					$imgTag = $GLOBALS['we_doc']->getField($attribs, 'img');
				}

				$checked = (!empty($_SESSION[$imgDataId]['doDelete'])) ? ' checked' : '';
				$inputstyle = ($size ? 'width:' . $size . 'em;' . $inputstyle : $inputstyle);
				return '<table class="weEditTable" style="border: 1px solid ' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $imgTag . '
			<input type="hidden" name="WE_UI_IMG_DATA_ID_' . $name . '" value="' . $imgDataId . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<input name="' . $fieldname . '" type="file" accept="' . implode(',', we_base_ContentTypes::getRealContentTypes(we_base_ContentTypes::IMAGE)) . '"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . '/>
		</td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;"><input style="border:0 solid black;" type="checkbox" id="WE_UI_DEL_CHECKBOX_' . $name . '" name="WE_UI_DEL_CHECKBOX_' . $name . '" value="1" ' . $checked . '/></td>
					<td><label for="WE_UI_DEL_CHECKBOX_' . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label></td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
			}

			$hidden = '<input type="hidden" name="WE_UI_IMG_DATA_ID_' . $name . '" value="' . $imgDataId . '" />';

			if(isset($_SESSION[$imgDataId]['serverPath'])){
				$src = '/' . ltrim(substr($_SESSION[$imgDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');

				return getHtmlTag('img', ['src' => $src,
						'alt' => '',
						'style' => 'width:' . $_SESSION[$imgDataId]["imgwidth"] . 'px;height:' . $_SESSION[$imgDataId]["imgheight"] . 'px',
					]) . $hidden;
			}

			if(!empty($_SESSION[$imgDataId]['doDelete'])){
				return $hidden;
			}

			if((!empty($_SESSION[$imgDataId]['id'])) || (isset($orgVal) && $orgVal)){//Fix #9835
				unset($attribs['width'], $attribs['height']);
				$attribs['id'] = $_SESSION[$imgDataId]['id'] ? $_SESSION[$imgDataId]['id'] : $orgVal;
				return $GLOBALS['we_doc']->getField($attribs, 'img') . $hidden;
			}

			return '';

		case 'flashmovie' :
			$flashmovieDataId = we_base_request::_(we_base_request::HTML, 'WE_UI_FLASHMOVIE_DATA_ID_' . $name, md5(uniqid(__FUNCTION__, true)));

			if($editable){
				if(($foo = attributFehltError($attribs, 'parentid', __FUNCTION__))){
					return $foo;
				}

				if(!isset($_SESSION[$flashmovieDataId])){
					$_SESSION[$flashmovieDataId] = [];
				}
				$_SESSION[$flashmovieDataId]['parentid'] = weTag_getAttribute('parentid', $attribs, 0, we_base_request::INT);
				//$_SESSION[$imgDataId]['maxfilesize'] = weTag_getAttribute('maxfilesize',$attribs);
				$_SESSION[$flashmovieDataId]['width'] = weTag_getAttribute('width', $attribs, 0, we_base_request::INT);
				$_SESSION[$flashmovieDataId]['height'] = weTag_getAttribute('height', $attribs, 0, we_base_request::INT);
				$_SESSION[$flashmovieDataId]['id'] = $orgVal ?: '';

				$bordercolor = weTag_getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
				$checkboxstyle = weTag_getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
				$inputstyle = weTag_getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
				$checkboxclass = weTag_getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
				$inputclass = weTag_getAttribute('inputclass', $attribs, '', we_base_request::STRING);
				$checkboxtext = weTag_getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::RAW);

				if($_SESSION[$flashmovieDataId]['id']){
					$attribs['id'] = $_SESSION[$flashmovieDataId]['id'];
				}

				if(isset($_SESSION[$flashmovieDataId]['serverPath'])){
					$src = '/' . ltrim(substr($_SESSION[$flashmovieDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');
					$flashmovieTag = '';
				} else {
					$attribs = removeAttribs($attribs, ['width', 'height']);
					// Include Flash class
					$flashmovieTag = (!empty($attribs['id']) ?
						$GLOBALS['we_doc']->getField($attribs, 'flashmovie') :
						'<img src="' . ICON_DIR . 'no_flashmovie.gif" alt="" width="64" height="64" />');
				}

				$checked = (!empty($_SESSION[$flashmovieDataId]['doDelete']) ? ' checked' : '');
				$inputstyle = ($size ? 'width:' . $size . 'em;' . $inputstyle : $inputstyle);
				return '<table class="weEditTable" style="border: 1px solid ' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $flashmovieTag . '<input type="hidden" name="WE_UI_FLASHMOVIE_DATA_ID_' . $name . '" value="' . $flashmovieDataId . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left"><input name="' . $fieldname . '" type="file" accept="application/x-shockwave-flash"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . '/></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;"><input style="border:0 solid black;" type="checkbox" id="WE_UI_DEL_CHECKBOX_' . $name . '" name="WE_UI_DEL_CHECKBOX_' . $name . '" value="1" ' . $checked . '/></td>
					<td><label for="WE_UI_DEL_CHECKBOX_' . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label></td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
			}
			$hidden = '<input type="hidden" name="WE_UI_FLASHMOVIE_DATA_ID_' . $name . '" value="' . $flashmovieDataId . '" />';

			if(isset($_SESSION[$flashmovieDataId]['serverPath'])){
				$src = '/' . ltrim(substr($_SESSION[$flashmovieDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');

				return $hidden;
			}
			if(!empty($_SESSION[$flashmovieDataId]['id'])){

				if(!empty($_SESSION[$flashmovieDataId]['doDelete'])){
					return $hidden;
				}
				$attribs = removeAttribs($attribs, ['width', 'height']);
				$attribs['id'] = $_SESSION[$flashmovieDataId]['id'];
				return $GLOBALS['we_doc']->getField($attribs, 'flashmovie') . $hidden;
			}

			return '';

		case 'binary' :
			$binaryDataId = we_base_request::_(we_base_request::HTML, 'WE_UI_BINARY_DATA_ID_' . $name, md5(uniqid(__FUNCTION__, true)));

			if($editable){
				if(($foo = attributFehltError($attribs, 'parentid', __FUNCTION__))){
					return $foo;
				}

				if(!isset($_SESSION[$binaryDataId])){
					$_SESSION[$binaryDataId] = [];
				}
				$_SESSION[$binaryDataId]['parentid'] = weTag_getAttribute('parentid', $attribs, 0, we_base_request::INT);
				//$_SESSION[$binaryDataId]['maxfilesize'] = weTag_getAttribute('maxfilesize',$attribs);

				$_SESSION[$binaryDataId]['id'] = $orgVal ?: '';

				$bordercolor = weTag_getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
				$checkboxstyle = weTag_getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
				$inputstyle = weTag_getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
				$checkboxclass = weTag_getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
				$inputclass = weTag_getAttribute('inputclass', $attribs, '', we_base_request::STRING);
				$checkboxtext = weTag_getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::RAW);

				if($_SESSION[$binaryDataId]['id']){
					$attribs['id'] = $_SESSION[$binaryDataId]['id'];
				}

				if(isset($_SESSION[$binaryDataId]['serverPath']) && strpos($_SESSION[$binaryDataId]['serverPath'], TEMP_PATH) === false){
					$src = '/' . ltrim(substr($_SESSION[$binaryDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');
					$imgTag = '<a href="' . $src . '" target="_blank">' . $src . '</a>';
				} else {
					$binaryTag = $GLOBALS['we_doc']->getField($attribs, 'binary');
					$t = explode('_', $binaryTag[0]);
					unset($t[1], $t[0]);
					$fn = implode('_', $t);
					$imgTag = '<a href="' . $binaryTag[1] . '" target="_blank">' . $fn . '</a>';
				}

				$checked = (!empty($_SESSION[$binaryDataId]['doDelete']) ? ' checked' : '');
				$inputstyle = ($size ? 'width:' . $size . 'em;' : '') . $inputstyle;
				return '<table class="weEditTable" style="border: 1px solid ' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' . $imgTag . '
			<input type="hidden" name="WE_UI_BINARY_DATA_ID_' . $name . '" value="' . $binaryDataId . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<input name="' . $fieldname . '" type="file" accept="application/*"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . '/>
		</td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;"><input style="border:0 solid black;" type="checkbox" id="WE_UI_DEL_CHECKBOX_' . $name . '" name="WE_UI_DEL_CHECKBOX_' . $name . '" value="1" ' . $checked . '/></td>
					<td><label for="WE_UI_DEL_CHECKBOX_' . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label></td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
			}
			if(!isset($_SESSION[$binaryDataId])){
				$_SESSION[$binaryDataId] = [];
			}
			$_SESSION[$binaryDataId]['id'] = $orgVal ?: '';
			if($_SESSION[$binaryDataId]['id']){
				$attribs['id'] = $_SESSION[$binaryDataId]['id'];
			}
			$hidden = '<input type="hidden" name="WE_UI_BINARY_DATA_ID_' . $name . '" value="' . $binaryDataId . '" />';

			if(isset($_SESSION[$binaryDataId]["serverPath"])){
				$src = '/' . ltrim(substr($_SESSION[$binaryDataId]['serverPath'], strlen($_SERVER['DOCUMENT_ROOT'])), '/');
				return $hidden;
			}
			if(empty($_SESSION[$binaryDataId]['id'])){
				return '';
			}
			if(!empty($_SESSION[$binaryDataId]['doDelete'])){
				return $hidden;
			}

			$attribs['id'] = $_SESSION[$binaryDataId]['id'];
			$binaryTag = $GLOBALS['we_doc']->getField($attribs, 'binary');
			$t = explode('_', $binaryTag[0]);
			unset($t[1], $t[0]);
			$fn = implode('_', $t);
			$imgTag = '<a href="' . $binaryTag[1] . '" target="_blank">' . $fn . '</a>';
			return $imgTag . $hidden;

		case 'textarea' :
			if(weTag_getAttribute('pure', $attribs, false, we_base_request::BOOL)){
				$atts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name',
					'values', 'hidden', 'editable', 'format', 'property', 'size', 'maxlength', 'width', 'height', 'fontnames',
					'bgcolor', 'editorcss', 'ignoredocumentcss', 'buttonpos', '_name_orig', 'hideautobr'
				]);
				return getHtmlTag('textarea', $atts, oldHtmlspecialchars(($content ?: $value)), true);
			}

			//$autobr = $autobrAttr ? 'on' : 'off';
			//$showAutobr = isset($attribs['autobr']);
			$charset = weTag_getAttribute('charset', $attribs, 'iso-8859-1', we_base_request::STRING);

			return (!$inlineedit ? STYLESHEET_MINIMAL : '') .
				we_html_forms::weTextarea($fieldname, ($content ?: $value), $attribs, false, '', false, false, $xml, $removeFirstParagraph, $charset, true, $name);
		case 'checkbox' :
			$atts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values',
				'hidden', 'editable', 'format', 'property', 'cols', 'rows', 'width', 'height', 'bgcolor', 'fontnames']);
			if((!$isset) && $checked){
				$content = 1;
			}
			return we_getInputCheckboxField($fieldname, $content, $atts);
		case 'date' :
			if($hidden){
				$attsHidden = ['type' => 'hidden',
					'name' => $fieldname,
					'value' => $orgVal ?: time(),
					'xml' => $xml
				];
				return getHtmlTag('input', $attsHidden);
			}
			$currentdate = weTag_getAttribute('currentdate', $attribs, false, we_base_request::BOOL);
			$orgVal = $orgVal ?: ($currentdate ? time() : 0);
			$year = date('Y');
			$minyear = weTag_getAttribute('minyear', $attribs, 0, we_base_request::INT);
			switch($minyear ? $minyear{0} : ''){
				case '+':
					$minyear = $year + intval(substr($minyear, 1));
					break;
				case '-':
					$minyear = $year - intval(substr($minyear, 1));
					break;
				default:
					$minyear = intval($minyear);
					break;
			}
			$maxyear = weTag_getAttribute('maxyear', $attribs, 0, we_base_request::INT);
			switch($maxyear ? $maxyear{0} : ''){
				case '+':
					$maxyear = $year + intval(substr($maxyear, 1));
					break;
				case '-':
					$maxyear = $year - intval(substr($maxyear, 1));
					break;
				default:
					$maxyear = intval($maxyear);
					break;
			}
			return we_html_tools::getDateInput('we_ui_' . (isset($GLOBALS['WE_FORM']) ? $GLOBALS['WE_FORM'] : '') . '[we_date_' . $name . ']', $orgVal, false, $format, '', '', $xml, $minyear, $maxyear);

		case 'country':
			$newAtts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden', 'editable', 'format', 'property',
				'rows', 'cols', 'fontnames', 'bgcolor', 'width', 'height', 'maxlength']);
			$docAttr = weTag_getAttribute('doc', $attribs, 'self', we_base_request::STRING);

			$doc = we_getDocForTag($docAttr);
			$lang = $doc->Language;
			$langcode = ($lang ?
				substr($lang, 0, 2) :
				array_search($GLOBALS['WE_LANGUAGE'], getWELangs()));


			if(WE_COUNTRIES_TOP){
				$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
				foreach($topCountries as $countrykey => &$countryvalue){
					$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
				}
				unset($countryvalue);
			} else {
				$topCountries = [];
			}
			if(WE_COUNTRIES_SHOWN){
				$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
				foreach($shownCountries as $countrykey => &$countryvalue){
					$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
				}
				unset($countryvalue);
			} else {
				$shownCountries = [];
			}
			$oldLocale = setlocale(LC_ALL, NULL);
			setlocale(LC_ALL, $lang . '.UTF-8');
			asort($topCountries, SORT_LOCALE_STRING);
			asort($shownCountries, SORT_LOCALE_STRING);
			setlocale(LC_ALL, $oldLocale);

			$options = '';
			if(WE_COUNTRIES_DEFAULT != ''){
				$options .= '<option value="--" ' . ($orgVal === '--' ? ' selected="selected">' : '>') . WE_COUNTRIES_DEFAULT . '</option>';
			}
			foreach($topCountries as $countrykey => &$countryvalue){
				$options .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($countryvalue) . '</option>';
			}
			if(!empty($topCountries) && !empty($shownCountries)){
				$options .= '<option value="-" disabled="disabled">----</option>';
			}
			foreach($shownCountries as $countrykey2 => &$countryvalue2){
				$options .= '<option value="' . $countrykey2 . '" ' . ($orgVal == $countrykey2 ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($countryvalue2) . '</option>';
			}
			unset($countryvalue, $countryvalue2);
			$newAtts['size'] = (isset($atts['size']) ? $atts['size'] : 1);
			$newAtts['name'] = $fieldname;
			return getHtmlTag('select', $newAtts, $options, true);
		case 'language':
			$newAtts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden', 'editable', 'format', 'property',
				'rows', 'cols', 'fontnames', 'bgcolor', 'width', 'height', 'maxlength']);

			$docAttr = weTag_getAttribute('doc', $attribs, 'self', we_base_request::STRING);
			$doc = we_getDocForTag($docAttr);
			$lang = $doc->Language;
			$langcode = ($lang ?
				substr($lang, 0, 2) :
				array_search($GLOBALS['WE_LANGUAGE'], getWELangs()) );

			$frontendL = $GLOBALS['we']['lang']['FrontendLanguages'];
			foreach($frontendL as &$lcvalue){
				$lccode = explode('_', $lcvalue);
				$lcvalue = $lccode[0];
			}
			$frontendLL = [];
			foreach($frontendL as &$lcvalue){
				$frontendLL[$lcvalue] = we_base_country::getTranslation($lcvalue, we_base_country::LANGUAGE, $langcode);
			}

			$oldLocale = setlocale(LC_ALL, NULL);
			setlocale(LC_ALL, $lang . '.UTF-8');
			asort($frontendLL, SORT_LOCALE_STRING);
			setlocale(LC_ALL, $oldLocale);
			$options = '';
			foreach($frontendLL as $langkey => &$langvalue){
				$options .= '<option value="' . $langkey . '" ' . ($orgVal == $langkey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($langvalue) . '</option>';
			}
			unset($langvalue);
			$newAtts['size'] = (isset($atts['size']) ? $atts['size'] : 1);
			$newAtts['name'] = $fieldname;
			return getHtmlTag('select', $newAtts, $options, true);
		case 'select' :
			$options = '';
			$atts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values',
				'hidden', 'editable', 'format', 'property', 'rows', 'cols', 'fontnames', 'bgcolor', 'width', 'height', 'maxlength'
			]);
			if($values){
				foreach($values as $txt){
					$options .= getHtmlTag('option', ($txt == $orgVal ? ['selected' => 'selected'] : []), trim($txt), true);
				}
			} elseif($object && !empty($object->DefArray['meta_' . $name])){
				foreach($object->DefArray['meta_' . $name]['meta'] as $key => $val){
					$atts2 = ($orgVal == $key ? ['value' => $key, 'selected' => 'selected'] : ['value' => $key]);
					$attsOption = removeAttribs(array_merge($atts, $atts2), ['class']);
					$options .= getHtmlTag('option', $attsOption, $val, true);
				}
			}
			$atts['size'] = (empty($atts['size']) ? 1 : $atts['size']);
			$atts['name'] = $fieldname;
			return getHtmlTag('select', $atts, $options, true);
		case 'radio' :
			$atts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden',
				'editable', 'format', 'property', 'rows', 'cols', 'width', 'height', 'bgcolor', 'fontnames'
			]);
			return (!$isset ?
				we_getInputRadioField($fieldname, ($checked ? $value : $value . 'dummy'), $value, $atts) :
				we_getInputRadioField($fieldname, $orgVal, $value, $atts));

		case 'hidden':
			return getHtmlTag('input', ['type' => 'hidden', 'name' => $fieldname, 'value' => oldHtmlspecialchars(weTag_getAttribute('value', $attribs, $content, we_base_request::STRING)),
				'xml' => $xml,]);
		case 'print' :
			return $orgVal;
		case 'choice' :
			$atts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values',
				'hidden', 'editable', 'format', 'property', 'cols', 'rows', 'width', 'height', 'bgcolor', 'fontnames', 'maxlength'
			]);
			$mode = weTag_getAttribute('mode', $attribs, '', we_base_request::STRING);
			return we_html_tools::inputChoiceField($fieldname, $orgVal, $values, $atts, $mode);
		case 'password':
			$atts = array_merge(removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden', 'editable', 'format',
				'property', 'cols', 'rows', 'width', 'height', 'bgcolor', 'fontnames']), ['type' => 'password', 'name' => $fieldname, 'value' => oldHtmlspecialchars($orgVal)]
			);

			return getHtmlTag('input', $atts);
		case 'textinput':
		default :
			$atts = removeAttribs($attribs, ['wysiwyg', 'commands', 'pure', 'type', 'value', 'checked', 'autobr', 'name', 'values', 'hidden', 'editable', 'format', 'property',
				'cols', 'rows', 'width', 'height', 'bgcolor', 'fontnames']);
			return we_getInputTextInputField($fieldname, $orgVal, $atts);
	}
}
